
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const SH3Hardware_cxx_Version =
    "$Id: SH3Hardware.cxx,v 1.22 2001/01/23 22:22:34 cktam Exp $";
/* stdlib */
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "VTime.hxx"
#include "voip.h"
#include "hitachiUser.h"    // provides interface to Hitachi's Libraries.
#include "SH3Hardware.h"
#include "ReqSig.h"

/** The following array provides mappings from MGCP encoded signals to
 ** the corresponding SH3Hardware signal.
 */
HardwareSignal SH3HardwareGenericSigOn[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericSigPatternDetected,
        SignalLocalRingbackStart,  //GenericSigRingbackTone,
        SignalNULL,  //GenericSigRingbackOnConnection,
        SignalNULL,  //GenericSigConfirmTone,
        SignalNULL,  //GenericSigNetworkCongestionTone,
        SignalNULL,  //GenericSigInterceptTone,
        SignalNULL,  //GenericSigPreemptionTone,
    };

HardwareSignal SH3HardwareGenericSigOff[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericSigPatternDetected,
        SignalLocalRingbackStop,  //GenericSigRingbackTone,
        SignalNULL,  //GenericSigRingbackOnConnection,
        SignalNULL,  //GenericSigConfirmTone,
        SignalNULL,  //GenericSigNetworkCongestionTone,
        SignalNULL,  //GenericSigInterceptTone,
        SignalNULL,  //GenericSigPreemptionTone,
    };

HardwareSignal SH3HardwareLineSigOn[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineSigAdsiDisplay,
        SignalNULL,  // LineSigVisualMsg,
        SignalNULL,  // LineSigAnswerTone,
        SignalBusyStart,  // LineSigBusyTone,
        SignalNULL,  // LineSigCallerId,
        SignalNULL,  // LineSigCallWaitTone,
        SignalDialToneStart,  // LineSigDialTone,
        SignalNULL,  // LineSigMsgWaitInd,
        SignalNULL,  // LineSigNetworkBusy,
        SignalRingStart,  // LineSigRinging,
        SignalNULL,  // LineSigDsnctRing0,
        SignalNULL,  // LineSigDsnctRing1,
        SignalNULL,  // LineSigDsnctRing2,
        SignalNULL,  // LineSigDsnctRing3,
        SignalNULL,  // LineSigDsnctRing4,
        SignalNULL,  // LineSigDsnctRing5,
        SignalNULL,  // LineSigDsnctRing6,
        SignalNULL,  // LineSigDsnctRing7,
        SignalNULL,  // LineSigRingSplash,
        SignalNULL,  // LineSigPromptTone,
        SignalNULL,  // LineSigErrorTone,
        SignalNULL,  // LineSigStutterTone,
        SignalNULL,  // LineSigAlertTone,
        SignalNULL,  // LineSigRecordWarnTone,
        SignalNULL,  // LineSigCallCardSrvTone,
        SignalNULL,  // LineSigOffhookWarnTone,
        SignalNULL,  // LineSigDsnctTonePattern,
    };

HardwareSignal SH3HardwareLineSigOff[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineSigAdsiDisplay,
        SignalNULL,  // LineSigVisualMsg,
        SignalNULL,  // LineSigAnswerTone,
        SignalBusyStop,  // LineSigBusyTone,
        SignalNULL,  // LineSigCallerId,
        SignalNULL,  // LineSigCallWaitTone,
        SignalDialToneStop,  // LineSigDialTone,
        SignalNULL,  // LineSigMsgWaitInd,
        SignalNULL,  // LineSigNetworkBusy,
        SignalRingStop,  // LineSigRinging,
        SignalNULL,  // LineSigDsnctRing0,
        SignalNULL,  // LineSigDsnctRing1,
        SignalNULL,  // LineSigDsnctRing2,
        SignalNULL,  // LineSigDsnctRing3,
        SignalNULL,  // LineSigDsnctRing4,
        SignalNULL,  // LineSigDsnctRing5,
        SignalNULL,  // LineSigDsnctRing6,
        SignalNULL,  // LineSigDsnctRing7,
        SignalNULL,  // LineSigRingSplash,
        SignalNULL,  // LineSigPromptTone,
        SignalNULL,  // LineSigErrorTone,
        SignalNULL,  // LineSigStutterTone,
        SignalNULL,  // LineSigAlertTone,
        SignalNULL,  // LineSigRecordWarnTone,
        SignalNULL,  // LineSigCallCardSrvTone,
        SignalNULL,  // LineSigOffhookWarnTone,
        SignalNULL,  // LineSigDsnctTonePattern,
    };

HardwareSignal SH3HardwareDtmfSigOn[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfSig0,
        SignalNULL,  //DtmfSig1,
        SignalNULL,  //DtmfSig2,
        SignalNULL,  //DtmfSig3,
        SignalNULL,  //DtmfSig4,
        SignalNULL,  //DtmfSig5,
        SignalNULL,  //DtmfSig6,
        SignalNULL,  //DtmfSig7,
        SignalNULL,  //DtmfSig8,
        SignalNULL,  //DtmfSig9,
        SignalNULL,  //DtmfSigHash,
        SignalNULL,  //DtmfSigStar,
        SignalNULL,  //DtmfSigA,
        SignalNULL,  //DtmfSigB,
        SignalNULL,  //DtmfSigC,
        SignalNULL,  //DtmfSigD
    };

HardwareSignal SH3HardwareDtmfSigOff[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfSig0,
        SignalNULL,  //DtmfSig1,
        SignalNULL,  //DtmfSig2,
        SignalNULL,  //DtmfSig3,
        SignalNULL,  //DtmfSig4,
        SignalNULL,  //DtmfSig5,
        SignalNULL,  //DtmfSig6,
        SignalNULL,  //DtmfSig7,
        SignalNULL,  //DtmfSig8,
        SignalNULL,  //DtmfSig9,
        SignalNULL,  //DtmfSigHash,
        SignalNULL,  //DtmfSigStar,
        SignalNULL,  //DtmfSigA,
        SignalNULL,  //DtmfSigB,
        SignalNULL,  //DtmfSigC,
        SignalNULL,  //DtmfSigD
    };



const int SLIC_DEBOUNCE_DELAY = 20;   // 20 ms


// 10ms * 8 samples/ms = 80
// 20ms * 8 samples/ms = 160
// 30ms * 8 samples/ms = 240

#define ULAW_PAYLOAD           160

// Change the RTP Rate based on the codec that is used.
// The changes here are strictly for the SH3 hardware platform.

// for linear16
#define RESID_RTP_RATE_160         160
#define NETWORK_RTP_RATE_160       160

// for g729A 
// #define RESID_RTP_RATE_80         80
// #define NETWORK_RTP_RATE_80       80


void*
deviceCallback(void* objectPtr)
{
    SH3Hardware* sh3HardwarePtr = static_cast < SH3Hardware * > (objectPtr);
    sh3HardwarePtr->deviceMain();
    return 0;
}


SH3Hardware::SH3Hardware ( const char* deviceName,
                           int numEndpoints,
                           Fifo < HardwareMsg > * inputQ,
                           Fifo < EndpointMsg > ** outputQ )
{
    VThread deviceThread;
    int errorcode;

    myFD = 0;
    myQ = inputQ;
    // myCodecType = CodecTypeULaw;
    myChannel[0].myCodecType = CodecTypeG729A;
    myChannel[1].myCodecType = CodecTypeG729A;

    myNumEndpoints = numEndpoints;

    // initialize the audio parameters
    for (int i = 0; i < myNumEndpoints; i++)
    {
        endpointQ[i] = outputQ[i];
        myChannel[i].audioActive = false;
        myChannel[i].audioHalfActive = true;
        myChannel[i].hasPlayed = false;


        // initialize the hook state

        if (checkHookState(i) == 0 )
        {
            myChannel[i].hookStateOffhook = false;
        }
        else
        {
            myChannel[i].hookStateOffhook = true;
        }

        setToneId(i, SILENCE);

    }


    // create Fifo queue and a new device thread
    // (for handling dtmf detection, tone generation, and audio).

    deviceQ = new Fifo < SH3DeviceMsg > ;

    errorcode = deviceThread.spawn(deviceCallback,
                                   (void *) this);

}  


SH3Hardware::~SH3Hardware (void)
{
    if (deviceQ)
    {
        delete deviceQ;
    }
}


void*
SH3Hardware::hardwareMain (void* parms)
{
    // process forever on behalf of null hardware
    fd_set readfds;
    struct timeval tv;
    bool operational = true;
    int maxFd = 32;
    int retval = 0;


    while (operational)
    {
        // reset file descriptor
        FD_ZERO(&readfds);
        addToFdSet(&readfds);

        // block on select but timeout needed to service myQ and deviceQ
        // Note:  The times used here can be change for blocking faster or
        //        slower.
        tv.tv_sec = 0;
        tv.tv_usec = 30;

        if ((retval = select(maxFd, &readfds, NULL, NULL, &tv)) < 0)
        {
            cpLog(LOG_ERR, "Select returned with an error - ignore");
        }
        else
        {
            // hardware input received
            if (process(&readfds) < 0)
            {
                cpLog(LOG_ERR, "Hardware failed - shutting down");
                operational = false;
            }
        }


        // process device msgs
        while (deviceQ->size() > 0)
        {
            processDeviceMsg(deviceQ->getNext());
        }

        // process msgs from endpoints (if any)
        while (myQ->size() > 0)
        {
            processEndpointMsg(myQ->getNext());
        }

    } // end while

    return 0;

}


int
SH3Hardware::process (fd_set* fd)
{
    // check hook state and report changes to endpoint.

    for (int i = 0; i < myNumEndpoints; i ++)
    {
        if (checkHookState(i) == 0 )
        {
            // Onhook.  Report change from offhook to onhook.

            if (myChannel[i].hookStateOffhook == true)
            {
                myChannel[i].hookStateOffhook = false;
                reportEvent(endpointQ[i], EventOnhook);
            }
        }
        else
        {
            // Offhook.  Report change from onhook to offhook

            if (myChannel[i].hookStateOffhook == false)
            {
                myChannel[i].hookStateOffhook = true;
                reportEvent(endpointQ[i], EventOffhook);
            }
        }
    }

    return 0;
} 



void
SH3Hardware::deviceMain()
{
    SH3DeviceMsg msg;
    int dtmfDigit = 0;

    initSH3();

    cpLog(LOG_DEBUG, "Device thread has started.\n");

    while (1)
    {
        semTake(ch1InSem, WAIT_FOREVER);

        // check for dtmf and play tones only when audio is not active
        // and the phone is offhook.

        for (int i = 0; i < myNumEndpoints; i ++)
        {

            if (!myChannel[i].audioActive && myChannel[i].hookStateOffhook == true)
            {
                dtmfDigit = getDtmf(i);

                if (dtmfDigit != -1)
                {
                    cpLog(LOG_DEBUG, "*** dtmf thread got a digit!   <%d> ***\n",
                          dtmfDigit);

                    msg.type = SH3DigitType;
                    msg.channelNum = i;
                    msg.digit = dtmfDigit;


                    deviceQ->add(msg);

                    setToneId(i, SILENCE);
                }

                playTone(i);
            }

            // do rtp stuff if the audio is active.

            if (myChannel[i].audioActive)
            {
                if (!myChannel[i].hasPlayed)
                {
                    printf("Begin audio\n");
                    myChannel[i].hasPlayed = true;
                }
                processRTP(i);
            }

        }
    }
}




int
SH3Hardware::handleDigitEvent(int dtmfDigit, int channelNum)
{
    HardwareEvent event = EventNULL;

    switch (dtmfDigit)
    {

        case 1:
        {
            event = EventDigit1;
        }
        break;

        case 2:
        {
            event = EventDigit2;
        }
        break;

        case 3:
        {
            event = EventDigit3;
        }
        break;

        case 4:
        {
            event = EventDigit4;
        }
        break;

        case 5:
        {
            event = EventDigit5;
        }
        break;

        case 6:
        {
            event = EventDigit6;
        }
        break;

        case 7:
        {
            event = EventDigit7;
        }
        break;

        case 8:
        {
            event = EventDigit8;
        }
        break;

        case 9:
        {
            event = EventDigit9;
        }
        break;

        case 0:
        {
            event = EventDigit0;
        }
        break;

        case 14:
        {
            event = EventDigitStar;
        }
        break;

        case 15:
        {
            event = EventDigitHash;

        }
        break;

        default:
        {
            // no events
            break;
        }
    } 

    if (event != EventNULL)
    {
        reportEvent(endpointQ[channelNum], event);
    }

    return 0;
}


int
SH3Hardware::addToFdSet (fd_set* fd)
{
    FD_SET(myFD, fd);
    return 0;
} 


int
SH3Hardware::sendSignal (HardwareSignal signal, int channelNum)
{
    switch (signal)
    {
        case SignalDialToneStart:
        {
            cpLog(LOG_INFO, "###   Start dialtone   ###\n");
            provideTone(DIAL_TONE, channelNum);
        }
        break;

        case SignalDialToneStop:
        {
            cpLog(LOG_INFO, "###   Stop dialtone   ###\n");
            provideTone(SILENCE, channelNum);
        }
        break;

        case SignalRingStart:
        {
            cpLog(LOG_INFO, "###   Start ringing   ###\n");
            while (checkHookState(channelNum) == 0 &&
                    !myQ->size())
            {
                ringChannel(channelNum);
            }
        }
        break;

        case SignalRingStop:
        {
            cpLog(LOG_INFO, "###   Stop ringing   ###\n");
            stopRingChannel(channelNum);
        }
        break;

        case SignalLocalRingbackStart:
        {
            cpLog(LOG_INFO, "###   Start local ringback   ###\n");
            provideTone(AUDRING_TONE, channelNum);
        }
        break;

        case SignalLocalRingbackStop:
        {
            cpLog(LOG_INFO, "###   Stop local ringback   ###\n");
            provideTone(SILENCE, channelNum);
        }
        break;

        case SignalBusyStart:
        {
            cpLog(LOG_INFO, "###   Start busy start   ###\n");
            provideTone(BUSY_TONE, channelNum);
        }
        break;

        case SignalBusyStop:
        {
            cpLog(LOG_INFO, "###   Stop busy stop   ###\n");
            provideTone(SILENCE, channelNum);
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unsupported hardware signal = %d", signal);
        }
        break;
    } 

    return 0;

} 




void
SH3Hardware::provideTone(int toneId, int channelNum)
{
    // Make sure that we are offhook before doing anything.
    printf("provideTone:  toneId = %d, channelNum = %d\n", toneId, channelNum);

    printf("myChannel:  audioActive = %d, hookState = %d\n",
           myChannel[channelNum].audioActive, checkHookState(channelNum) );

    if (!myChannel[channelNum].audioActive && checkHookState(channelNum) == 1)
    {
        setToneId(channelNum, toneId);
    }
    else
    {
        printf("Didn't set the tone!\n");
    }

} 


void
SH3Hardware::processRTP(int channelNum)
{
    // if the audioStack and the outRtpPkt has not been created yet
    // then return.  Audio is not ready yet.

    if ((myChannel[channelNum].audioStack == 0) &&
            (myChannel[channelNum].outRtpPkt == 0)) return ;

    // Get packet from network (rtp session)

    if (myChannel[channelNum].audioStack)
    {
        myChannel[channelNum].inRtpPkt =
            myChannel[channelNum].audioStack->receive();
    }

    // Write packet to the device

    if (myChannel[channelNum].inRtpPkt)
    {
        processInPkt(myChannel[channelNum].inRtpPkt->getPayloadLoc(),
                     channelNum);
    }
    else
    {
        cpLog(LOG_DEBUG, "inRtpPkt doesn't exist!!!");
    }

    // Get packet from device

    if (myChannel[channelNum].outRtpPkt)
    {
        processOutPkt(myChannel[channelNum].outRtpPkt->getPayloadLoc(),
                      channelNum);
    }
    else
    {
        cpLog(LOG_DEBUG, "outRtpPkt doesn't exist!!!");
    }

    // Transmit the packet to the network

    if ( (myChannel[channelNum].outRtpPkt) &&
            (myChannel[channelNum].audioStack) )
    {
        if (myChannel[channelNum].myCodecType == CodecTypeG729A)
        {
            myChannel[channelNum].outRtpPkt->setPayloadUsage (10);
            myChannel[channelNum].audioStack->transmit (myChannel[channelNum].outRtpPkt);

        }
        else
        {
            myChannel[channelNum].outRtpPkt->setPayloadUsage (160);
            myChannel[channelNum].audioStack->transmit (myChannel[channelNum].outRtpPkt);
        }

    }

}

void
SH3Hardware::processInPkt(char* inBuffer, int channelNum)
{
    if (myChannel[channelNum].myCodecType == CodecTypeG729A)
    {
        writeDevicePkt(channelNum,
                       inBuffer,
                       10);

    }
    else
    {
        writeDevicePkt(channelNum,
                       inBuffer,
                       160);
    }

}


void
SH3Hardware::processOutPkt(char* outBuffer, int channelNum)
{
    // always 160 because that's the size that the device reads.

    readDevicePkt(channelNum,
                  outBuffer );
}


void
SH3Hardware::processDeviceMsg(const SH3DeviceMsg& deviceMsg)
{
    cpLog(LOG_DEBUG, "SH3HW:  Got Device Msg\n");
    switch (deviceMsg.type)
    {
        case SH3DigitType:
        {
            handleDigitEvent(deviceMsg.digit, deviceMsg.channelNum);
        }
        break;

        default:
        {
            cpLog(LOG_ERR, "Unknown SH3 Device Event");
        }
        break;
    }
}


int
SH3Hardware::audioStart (const HardwareAudioRequest& request,
                         const int channelNum)
{
    RtpPayloadType apiPayloadType;
    RtpPayloadType networkPayloadType;

    cpLog(LOG_INFO, "### Establishing audio ###\n");

    cpLog(LOG_INFO, "Listening on port: %d ", request.localPort);
    cpLog(LOG_INFO, "Sending to host:   %s ", request.remoteHost);
    cpLog(LOG_INFO, "Sending to port:   %d ", request.remotePort);
    cpLog(LOG_INFO, "Using codec:       %d ", request.codecType);

    // initialize the device

    setToneId(channelNum, SILENCE);


    // Initialize codec and rtp parameters according to SDP info.

    switch (request.codecType)
    {
        case 0:
        {
            cpLog(LOG_DEBUG, "*** ULAW ***\n");

            // Need to convert to ULAW
            setCodecType(channelNum, SH3CodecTypeLinear16);
            apiPayloadType = rtpPayloadL16_mono;
            networkPayloadType = rtpPayloadPCMU;

        }
        break;

        case 11:
        {
            cpLog(LOG_DEBUG, "*** Linear16 ***\n");

            setCodecType(channelNum, SH3CodecTypeLinear16);
            apiPayloadType = rtpPayloadL16_mono;
            networkPayloadType = rtpPayloadL16_mono;

        }
        break;

        case 18:
        {
            cpLog(LOG_DEBUG, "*** G729A ***\n");
            initG729A(channelNum);
            setCodecType(channelNum, SH3CodecTypeG729A);

            apiPayloadType = rtpPayloadG729;
            networkPayloadType = rtpPayloadG729;

        }
        break;

        default:
        {
            cpLog(LOG_DEBUG, "No codecs specified... using ULAW\n");

            // Need to convert to ULAW
            setCodecType(channelNum, SH3CodecTypeLinear16);
            apiPayloadType = rtpPayloadL16_mono;
            networkPayloadType = rtpPayloadPCMU;
        }
        break;
    }

    // creating new RTP session

    myChannel[channelNum].audioStack = new RtpSession(request.remoteHost,
                                       request.remotePort,
                                       request.localPort,
                                       request.remotePort + 1,
                                       request.localPort + 1,
                                       apiPayloadType,
                                       networkPayloadType,
                                       5);

    // allocate RTP packet spaces
    if (myChannel[channelNum].myCodecType == CodecTypeG729A)
    {
        myChannel[channelNum].outRtpPkt =
            myChannel[channelNum].audioStack->createPacket(10);
        myChannel[channelNum].audioStack->setApiPktSize (10);
        myChannel[channelNum].audioStack->setNetworkPktSize (10);
    }
    else
    {
        myChannel[channelNum].outRtpPkt =
            myChannel[channelNum].audioStack->createPacket(ULAW_PAYLOAD * 2);
        myChannel[channelNum].audioStack->setApiPktSize (RESID_RTP_RATE_160);
        myChannel[channelNum].audioStack->setNetworkPktSize (NETWORK_RTP_RATE_160);
    }


    assert(myChannel[channelNum].outRtpPkt);
    myChannel[channelNum].inRtpPkt = 0;

    // always turn echo cancellation on.
    setEchoCancel(channelNum);

    // mark audio as active
    myChannel[channelNum].audioActive = true;
    myChannel[channelNum].audioHalfActive = true;
    myChannel[channelNum].hasPlayed = false;

    rngFlush(ch1InBufCodec);
    rngFlush(ch1InBufNet);
    rngFlush(ch2InBufCodec);
    rngFlush(ch2InBufNet);

    return 0;
}


int
SH3Hardware::audioStop (const int channelNum)
{
    printf("### Stopping audio ###\n");

    // audio is not active!  do nothing.
    if (!myChannel[channelNum].audioActive)
    {
        return 1;
    }

    myChannel[channelNum].audioActive = false;
    myChannel[channelNum].audioHalfActive = true;
    myChannel[channelNum].hasPlayed = false;

    // wait 10ms before cleaning up.  Need to wait until the rtp stuff
    // is done before deleting.
    taskDelay(10);


    // close RTP session, delete rtp stack & packets.

    myChannel[channelNum].audioStack->transmitRTCPBYE();

    if (myChannel[channelNum].outRtpPkt)
    {
        delete myChannel[channelNum].outRtpPkt;
        myChannel[channelNum].outRtpPkt = 0;
    }

    if (myChannel[channelNum].audioStack)
    {
        delete myChannel[channelNum].audioStack;
        myChannel[channelNum].audioStack = 0;
    }

    if (myChannel[channelNum].inRtpPkt)
    {
        delete myChannel[channelNum].inRtpPkt;
        myChannel[channelNum].inRtpPkt = 0;
    }

    return 0;
}



HardwareSignal
SH3Hardware::getSignalOn (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return SH3HardwareGenericSigOn[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return SH3HardwareLineSigOn[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return SH3HardwareDtmfSigOn[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unknown package = %d", package);
            return SignalNULL;
        }
        break;
    } 

    return SignalNULL;

} 


HardwareSignal
SH3Hardware::getSignalOff (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return SH3HardwareGenericSigOff[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return SH3HardwareLineSigOff[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return SH3HardwareDtmfSigOff[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cpLog(LOG_ERR, "Unknown package = %d", package);
            return SignalNULL;

        }
        break;
    } 

    return SignalNULL;

} 
