/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpException_cxx_Version = 
    "$Id: HttpException.cxx,v 1.6 2001/11/05 18:07:36 icahoon Exp $";


#include "HttpException.hxx"
#include "Verify.hxx"


using Vocal::Http::Exception;


size_t  Exception::myTotal = 0;
        

const Exception::Error  Exception::Internal         = myTotal++;
const Exception::Error  Exception::Incomplete       = myTotal++;
const Exception::Error  Exception::IncompleteInBody = myTotal++;
const Exception::Error  Exception::BadRequestLine   = myTotal++;
const Exception::Error  Exception::BadMethod        = myTotal++;
const Exception::Error  Exception::BadStatus        = myTotal++;
const Exception::Error  Exception::BadUrl           = myTotal++;
const Exception::Error  Exception::BadVersion       = myTotal++;
const Exception::Error  Exception::BadHeader        = myTotal++;


const string Exception::InternalErrMsg 
    = "Internal error.";

const string Exception::IncompleteErrMsg 
    = "Incomplete HTTP message.";

const string Exception::IncompleteInBodyErrMsg 
    = "Incomplete HTTP message, while parsing message body.";

const string Exception::BadRequestLineErrMsg
    = "Malformed Request-Line or Status-Line.";

const string Exception::BadMethodErrMsg
    = "Malformed Method.";

const string Exception::BadStatusErrMsg
    = "Malformed Status.";

const string Exception::BadUrlErrMsg 
    = "Malformed HTTP Request-URI.";

const string Exception::BadVersionErrMsg
    = "Malformed HTTP version.";

const string Exception::BadHeaderErrMsg
    = "Malformed HTTP header.";


const string *  Exception::Msg[] =
{
    &InternalErrMsg,
    &IncompleteErrMsg,
    &IncompleteInBodyErrMsg,
    &BadRequestLineErrMsg,
    &BadMethodErrMsg,
    &BadStatusErrMsg,
    &BadUrlErrMsg,
    &BadVersionErrMsg,
    &BadHeaderErrMsg
};    


Exception::Exception(  
    Error 	    	    	error,
    const string    	    & 	file,
    const int 	    	    	line
) 
    :   VException( ( error < total() ? *Msg[error] : *Msg[Internal] ), 
                    file, line, static_cast<int>(error) ),
        myBytesRemaining(0)                    
{
    test_covered("0");
    
    assert( sizeof(Msg)/sizeof(Msg[0]) == total() );
}
    

string      	    	    
Exception::getName() const
{
    test_covered("1");

    return ( "Vocal::Http::Exception" );
}


Exception::Error
Exception::error() const
{
    test_covered("2");
    
    return ( static_cast<Error>(errorCode) );
}


const string &  
Exception::message() const
{
    Error   error = static_cast<Error>(errorCode);

    test_covered("3");

    return ( error < total() ? *Msg[error] : *Msg[Internal] );
}


void    
Exception::bytesRemaining(size_t bytes)
{
    if ( static_cast<Error>(errorCode) == IncompleteInBody )
    {
        myBytesRemaining = bytes;
        
        test_covered("4");
    }
}
        

size_t  
Exception::bytesRemaining() const
{
    test_covered("5");

    return ( myBytesRemaining );
}
        
        
size_t   
Exception::total()
{
    test_covered("6");

    return ( myTotal );
}
