/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpMessage_cxx_Version =
    "$Id: HttpMessage.cxx,v 1.7 2001/05/11 18:53:50 icahoon Exp $";


#include "HttpMessage.hxx"
#include "Http.hxx"
#include <strstream>
#include <cstdlib>


using Vocal::Http::Message;
using Vocal::Http::Method;
using Vocal::Http::Url;
using Vocal::Http::Version;
using Vocal::Http::Status;
using Vocal::Http::Header;
using Vocal::Http::Exception;
using Vocal::Http::CRLF;
using Vocal::Http::SP;
using Vocal::Http::WS;
using Vocal::Http::DOT;
using Vocal::Http::DIGIT;
using Vocal::Http::getToken;
using Vocal::Http::removeLeading;
using std::list;
using std::string;
using std::strstream;


Message::Message()
    :   myType(Unknown),
        myDirty(true)
{
}


Message::Message(
    const Method        &   pMethod,
    const Url           &   pUrl,
    const list<Header>  &   pHeaders,
    const string        &   pBody
)
    :   myType(Request),
        myMethod(pMethod),
        myUrl(pUrl),
        myHeaders(pHeaders),
        myBody(pBody),
        myDirty(true)
{
}


Message::Message(
    const Status        &   pStatus,
    const list<Header>  &   pHeaders,
    const string        &   pBody
)
    :   myType(Response),
        myStatus(pStatus),
        myHeaders(pHeaders),
        myBody(pBody),
        myDirty(true)
{
}


Message::Message(const Message & src)
    :   myMethod(src.myMethod),
        myUrl(src.myUrl),
        myVersion(src.myVersion),
        myStatus(src.myStatus),
        myHeaders(src.myHeaders),
        myBody(src.myBody),
        myDirty(src.myDirty)
{
}


Message::~Message()
{
}


const Message & 
Message::operator=(const Message & src)
{
    if ( this != &src )
    {
        myMethod    = src.myMethod;
        myUrl       = src.myUrl;
        myVersion   = src.myVersion;
        myStatus    = src.myStatus;
        myHeaders   = src.myHeaders;
        myBody      = src.myBody;
        myEncoded   = src.myEncoded;
        myDirty     = src.myDirty;
    }
    return ( *this );
}
        
        
bool 
Message::operator==(const Message & src)
{
    return  (   myType      == src.myType
            &&  myMethod    == src.myMethod
            &&  myUrl       == src.myUrl
            &&  myVersion   == src.myVersion
            &&  myStatus    == src.myStatus
            &&  myHeaders   == src.myHeaders
            &&  myBody      == src.myBody
            );
}

bool 
Message::operator<(const Message & src)
{
    switch ( myType )
    {
        case Request:
        {
            if  (   myType < src.myType
                &&  myMethod < src.myMethod
                &&  myUrl < src.myUrl
                &&  myVersion < src.myVersion
                &&  myHeaders < src.myHeaders
                &&  myBody < src.myBody
                )
            {
                return ( true );
            }
            break;
        }
        case Response:
        {
            if  (   myType < src.myType
                &&  myVersion < src.myVersion
                &&  myStatus < src.myStatus
                &&  myBody < src.myBody
                )
            {
                return ( true );
            }
            break;
        }
        default:
        {
            break;
        }
    }
    return ( false );
}
        

Message::Type    
Message::type() const
{
    return ( myType );
}


void
Message::type(Type pType)
{
    myType = pType;
    myDirty = true;
}


const Method &  
Message::method() const
{
    return ( myMethod );
}


void
Message::method(const Method & pMethod)
{
    myMethod  = pMethod;
    myDirty = true;
}


const Url &     
Message::url() const
{
    return ( myUrl );
}


void
Message::url(const Url & pUrl)
{
    myUrl = pUrl;
    myDirty = true;
}


const Version & 
Message::version() const
{
    return ( myVersion );
}
        
        
void
Message::version(const Version & pVersion)
{
    myVersion = pVersion;
    myDirty = true;
}


const Status&
Message::status() const
{
    return myStatus;
}

void  
Message::status(const Status & pStatus)
{
    myStatus = pStatus;
    myDirty = true;
}


const list<Header> &    
Message::headers() const
{
    return ( myHeaders );
}


void
Message::headers(const list<Header> & pHeaders)
{
    myHeaders = pHeaders;
    myDirty = true;
}


const string &      
Message::body() const
{
    return ( myBody );
}


void
Message::body(const string & pBody)
{
    myBody = pBody;
    myDirty = true;
}


ostream &   
Message::writeTo(ostream & out) const
{
    if ( myDirty == false )
    {
        return ( out << myEncoded );
    }
    
    switch ( myType )
    {
        case Request:
        {
            out << myMethod << SP << myUrl << SP << myVersion << CRLF;
            break;
        }
        case Response:
        {
            out << myVersion << SP << myStatus << CRLF;
            break;
        }
        default:
        {
            return ( out );
        }
    }

    for ( list<Header>::const_iterator i = myHeaders.begin();
            i != myHeaders.end();
            ++i
        )
    {
        out << *i << CRLF;
    }

    out << CRLF << myBody;

    return ( out );
}


const string &      
Message::encode()
throw ( Vocal::Http::Exception )
{
    if ( myDirty == true )
    {
        strstream   buffer;
        buffer << *this << ends;
        myEncoded = buffer.str();
        buffer.freeze(0);
        myDirty = false;
    }
    
    return ( myEncoded );
}


size_t
Message::decode(const string & raw)
throw ( Vocal::Http::Exception )
{
    string  decodeToDigitOrWS(DIGIT);
    decodeToDigitOrWS += WS;
    
    string  allWS(WS);
    allWS += CRLF;
    
    string  current = myEncoded = raw;

    // Lets remove all the leading whitespace, per the recommendation.
    //
    size_t  byteCount = removeLeading(current, allWS);
        
    string  token = getToken(current, byteCount, decodeToDigitOrWS);

    // Let's see if it's a response first. If not, it's a request line.
    //    
    if ( !decodeStatusLine(token, current, byteCount) )
    {
        decodeRequestLine(token, current, byteCount);
    }

    // Decode the headers.
    //
    size_t  contentLength = decodeHeaders(token, current, byteCount);
    
    // Decode the message body. This is a noop if the content length is zero.
    //
    decodeMessageBody(contentLength, current, byteCount);
    
    // If we have an extra piece of message at the end, erase it.
    //
    if ( byteCount != myEncoded.size() )
    {
        myEncoded.erase(byteCount);
    }

    // Since we are reusing the input buffer, we don't have to
    // regenerate it until a parameter is changed.
    //
    myDirty = false;
    
    return ( byteCount );
}


bool    
Message::decodeStatusLine(
    Token           &   token, 
    Token           &   current, 
    size_t          &   byteCount
) 
throw ( Vocal::Http::Exception )
{
    if ( token != Version::TokenHTTP )
    {
        return ( false );
    }
    
    myType = Response;

    // Finish version
    //
    myVersion.majorNumber(getToken(current, byteCount, DOT));
    byteCount += removeLeading(current, DOT);
    myVersion.minorNumber(getToken(current, byteCount, WS));

    // Decode status. The reason phrase is optional, so we could
    // potentially hit the end of the line.
    //        
    myStatus.code(getToken(current, byteCount, WS, true));

    // We can have zero or more reason phrases
    //
    token = getToken(current, byteCount, CRLF, true);

    if ( token.size() != 0 )
    {
        if ( myStatus.isExtension() )
        {
            myStatus.reasonPhrase(token);
        }
    }   
    return ( true );
}


void    
Message::decodeRequestLine(
    Token           &   token,
    Token           &   current, 
    size_t          &   byteCount
) 
throw ( Vocal::Http::Exception )
{
    myType = Request;    

    // Finish method
    //
    myMethod.type(token);

    // Decode url
    //
    myUrl = getToken(current, byteCount, WS);

    // Decode version
    //
    token = getToken(current, byteCount, DIGIT);

    if ( token != Version::TokenHTTP )
    {
        throw Vocal::Http::Exception(Exception::BadRequestLine);
    }

    myVersion.majorNumber(getToken(current, byteCount, DOT));
    byteCount += removeLeading(current, DOT);
    myVersion.minorNumber(getToken(current, byteCount, CRLF, true));
}
                    

size_t  
Message::decodeHeaders(
    Token           &   token,
    Token           &   current, 
    size_t          &   byteCount
) 
throw ( Vocal::Http::Exception )
{
    size_t  contentLength = 0;
    
    byteCount += removeLeading(current, WS);
    
    Header  header;
    
    // Decode the headers
    //
    while ( CRLF.find_first_of(current[0]) == Token::npos )
    {
        token = getToken(current, byteCount, COLON);
        
        header.type(token);

        byteCount += removeLeading(current, COLON);
                
        token = getToken(current, byteCount, CRLF, true);
        
        header.value(token);

        while ( WS.find_first_of(current[0]) != Token::npos )
        {
            token = getToken(current, byteCount, CRLF, true);
            
            header.continueValue(token);
        }
        
        myHeaders.push_back(header);
        
        if ( header.type() == Header::ContentLength )
        {
            contentLength = atoi(header.value().c_str());
        }
    }
    
    byteCount += removeCRLF(current);
 
    // TODO
    //
    return ( contentLength );
}


void
Message::decodeMessageBody(
    size_t              contentLength,
    Token           &   current,
    size_t          &   byteCount
)
throw ( Vocal::Http::Exception )
{
    // If we not have a content length, just return.
    //
    if ( contentLength == 0 )
    {
        return;
    }

    // If the content length is greater than the amount remaining,
    // then we have an incomplete message, but we know how much more
    // to expect.
    //
    size_t  size = current.size();

    if ( contentLength > size )
    {
        Exception   e(Exception::IncompleteInBody);

        e.bytesRemaining(size-contentLength);

        throw ( e );
    }
    
    // We have a completed message
    //
    myBody = current.substr(0, contentLength);

    byteCount += contentLength;
}
