/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpVersion_cxx_Version =
    "$Id: HttpVersion.cxx,v 1.5 2001/11/05 18:07:36 icahoon Exp $";


#include "HttpVersion.hxx"
#include "Verify.hxx"
#include <cstdlib>


using Vocal::Http::Version;


const string     Version::TokenHTTP("HTTP/");


Version::Version(u_int32_t pMajor, u_int32_t pMinor)
    :   myMajor(pMajor),
        myMinor(pMinor)
{
    test_covered("1");
}



Version::Version(const Version & src)
    :   myMajor(src.myMajor),
        myMinor(src.myMinor)
{
    test_covered("2");
}


Version::~Version()
{
    test_covered("3");
}


const Version &  
Version::operator=(const Version & src)
{
    if ( this != &src )
    {
        myMajor = src.myMajor;
        myMinor = src.myMinor;

        test_covered("4");
    }

    test_covered("5");

    return ( *this );
}


bool            
Version::operator==(const Version & src) const
{
    test_covered("6");

    return ( myMajor == src.myMajor && myMinor == src.myMinor );
}


bool            
Version::operator<(const Version & src) const
{
    test_covered("7");

    return ( myMajor != src.myMajor ? myMajor < src.myMajor : myMinor < src.myMinor );
}


const string &  
Version::text()
{
    test_covered("8");
    
    return ( TokenHTTP );
}


u_int32_t        
Version::majorNumber() const
{
    test_covered("9");

    return ( myMajor );
}


void        
Version::majorNumber(u_int32_t major)
{
    myMajor = major;

    test_covered("10");
}


void        
Version::majorNumber(const string & major)
{
    myMajor = atoi(major.c_str());

    test_covered("11");
}


u_int32_t        
Version::minorNumber() const
{
    return ( myMinor );

    test_covered("12");
}


void
Version::minorNumber(u_int32_t minor)
{
    myMinor = minor;

    test_covered("13");
}


void
Version::minorNumber(const string & minor)
{
    myMinor = atoi(minor.c_str());

    test_covered("14");
}


ostream &       
Version::writeTo(ostream & out) const
{
    test_covered("15");

    return ( out << text() << majorNumber() << '.' << minorNumber() );
}
