#if !defined(VOCAL_HTTP_VERSION_HXX)
#define VOCAL_HTTP_VERSION_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpVersion_hxx_Version =
    "$Id: HttpVersion.hxx,v 1.4 2001/11/05 18:07:36 icahoon Exp $";


#include "VocalCommon.hxx"
#include "Writer.hxx"
#include <string>


/** Infrastructure common to VOCAL.<br><br>
 */
namespace Vocal 
{


/** Http Protocol Libraries<br><br>
 */
namespace Http
{


/** Implementation of RFC 2616, Section 3-1.<br><br>
 */
class Version : public Vocal::IO::Writer
{
    public:

        
        static const string     TokenHTTP;


        /** Create the Version given the optional major and minor numbers.
         */
        Version(u_int32_t major = 1, u_int32_t minor = 1);
        

        /** Copy constructor
         */
        Version(const Version &);


        /** Virtual destructor
         */
        virtual ~Version();


        /** Assignment operator
         */
        const Version & operator=(const Version &);


        /** Equality operator
         */
        bool    operator==(const Version &) const;


        /** Less than relational operator
         */
        bool    operator<(const Version &) const;


        /** Return the version token text
         */
        static const string &   text();


        /** Major version number accessor
         */
        u_int32_t   majorNumber() const;


        /** Major version number mutator
         */
        void        majorNumber(u_int32_t);


        /** Major version number mutator
         */
        void        majorNumber(const string &);


        /** Minor version number accessor
         */
        u_int32_t   minorNumber() const;


        /** Minor version number mutator
         */
        void        minorNumber(u_int32_t);
        

        /** Minor version number mutator
         */
        void        minorNumber(const string &);
        

        /** Write the version to a stream
         */
        ostream &   writeTo(ostream &) const;


    protected:
    
        u_int32_t   myMajor;
        u_int32_t   myMinor;
};


} // namespace Http
} // namespace Vocal


#endif // !defined(VOCAL_HTTP_VERSION_HXX)
