/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpMethodTest_cxx_Version = 
    "$Id: HttpMethodTest.cxx,v 1.5 2001/07/12 20:13:11 icahoon Exp $";


#include "Application.hxx"
#include "HttpMethod.hxx"
#include "Verify.hxx"


using namespace Vocal;


class TestApplication : public Application
{
    public:
        TestApplication() {}
        virtual ~TestApplication() {}
        
    	ReturnCode  	run();
};


Application *	Application::create()
{
    return ( new TestApplication );
}


int main(int argc, char ** argv, char ** arge)
{
    Application::main(argc, argv, arge);
    
    return ( test_return_code(39) );
}


ReturnCode TestApplication::run()
{
    for ( Http::Type i = 0; i < Http::Method::totalTypes(); i++ )
    {
        Http::Method    m = Http::Method(i);
        
        cout << Http::Method(i) << endl;
        
        test_verify( m.isExtension() == false );
    }
    
    // Check construction
    //
    Http::Method    m(Http::Method::GET);

    test_verify( m.isExtension() == false );
    test_verify( m.type() == Http::Method::GET );
    test_verify( m.token() == Http::Method::TokenGET );

    Http::Method    x(Http::Method::TokenCONNECT);
    
    test_verify( x.isExtension() == false );
    test_verify( x.type() == Http::Method::CONNECT );
    test_verify( x.token() == Http::Method::TokenCONNECT );

    // Check copy ctor
    //
    Http::Method    n(m);
    
    test_verify( n.isExtension() == false );
    test_verify( n.type() == Http::Method::GET );
    test_verify( n.token() == Http::Method::TokenGET );
    
    test_verify( n == m );
    test_verify( !(n < m) );
    
    
    // Check assignment
    //
    Http::Method    o(Http::Method::PUT);
    
    test_verify( o.isExtension() == false );
    test_verify( o.type() == Http::Method::PUT );
    test_verify( o.token() == Http::Method::TokenPUT );
    
    m = o;

    test_verify( m.isExtension() == false );
    test_verify( m.type() == Http::Method::PUT );
    test_verify( m.token() == Http::Method::TokenPUT );
    
    
    // Check self assignment
    //
    m = m;
    
    test_verify( m.isExtension() == false );
    test_verify( m.type() == Http::Method::PUT );
    test_verify( m.token() == Http::Method::TokenPUT );
    
    // Check extension methods
    //
    Http::Method head("HEAD"), ext("EXTENSION");
    
    try
    {   
        Http::Method    bad;
        bad.type("");

        test_verify(false);
    }
    catch ( Http::Exception & e )
    {
        test_verify( e.error() == Http::Exception::BadMethod );
    }
        
    
    test_verify( head.type() == Http::Method::HEAD );
    test_verify( head.token() == Http::Method::TokenHEAD );
    
    test_verify( ext.type() == Http::EXTENSION );
    test_verify( ext.token() == string("EXTENSION") );


    // Check less than for two extensions
    //
    Http::Method p(Http::EXTENSION);

    p.type("FOO");
    
    test_verify( !(p == m ) );

    test_verify( m < p );
    test_verify( ext < p );
    test_verify( !(ext == p) );
    test_verify( !( m == n ) );
    
    // Check setting a bogus type
    //
    
    p.type(77);

    test_verify( p.type() == Http::EXTENSION );

    return ( SUCCESS );
}
