/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpStatusTest_cxx_Version = 
    "$Id: HttpStatusTest.cxx,v 1.6 2001/07/12 20:13:11 icahoon Exp $";


#include "Application.hxx"
#include "HttpStatus.hxx"
#include "Verify.hxx"


using namespace Vocal;


class TestApplication : public Application
{
    public:
        TestApplication() {}
        virtual ~TestApplication() {}
        
    	ReturnCode  	run();
};


Application *	Application::create()
{
    return ( new TestApplication );
}


int main(int argc, char ** argv, char ** arge)
{
    ReturnCode count = Application::main(argc, argv, arge);

    return ( test_return_code(count) );
}


ReturnCode TestApplication::run()
{
    int test_count = 0;
    
    u_int32_t       status_code[] =
    {
        100, 101,
        200, 201, 202, 203, 204, 205, 206,
        300, 301, 302, 303, 304, 305, 307,
        400, 401, 402, 403, 404, 405, 406, 407, 408, 409, 410,
        411, 412, 413, 414, 415, 416, 417,
        500, 501, 502, 503, 504, 505
    };
    
    u_int32_t number_status_codes = sizeof(status_code)/sizeof(status_code[0]);
    
    
    for ( u_int32_t i = 0; i < number_status_codes; i++ )
    {
        Http::Status    s = Http::Status(status_code[i]);

        cout << s << endl;
        
        test_verify( s.isExtension() == false ); ++test_count;
    }

    Http::Status    s(599);
    
    s.reasonPhrase("Hi Mom!!");
    
    test_verify( s.isExtension() == true ); ++test_count;
    test_verify( s.code() == 599 ); ++test_count;
    test_verify( s.token() == "599" ); ++test_count;
    test_verify( s.reasonPhrase() == "Hi Mom!!" ); ++test_count;
    test_verify( s.isServerError() ); ++test_count;

    cout << s << endl;
    
    s.code(499);
    
    s.reasonPhrase("Go Away");

    test_verify( s.isExtension() == true ); ++test_count;
    test_verify( s.code() == 499 ); ++test_count;
    test_verify( s.token() == "499" ); ++test_count;
    test_verify( s.reasonPhrase() == "Go Away" ); ++test_count;
    test_verify( s.isClientError() ); ++test_count;

    cout << s << endl;

    s.code(100);

    test_verify( s.isExtension() == false ); ++test_count;
    test_verify( s.code() == 100 ); ++test_count;
    test_verify( s.token() == Http::Status::Token100 ); ++test_count;
    test_verify( s.reasonPhrase() == Http::Status::ReasonPhrase100 ); ++test_count;
    test_verify( s.isInformational() ); ++test_count;
    
    cout << s << endl;

    s.code(794);

    test_verify( s.isExtension() == true ); ++test_count;
    test_verify( s.code() == 794 ); ++test_count;
    test_verify( s.token() == string("794") ); ++test_count;
    test_verify( s.reasonPhrase() == string("") ); ++test_count;

    cout << s << endl << endl;

    // Test token ctor
    //
    Http::Status  j("302");
    
    test_verify( j.isExtension() == false ); ++test_count;
    test_verify( j.code() == 302 ); ++test_count;
    test_verify( j.token() == Http::Status::Token302 ); ++test_count;
    test_verify( j.reasonPhrase() == Http::Status::ReasonPhrase302 ); ++test_count;
    test_verify( j.isRedirection() ); ++test_count;
    
    // Test code / phrase ctors
    //
    Http::Status::ReasonPhrase 
        new206Value("Almost received the whole content. Try again.");
        
    Http::Status    k(206, new206Value);
    Http::Status    l("206", Http::Status::ReasonPhrase206);
    
    test_verify( k.isExtension() == false ); ++test_count;
    test_verify( k.code() == 206 ); ++test_count;
    test_verify( k.token() == Http::Status::Token206 ); ++test_count;
    test_verify( k.reasonPhrase() == new206Value ); ++test_count;
    test_verify( k.isSuccess() ); ++test_count;

    test_verify( l.isExtension() == false ); ++test_count;
    test_verify( l.code() == 206 ); ++test_count;
    test_verify( l.token() == Http::Status::Token206 ); ++test_count;
    test_verify( l.reasonPhrase() == Http::Status::ReasonPhrase206 ); ++test_count;
    test_verify( l.isSuccess() ); ++test_count;
    
    test_verify( k == k ); ++test_count;
    test_verify( k < j ); ++test_count;
    test_verify( l == k ); ++test_count;
    test_verify( !(j < k) ); ++test_count;
    test_verify( !(k < k) ); ++test_count;
    test_verify( !(j == k) ); ++test_count;
    
    
    // Test copy ctor
    //
    Http::Status    s1(s);

    // Test assignment operator
    //
    Http::Status    s2;
    s2 = s;
    
    test_verify( s2 == s1 ); ++test_count;
    
    // Test self assignment
    //
    s2 == s2;

    test_verify( s2 == s ); ++test_count;
    
    // Test comparison
    //
    Http::Status    x("100"), y(200);

    test_verify( x < y );  ++test_count;
    test_verify( !(y < x) );  ++test_count;

    // Test reseting code to valid
    //
    x.code("300");
    
    test_verify( x.code() == 300 ); ++test_count;
    test_verify( x.isExtension() == false ); ++test_count;
    test_verify( x.token() == Http::Status::Token300 ); ++test_count;
    test_verify( x.reasonPhrase() == Http::Status::ReasonPhrase300 ); ++test_count;
    
    Http::Status::ReasonPhrase  foobar("foobar");
    x.reasonPhrase(foobar);
    
    test_verify( x.reasonPhrase() == foobar ); ++test_count;
    
    // Test reseting code in extension
    //
    x.code(399);

    test_verify( x.code() == 399 ); ++test_count;
    test_verify( x.isExtension() == true ); ++test_count;
    test_verify( x.token() == string("399") ); ++test_count;
    test_verify( x.reasonPhrase() == string("") ); ++test_count;
    
    Http::Status::ReasonPhrase  feh("feh");
    x.reasonPhrase(foobar);
    
    test_verify( x.reasonPhrase() == foobar ); ++test_count;
    
    x.reasonPhrase(feh);
    
    test_verify( x.reasonPhrase() == feh ); ++test_count;

    // Test less than with two extensions
    //
    Http::Status    small("777");
    Http::Status    big("999");
    
    test_verify( small < big ); ++test_count;
    
    // Again
    //
    x.code("499");

    test_verify( x.code() == 499 ); ++test_count;
    test_verify( x.isExtension() == true ); ++test_count;
    test_verify( x.token() == string("499") ); ++test_count;
    test_verify( x.reasonPhrase() == string("") ); ++test_count;
    
        
    // Test invalid status code
    //
    Http::Token   token;
    
    token = "1000";

    try
    {
        y.code("559");
        y.code(token);

        test_verify(false); ++test_count;
    }
    catch ( Vocal::Http::Exception & e )
    {
        test_verify( e.error() == Http::Exception::BadStatus ); ++test_count;
    }
        
    return ( test_count );
}
