
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGBuilder.hxx"
#include "LGStateMcData.hxx"
#include "LGEngine.hxx"
#include "LGState.hxx"
#include "CallInfo.hxx"
#include "LGParam.hxx"
#include "SampleMsg.hxx"
#include "LGEvent.hxx"
#include "InviteMsg.hxx"
#include "SipProxyEvent.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "SipTransceiver.hxx"
#include "SampleMsg.hxx"
#include "LGCallLeg.hxx"

LGBuilder::LGBuilder()
    : sipStack(0), cpsEngine(0), fifo(0)
{
    cpsEngine = new LGEngine();
    
}

LGBuilder::~LGBuilder()
{
    sipStack = 0;
    delete cpsEngine;
    fifo = 0;
}

void
LGBuilder::setSipStack(const Sptr<SipTransceiver>& stack)
{
    sipStack = stack;
}

void
LGBuilder::setFifo(const Sptr<Fifo<Sptr<SipProxyEvent> > >& f)
{
    fifo = f;
/////////////////may need to move out to//////////////////////////
//    fifo->addDelayMs(new LGEvent(LGEvent::E_START_CALL, fifo),
//		     LGParam::Call2CallDelay);
/////////////////////seperate firing thread///////////////////////
}


void
LGBuilder::process(const Sptr<SipProxyEvent> event)
{
    Sptr<LGEvent> lgEvent;

    if(lgEvent.dynamicCast(event)!=0)
    {
	if(lgEvent->type == LGEvent::E_START_CALL)
	    startCall(lgEvent);
	else assert(0);
    }
    else
    {
	processCurr(event);
    }
}


void
LGBuilder::startCall(const Sptr<LGEvent>& lgEvent)
{
 /////////////////may need to move out to//////////////////////////
    //// Trigger Another Call
//    fifo->addDelayMs(new LGEvent(LGEvent::E_START_CALL,fifo),
//		     LGParam::Call2CallDelay);
/////////////////////seperate firing thread///////////////////////

    cpLog(LOG_DEBUG,"starting in LGBuilder::startCall<---------");
    // get the request message for new call
    SampleMsg::MsgContainer* msg =  SampleMsg::getNextMsg();
    Sptr<SipCommand> sipMsg = msg->nextMsg;

    // creat new call info
    Sptr<SipCallLeg> callLeg = new SipCallLeg(sipMsg->computeCallLeg());
    if(LGParam::Function == LGParam::REGISTRATION)
	callLeg->setCSeq(sipMsg->getCSeq());
///////    Sptr<SipCallLeg> callLeg = new LGCallLeg(*sipMsg);
    cpLog(LOG_DEBUG,"Creating a new callleg entry in call container");
    Sptr<CallInfo> callInfo = myCallContainer->getCall(callLeg);
    cpLog(LOG_DEBUG,"done creating new entry");
    callInfo->setSipCommand(sipMsg);
    LGStateMcData * callData = new LGStateMcData();
    callInfo->setCallStateData(callData);

    // feed the call data to this event
    lgEvent->setCallInfo(callInfo, myCallContainer);
    lgEvent->sipStack = sipStack;

    // start the new call
//     if(LGParam::URIHost != LGParam::NextHopHost)
// 	    sipStack->sendAsync(sipMsg,LGParam::NextHopHost,
// 				LGParam::NextHopPort);
    if(LGParam::ToHost != LGParam::NextHopHost)
	    sipStack->sendAsync(sipMsg,LGParam::NextHopHost,
				LGParam::NextHopPort);
    else
	    sipStack->sendAsync(sipMsg);
    callData->secret = msg->secret;
    delete msg;
    callData->currState = cpsEngine->process(lgEvent);
}

void
LGBuilder::processCurr(const Sptr<SipProxyEvent>& event)
{
    Sptr<CallInfo> callInfo;
    static Sptr<TimerEvent> bogus = new TimerEvent(fifo);

    cpLog(LOG_DEBUG,"processing in LGBuilder::processCurr<---------");
    Sptr<SipEvent> sEvent;
    if(sEvent.dynamicCast(event)!=0)
    {
	cpLog(LOG_DEBUG,"finding sipevent's callleg entry in call container");
	Sptr<SipCallLeg> callLeg = sEvent->getSipCallLeg();
	if(LGParam::Function == LGParam::REGISTRATION)
	    callLeg->setCSeq(sEvent->getSipMsg()->getCSeq());
	callInfo = myCallContainer->findCall(callLeg);
////////	callInfo = myCallContainer->findCall(new
////////       				     LGCallLeg(*sEvent->getSipMsg()));
	cpLog(LOG_DEBUG,"%sfound callleg entry",
	      callInfo == 0 ? "not " : "\0" );
	if(callInfo == 0)
	{
	    if(LGParam::Function != LGParam::TERMINATING)
	    {
		//		cpLog(LOG_CRIT,
		//      "ERROR: no c/l found for received message\n%s",
		//      sEvent->getSipMsg()->encode().getData());
		return;
	    }
	    else
	    {
		Sptr<SipCommand> command;
		command.dynamicCast(sEvent->getSipMsg());
		if(command == 0)
		    return;
		Sptr<SipCallLeg> callLeg = new SipCallLeg(
		    command->computeCallLeg());
		cpLog(LOG_DEBUG,
		      "Creating a new callleg entry in call container");
		callInfo = myCallContainer->getCall(callLeg);
		cpLog(LOG_DEBUG,"done creating new entry");

		// save the request for state m/c info
		callInfo->setSipCommand(command);
		LGStateMcData * callData = new LGStateMcData();
		callInfo->setCallStateData(callData);
	    }
	}
	else
	{
	    if(LGParam::Function == LGParam::TERMINATING)
		{
		    Sptr<InviteMsg> invite;
		    if(invite.dynamicCast(sEvent->getSipMsg())!=0)
			cpLog(LOG_CRIT,
			  "ERROR: RECIEVED new INVITE with existing c/l!!!");
		}
	}
		    
	sEvent->setCallInfo(callInfo, myCallContainer);
    }
    else
    {
	Sptr<TimerEvent> tEvent;
	if(tEvent.dynamicCast(event)!=0)
	{
	    callInfo = event->getCallInfo();
	    tEvent->setSipStack(sipStack);
	}
	else
	{
	    assert(0);
	}
    }

    Sptr<LGStateMcData> callData;
    if(callData.dynamicCast(callInfo->getCallStateData())!=0)
    {
	if(callData->currState == 0 &&
	   LGParam::Function == LGParam::TERMINATING)
	    callData->currState = cpsEngine->process(event);
	else
	    callData->currState = callData->currState->process(event);
	if(callData->currState==0)
	{
	    cpLog(LOG_DEBUG,"deleting in LGBuilder::processCurr<---------");
	    Sptr<SipCallLeg> callLeg = 
		new SipCallLeg(callInfo->getSipCommand()->computeCallLeg());
	    if(LGParam::Function == LGParam::REGISTRATION)
		callLeg->setCSeq(callInfo->getSipCommand()->getCSeq());
///////    Sptr<SipCallLeg> callLeg = new LGCallLeg(*sipMsg);
///////		new LGCallLeg(*callInfo->getSipCommand());
	    cpLog(LOG_DEBUG,"removing callleg entry from call container");
	    myCallContainer->removeCall(callLeg);
	    cpLog(LOG_DEBUG,"done removing call leg entry");
	    callInfo->setTimer(bogus);
	}
    }
    else
    {
	assert(0);
    }
}
