
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <stdlib.h>
#include <string.h>
#include "LGUserRange.hxx"
#include "LGParam.hxx"
#include "cpLog.h"

#define INCR(A,B,C) ((A)>=(C) ? (A=(B)) : (A++))

VeryLong atovl(char *c)
{
    VeryLong ll=0;
    for(;c && *c >= '0' && *c <= '9'; c++)
	ll = ll*10 + *c - '0';
    return ll;
}

LGUserRange::LGUserRange(VeryLong fromStart, VeryLong fromEnd,
			 VeryLong toStart, VeryLong toEnd)
    : fromStart(fromStart), toStart(toStart), fromEnd(fromEnd), toEnd(toEnd),
      currFrom(fromEnd), currTo(toEnd), cseqCount(0)
{
	cpLog(LOG_INFO,"From [%d .. %d] <-----------> To [%d .. %d]",
	      fromStart, fromEnd, toStart, toEnd);
}

LGUserRange::~LGUserRange()
{}

char*
LGUserRange::itoa(VeryLong i)
{
    int j;
    VeryLong k=10;
    for(j=1;i/k;j++,k*=10);

    char * r = new char[j+1];
    for(int l=0;l<j;l++)
    {
        k=k/10;
        r[l]='0'+(i/k);
        i%=k;
    }
    r[j]=0;
    return r;
}

PairInfo*
LGUserRange::nextPair()
{
    static int secretLen(LGParam::Secret ? strlen(LGParam::Secret) : 0);
    static long fromDiff = fromEnd-fromStart, toDiff = toEnd-toStart;

    PairInfo * retPair = new PairInfo();

    lock.lock();
    switch (LGParam::Function)
    {
    case LGParam::ORIGINATING:
	//	if(currFrom >= fromEnd)
	//	{
	//	    currFrom = fromStart;
	//	    currTo >= toEnd ? (currTo = toStart) :  currTo++;
	//	}
	//	else currFrom++;
	if(LGParam::Random)
	{
		currFrom = fromStart + (fromDiff ? random()%fromDiff : 0);
		currTo   = toStart   + (toDiff   ? random()%toDiff   : 0);
	}
	else
	{
		currFrom >= fromEnd ? (currFrom = fromStart) :  currFrom++;
		currTo >= toEnd ? (currTo = toStart) :  currTo++;
	}
	break;

    case LGParam::REGISTRATION:
	if(currTo >= toEnd)
	{
	    cseqCount++;
	    currTo = toStart;
	}
	else currTo++;
	currFrom >= fromEnd ? (currFrom = fromStart) :  currFrom++;
	break;

    default:
	break;
    }
    retPair->to = itoa(currTo);
    retPair->from = itoa(currFrom);
    cpLog(LOG_INFO,"From [%s] <-----------> To [%s]",retPair->to,
	  retPair->from);
    retPair->secret = new char[secretLen+1];
    strcpy(retPair->secret,LGParam::Secret);
    retPair->cseqCount = cseqCount;
    lock.unlock();
    return retPair;
}
