
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PEPClientData_cxx_Version =
    "$Id: PEPClientData.cxx,v 1.6 2000/12/18 23:42:51 bko Exp $";
#include "PEPClientData.hxx"


///
PEPClientData::PEPClientData(const string& path, const string& fileName)
{
    VFile vf( path, fileName);
    parse( vf );
}

///
PEPClientData& PEPClientData::operator=(const PEPClientData& other)
{
    if (this != &other)
    {
        _Id = other._Id;
        _clientType = other._clientType;
        _noResponseTimer = other._noResponseTimer;
        _pdpHost = other._pdpHost;   // copy the other list
    }

    return (*this);
}

///
void PEPClientData::parse(VXmlParser& parser)
{
    _Id = parser.getContent("PEPClient", "identifier");
    _clientType = (PepType) atoi( parser.getContent("PEPClient", "clientType").c_str() );
    xmlNodePtr root = parser.findNode(NULL, "PEPClient");
    if (root != NULL)
    {
        parser.getContent( root, "PDPHost", "", _pdpHost );
    }
    else
    {
        cpLog(LOG_ALERT, "parse() cannot findNode() %s", "PEPClient");
    }
    _noResponseTimer = atoi(
                           parser.getContent("PEPClient", "noResponseTimer").c_str());

};

///
ostream& operator<<(ostream& os, const PEPClientData& data)
{
    os << "PEPClient:- Id:" << data.getId() << endl;
    os << "Client Type:" << (int) data.getType() << endl;
    os << "noResponseTimer:" << data.getNoResponseTimer() << endl;
    int i = 1;
    for ( list < string > ::const_iterator itr = data._pdpHost.begin();
            itr != data._pdpHost.end(); itr++)
    {
        os << "PDPHost " << i << ":" << itr->c_str() << endl;
        i++;
    }
    return os;
}
