#ifndef  PEPClientData_hxx
#define  PEPClientData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PEPClientData_hxx_Version =
    "$Id: PEPClientData.hxx,v 1.6 2000/12/18 23:42:51 bko Exp $";



#include <string>
#include <list>

#include "VXmlParser.hxx"
#include "VFile.hxx"
#include "cpLog.h"


typedef enum PepType
{
    SIP_PEP = 0,
    RSVP_PEP,
    MAX_PEP_TYPE
} PEP_type;

class PEPClientData
{
    public:
        ///
        PEPClientData(void)
        {
            _Id = "uninitialized";
            _clientType = SIP_PEP;
            _noResponseTimer = 0;
        }

        ///
        PEPClientData(const VFile& xmlFile)
        {
            parse(xmlFile);
        }

        /// called by PolicyCfg:
        PEPClientData(VXmlParser& parser)
        {
            parse(parser);
        }

        ///
        PEPClientData(const string& path, const string& fileName);

        ///
        PEPClientData& operator=(const PEPClientData& other);

        ///
        friend ostream& operator<<(ostream& os, const PEPClientData& data);

        ///
        const string& getId(void ) const
        {
            return _Id;
        }
        ///
        enum PepType getType(void ) const
        {
            return _clientType;
        }
        ///
        const list < string > & getPDPHost( void ) const
        {
            return _pdpHost;
        }
        ///
        int getNoResponseTimer( void ) const
        {
            return _noResponseTimer;
        }

        ///
        void parse(VXmlParser& parser);
        ///
        void parse(const VFile& vf)
        {
            VXmlParser parser( vf );
            parse( parser );
        }

    private:
        string _Id;                        // identifier, e.g. PEP1:5060
        PepType _clientType;                 // default = SIP_PEP
        list < string > _pdpHost;
        int _noResponseTimer;             // in ms
}
;

#endif
