
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PolicyCfg_cxx_Version =
    "$Id: PolicyCfg.cxx,v 1.19 2001/01/12 01:35:50 wbreu Exp $";
// #include <fstream.h>
// #include <strstream.h>
#include <errno.h>

#include "cpLog.h"
#include "PolicyCfg.hxx"
#include "ProvisionInterface.hxx"
#include "VException.hxx"

#define PDPCONFIG "ListOfPdpServers.xml"
#define OSPCONFIG "ospServer.xml"

PolicyCfg* PolicyCfg::_instance = 0;

///
PolicyCfg::PolicyCfg( const char* psHost, int psPort,
                      const char* altHost, int altPort)
{
    _ospDataChanged = false;
    _pdpDataChanged = false;
#if defined (_linux_)
    if (atexit( PolicyCfg::destroy ) < 0)
    {
        cpLog( LOG_ALERT, "unable to register PolicyCfg::destroy with atexit");
    }
#endif
    //Interface to provision server
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance(psHost, psPort,
                                  altHost, altPort);
        pif.getOspServer(_ospServerData);
        pif.registerForUpdate(PolicyCfg::notifyOspUpdate,
                              OSPCONFIG,
                              ProvisionInterface::VCONFIG);
        pif.registerForUpdate(PolicyCfg::notifyPdpUpdate,
                              PDPCONFIG,
                              ProvisionInterface::VCONFIG);
    }
    catch (VException& e)
    {
        cpLog(LOG_ALERT, "Failed to get Provisioning data, reason:%s",
              e.getDescription().c_str());
    }
}

void PolicyCfg::destroy( void )
{
    if (PolicyCfg::_instance != 0 )
    {
        delete PolicyCfg::_instance;
        PolicyCfg::_instance = 0;
    }
}

///
PolicyCfg& PolicyCfg::operator=(const PolicyCfg& other)
{
    if (this != &other)
    {
        _ospServerData = other._ospServerData;
    }
    return (*this);
}

///
ostream& operator<<(ostream& os, const PolicyCfg& data)
{
    os << "Policy Configurator:" << endl;
    os << data._ospServerData << endl;
    return os;
}


void
PolicyCfg::notifyOspUpdate(const string& data, const string& fileName, bool delFlg)
{
    cpLog(LOG_DEBUG,
          "Received update notification for file %s", fileName.c_str());

    if (delFlg)
    {
        cpLog(LOG_DEBUG,
              "File %s being deleted, ignore", fileName.c_str());
        return ;
    }
    //Get the OSP Data
    try
    {
       VOspServerData sdata;
       ProvisionInterface::instance().getOspServer(sdata);
       PolicyCfg::Instance().setOspServerData(sdata);
       PolicyCfg::Instance().ospDataChanged(true);
    }
    catch (VException& e)
    {
       cpLog(LOG_ALERT, "Failed to get OSP Server data, reason:%s",
		       e.getDescription().c_str());
    }
}

void
PolicyCfg::notifyPdpUpdate(const string& data, const string& fileName, bool delFlg)
{
    cpLog(LOG_DEBUG,
          "Received update notification for file %s", fileName.c_str());
    if (delFlg)
    {
        cpLog(LOG_DEBUG,
              "File %s being deleted, ignore", fileName.c_str());
        return ;
    }

    PolicyCfg::Instance().pdpDataChanged(true);
}
