
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSContext_cxx_Version =
    "$Id: COPSContext.cxx,v 1.9 2001/05/13 07:26:01 icahoon Exp $";

#include "VocalCommon.hxx"
#include "vin.h"
#include <netinet/in.h>

#include "COPSContext.hxx"

COPSContext::COPSContext(u_int16_t rType, u_int16_t mType )
{
    _objHdr.setCNum(COPS_CONTEXT);
    _objHdr.setCType(1);
    _dataRep._rType = rType;
    _dataRep._mType = mType;
    _objHdr.setDataLength(4);
}

COPSContext::COPSContext(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    u_int16_t rType;
    u_int16_t mType;
    memcpy(&rType, dataPtr + 4, 2);
    memcpy(&mType, dataPtr + 6, 2);
    _dataRep._rType = ntohs(rType);
    _dataRep._mType = ntohs(mType);
    _objHdr.setDataLength(4);
}

void
COPSContext::dump(ostream& os) const
{

    os << _objHdr << endl;
    os.width(25);
    os << "context: " << getDescription().c_str() << "," << _dataRep._mType << endl;
}


string
COPSContext::getDescription()const
{
    string retStr;
    if (_dataRep._rType & 0x01)
    {
        retStr += (retStr.size()) ? "," : "";
        retStr += "Incoming Message/Admission Control";
    }
    if (_dataRep._rType & 0x02)
    {
        retStr += (retStr.size()) ? "," : "";
        retStr += "Resource allocation";
    }
    if (_dataRep._rType & 0x04)
    {
        retStr += (retStr.size()) ? "," : "";
        retStr += "Outgoing message";
    }
    if (_dataRep._rType & 0x08)
    {
        retStr += (retStr.size()) ? "," : "";
        retStr += "Configuration";
    }
    return retStr;
}

COPSContext&
COPSContext::operator=(const COPSContext& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _dataRep._rType = other._dataRep._rType;
        _dataRep._mType = other._dataRep._mType;
    }
    return (*this);
}

u_int16_t
COPSContext::getDataLength()const
{
    //Add the size of the header also
    return (_objHdr.getDataLength());
}


void
COPSContext::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSContext::writeData");
    _objHdr.writeData(id);
    u_int16_t rType = htons(_dataRep._rType);
    u_int16_t mType = htons(_dataRep._mType);
    COPSUtil::writeData(id, &rType, 2);
    COPSUtil::writeData(id, &mType, 2);
}

bool
COPSContext::operator==(const COPSContext& other) const
{
    if (_objHdr == other._objHdr)
    {
        if (_dataRep._rType == other._dataRep._rType)
        {
            if (_dataRep._mType == other._dataRep._mType)
            {
                return true;
            }
        }
    }
    return false;
}

bool
COPSContext::operator<(const COPSContext& other) const
{
    if (_objHdr < other._objHdr)
    {
        if (_dataRep._rType < other._dataRep._rType)
        {
            if (_dataRep._mType < other._dataRep._mType)
            {
                return true;
            }
        }
    }
    return false;
}
