
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSData_cxx_Version =
    "$Id: COPSData.cxx,v 1.8 2000/12/18 23:42:53 bko Exp $";
#include <string.h>

#include "COPSData.hxx"

COPSData::COPSData(const char* dPtr, int dLen)
{
    _dataBuf = new char[dLen];
    memcpy(_dataBuf, dPtr, dLen);
    _dLen = dLen;
}

///
COPSData::COPSData(const string& data)
{
    _dataBuf = new char[data.length()];
    memcpy(_dataBuf, data.c_str(), data.length());
    _dLen = data.length();
};

///
COPSData::COPSData(const COPSData& other)
{
    _dataBuf = new char[other._dLen];
    memcpy(_dataBuf, other._dataBuf, other._dLen);
    _dLen = other._dLen;
}


COPSData&
COPSData::operator=(const COPSData& other)
{
    if (this != &other)
    {
        _dataBuf = new char[other._dLen];
        memcpy(_dataBuf, other._dataBuf, other._dLen);
        _dLen = other._dLen;
    }
    return *this;
}

///
bool
COPSData::operator==(const COPSData& other)
{
    if (_dLen != other._dLen) return false;

    if (memcmp(_dataBuf, other._dataBuf, other._dLen) == 0)
        return true;

    return false;
}

///
bool
COPSData::operator<(const COPSData& other)
{
    if (memcmp(_dataBuf, other._dataBuf, other._dLen) < 0)
        return true;

    return false;
}

string
COPSData::str() const
{
    string retStr;
    for (int i = 0; i < _dLen; i++)
    {
        retStr += _dataBuf[i];
    }
    return retStr;
}

bool operator==(const COPSData& left, const COPSData& right)
{
    if (left._dLen != right._dLen) return false;

    if (memcmp(left._dataBuf, right._dataBuf, right._dLen) == 0)
        return true;

    return false;
}
