#ifndef COPSDecisionMsg_hxx
#define COPSDecisionMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSDecisionMsg_hxx_Version =
    "$Id: COPSDecisionMsg.hxx,v 1.10 2000/12/18 23:42:53 bko Exp $";

#include <vector>
#include <map>

#include "Sptr.hxx"
#include "COPSMsg.hxx"


#include  "COPSHandle.hxx"
#include  "COPSContext.hxx"
#include  "COPSError.hxx"
#include  "COPSDecision.hxx"
#include  "COPSIntegrity.hxx"
#include  "VNetworkException.hxx"


///COPSDecisionMsg

class COPSDecisionMsg : public COPSMsg
{
    public:
        ///
        typedef map < COPSContext, vector < COPSDecision > , less < COPSContext > > DecisionMap;
        ///
        COPSDecisionMsg();

        ///
        virtual ~COPSDecisionMsg();


        /** Checks the sanity of COPS message and throw an
          * COPSBadDataException when data is bad.
          */
        void checkSanity() const throw (COPSBadDataException&);

        ///Add header to the message
        COPSDecisionMsg& operator<<(const COPSHeader& obj);

        ///Add client handle to the message
        COPSDecisionMsg& operator<<(const COPSHandle& obj);

        ///Add an Error object
        COPSDecisionMsg& operator<<(const COPSError& obj);

        /**Add one or more local decision object for a given decision context
         * the context is optional, if null all decision object are tided to
         * message context
         */
        void addDecision(const COPSDecision& obj, const COPSContext& cntxt);

        ///Add integrity object
        COPSDecisionMsg& operator<<(const COPSIntegrity& obj);


        ///Dumps the whole object to a output stream
        void dump(ostream& os) const;

        ///Writes data to given file descriptor
        void writeData(int id) const throw (VNetworkException&);

        ///
        string className() const
        {
            return "COPSDecisionMsg";
        };


        ///Accessor
        const COPSHeader& getHeader() const
        {
            return *_hdr;
        };
        ///
        const COPSHandle& getClientHandle() const
        {
            return *_clientHandle;
        };

        ///Returns true if it has error object
        bool hasError() const
        {
            return (_error != 0);
        };

        ///Should check hasError() before calling
        const Sptr < COPSError > & getError() const
        {
            return _error;
        };

        /**Returns a map of decision for which is an arry of context and vector
           of associated decision object.
         */
        const DecisionMap& getDecisions() const
        {
            return _decisions;
        };

        ///Returns true if it has integrity object
        bool hasIntegrity() const
        {
            return (_integrity.getPtr() != 0);
        };

        ///Should check hasIntegrity() before calling
        const Sptr < COPSIntegrity > & getIntegrity() const
        {
            return _integrity;
        };

    private:
        ///
        friend class COPSMsgParser;
        ///
        COPSDecisionMsg(const char* data);

        ///Parses the data and fills COPSDecisionMsg with its constituents
        void parse(const char* data) throw (COPSBadDataException&);

        ///Parses the data and fills that follows the header hdr and fills COPSDecisionMsg
        void parse(const COPSHeader&hdr, const char* data) throw (COPSBadDataException&);

        ///
        void setMsgLength();

        /* COPSHeader coming from base class */
        Sptr < COPSHandle > _clientHandle;
        Sptr < COPSError > _error;
        DecisionMap _decisions;
        Sptr < COPSIntegrity > _integrity;
        Sptr < COPSContext > _decContext;
};

#endif
