#ifndef COPSError_hxx
#define COPSError_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSError_hxx_Version =
    "$Id: COPSError.hxx,v 1.7 2000/12/18 23:42:53 bko Exp $";

#include <sys/types.h>
#include <string>

#include "COPSObjBase.hxx"
#include "COPSObjHeader.hxx"

typedef enum {
   COPS_ERR_BAD_HANDLE=1,
   COPS_ERR_BAD_HANDLE_REF,
   COPS_ERR_BAD_MSG_FORMAT,
   COPS_ERR_FAIL_PROCESS,
   COPS_ERR_MISSING_INFO,
   COPS_ERR_UNSUPPORTED_CLIENT_TYPE,
   COPS_ERR_MANDATORY_OBJECT_MISSING,
   COPS_ERR_CLIENT_FAILURE,
   COPS_ERR_COMM_FAILURE,
   COPS_ERR_UNKNOWN,
   COPS_ERR_SHUTTING_DOWN,
   COPS_ERR_PDP_REDIRECT,
   COPS_ERR_UNKNOWN_OBJECT,
   COPS_ERR_AUTH_FAILURE,
   COPS_ERR_AUTH_REQUIRED,
   COPS_ERR_MAX
} COPSErrCode;


///COPSError

class COPSError : public COPSObjBase
{
    public:
        ///
        COPSError(u_int16_t errCode, u_int16_t subCode);

        ///
        COPSError(const COPSError& other);

        ///
        COPSError& operator=(const COPSError& other);

        ///Returns size in number of octects
        u_int16_t getDataLength() const
        {
            return (_objHdr.getDataLength());
        };

        ///
        string getDescription();

        ///
        void dump(ostream& os)const;

        ///
        string className() const
        {
            return "COPSError";
        };

        ///
        bool isError() const
        {
            return true;
        };

        ///Writes object to given network socket in network byte order
        void writeData(int id) const throw(VNetworkException&);

    private:
        friend class COPSDecisionMsg;
        friend class COPSClientCloseMsg;
        ///
        COPSError(const char* dataPtr);

        COPSObjHeader _objHdr;
        struct
        {
            u_int16_t _errCode;
            u_int16_t _errSubCode;
        }
        _dataRep;
};

#endif
