#ifndef COPSIpv4PdpAddress_hxx
#define COPSIpv4PdpAddress_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSIpv4PdpAddress_hxx_Version =
    "$Id: COPSIpv4PdpAddress.hxx,v 1.8 2001/05/13 07:26:01 icahoon Exp $";

#include <sys/types.h>
#include <string>

#include "COPSPdpAddress.hxx"
#include "COPSObjHeader.hxx"
#include "COPSIpv4Address.hxx"

///COPSIpv4PdpAddress

class COPSIpv4PdpAddress : public COPSPdpAddress
{
    public:
        ///
        void setIpAddress(const string& hostName);
        ///
        void setTcpPort(u_int16_t port)
        {
            _dataRep._tcpPort = port;
        };

        ///
        string getIpName() const;
        ///
        u_int16_t getTcpPort() const
        {
            return _dataRep._tcpPort;
        };

        ///
        u_int32_t getIpAddress() const;

        ///Returns size in number of octects, including header
        u_int16_t getDataLength() const;

        ///
        COPSIpv4PdpAddress& operator=(const COPSIpv4PdpAddress& other);

        ///
        bool isIpv4PdpAddress() const
        {
            return true;
        };

        ///Write data to given network socket
        void writeData(int id) const throw (VNetworkException&);

        ///
        void dump(ostream& os) const;

    protected:
        ///
        COPSIpv4PdpAddress();

        ///
        COPSIpv4PdpAddress(const char* dataPtr);

        COPSIpv4PdpAddress(const COPSIpv4PdpAddress& other);

        COPSObjHeader _objHdr;
        struct
        {
            COPSIpv4Address _addr;
            u_int16_t _reserved;
            u_int16_t _tcpPort;
        }
        _dataRep;
};

#endif
