#ifndef COPSMsg_hxx
#define COPSMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSMsg_hxx_Version =
    "$Id: COPSMsg.hxx,v 1.13 2000/12/18 23:42:54 bko Exp $";

#include <sys/types.h>
#include <iostream.h>
#include <string>

#include "COPSBadDataException.hxx"
#include "COPSHeader.hxx"
#include "VNetworkException.hxx"
#include "Sptr.hxx"

///COPSMsg

class COPSMsg
{
    public:
        ///
        const COPSHeader& getHeader() const
        {
            return *_hdr;
        };
        ///
        virtual void dump(ostream& os)const = 0;

        ///
        virtual void writeData(int id) const throw(VNetworkException&) = 0;

        ///
        virtual string className() const = 0;

        ///
        u_int16_t getMsgLength() const
        {
            return _hdr->getMsgLength();
        };

        ///
        virtual ~COPSMsg()
        { }
        ;


    protected:
        ///
        virtual void parse(const COPSHeader& hdr,
                           const char* data) throw (COPSBadDataException&) = 0;
        ///
        virtual void parse(const char* data) throw (COPSBadDataException&) = 0;

        ///
        void parseHeader(const char* data) throw (COPSBadDataException&);

        COPSMsg() : _hdr(0)
        { }
        ;


        /** Checks the sanity of COPS message and throw an
          * COPSBadDataException when data is bad.
          */
        virtual void checkSanity() const throw (COPSBadDataException&);

        ///
        friend ostream& operator<<(ostream& os, const COPSMsg& msg);

        Sptr < COPSHeader > _hdr;
        u_int16_t _dataLength;
        u_int16_t _dataStart;

        friend class COPSTransceiver;
        friend class COPSMsgParser;
};

#endif
