
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSObjHeader_cxx_Version =
    "$Id: COPSObjHeader.cxx,v 1.10 2001/05/13 07:26:01 icahoon Exp $";

#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSObjHeader.hxx"
#include "cpLog.h"

void
COPSObjHeader::setDataLength(u_int32_t len)
{
    //Add the length of the header also
    _dataRep._len = len + 4;
}

void
COPSObjHeader::dump(ostream& os) const
{
    os << "**" << getStrCNum() << "**" << endl;
    os.width(25);
    os << dec << endl;
    os << "Length: " << _dataRep._len << endl;
    os.width(25);
    os << "C-num: " << static_cast < int > (_dataRep._cNum) << endl;
    os.width(25);
    os << "C-type: " << static_cast < int > (_dataRep._cType) << endl;
}

COPSObjHeader::COPSObjHeader(const COPSObjHeader& other)
{
    _dataRep._len = other._dataRep._len;
    _dataRep._cNum = other._dataRep._cNum;
    _dataRep._cType = other._dataRep._cType;
}

COPSObjHeader&
COPSObjHeader::operator=(const COPSObjHeader& other)
{
    if (this != &other)
    {
        _dataRep._len = other._dataRep._len;
        _dataRep._cNum = other._dataRep._cNum;
        _dataRep._cType = other._dataRep._cType;
    }
    return (*this);
}

string
COPSObjHeader::getStrCNum() const
{
    switch (getCNum())
    {
        case COPS_HANDLE:
        return ("Client-handle");
        break;
        case COPS_CONTEXT:
        return ("Context");
        break;
        case COPS_ININTF:
        return ("In-Interface");
        break;
        case COPS_OUTINTF:
        return ("Out-Interface");
        break;
        case COPS_REASON_CODE:
        return ("Reason");
        break;
        case COPS_DEC:
        return ("Decision");
        break;
        case COPS_LPDP_DEC:
        return ("Local-Decision");
        break;
        case COPS_ERROR:
        return ("Error");
        break;
        case COPS_CSI:
        return ("Client-SI");
        break;
        case COPS_KA:
        return ("KA-timer");
        break;
        case COPS_PEPID:
        return ("PEP-id");
        break;
        case COPS_RPT:
        return ("Report");
        break;
        case COPS_PDP_REDIR:
        return ("Redirect PDP addr");
        break;
        case COPS_LAST_PDP_ADDR:
        return ("Last PDP addr");
        break;
        case COPS_ACCT_TIMER:
        return ("Account-Timer");
        break;
        case COPS_MSG_INTEGRITY:
        return ("Message-Integrity");
        break;
        default:
        return ("Unknown");
        break;
    }
}

void
COPSObjHeader::checkDataLength() throw (COPSBadDataException&)
{}
;


///Writes data to a given network socket
void
COPSObjHeader::writeData(int id) const throw (VNetworkException&)
{
    u_int16_t len = htons(_dataRep._len);
    COPSUtil::writeData(id, &len, 2);
    COPSUtil::writeData(id, &_dataRep._cNum, 2);
}


void
COPSObjHeader::parse(const char* data)
{
    u_int16_t a;
    memcpy(&a, data, 2);
    a = ntohs(a);
    memcpy(&_dataRep._len, &a, 2);
    memcpy(&_dataRep._cNum, data + 2, 2);
}
