#ifndef COPSObjHeader_hxx
#define COPSObjHeader_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSObjHeader_hxx_Version =
    "$Id: COPSObjHeader.hxx,v 1.6 2000/12/18 23:42:54 bko Exp $";

#include <sys/types.h>

#include "COPSObjBase.hxx"
#include "COPSBadDataException.hxx"

typedef enum {
    COPS_HANDLE = 1,
    COPS_CONTEXT,
    COPS_ININTF,
    COPS_OUTINTF,
    COPS_REASON_CODE,
    COPS_DEC,
    COPS_LPDP_DEC,
    COPS_ERROR,
    COPS_CSI,
    COPS_KA,
    COPS_PEPID,
    COPS_RPT,
    COPS_PDP_REDIR,
    COPS_LAST_PDP_ADDR,
    COPS_ACCT_TIMER,
    COPS_MSG_INTEGRITY
} COPSC_num;

///COPSObjHeader

class COPSObjHeader : public COPSObjBase
{
    public:
        ///
        COPSObjHeader(COPSC_num cNum, u_int8_t cType)
        {
            _dataRep._len = 4;
            _dataRep._cNum = cNum;
            _dataRep._cType = cType;
        }

        ///
        COPSObjHeader()
        {
            _dataRep._len = 4;
            _dataRep._cNum = 0;
            _dataRep._cType = 0;
        }

        ///
        COPSObjHeader(const COPSObjHeader& other);


        ///
        bool operator==(const COPSObjHeader& other) const
        {
            if (_dataRep._len == other._dataRep._len)
            {
                if (_dataRep._cNum == other._dataRep._cNum)
                {
                    if (_dataRep._cType == other._dataRep._cType)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        ///
        bool operator<(const COPSObjHeader& other) const
        {
            if (_dataRep._len < other._dataRep._len)
            {
                if (_dataRep._cNum < other._dataRep._cNum)
                {
                    if (_dataRep._cType < other._dataRep._cType)
                    {
                        return true;
                    }
                }
            }
            return false;
        }


        ///Accessor

        ///Get the data length in number of octets
        u_int16_t getDataLength() const
        {
            return _dataRep._len;
        };

        ///Get the class information identifier cNum
        COPSC_num getCNum() const
        {
            return static_cast < COPSC_num > (_dataRep._cNum);
        };

        ///Get the type per cNum
        u_int8_t getCType() const
        {
            return _dataRep._cType;
        };

        ///Get stringified CNum
        string getStrCNum() const;


        ///Mutator

        /**Set the obj length, the length is the length of the data following
          * the object header.The length of the object header (4 bytes) is added
          * to the length passed.
          */
        void setDataLength(u_int32_t len);

        ///Set the class of information cNum
        void setCNum(COPSC_num cNum)
        {
            _dataRep._cNum = cNum;
        } ;

        ///Set the  type defined per cNum
        void setCType(u_int8_t cType)
        {
            _dataRep._cType = cType;
        } ;

        ///
        void dump(ostream& os) const;

        ///
        COPSObjHeader& operator=(const COPSObjHeader& other);

        /** Check the object length, if does not fall in 32 bit
            boundry throw COPSBadDataException.
         */
        void checkDataLength() throw (COPSBadDataException&);

        ///Writes data to a given network socket
        void writeData(int id) const throw (VNetworkException&);

        ///
        void parse(const char* data);

    private:
        friend class COPSReqMsg;
        friend class COPSClientOpenMsg;
        ///
        COPSObjHeader(const char* data)
        {
            parse(data);
        }

        struct
        {
            u_int16_t _len;
            u_int8_t _cNum;
            u_int8_t _cType;
        }
        _dataRep;
};

#endif
