
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSPolElementHeader_cxx_Version =
    "$Id: COPSPolElementHeader.cxx,v 1.4 2001/05/13 07:26:01 icahoon Exp $";


#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSPolElementHeader.hxx"
#include "COPSUtil.hxx"
#include "cpLog.h"

void
COPSPolElementHeader::setDataLength(u_int32_t len)
{
    //Add the length of the header also
    _dataRep._len = len + 4;
}

COPSPolElementHeader::COPSPolElementHeader(const COPSPolElementHeader& other)
{
    _dataRep._len = other._dataRep._len;
    _dataRep._pType = other._dataRep._pType;
}

COPSPolElementHeader&
COPSPolElementHeader::operator=(const COPSPolElementHeader& other)
{
    if (this != &other)
    {
        _dataRep._len = other._dataRep._len;
        _dataRep._pType = other._dataRep._pType;
    }
    return (*this);
}

///Writes data to a given network socket
void
COPSPolElementHeader::writeData(int id) const throw (VNetworkException&)
{
    PolDataHeader pHeader;
    pHeader._len = htons(_dataRep._len);
    pHeader._pType = htons(_dataRep._pType);
    COPSUtil::writeData(id, &pHeader, sizeof(PolDataHeader));
}


void
COPSPolElementHeader::parse(const char* data)
{
    PolDataHeader pHeader;
    memcpy(&pHeader, data, sizeof(PolDataHeader));
    _dataRep._len = ntohs(pHeader._len);
    _dataRep._pType = ntohs(pHeader._pType);
}

string 
COPSPolElementHeader::getStrPType() const
{
    string retStr;
    switch(_dataRep._pType)
    {
        case COPS_POL_EL_RSVP_PROXY_SUPPORT:
        {
            retStr = "RSVP_PROXY_SUPPORT";
        }
        break;
        case COPS_POL_EL_POLICING_SUPPORT:
        {
            retStr = "POLICING_SUPPORT";
        }
        break;
        case COPS_POL_EL_MARKING_SUPPORT:
        {
            retStr = "MARKING_SUPPORT";
        }
        break;
        case COPS_POL_EL_IN_ROLE_COMB:
        {
            retStr = "IN_ROLE_COMB";
        }
        break;
        case COPS_POL_EL_OUT_ROLE_COMB:
        {
            retStr = "OUT_ROLE_COMB";
        }
        break;
        default:
             retStr = "Unknown";
    }
    return retStr;
}

