#ifndef COPSPrGlobalError_hxx
#define COPSPrGlobalError_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSPrGlobalError_hxx_Version =
    "$Id: COPSPrGlobalError.hxx,v 1.4 2000/12/18 23:42:55 bko Exp $";

#include "COPSPrError.hxx"
#include "COPSObjHeader.hxx"
#include "COPSData.hxx"


///COPSPrGlobalError

typedef enum {
    G_availMemoryLow = 1,
    G_availMemoryExhausted = 2,
    G_unknownASN1Tag = 3,
    G_maxMsgSizeExceeded = 4,
    G_unknownError = 5,
    G_maxReqStateOpen = 6,
    G_invalidASN1Length = 7,
    G_invalidObjPad = 8,
    G_unknownPIBData = 9,
    G_unknownCOPSPrObj = 10,
    G_malformedDec = 11,
    G_errmax = 12
} GPERR_code;


static const char* GerrTable[G_errmax] =
    {
        "Reserved",
        "Available memory low",
        "Available memory exhausted",
        "Unknown ASN.1 tag",
        "Max. message size exceeded",
        "Unknown error",
        "No more Request-states can be created by the PEP",
        "ASN.1 object length was incorrect",
        "Object was not properly padded",
        "Unknown PIB data",
        "Unknown COPS-PR object",
        "Melformed decision"
    };

class COPSPrGlobalError : public COPSPrError
{
    public:
        ///
        COPSPrGlobalError(u_int16_t eCode, u_int16_t eSubCode) :
        COPSPrError(eCode, eSubCode)
        {
            assert(eCode < G_errmax);
            _sNum = COPSPrObjBase::PR_GPERR;
            _sType = COPSPrObjBase::PR_BER;
        }

        ///
        COPSPrGlobalError& operator=(const COPSPrGlobalError& other)
        {
            if (this != &other)
            {
                COPSPrError::operator=(other);
            }
            return *this;
        }

        ///
        string className() const
        {
            return "COPSPrGlobalError";
        };

        ///
        bool isGlobalPrError() const
        {
            return true;
        }

        ///
        string strError() const
        {
            return GerrTable[_errCode];
        };

    private:
        friend class COPSPrObjParser;
        friend class COPSReqMsg;
        friend class COPSReportMsg;

        ///Parse the data and create a PrGlobalError object
        COPSPrGlobalError(const char* dataPtr) : COPSPrError(dataPtr)
        { }
        ;
};

#endif
