
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSMsgParser_cxx_Version =
    "$Id: COPSPrObjParser.cxx,v 1.4 2000/12/18 23:42:55 bko Exp $";
#include <assert.h>

#include "COPSPrObjParser.hxx"

Sptr < COPSPrObjBase >
COPSPrObjParser::parse(const COPSData& data) throw (COPSBadDataException&)
{
    COPSPrObjBase* copsPrObject = 0;

    if (data.length() < 4)
    {
        cpLog(LOG_ALERT, "BAD COPS Object");
        throw COPSBadDataException("Bad COPS Object", __FILE__, __LINE__, 0);
    }

    u_int8_t snum;
    memcpy(&snum, data.getData() + 2, 1);
    COPSPrObjBase::PrSNum sNum = static_cast < COPSPrObjBase::PrSNum > (snum);
    switch (sNum)
    {
        case COPSPrObjBase::PR_PRID:
        {
            cpLog(LOG_DEBUG_STACK, "Creating PRID object");
            copsPrObject = new COPSPrID(data.getData());
        }
        break;
        case COPSPrObjBase::PR_PPRID:
        {
            cpLog(LOG_DEBUG_STACK, "Creating PRID Prefix object");
            copsPrObject = new COPSPrefixPrID(data.getData());
        }
        break;
        case COPSPrObjBase::PR_EPD:
        {
            cpLog(LOG_DEBUG_STACK, "Creating EPD object");
            copsPrObject = new COPSPrEPD(data.getData());
        }
        break;
        case COPSPrObjBase::PR_GPERR:
        {
            cpLog(LOG_DEBUG_STACK, "Creating Global Error object");
            copsPrObject = new COPSPrGlobalError(data.getData());
        }
        break;
        case COPSPrObjBase::PR_CPERR:
        {
            cpLog(LOG_DEBUG_STACK, "Creating class error object");
            copsPrObject = new COPSPrClassError(data.getData());
        }
        break;
        case COPSPrObjBase::PR_IDERR:
        {
            cpLog(LOG_DEBUG_STACK, "Creating Error PRID object");
            copsPrObject = new COPSPrErrorPRID(data.getData());
        }
        break;
        default:
        cpLog(LOG_DEBUG_STACK, "Not a PR object type");
        break;
    }
    return copsPrObject;
}
