
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPAuthRespInfo_cxx_Version =
    "$Id: OSPAuthRespInfo.cxx,v 1.10 2001/01/20 01:33:06 wbreu Exp $";
#include <sys/types.h>
#include <string.h>
#include <string>

#include "cpLog.h"
#include "OSPAuthRespInfo.hxx"
#include "OSPToken.hxx"


void OSPAuthRespInfo::initData(
    unsigned int ts, unsigned int cs, unsigned int cns,
    unsigned int ds, unsigned int dds, unsigned int tks )
{
    assert(ts);
    assert(cs);
    assert(cns);
    assert(ds);
    assert(tks);
    _ospAuthResp._tid = OSPInvalidTransHandle;
    _ospAuthResp._status = 0;
    _ospAuthResp._sizeOfTimeStamp = ts;
    _ospAuthResp._sizeOfCallId = cs;
    _ospAuthResp._sizeOfCalledNumber = cns;
    _ospAuthResp._sizeOfDestination = ds;
    _ospAuthResp._sizeOfDestDevice = dds;
    _ospAuthResp._sizeOfToken = tks;
    _ospAuthResp._b64Token = "NULLtoken";

    _ospAuthResp._validAfter = new char[ts];
    assert(_ospAuthResp._validAfter);
    memset(_ospAuthResp._validAfter, 0, ts);

    _ospAuthResp._validUntil = new char[ts];
    assert( _ospAuthResp._validUntil );
    memset( _ospAuthResp._validUntil, 0, ts );

    _ospAuthResp._timeLimit = 0;

    _ospAuthResp._callId = new char[cs];
    assert( _ospAuthResp._callId );
    memset( _ospAuthResp._callId, 0, cs );

    _ospAuthResp._calledNumber = new char[cns];
    assert( _ospAuthResp._calledNumber );
    memset( _ospAuthResp._calledNumber, 0, cns );

    _ospAuthResp._destination = new char[ds];
    assert( _ospAuthResp._destination );
    memset( _ospAuthResp._destination, 0, ds );

    _ospAuthResp._token = new char[tks];
    assert( _ospAuthResp._token );
    memset( _ospAuthResp._token, 0, tks );

    if (dds == 0)
        _ospAuthResp._destinationDevice = 0;
    else
    {
        _ospAuthResp._destinationDevice = new char[dds];
        assert( _ospAuthResp._destinationDevice );
        memset( _ospAuthResp._destinationDevice, 0, dds );
    }

}

///
OSPAuthRespInfo::~OSPAuthRespInfo()
{
    if (_ospAuthResp._validAfter) delete [] _ospAuthResp._validAfter;
    if (_ospAuthResp._validUntil) delete [] _ospAuthResp._validUntil;
    if (_ospAuthResp._callId) delete [] _ospAuthResp._callId;
    if (_ospAuthResp._calledNumber) delete [] _ospAuthResp._calledNumber;
    if (_ospAuthResp._destination)delete [] _ospAuthResp._destination;
    if (_ospAuthResp._destinationDevice) delete [] _ospAuthResp._destinationDevice;
    if (_ospAuthResp._token) delete [] _ospAuthResp._token;

    cpLog(LOG_DEBUG, "OSPAuthRespInfo dtor invoked");
}

///  convert the binary osp token to Base64 encoding format:
bool OSPAuthRespInfo::encodeToken( void )
{
    OSPToken tok;
    cpLog( LOG_DEBUG, "encodeToken invoked");
    return ( tok.encodeToken(
                 reinterpret_cast < const unsigned char* > (_ospAuthResp._token),
                 _ospAuthResp._sizeOfToken, _ospAuthResp._b64Token ));
}

///
void OSPAuthRespInfo::dump(void )
{
    string s;
    s = "OSP Authorization Response: \n";
    s += "Valid After ts = |";
    s += _ospAuthResp._validAfter;
    s += "|\nValid Until ts = |";
    s += _ospAuthResp._validUntil;
    s += "|\nCallId = |";
    s += _ospAuthResp._callId;
    s += "|  Called Number = |";
    s += _ospAuthResp._calledNumber;
    s += "|\nDest = |";
    s += _ospAuthResp._destination;
    s += "|  Dest dev = |";
    s += _ospAuthResp._destinationDevice;
    /* can't print out binary token:
    s += "|\nToken = |";
    s += static_cast<char *>(_ospAuthResp._token);
    */
    s += "|\nB64 Token = |";
    s += _ospAuthResp._b64Token.c_str();
    s += "|\n";

    cpLog(LOG_DEBUG, "%s   TimeLimit = %d   Called Number size = %d "
          "\n   CallId size = %d  Dest size = %d"
          "\n   Dest dev size = %d  Token size = %d",
          s.c_str(), _ospAuthResp._timeLimit, _ospAuthResp._sizeOfCalledNumber,
          _ospAuthResp._sizeOfCallId, _ospAuthResp._sizeOfDestination,
          _ospAuthResp._sizeOfDestDevice, _ospAuthResp._sizeOfToken);
}


// end of File
