
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#ifndef OSPAUTHRESPINFO_HXX
#define OSPAUTHRESPINFO_HXX

static const char* const OSPAuthRespInfo_hxx_Version =
    "$Id: OSPAuthRespInfo.hxx,v 1.14 2001/01/20 01:33:06 wbreu Exp $";



#include <sys/types.h>
#include <string>
#include "ospb64.h"
#include "OSPDefines.hxx"
#include "OSPCfg.hxx"

static const int DEF_TS_SIZE = 32;     // default timestamp size
static const int DEF_CN_SIZE = 20;     // default called number size
static const int DEF_DEST_SZ = 64;     // default destination size
static const int DEF_TOK_SZ = 2096*2;    // default token size


/// Information class needed for authorization request
class OSPAuthRespInfo
{
    public:
        /// default constructor
        OSPAuthRespInfo()
        {
            initData( DEF_TS_SIZE, CALL_ID_SZ, DEF_CN_SIZE,
                      DEF_DEST_SZ, DEF_DEST_SZ, DEF_TOK_SZ );
        }

	/// constructor
        OSPAuthRespInfo( unsigned int ts, unsigned int cs, unsigned cns,
                         unsigned int ds, unsigned int dds, unsigned tks )
        {
            initData( ts, cs, cns, ds, dds, tks );
        }

        /// destructor
        virtual ~OSPAuthRespInfo();

        ///
        // access methods
        // note that there are no "set" methods as most data members are
        // updated by the OSP api.
        //
        /// get id associated with the OSP transaction
        OSPTransactionHandle getTid( void ) const
        {
            return ( _ospAuthResp._tid );
        }

	/// set transaction id
        void setTid( OSPTransactionHandle id )
        {
            _ospAuthResp._tid = id;
        }

	/// get authorization status
        const int getStatus( void ) const
        {
            return ( _ospAuthResp._status );
        }

	/// set authorization status
        void setStatus( int s )
        {
            _ospAuthResp._status = s;
        }

	/// 
        const unsigned int getTimeStampSize( void ) const
        {
            return (_ospAuthResp._sizeOfTimeStamp);
        }

	///
        char * getValidAfterTs( void ) const
        {
            return ( _ospAuthResp._validAfter );
        }

	///
        char * getValidUntilTs( void ) const
        {
            return ( _ospAuthResp._validUntil );
        }

	///
        unsigned int *getTimeLimitLoc( void )
        {
            return ( &_ospAuthResp._timeLimit );
        }

	///
        const unsigned int getTimeLimit( void ) const
        {
            return ( _ospAuthResp._timeLimit );
        }

	///
        unsigned int *getCallIdSizeLoc( void )
        {
            return ( &_ospAuthResp._sizeOfCallId );
        }

	///
        char * getCallId( void ) const
        {
            return ( _ospAuthResp._callId );
        }

	///
        const unsigned int getCalledNumSize( void ) const
        {
            return ( _ospAuthResp._sizeOfCalledNumber );
        }

	///
        char * getCalledNum( void ) const
        {
            return ( _ospAuthResp._calledNumber );
        }

	///
        const unsigned int getDestSize( void ) const
        {
            return ( _ospAuthResp._sizeOfDestination );
        }

	///
        char * getDestination( void ) const
        {
            return ( _ospAuthResp._destination );
        }

	///
        const unsigned int getDestDeviceSize( void ) const
        {
            return ( _ospAuthResp._sizeOfDestDevice );
        }

	///
        char * getDestDevice( void ) const  // optional, value may be empty
        {
            return ( _ospAuthResp._destinationDevice );
        }

	///
        unsigned int * getTokenSizeLoc( void )
        {
            return ( &_ospAuthResp._sizeOfToken );
        }
	///
        const unsigned int getTokenSize( void ) const
        {
            return ( _ospAuthResp._sizeOfToken );
        }

        /// get opaque token, a binary object
        void * getToken( void ) const
        {
            return ( static_cast < void * > (_ospAuthResp._token));
        }

        /// get base 64 encoded token:
        string& getB64Token( void )
        {
            return ( _ospAuthResp._b64Token );
        }

        /// convert binary token into Base 64 encoding format:
        bool encodeToken( void );

        /// debug only
        void dump(void );

    private:
        ///  equal operator - not in use
        bool operator==( const OSPAuthRespInfo& rhs );
        /// copy constructor - not in use
        OSPAuthRespInfo( const OSPAuthRespInfo& rhs );
         
        /// assignment operator - not in use
        OSPAuthRespInfo& operator=( const OSPAuthRespInfo& rhs );

        ///  initialize private data members:
        void initData( unsigned int ts, unsigned int cs, unsigned cns,
                       unsigned int ds, unsigned int dds, unsigned tks );
        // note: all "_size*" data members represents a char string size
        //       including the terminating '\0'
	/// structure containing info extracted from the auth response
        struct
        {
            OSPTransactionHandle _tid;       // OSP transaction id
            int _status;            // request authorized?
            unsigned int _sizeOfTimeStamp;   // size of char string incl. '\0'
            char * _validAfter;
            char * _validUntil;
            unsigned int _timeLimit;     //  updated parm. in seconds
            unsigned int _sizeOfCallId;  //  actual size of call id
            char* _callId;
            unsigned int _sizeOfCalledNumber;
            char* _calledNumber;
            unsigned int _sizeOfDestination;
            char* _destination;
            unsigned int _sizeOfDestDevice;     // optional, OK to be 0
            char* _destinationDevice;         // optional, OK to be empty
            unsigned int _sizeOfToken;   // size of memory buffer in which
            // the auth token is to be stored,
            // on output, it indicates the actual
            // size of the authorization token
            char* _token;             // used as opaque binary object
            string _b64Token;          // Base 64 Encoded token
        }

        _ospAuthResp;
};
#endif

// end of File
