
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 * ====================================================================
 * Some code in this file has been adapted from the OSP Toolkit:
 * ====================================================================
 * THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS TRANSNEXUS 
 * PUBLIC LICENSE ("AGREEMENT").  ANY USE OF THE PROGRAM OR USE, REPRODUCTION 
 * OR DISTRIBUTION OF A DERIVATIVE WORK OF THE PROGRAM CONSTITUTES LICENSEE'S 
 * ACCEPTANCE OF THIS AGREEMENT.
 * 1. GRANT OF RIGHTS
 * a) Subject to the terms of this Agreement, TransNexus grants to Licensee 
 * a limited, non-exclusive, worldwide, royalty-free copyright license to 
 * use version 2.5 of the TransNexus OSP Toolkit ("Program") to prepare 
 * derivative works of the Program ("Derivative Works") and to reproduce, 
 * use, distribute and sublicense the Derivative Works.
 * b) Licensee understands that although TransNexus grants the license as 
 * set forth herein, no assurances are provided by TransNexus that the 
 * Program does not infringe the patent or other intellectual property 
 * rights of any other entity. TransNexus disclaims any liability to 
 * Licensee for claims brought by any other entity based on infringement 
 * of intellectual property rights or otherwise. As a condition to 
 * exercising the rights and licenses granted hereunder, each Licensee 
 * hereby assumes sole responsibility to secure any other intellectual 
 * property rights needed, if any. For example, if a third party patent 
 * license is required to allow Licensee to use the Program or to use, 
 * reproduce or distribute Derivative Works, it is Licensee's responsibility 
 * to acquire that license before taking any such action.
 * 2. REQUIREMENTS
 * A Licensee may choose to distribute Derivative Works under its own 
 * license agreement, provided that:
 * a) The name "TransNexus" must not be used to endorse or promote products 
 * derived from the Program without prior written permission.
 * b) Products derived from the Program may not be called "TransNexus", nor 
 * may "TransNexus" appear in their name, without prior written permission 
 * of TransNexus.
 * 3. COMMERCIAL DISTRIBUTION
 * Commercial distributors of software may accept certain responsibilities 
 * with respect to end users, business partners and the like. While this 
 * license is intended to facilitate the commercial preparation, use and 
 * distribution of Derivative Works, the Licensee who includes one or more 
 * Derivative Works in a commercial product offering should do so in a 
 * manner which does not create potential liability for TransNexus. 
 * Therefore, if a Licensee includes a Derivative Work in a commercial 
 * product offering, such Licensee ("Commercial Licensee") hereby agrees 
 * to defend and indemnify TransNexus against any losses, damages and 
 * costs (collectively "Losses") arising from claims, lawsuits and other 
 * legal actions brought by a third party against TransNexus to the 
 * extent caused by the acts or omissions of such Commercial Licensee in 
 * connection with its distribution of the Derivative Work in a commercial
 * product offering. TransNexus will: a) promptly notify the Commercial 
 * Licensee in writing of such claim, and b) allow the Commercial Licensee 
 * to control, and cooperate with the Commercial Licensee in, the defense 
 * and any related settlement negotiations. TransNexus may participate in 
 * any such claim at its own expense.
 * 
 * 
 */
static const char* const OSPCfg_cxx_Version =
    "$Id: OSPCfg.cxx,v 1.13 2001/03/08 16:33:26 wbreu Exp $";
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "global.h"
#include <iostream>
#include <string>
#include <cassert>



#include "osp.h"
#include "ospprovider.h"
#include "osppkcs1.h"
#include "ospfail.h"
#include "osptnlog.h"
#include "ospasn1.h"
#include "ospsecurity.h"
#include "osppkcs8.h"
#include "ospx509.h"
#include "OSPCfg.hxx"

#include "cpLog.h"
#include "VIoException.hxx"
#include "OSPProvider.hxx"
#include "OSPErrStrings.hxx"
#include "PolicyCfg.hxx"

int verbose = 0;     /* wbr: turn on verbose msg */

/************ Local prototypes */
static off_t FileSizeGet(int fd);
static int FileArrayRead(unsigned char ***array, size_t *arrayNum,
                         int **filesizes, const char *baseName,
                         const char *ext, size_t max);
static void FileArrayDelete(unsigned char ***array, int **filesizes,
                            size_t num);

OSPCfg* OSPCfg::_instance = 0;

const char* const OSPCfg::osptag = "OSP";
        ///
const char* const OSPCfg::servertag = "OSPSERVER";
        ///
const char* const OSPCfg::customeridtag = "CUSTOMERID";


OSPCfg & OSPCfg::Instance( void )
{
    if (_instance == 0)
    {
        _instance = new OSPCfg();
        assert( _instance );
    }
    return ( *_instance);
}

OSPCfg::OSPCfg()
{
    cpLog(LOG_DEBUG, "OSPCfg ctor invoked");

    custid = "1000";
    devid = "1000";
    num_serv_points = 0;
    NUM_CA_CERTS = 0;
    NUM_LOCAL_CERTS = 0;
    NUM_PKEYS = 0;
    certs = 0;
    certssizes = 0;
    localcerts = 0;
    localcertssizes = 0;
    pkeys = 0;
    pkeyssizes = 0;
    privatekey.PrivateKeyData = 0;
    privatekey.PrivateKeyLength = 0;

    readCfgFile();     // read config data to set up OSPProvider
}


void OSPCfg::destroy( void )
{
    cpLog(LOG_DEBUG, "OSPCfg::destroy invoked");
    delete OSPCfg::_instance;
    OSPCfg::_instance = 0;
}

/// 
void OSPCfg::readCfgFile( void )
{
    cpLog(LOG_DEBUG,
          "OSPCfg - readCfgFile() invoked" );

    // get server characteristics:
    try
    {
        getServerConfiguration();
    }
    catch (VMissingDataException &e)
    {   // do not continue
        cpLog(LOG_DEBUG, "getServerConfiguration error");
        return ;
    }

    // for debug:
    display();

    // get certificate info:
    int i = 0;
    char msg[100];

    FileArrayDelete(&certs, &certssizes, NUM_CA_CERTS);
    (void)FileArrayRead(&certs, &NUM_CA_CERTS, &certssizes, "cert", "dat", 99);
    if (!verbose)
        cpLog(LOG_INFO, "Loaded %u CA certificates\n", NUM_CA_CERTS);

    FileArrayDelete(&localcerts, &localcertssizes, NUM_LOCAL_CERTS);
    (void)FileArrayRead(&localcerts, &NUM_LOCAL_CERTS, &localcertssizes,
                        "localcert", "dat", 1);
    if (!verbose)
        cpLog(LOG_INFO, "Loaded %u local certificates\n", NUM_LOCAL_CERTS);

    FileArrayDelete(&pkeys, &pkeyssizes, NUM_PKEYS);
    (void)FileArrayRead(&pkeys, &NUM_PKEYS, &pkeyssizes, "pkey", "dat", 1);
    if (!verbose)
        cpLog(LOG_INFO, "Loaded %u private keys\n", NUM_PKEYS);

    privatekey.PrivateKeyData = pkeys[0];
    privatekey.PrivateKeyLength = pkeyssizes[0];

    if (!verbose)
    {
        for (i = 0 ; i < (int)NUM_CA_CERTS ; i++)
        {
            sprintf(msg, "Loaded Authority Certificate %02d", i);
            OSPTNLOGDUMP(certs[i], certssizes[i], msg);
        }

        OSPTNLOGDUMP(localcerts[0], localcertssizes[0], "Loaded Local Certificate");
        OSPTNLOGDUMP(pkeys[0], pkeyssizes[0], "Loaded Private Key");
    }

}

OSPCfg::~OSPCfg()
{
    cpLog( LOG_DEBUG, "OSPCfg dtor invoked");
    // clean up:
    FileArrayDelete(&certs, &certssizes, NUM_CA_CERTS);
    FileArrayDelete(&localcerts, &localcertssizes, NUM_LOCAL_CERTS);
    FileArrayDelete(&pkeys, &pkeyssizes, NUM_PKEYS);

    NUM_CA_CERTS = 0;
    NUM_LOCAL_CERTS = 0;
    NUM_PKEYS = 0;

    int i;
    // cleanup Service points
    for (i = 0; i < num_serv_points; i++)
    {
        free(servicepoints[i]);
    }

}

// adapted from OSP toolkit:
static int FileArrayRead(unsigned char ***array, size_t *arrayNum, int *filesizes[], const char *baseName, const char *ext, size_t maxFiles)
{
    int Fretval;
    char file[1024 + 1];
    int doIt;
    int fd;
    int *fs;
    unsigned char **cp;

    Fretval = ( -1);
    doIt = 1;
    if ((array != 0) && (arrayNum != 0))
    {
        *array = 0;
        *arrayNum = 0;
        *filesizes = (int *)0;
        if (((cp = (unsigned char **)calloc(1, maxFiles * sizeof(cp))) != 0)
                && ((fs = (int *)malloc(maxFiles * sizeof(int))) != 0))
        {
            size_t x;

            Fretval = 0;
            for (x = 0; (x < maxFiles) && (Fretval == 0); x++)
            {
                sprintf(file, "%s_%05d.%s", baseName, x + 1, ext);
                if ((fd = _Open(file, O_RDONLY | O_BINARY)) >= 0)
                {
                    int s;
                    unsigned char *data;

                    if ((s = FileSizeGet(fd)) > 0)
                    {
                        if ((data = (unsigned char *)malloc(s + 1)) != 0)
                        {
                            cp[x] = data;
                            fs[x] = s;
                            data[s] = 0;   /* So it looks like a string */
                            if (_Read(fd, data, s) != s)
                            {
                                cpLog(LOG_ERR, "Unable to read file %s size %u\n", file, s);
                                Fretval = ( -1);
                            }
                        }
                        else
                        {
                            cpLog(LOG_ERR, "Unable to allocate %u bytes for file %s\n", s, file);
                            Fretval = ( -1);
                        }
                    }
                    else
                    {
                        cpLog(LOG_ERR, "Unable to get file size of <%s>\n", file);
                        Fretval = ( -1);
                    }
                    (void)_Close(fd);
                }
                else
                {
                    break;
                }
            } /* For every file */
            if (Fretval == 0)
            {
                *arrayNum = x;
                *array = cp;
                *filesizes = fs;
            }
            else
            {
                FileArrayDelete(&cp, &fs, maxFiles);
            }
        }
        else
        {
            cpLog(LOG_ERR, "Unable to allocate file pointers\n");
        }
    }
    else
    {
        cpLog(LOG_ERR, "Bad function parameters\n");
    }
    return (Fretval);
} /* FileArrayRead */

static void FileArrayDelete(unsigned char ***array, int **filesizes, size_t num)
{
    unsigned char **p;
    size_t x;

    if ((array != 0) && (num >= 1) && (filesizes != 0))
    {
        p = *array;
        for (x = 0; x < num; x++)
        {
            if (p[x] != 0)
            {
                free(p[x]);
            }
        }
        free(p);
        *array = 0;
        free(*filesizes);
        *filesizes = 0;
    }
    return ;
}

/********************************************************
*
* Function name : FileSizeGet
* Description   :
* Returns       : -1: error else file size
*
********************************************************/
static off_t FileSizeGet(int fd)
{
    off_t x;
    off_t Fretval;

    Fretval = ( -1);
    if (fd != ( -1))
    {
        if ((x = _Lseek(fd, 0, SEEK_CUR)) != ( -1))
        {
            if (_Lseek(fd, 0, SEEK_SET) != ( -1))
            {
                Fretval = _Lseek(fd, 0, SEEK_END);
            }
            if (_Lseek(fd, x, SEEK_SET) == ( -1))
            {
                Fretval = ( -1);
            }
        }
    }
    return (Fretval);
} /* FileSizeGet */


/// get OSP Service Provider data (OSPServerData): let
//  policyCfg do the parsing, just shadow the data here
//  to make the data compatible to the api for the SDK:
void
OSPCfg::getServerConfiguration( void ) throw (VMissingDataException& )
{
    int errorcode = 0;

    try
    {
        const PolicyCfg& cData = PolicyCfg::Instance();
        const VOspServerData& sData = cData.getOSPServerData();

        // servicepoints[10] get service point URLs:
        // const list<string>& sUrl = sData.getServicePoint();
        list < string > sUrl;
        sData.getSvcPoints( sUrl );
        num_serv_points = 0;
        for ( list < string > ::const_iterator itr = sUrl.begin();
                itr != sUrl.end(); itr++ )
        {
            servicepoints[num_serv_points++] = _Strdup((*itr).c_str());
            if (num_serv_points == MAX_SERVICE_POINTS)
            {
                cpLog( LOG_ALERT,
                       "Too many service points configured. Max = %d\n",
                       MAX_SERVICE_POINTS);
                break;        
            }
        }

        localValidation = sData.getlocalValidation();
        sslLifeTime = sData.getSslLifetime();
        httpMaxConnections = sData.getHttpMaxConnections();
        httpPersistence = sData.getHttpPersistence();
        httpRetryDelay = sData.getHttpRetryDelay();
        httpRetryLimit = sData.getHttpRetryLimit();
        httpTimeout = sData.getHttpTimeout();

        custid = sData.getCustomerID();
        devid = sData.getDeviceID();
        
        auditurl = sData.getAuditURL();
    }
    catch (VIoException & e)
    {
        cpLog(LOG_ALERT, "VXMLParser exception: %s",
              e.getDescription().c_str());
    }

    if (num_serv_points == 0 || auditurl.size() == 0)
    {
        string reason =
            "Both service points and an audit URL must be configured\n";
        cpLog( LOG_ALERT, reason.c_str());
        throw VMissingDataException(reason.c_str(), __FILE__,
                                    __LINE__, errorcode);
    }
    
}

/// for debug only
void OSPCfg::display( void )
{
    cpLog(LOG_DEBUG, "\nOSPCfg Data:\n"
          "custid:%s     devid:%s\n"
          "auditurl:%s\n"
          "localValidation: %d\n"
          "sslLifeTime: %d secs\n"
          "httpMaxConnections: %d\n"
          "httpPersistence: %d secs\n"
          "httpRetryDelay: %d secs\n"
          "httpRetryLimit: %d times\n"
          "httpTimeout: %d ms\n"
          "num_serv_points: %d\n",
          custid.c_str(), devid.c_str(), auditurl.c_str(),
          localValidation, sslLifeTime, httpMaxConnections,
          httpPersistence, httpRetryDelay, httpRetryLimit,
          httpTimeout, num_serv_points);

    for (int i = 0; i < num_serv_points; i++)
    {
        cpLog(LOG_DEBUG, "servicepoint %d: |%s|", i, servicepoints[i]);
    }

}

// end of File
