
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#ifndef OSPUSAGEINFO_HXX
#define OSPUSAGEINFO_HXX


static const char* const OSPUsageInfo_hxx_Version =
    "$Id: OSPUsageInfo.hxx,v 1.11 2001/01/20 01:33:06 wbreu Exp $";


#include "cpLog.h"
#include "OSPDefines.hxx"


/// Information class needed for sending Usage Indication
class OSPUsageInfo
{
    public:
        /// default constructor
        OSPUsageInfo( void )
        {
            initData( -1, 0, 0, -1, 0, -1 );
        }

        /** constructor.
         * input: id = transaction Id
         *       d  = duration in seconds
         *       lps = loss packets sent
         *       lfs = loss fraction sent
         *       lpr = loss packets received
         *       lfr = loss fraction received
	 */
        OSPUsageInfo( int id, unsigned int d, unsigned int lps, int lfs,
                      unsigned int lpr, int lfr )
        {
            initData( id, d, lps, lfs, lpr, lfr );
        }

        /// destructor
        virtual ~OSPUsageInfo()
        {}


	/// initialize usage info in this class
        void initData( int id, unsigned int d, unsigned int lps, int lfs,
                       unsigned int lpr, int lfr )
        {
            _ospUsageInfo._tid = id;
            _ospUsageInfo._duration = d;
            _ospUsageInfo._lossPacketSent = lps;
            _ospUsageInfo._lossFractionSent = lfs;
            _ospUsageInfo._lossPacketReceived = lpr;
            _ospUsageInfo._lossFractionReceived = lfr;
        }

        /// get transaction handle
        OSPTransactionHandle getID( void ) const
        {
            return (_ospUsageInfo._tid);
        }

	/// set transaction handle
        void setID( OSPTransactionHandle id )
        {
            _ospUsageInfo._tid = id;
        }

        /// get call duration in seconds
        unsigned int getDuration( void ) const
        {
            return ( _ospUsageInfo._duration);
        }
        /// set call duration
        void setDuration( unsigned int d )
        {
            _ospUsageInfo._duration = d;
        }

        /// get # sent packets loss - not used
        unsigned int getLossPacketSent( void ) const
        {
            return ( _ospUsageInfo._lossPacketSent );
        }
        /// set # sent packets loss - not used
        void setLossPacketSent( unsigned int lps )
        {
            _ospUsageInfo._lossPacketSent = lps;
        }

        /// get fraction of sent packets loss
        int getLossFractionSent( void ) const
        {
            return ( _ospUsageInfo._lossFractionSent );
        }
        /// set fraction of sent packets loss
        void setLossFractionSent( unsigned int lfs )
        {
            _ospUsageInfo._lossFractionSent = lfs;
        }

        /// get # received packets lost - not used
        unsigned int getLossPacketReceived( void ) const
        {
            return ( _ospUsageInfo._lossPacketReceived );
        }
        /// set # received packets lost - not used
        void setLossPacketReceived( unsigned int lpr )
        {
            _ospUsageInfo._lossPacketReceived = lpr;
        }

        /// get fraction of lost received packets
        int getLossFractionReceived( void ) const
        {
            return ( _ospUsageInfo._lossFractionReceived );
        }
        /// set fraction of lost received packets
        void setLossFractionReceived( unsigned int lfr )
        {
            _ospUsageInfo._lossFractionReceived = lfr;
        }

        /// debug only
        void dump(void ) 
        {
            cpLog(LOG_DEBUG,
            "OSPUsageInfo - (transid: %d, duration:%d, lpSent:%d, lfSent:%d, lpRevd:%d, lfRcvd:%d)",
            _ospUsageInfo._tid,
            _ospUsageInfo._duration, _ospUsageInfo._lossPacketSent,
            _ospUsageInfo._lossFractionSent, _ospUsageInfo._lossPacketReceived,
            _ospUsageInfo._lossFractionReceived );
        }

    private:
        /// not used: equal operator
        bool operator==( const OSPUsageInfo& rhs );
        /// not used: copy constructor
        OSPUsageInfo( const OSPUsageInfo& rhs );
        /// not used: assignment operator
        OSPUsageInfo& operator=( const OSPUsageInfo& rhs );

        /// structure containing info for a Usage Report
        struct
        {
            OSPTransactionHandle _tid;  // transaction id
            unsigned int _duration;    // duration of the call in seconds
            unsigned int _lossPacketSent;
            int _lossFractionSent;    // if -ve, assume no statistics
            // for packets sent available
            unsigned int _lossPacketReceived;
            int _lossFractionReceived;   // if -ve, assume no statistics
            // for packets rec'vd avail.
        }

        _ospUsageInfo;

};

#endif

// end of File
