
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PolicyTranslator_hxx_Version =
    "$Id: PolicyTranslator.hxx,v 1.10 2001/01/20 01:33:06 wbreu Exp $";
#ifndef POLICYTRANSLATOR_HXX
#define POLICYTRANSLATOR_HXX




#include "COPSMsg.hxx"
#include "COPSHeader.hxx"
#include "PolicyTransaction.hxx"
#include "PolicyBadDataException.hxx"
#include "OSPException.hxx"

/**
 * Object of this class acts as a conduit between the PDP Agent
 * (COPS Server) and OSP Client
 */
class PolicyTranslator
{

    public:
        /** constructor
	 */
        PolicyTranslator(void);

        /** destructor
	 */
        virtual ~PolicyTranslator();

        /** send an OSP request (either authorization or validation) to
	 *  the OSP Server
	 */
        int sendReq( const COPSMsg & cMsg, PolicyTransaction & pol ) throw
        ( PolicyBadDataException & );

        /** send an OSP Usage report
	 */
        int sendUsage( const COPSMsg & cMsg, PolicyTransaction & pol ) throw
        ( PolicyBadDataException & , OSPException& );

        /** transaction (call) is aborted
	 */
        int abandonTrans( const string & reason, PolicyTransaction & pol );


    protected:
        /**  translate COPS request to an OSP authorization req for an 
	 *   outgoing call
	 */
        int translateRequest(const COPSMsg &, PolicyTransaction & pol ) throw
        ( PolicyBadDataException & , OSPException&);

        /** translate COPS usage req to an OSP Usage report when a call
	 *  terminates
	 */
        int translateUsage(const COPSMsg &, PolicyTransaction & pol ) throw
        ( PolicyBadDataException & , OSPException& );

        ///  validate a requested incoming call
        int translateValRequest( const COPSMsg &, PolicyTransaction & pol ) throw
        ( PolicyBadDataException & ) ;

    private:

	/// equal operator - illegal to use
        bool operator==( const PolicyTranslator& rhs );
           
	/// copy constructor - illegal to use
        PolicyTranslator( const PolicyTranslator& rhs );
         
        /// assignment operator - illegal to use
        PolicyTranslator& operator=( const PolicyTranslator& rhs );

};


#endif
