#ifndef COPSAgent_hxx
#define COPSAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSAgent_hxx_Version =
    "$Id: COPSAgent.hxx,v 1.10.22.1 2002/05/15 01:24:05 sprajpat Exp $";

#include "VBadDataException.hxx"
#include "VNetworkException.hxx"

class COPSAgent;
class PolicyTransaction;
class COPSReqMsg;
class COPSDecisionMsg;
class COPSReportMsg;
class COPSDecision;

/** COPSAgent class is the base class for handling various
 *  types of COPS requests
 */
class COPSAgent
{
    public:
        ///
        virtual void processRequest(const COPSReqMsg& reqMsg)
        throw (VBadDataException&, VNetworkException&) = 0;
        ///
        virtual void processReport(const COPSReportMsg& repMsg)
        throw (VBadDataException&, VNetworkException&) = 0;

        ///
        void setConnId(int connId)
        {
            _connId = connId;
        };

    protected:
	///
        int _connId;
};

#ifdef HAS_OSP
/// specific COPS Agent handling COPS messages destined for the OSP Client
class COPSOSPAgent : public COPSAgent
{
    public:
        ///
        void processRequest(const COPSReqMsg& reqMsg)
        throw (VBadDataException&, VNetworkException&);
        ///
        void processReport(const COPSReportMsg& repMsg)
        throw (VBadDataException&, VNetworkException&);
        ///
        void sendOSPDecision(const PolicyTransaction& polT, const COPSReqMsg& msg) const;

};
#endif

/// COPS Agent handling messaging between PDP and RSVP COPS client
class COPSRSVPAgent : public COPSAgent
{
    public:
        ///
        void processRequest(const COPSReqMsg& reqMsg)
        throw (VBadDataException&, VNetworkException&);

        ///
        void checkForQoS(const COPSReqMsg& msg, COPSDecision& dec1);

        ///
        void processReport(const COPSReportMsg& repMsg)
        throw (VBadDataException&, VNetworkException&);
};

/// COPS Agent handling QoS requests between the SIP/PEP and PDP
class COPSQoSAgent : public COPSAgent
{
    public:
        ///
        void processRequest(const COPSReqMsg& reqMsg)
        throw (VBadDataException&, VNetworkException&);
        ///
        void processReport(const COPSReportMsg& repMsg)
        throw (VBadDataException&, VNetworkException&);
};

/// COPS Agent used to interface with Provisioning
class COPSProvisioningAgent : public COPSAgent
{
    public:
        ///
        void processRequest(const COPSReqMsg& reqMsg)
        throw (VBadDataException&, VNetworkException&);
        ///
        void processReport(const COPSReportMsg& repMsg)
        throw (VBadDataException&, VNetworkException&);
};

/// generic vanilla COPS Agent
class COPSDefaultAgent : public COPSAgent
{
    public:
        ///
        void processRequest(const COPSReqMsg& reqMsg)
        throw (VBadDataException&, VNetworkException&);
        ///
        void processReport(const COPSReportMsg& repMsg)
        throw (VBadDataException&, VNetworkException&);
};

#endif
