#ifndef  PdpAgent_hxx
#define PdpAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PdpAgent_hxx_Version =
    "$Id: PdpAgent.hxx,v 1.19 2001/06/29 21:02:40 bko Exp $";

#include "global.h"
#include <sys/time.h>
#include <sys/types.h>
#include <map>
#include <string>

#include "HeartbeatTxThread.hxx"

#include "Tcp_ServerSocket.hxx"
#include "Sptr.hxx"
#include "VMutex.h"
#include "COPSDecisionMsg.hxx"
#include "COPSData.hxx"
#include "VThreadPool.hxx"
#include "QoSData.hxx"
#include "VNetworkException.hxx"
#include "VMissingDataException.hxx"
#include "VException.hxx"

class Connection;
class COPSMsg;

/**
    Front end for any COPS Connection coming from a PEP.
*/
class PdpAgent
{
    public:
        ///Interface to get ref. to singelton object.
        static PdpAgent& instance();

        ///
        string className()
        {
            return "PdpAgent";
        }

        ///
        void run();

        ///Destructor
        ~PdpAgent();

        ///Signal handler for SIGTERM
        static void sigTerm(int signo);
        ///Signal handler to cleanup the child
        static void sigChld(int signo);

    protected:
        ///
        Sptr < HeartbeatTxThread > itsHeartbeatTxThread;

    private:

        ///
        PdpAgent();

        ///
        bool isQoSAllowed(const string ip1, const string& ip2);

        ///
        void processRequest(Connection& conn);

        ///
        static void processRequestFunc(void* conn) ;

        ///
        void insertDecision(const COPSData& key, Sptr < COPSDecisionMsg > decMsg);

        ///
        void removeDecision(const COPSData& key)
            throw (VMissingDataException&);

        ///
        void insertLocalPolicy(const string& callId, const QoSData& qData);

        ///
        void removeLocalPolicy(const string& callId);

        ///
        const COPSContext& findDecisionContext(const COPSData& key)
             throw (VMissingDataException&);


        ///Validates the client type in each COPS OPEN Msg
        void validateClientType(const Connection& conn, const COPSMsg& msg)
             throw (COPSBadDataException&);


     

        ///Handle Client Open Message
        void handleClientOpenMsg(const Connection& conn, const COPSMsg& msg);

        ///Handle Client Close Message, close the passed connection
        void handleClientCloseMsg(Connection& conn, const COPSMsg& msg);

        ///
        void handleDeleteRequestMsg(const Connection& conn, const COPSMsg& msg);

        ///
        void handleRequestMsg(const Connection& conn, const COPSMsg& msg);

        ///
        void handleKeepAliveMsg(const Connection& conn, const COPSMsg& msg);


        /**
           Frees the memory associated with singelton instance.
           gets register to atexit() function at the time of creation.
        */
        static void destroy(void);

        static PdpAgent* _instance;

        Sptr < TcpServerSocket > _serverSocket;
        Sptr < VThreadPool > _threadPool;

        ///Maps a PEPID to a connection
        typedef map < string, Connection > ConnectionMap;

        ///Maps a client handle to a decision message
        typedef map < COPSData, Sptr < COPSDecisionMsg > > PdpDecisionMap;
        PdpDecisionMap _decisionMap;

        ///Holds the list of clients, notification will be sent out.
        static ConnectionMap _connectionMap;

        typedef map < string, QoSData > LocalPolicyData;
        LocalPolicyData _localPolicyData;

        VMutex  _lock;

        ///KeepAlive timer value in seconds
        int _kaTimerValue;

        ///PDP Host IP
        string _pdpHostIp;

        friend class COPSProcessor;
        friend class COPSOSPAgent;
        friend class COPSRSVPAgent;
        friend class COPSQoSAgent;
};
#endif
