
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const testOSPMsg_cxx_Version =
    "$Id: testOSPMsg.cxx,v 1.10 2001/02/06 21:54:33 wbreu Exp $";
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>

#include "PepAgent.hxx"
#include "PolicyCfg.hxx"
#include "ProvisionInterface.hxx"
#include "CommandLine.hxx"
#include "OSPTime.hxx"

void callThread(void*)
{
    cpLog(LOG_DEBUG, "Running thread (%d)", pthread_self());
    int i = 50;
    while (i)
    {
        char buf[56];
        int j = 1 + (int) (10.0 * rand() / (RAND_MAX + 1.0));
        sleep(j);
        //make call
        sprintf(buf, "%d-%d-%d", j, (int)pthread_self(), (int)OSPTime::getCurrentTime());
        //        sprintf(buf, "%s", "E944D9A6-88C509CE-0-2888184C@192.168.6.210");
        //sprintf(buf, "%s", "E944D9A6-88C509CE-0-2888184C@19");
        PepAgent::instance().auth("14048724887", "14048729515", buf);
        i--;
    }
}

void
callValidation(void*)
{
    ifstream ifin("./token.bin");
    if (!ifin.good())
    {
        cerr << "Failed to open ./token.bin" << endl;
        return ;
    }
    char cBuf[2000];
    int i = 0;
    int chk;
    while ((chk = ifin.get()) != EOF)
    {
        cBuf[i++] = chk;
    }
    COPSData cData(cBuf, i);

    cerr << "Token :[" << cData.str().c_str() << "]" << endl;

    PepAgent::instance().validateToken("14082422211", "14082422211",
                                       "234567", cData);

    int sCnt = 0;
    while (true)
    {
        sCnt++;
        if (sCnt > 10) break;
    }
}


void notifyFunc(PolRequest& polReq)
{
    cerr << "***notifyFunc***" << endl;

    if (polReq.getStatus() == PolRequest::STATUS_OK)
    {
        cerr << "Token:" << polReq.getToken() << endl;
        //polReq.implAbort();
        polReq.implStart();
        int j = 1 + (int) (10.0 * rand() / (RAND_MAX + 1.0));
        sleep(j);

        string callId = polReq.getCallId();
        PolRequest& req2 = PepAgent::instance().getPolReq(callId);
        req2.implFinish();
    }
    else
    {
        cerr << "AUTHENTICATION FAILURE" << endl;
    }
}

int
main(int argc, const char** argv)
{
    try
    {
        cpLogSetPriority(LOG_DEBUG);

        Sptr < CommandLine > cIn = CommandLine::instance(argc, argv, "Usage: testOSP -d -v LOG_DEBUG  pserver:port pserver:port xxxx");
        string pHost = cIn->getString("pshostname");
        int pPort = cIn->getInt("psremoteport");
        cpLog(LOG_DEBUG, "ProvisionServer:%s", pHost.c_str());
        cpLog(LOG_DEBUG, "ProvisionServer port:%d", pPort);

        PolicyCfg& pcfg = PolicyCfg::Instance(pHost.c_str(), pPort, 
												                      pHost.c_str(),pPort);
        PepAgent::instance(pHost.c_str(), pPort).registerCallback(notifyFunc);

        cerr << "POLICY CFG:" << pcfg << endl;

        VThreadPool pool(5);
        VFunctor functor(callThread, NULL);
        pool.addFunctor(functor);

        while (true)
        {
            sleep(1);
        }
    }
    catch (VException& e)
    {
        cerr << "Caught exception:" << e.getDescription() << endl;
        exit( -1);
    }
}
