#ifndef _SERVERTABLE_H
#define _SERVERTABLE_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ServerTable_hxx_Version = "";

/**
 * this class keeps the information on applications (application table) that 
 * is needed to implement the networkApplicationMib
 */

#ifdef __cplusplus
#include "VMutex.h"
#include <set>
#endif 

/*  the following structure is the only common portion between
    'c' and 'c++' */
    typedef struct {
	unsigned long index;
	char hostName[256];
	int port;
	int type;
	int status;
    } serverInfoT;



#ifdef __cplusplus

class tableElement
{
public:
    tableElement(unsigned long index)
	{ 
	    element.index = index; 
	};
    tableElement(serverInfoT tmpServer);

    ~tableElement(){};

    bool operator ==(const tableElement& x)
	{
	    return element.index == x.element.index;
	}
    bool operator <(const tableElement& x) const
	{
	    return element.index < x.element.index;
	}

    serverInfoT element;
};


class ServerTable
{
    public :
        ///
        ServerTable();
        ///
        ~ServerTable();
    ///
    /// return a pointer to the server with index.  
    /// return null if nune available.
    serverInfoT getServerFromIndex(unsigned long index);
    ///
    /// return the value of the next index within the server table's row.
    unsigned long getNextServerIndex(unsigned long index);

    ///
    /// delete all entries in the server cache.
    void clearServerCache();

    ///
    /// create a new server in the server table and place it in 
    /// the proper order based on index.
    void insertServer(serverInfoT aServer);
private :

    set < tableElement, less < tableElement > > serverCache;
    set < tableElement, less <tableElement> > ::iterator apI;

    VMutex tableLock;

};

#endif /* ifdef __cplusplus */

#endif /* _SERVERTABLE_H */
