/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const FsBuilder_cxx_Version =
    "$Id: FsBuilder.cxx,v 1.33 2002/03/19 21:39:39 jason Exp $";
#include "FsBuilder.hxx"
#include "FsUsers.hxx"
#include "SipEvent.hxx"
#include "FsProvisioning.hxx"
#include "InviteMsg.hxx"
#include "TimerEvent.hxx"
#include "CPLFeatureBuilder.hxx"
#include "FsFeature.hxx"
#include "CPLStartStateHolder.hxx"
#include "FsStateMachnData.hxx"
#include "AckMsg.hxx"

using namespace Vocal;


FsBuilder::FsBuilder()
{
    FsProvisioning::Instance()->initialize();

    createDefaultFeature();

    FsUsers::instance()->showUsers();
}



FsBuilder::~FsBuilder()
{
}


/// This method is the entry point for the Feature Server call processing:
/// all events (timer and sip events) are handled by this method.
/// Once the call data information and the feature associated with the user
/// have been retrieved then the event is passed to the user's feature
/// state machine (feature::process method)

void
FsBuilder::process( const Sptr < SipProxyEvent > nextEvent )
{
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( nextEvent );

    if ( sipEvent != 0 )
    {
        cpLog (LOG_DEBUG, "SIP event" );

        string featureCode;

        //First search to see if CallInfo exists
        Sptr < CallInfo > callInfo =
                myCallContainer->findCall( sipEvent->getSipCallLeg() );

        if ( callInfo == 0)
        {
            cpLog (LOG_DEBUG, "No CallInfo found" );

            Sptr < InviteMsg > inviteMsg;
            inviteMsg.dynamicCast( sipEvent->getSipMsg() );
            if ( inviteMsg != 0 )
            {
                cpLog (LOG_DEBUG, "INVITE, so create new callInfo" );
                callInfo = myCallContainer->getCall( sipEvent->getSipCallLeg() );
                assert( callInfo != 0 );

                Sptr < FsStateMachnData > callData = new FsStateMachnData;
                callInfo->setCallStateData( callData );

                //Strip out any feature codes
                getFeatureCode( nextEvent, callInfo, featureCode );

                if (featureCode.length() > 0 )
                    callData->setFeatureCode( featureCode );

            } /*if*/
            else
            {
               Sptr < AckMsg > ack;
               Sptr < StatusMsg > status;

                ///if status is 487 deal with it
               if ( status.dynamicCast( sipEvent->getSipMsg() ) != 0)
               {
                    cpLog (LOG_DEBUG, "Got 487, handling it" );
                    assert(status != 0);
                    if(status->getStatusLine().getStatusCode() == 487)
                    {
                       status->removeVia(0);
                       sipEvent->getSipStack()->sendReply( status ); 
                       //Create the ACK msg of the original Invite and send
                       Sptr<InviteMsg> invMsg = sipEvent->getInvite();
                       assert(invMsg != 0);
                       Sptr<AckMsg> ackMsg = new AckMsg(*status);
                       ackMsg->flushViaList();

                       SipVia via = invMsg->getVia(0);
                       ackMsg->setVia(via, 0);

                       Sptr<SipUrl> sUrl;
                       sUrl.dynamicCast(invMsg->getRequestLine().getUrl());
                       SipRequestLine rLine(SIP_ACK, sUrl);
                       ackMsg->setRequestLine(rLine);

                       sipEvent->getSipStack()->sendAsync( ackMsg );
                    }
                }
               else if (ack.dynamicCast( sipEvent->getSipMsg() ) != 0 )
               {
                  cpLog(LOG_DEBUG, "Default Action: proxying ACK - not implemented");
               }
               else
               {
                  cpLog (LOG_DEBUG, "Not INVITE, so ignore event" );
               }
               return ;   //thereby discard handling this event
            }
        }  /*if*/

        Sptr < StatusMsg > statusMsg;
        statusMsg.dynamicCast( sipEvent->getSipMsg() );
        if ( statusMsg != 0 )
        {
            cpLog (LOG_DEBUG, "Status message" );
            LocalScopeAllocator lo;
            Data method = statusMsg->getCSeq().getMethod();
            if ((statusMsg->getStatusLine().getStatusCode() == 302) &&
                    (strcmp(sipEvent->getSipMsg()->
                            getFrom().getUser().getData(lo),  DisableCallerID ) == 0))
            {
                Sptr<FsStateMachnData> fsCallData;
                fsCallData.dynamicCast( callInfo->getCallStateData() );
                assert( fsCallData != 0 );

                SipFrom from = fsCallData->getOrigFrom();
                cpLog(LOG_DEBUG, "Found the saved From is: %s",
                      from.encode().logData());
                //remove the added entry in callcontainer
                myCallContainer->removeCall( sipEvent->getSipCallLeg());
                sipEvent->getSipMsg()->setFrom(from);
                   
                //Using the new 'From', reset the SipCallLeg using this: 
                sipEvent->setSipMsg( sipEvent->getSipMsg() );
                sipEvent->setCallLeg();

                // reset callInfo
                callInfo = myCallContainer->findCall(
                               sipEvent->getSipCallLeg() );
                if (callInfo == 0)
                {
                    cpLog (LOG_DEBUG, "CallInfo not found, so discard event" );
                    return ;   //thereby discard handling this event
                }

                else
                {
                    SipFrom resetOrigFrom;
                    fsCallData->setOrigFrom(resetOrigFrom);
                }
            }
        }

        // For FS each user has a different feature
        Sptr < Feature > feature = callInfo->getFeature();

        //TODO: move these few line to where callInfo is allocated?
        //If CallInfo is brand new, then a feature needs to be retrieved.
        if (feature == 0)
        {
            feature = FsBuilder::getFeature( nextEvent, featureCode );
            assert( feature != 0 );
            callInfo->setFeature(feature);
        }
        sipEvent->setCallInfo ( callInfo, myCallContainer );

        //Pass the event to the user's feature for processing.
        callInfo->getFeature()->process(nextEvent);

        //Detach CallInfo from SipEvent, otherwise smart pointer for
        //CallInfo will never decrement to zero.
        sipEvent->removeCallInfo();

    } /*if sipEvent != 0*/
    else
    {
        Sptr < TimerEvent > timerEvent;
        timerEvent.dynamicCast( nextEvent );
        if (timerEvent != 0 )
        {
            cpLog( LOG_DEBUG, "Timer event[%x]",
                   timerEvent.getPtr() );
            Sptr < CallInfo > callInfo = timerEvent->getCallInfo();
            assert ( callInfo != 0 );

            callInfo->getFeature()->process(nextEvent);

            //Detach event from Callinfo to force smart pointer to
            //TimerEvent to zero where it can be destructed.
            //However, we would remove the TimerEvent ONLY if a new one
            //has not been added since calling the State Machine operators.
            Sptr < TimerEvent > timerCheck;
            timerCheck = timerEvent->getCallInfo()->getTimer();
            if ( timerCheck != 0 )
                if ( timerEvent == timerCheck )
                    callInfo->removeTimer();

            //Detach CallInfo from event, otherwise smart pointer
            //for CallInfo will never reach zero.
            timerEvent->removeCallInfo();

        }
        else
            cpLog( LOG_ERR, "Unknown Event." );
    } /*else*/


    cpLog( LOG_DEBUG, "Done Processing Event." );

}

///
bool
FsBuilder::getFeatureCode( Sptr < SipProxyEvent > nextEvent,
                           Sptr < CallInfo > callInfo,
                           string& ftrCode )
{
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( nextEvent );

    Data prefix;
    SipTo savedSipTo;
    bool unBlk;

    ftrCode = preprocess(sipEvent, prefix, savedSipTo, unBlk);

    Sptr< FsStateMachnData> fsCallData;
    fsCallData.dynamicCast( callInfo->getCallStateData() );
    assert( fsCallData != 0 );

    fsCallData->setPrefixCIDBlk(prefix);
    fsCallData->setPrefixTo(savedSipTo);
    fsCallData->setNoCIDBlk(unBlk);

    // if Calleg is still the same, addCall automatically won't add more entry
    myCallContainer->addCall(sipEvent->getSipCallLeg(), callInfo);

    return true;

}  //getFeatureCode


///
Sptr < Feature >
FsBuilder::getFeature( Sptr < SipProxyEvent > event, const string& ftrCode )
{
    Sptr < Feature > ftr;

    ftr = getSpecialFeature( event, ftrCode );
    if (ftr != 0 ) return ftr;

    ftr = getCPLFeature( event );
    if (ftr != 0 ) return ftr;

    return getDefaultFeature( event );

}  //getFeature

/// A special feature is a non-CPL based feature
Sptr < Feature >
FsBuilder::getSpecialFeature( Sptr < SipProxyEvent > event, const string& ftrCode )
{
    if (ftrCode.length() > 0)
    {
        Sptr < Feature > ftr = 0;

        Sptr < SipEvent > sipEvent;
        sipEvent.dynamicCast( event );

        assert ( sipEvent != 0 );

        Sptr < FsUserData > userData;
        string userTo = ftrCode;
        userData = FsUsers::instance()->getUserData( userTo );

        // assume ftrCode is always using Called ftr
        if ( userData != 0 )
        {
            ftr = userData->getCalledFeature();
            if ( ftr != 0 )
            {
                cpLog( LOG_DEBUG, "Found special feature for %s",
                       ftrCode.c_str());
                return ftr ;
            }
        } /*if*/
        else
        {
            cpLog(LOG_DEBUG, "No UserData found");
        }

        cpLog( LOG_ERR, "No special feature for %s", ftrCode.c_str());

        return ftr;
    }
    return 0;

}  //getSpecialFeature


///
Sptr < Feature >
FsBuilder::getCPLFeature( Sptr < SipProxyEvent > event )
{
    Sptr < Feature > feature = 0;

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    if ( sipEvent == 0 )
    {
        assert ( 0 );
    }
    else 
    {
        Sptr < InviteMsg > inviteMsg;
        LocalScopeAllocator lo;
        inviteMsg.dynamicCast( sipEvent->getSipMsg() );
        if ( inviteMsg != 0 )
        {
            //The feature server may have a feature configured for the
            //caller (outgoing feature) or a feature configured for the
            //called (incoming feature) party.   The feature server can
            //only run 1 feature if both are available.  The outgoing
            //feature has higher precedence.
            string userFrom;
            string userTo;
            Sptr < FsUserData > userData;
            LocalScopeAllocator lo;

            //Look for an outgoing feature
            userFrom = inviteMsg->getFrom().getUrl()->getUserValue().getData(lo);
            userFrom[ userFrom.size() ] = 0;
            userData = FsUsers::instance()->getUserData( userFrom );
            if ( userData != 0 )
            {
                //Search for the originating-party's outgoing feature.
                //It may need to be downloaded on-demand if it wasn't
                //already downloaded.  After the download, if the user
                //still doesn't have an outgoing feature, then try to
                //find the destination party's incoming Feature.
                feature = userData->getCallingFeature();
                if ( feature == 0 )
                {
                    //Does this user have ANY features??
                    if ( userData->getCalledFeature() == 0 )
                    {
                        //Maybe feature hasn't been downloaded yet.  Try to
                        //contact Provisioning Server for user's features.
                        FsProvisioning::Instance()->retrieveUserCPLFeatures( userData->getUserFileName().getData(lo) );
                        feature = userData->getCallingFeature();
                        if ( feature != 0 )
                            return feature;
                    } /*if*/
                } /*if*/
                else
                    return feature;
            } /*if*/

            //Look for a incoming feature
            userTo = inviteMsg->getRequestLine().getUrl()->getUserValue().getData(lo);
            userTo[ userTo.size() ] = 0;
            userData = FsUsers::instance()->getUserData( userTo );
            if ( userData != 0 )
            {
                //Search for the destination party's incoming feature.
                //It may need to be downloaded on-demand if it wasn't
                //already downloaded.
                feature = userData->getCalledFeature();
                if ( feature == 0 )
                {
                    //Does this user have ANY features?
                    if ( userData->getCallingFeature() == 0 )
                    {
                        //Maybe feature hasn't been downloaded yet.  Try to
                        //contact Provisioning Server for user's features.
                        FsProvisioning::Instance()->retrieveUserCPLFeatures(userData->getUserFileName().getData(lo) );
                        feature = userData->getCalledFeature();
                        if ( feature != 0 )
                            return feature;
                    } /*if*/
                } /*if*/
                else
                    return feature;
            } /*if*/

            cpLog( LOG_ERR, "No outgoing feature for %s.", userFrom.c_str());
            cpLog( LOG_ERR, "No incoming feature for %s.", userTo.c_str());
        }
        else
        {
            cpLog( LOG_ERR, "Not an INVITE message" );
        }
    }

    return feature;
}    // FsBuilder::getCPLFeature


/// The default features are created in createDefaultFeature and consists
/// of a reject cpl feature
Sptr < Feature >
FsBuilder::getDefaultFeature( Sptr < SipProxyEvent > event )
{
    return myDefaultFeature;

} //getDefaultFeature


///
void
FsBuilder::createDefaultFeature()
{
#if 0 //Uncomment for debugging only
    cpLog( LOG_DEBUG, "Creating default feature." );
#endif

    string userFilename( "DEFAULT" );
    string cplScript("<?xml version=\"1.0\" ?> <!DOCTYPE cpl SYSTEM \"cpl.dtd\"> <cpl> <incoming> <reject status=\"error\" reason=\"Invalid or unregistered user.\"/> </incoming> </cpl> " );

    Sptr < CPLStartStateHolder > featureStartState;
    featureStartState = CPLFeatureBuilder::Instance()->create(cplScript);

    if (featureStartState == 0)
    {
        cpLog( LOG_ERR, "Could not create default feature." );
        assert ( 0 );
    } /*if*/

    //Store the incoming feature if available
    if ( featureStartState->getIncoming() != 0 )
    {
        Sptr < FsFeature > newFeature = new FsFeature;
        newFeature->setFeatureScript( userFilename );
        newFeature->addState( featureStartState->getIncoming() );
        myDefaultFeature = newFeature;
        return ;
    } /*if*/

    //Store the outgoing feature if available
    if ( featureStartState->getOutgoing() != 0 )
    {
        Sptr < FsFeature > newFeature = new FsFeature;
        newFeature->setFeatureScript( userFilename );
        newFeature->addState( featureStartState->getOutgoing() );
        myDefaultFeature = newFeature;
        return ;
    } /*if*/

    cpLog( LOG_ERR, "No Start states were found." );
    assert( 0 );
}  //createDefaultFeature

string
FsBuilder::preprocess( Sptr < SipProxyEvent > nextEvent, Data& prefix, SipTo& sipTo, bool& unblock )
{
#if 0 //Change to 1 if needed for debugging
    cpLog(LOG_DEBUG, "FsBuilder::preprocess() *******");
#endif

    string ret;

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( nextEvent );

    Sptr < SipMsg > sipMsg;
    sipMsg = sipEvent->getSipMsg();
    assert (sipMsg != 0 );

    unblock = false;

    SipTo to = sipMsg->getTo();
    Sptr<BaseUrl> toUrl = to.getUrl();
#if 0 //Change to 1 if needed for debugging
    cpLog(LOG_DEBUG, "the callee orig user is %s", toUrl.getUserValue().logData());
#endif

    Data thePrefix;
    if (toUrl->getUserValue().match( EnableCallerID , &thePrefix, false) == FIRST)
    {
        cpLog(LOG_DEBUG, "found thePrefix %s in ToUrl",  EnableCallerID );
        unblock = true;
        sipTo = to;
        prefix = Data( EnableCallerID );
        removePrefix(sipMsg, Data( EnableCallerID ));
    }
    if (toUrl->getUserValue().match( DisableCallerID , &thePrefix, false) == FIRST)
    {
        cpLog(LOG_DEBUG, "found thePrefix %s in ToUrl",  DisableCallerID );

        ret = string( DisableCallerID );

        sipTo = to;
        prefix = Data( DisableCallerID );
        removePrefix(sipMsg, Data( DisableCallerID ));
    }

    sipEvent->setSipMsg(sipMsg);  //reset CallLeg
    sipEvent->setCallLeg();

    return ret;
}

void
FsBuilder::removePrefix(const Sptr < SipMsg > sipMsg , const Data& thePrefix) 
{
    SipTo to = sipMsg->getTo();
    Sptr<BaseUrl> toUrl = to.getUrl();

    Data prefix;
    {
        LocalScopeAllocator lo;
        Data tempUser = toUrl->getUserValue();
        tempUser.match(thePrefix.getData(lo), &prefix, true, "");
        toUrl->setUserValue(tempUser);
        to.setUrl(toUrl);

        Data dispName = to.getDisplayName();
        dispName.match(thePrefix.getData(lo), &prefix, true, "");
        to.setDisplayName(dispName);

        sipMsg->setTo(to);
        cpLog(LOG_DEBUG, "The modified To Url is:%s",
              sipMsg->getTo().getUrl()->encode().logData());

        Sptr < InviteMsg > inviteMsg;
        inviteMsg.dynamicCast( sipMsg );
        if ( inviteMsg != 0 )
        {
            SipRequestLine reqLine = inviteMsg->getRequestLine();
            Sptr<BaseUrl> reqUrl = reqLine.getUrl();
            Data tempReqUser = reqUrl->getUserValue();
            tempReqUser.match(thePrefix.getData(lo), &prefix, true, "");
            reqUrl->setUserValue(tempReqUser);
            reqLine.setUrl(reqUrl);
            inviteMsg->setRequestLine(reqLine);
            cpLog(LOG_DEBUG, "The modified RequestLine Url is:%s",
                  inviteMsg->getRequestLine().getUrl()->encode().logData());
        }

    }
}
// End of File
