#ifndef FS_BUILDER_HXX
#define FS_BUILDER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const FsBuilder_hxx_Version =
    "$Id: FsBuilder.hxx,v 1.14 2002/03/19 21:39:40 jason Exp $";


#include <cassert>
#include "Builder.hxx"


namespace Vocal
{

/* CONSTANTS - defined globally because SUN Compiler won't allow string
               constants in classes. */
//Feature Code for Disabling Caller ID per call 
static const char DisableCallerID[] = "*67";

//Feature Code for Enabling Caller ID per call
static const char EnableCallerID[] = "*82"; 
          

/**
  *  This class handles the reception of sip messages for the feature server. 
  *  When a sip message is received, it gets the data associated with the
  *  call as well as the feature associated with the user (this might be a
  *  CPL or a non-CPL feature). If this is a new call the feature is
  *  retrieved from the configuration
  */

class FsBuilder : public Builder
{
    public:
        /**@name Initialization & Deallocation Functions */
        //@{
        /// Constructor
        FsBuilder();
        /// Destructor
        virtual ~FsBuilder();
        //@}

        /// Handles sip and timer events
        //void process(class Sptr < SipProxyEvent > );
         void process( Sptr < SipProxyEvent > );

    private:

        /// Strips out any feature codes (e.g. *67) and sets ftrCode
        bool getFeatureCode( Sptr < SipProxyEvent > sipEvent, 
                             Sptr < CallInfo > callInfo,
                             string &ftrCode );
        /// Get the feature associated with the users involved in the call
        Sptr < Feature > getFeature( Sptr < SipProxyEvent > event, 
                                            const string &ftrCode );
        /// Get a non-CPL feature
        Sptr < Feature > getSpecialFeature( Sptr < SipProxyEvent > event, 
                                                   const string &ftrCode );     
           
        /// Get a CPL feature, from provisioning if needed
        Sptr < Feature > getCPLFeature( Sptr < SipProxyEvent > event );
          
        /// Get the default feature
        Sptr < Feature > getDefaultFeature( Sptr < SipProxyEvent > event );

        /// CLID blocking specific code: removes *67 or *82 from call leg
        string preprocess( Sptr < SipProxyEvent > sipEvent, Data& prefix,
                           SipTo& sipTo, bool& unblock );
        /// CLID blocking specific code: removes a given prefix from a sipMsg
        void removePrefix( const Sptr < SipMsg > sipMsg ,const Data& thePrefix);

        /// Creates a default CPL feature
        void createDefaultFeature();
        ///Used when FS is unable to find a feature for a user.
        Sptr < Feature > myDefaultFeature;
};
 
}

#endif // FS_BUILDER_HXX
