
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLBuildStatus_cxx_Version =
    "$Id: CPLBuildStatus.cxx,v 1.16 2002/03/19 21:39:40 jason Exp $";
#include "CPLBuildStatus.hxx"
#include "cpLog.h"
#include "CallProcessingLanguage.hxx"

using namespace Vocal;

/************SINGLETON***************/
CPLBuildStatus* CPLBuildStatus::myInstance = NULL;
/************************************/


///
CPLBuildStatus*
CPLBuildStatus::Instance()
{
    if ( myInstance == 0 )
        myInstance = new CPLBuildStatus;

    return myInstance;
}  //Instance


///
CPLBuildStatus::CPLBuildStatus()
{
    myLastOp = 0;
    myLastNode = CPLNodeEndofNodes;
} //Constructor


///
CPLBuildStatus::~CPLBuildStatus()
{
    myLastOp = 0;
} //Destructor


///
void
CPLBuildStatus::appendToStateMachine( Sptr < CPLOperator > op,
                                      int currentNode )
{
    if ( myLastOp != 0 )
        myLastOp->setNextOperator( op, myLastNode );
    myLastOp = op;

    //We need to do 1 extra step if previous node was
    //SubAction.  We need to add an Operator to the
    //element on top of the list.
    if ( myLastNode == CPLNodeSubaction )
    {
        mySubActionList.back().setOp( op );
    } /*if*/

    myLastNode = currentNode;
}  //appendToStateMachine


///
bool
CPLBuildStatus::nodeEntry( Sptr < CPLOperator > op, int node )
{
    switch ( node )
    {
        case CPLNodeSubaction:
        {
            resetBuildStatus();

            //Save subAction info so we can reference when
            //we encounter a <sub>.
            string subActionID( op->name() );
            CPLListElement newSubAction;
            newSubAction.setElement( subActionID, 0 );
            mySubActionList.push_back( newSubAction );

            myLastNode = CPLNodeSubaction;
        }
        break;

        case CPLNodeTimezone:
        {
            CPLListElement elem;
            string opID = op->name();
            elem.setElement(opID, op);
            myTimeZoneList.push_back(elem);
        }
        break;

        case CPLNodeStandard:
        case CPLNodeDaylight:
        if (myTimeZoneList.size() == 0)
        {
            cpLog(LOG_DEBUG, "Empty time zone list.");
            return false;
        }
        myTimeZoneList.back().getOp()->setNextOperator(op);
        break;

        case CPLNodeBusy:
        case CPLNodeFailure:
        case CPLNodeNoanswer:
        case CPLNodeRedirection:
        case CPLNodeDefault:
        case CPLNodeNotfound:
        case CPLNodeNotpresent:
        case CPLNodeSuccess:
        case NoOpChild:
        {
            //Check for possible errors.
            if ( myCompoundOperatorList.size() == 0 )
            {
                cpLog( LOG_DEBUG, "Empty compoundOperatorList." );
                return false;
            } /*if*/

            //An output node was found.  Need to find
            //the parent operator.
            myLastOp = myCompoundOperatorList.back().getOp();

            //Save this information, so the next node
            //knows where to put it.
            myLastNode = node;
        } /* case */
        break;

        case CPLNodeBusyExt:
        case CPLNodeFailureExt:
        case CPLNodeNoanswerExt:
        case CPLNodeRedirectionExt:
        case CPLNodeDefaultExt:
        case CPLNodeAddress:
        case CPLNodeOtherwise:
        case CPLNodePriority:
        case CPLNodeString:
        case CPLNodeTime:
        case CompoundOpChild:
        {
            //Check for possible errors.
            if ( myCompoundOperatorList.size() == 0 )
            {
                cpLog( LOG_DEBUG, "Empty compoundOperatorList." );
                return false;
            } /*if*/

            //An output node was found.  Need to add to
            //the parent operator.
            myCompoundOperatorList.back().getOp()->setNextOperator( op );

            myLastOp = op;
        } /* case */
        break;


        case CPLNodeIncoming:
        case CPLNodeOutgoing:
        {
            resetBuildStatus();
            appendToStateMachine( op, node );
        }
        break;

        case CPLNodeProxy:
        case CPLNodeLookup:
        case CPLNodeTimeswitch :
        case CPLNodeStringswitch :
        case CPLNodeAddressswitch :
        case CPLNodePriorityswitch :
        case CompoundOp:
        {
            CPLListElement elem;
            string dontcareString("");
            elem.setElement( dontcareString, op );
            myCompoundOperatorList.push_back( elem );

            appendToStateMachine( op, node );
        }
        break;

        case SimpleOp:
        default:
        {
            appendToStateMachine( op, node );
        } /*default*/
    } /*switch*/

    return true;
}  //nodeEntry


///
bool
CPLBuildStatus::nodeExit( int node )
{
    switch ( node )
    {
        case CPLNodeAddressswitch :
        case CPLNodeLookup:
        case CPLNodePriorityswitch :
        case CPLNodeProxy:
        case CPLNodeStringswitch :
        case CPLNodeTimeswitch :
        {
            //Done with this node, remove it from list.
            myCompoundOperatorList.pop_back();
        } /*case*/
        break;
        case CPLNodeCpl:
        {
            //We reset the member variables to avoid
            //the smart pointers from staying attached
            //to an Operator.
            resetBuildStatus();

            //Clean-up subaction list
            while ( !mySubActionList.empty() )
                mySubActionList.pop_back();
            //Clean-up timezone list too
            while ( !myTimeZoneList.empty() )
                myTimeZoneList.pop_back();
        } /*case CPL*/
        break;
        default:
        ;  //No special processing for the remaining nodes.
    } /* switch */
    return true;
}  //nodeExit


///
void
CPLBuildStatus::resetBuildStatus()
{
    myLastOp = 0;
    myLastNode = CPLNodeEndofNodes;
    if (isBuildClean() == false)
    {
        cpLog (LOG_ERR, "%d elems still on CompoundList!",
               myCompoundOperatorList.size() );
        while ( !myCompoundOperatorList.empty() )
        {
            cpLog (LOG_ERR, "List Residue '%s'",
                   myCompoundOperatorList.back().getOp()->name() );
            myCompoundOperatorList.pop_back();
        } /*while*/
    } /*if*/

}  //resetBuildStatus


///
Sptr < CPLOperator >
CPLBuildStatus::getSubAction( const string& id ) const
{
    if ( mySubActionList.empty() ) return 0;

    //Look up id in subActionList and return the operator
    vector < CPLListElement > ::const_iterator op = mySubActionList.begin();
    vector < CPLListElement > ::const_iterator listEnd = mySubActionList.end();
    Sptr < CPLOperator > subActionOp;

    while (op != listEnd )
    {
        subActionOp = op->findID( id );
        if ( subActionOp != 0) return subActionOp;
        ++op;
    } /*while*/

    cpLog( LOG_DEBUG, "Unable to build subaction '%s'.", id.c_str() );
    return 0;
}  //getSubAction


///
Sptr < CPLOperator >
CPLBuildStatus::getTimeZone( const string& id ) const
{
    if ( myTimeZoneList.empty() ) return 0;

    //Look up id in subActionList and return the operator
    vector < CPLListElement > ::const_iterator op = myTimeZoneList.begin();
    vector < CPLListElement > ::const_iterator listEnd = myTimeZoneList.end();
    Sptr < CPLOperator > timeZoneOp;

    while (op != listEnd )
    {
        timeZoneOp = op->findID( id );
        if ( timeZoneOp != 0)
        {
            cpLog(LOG_DEBUG, "time zone \"%s\" found...", timeZoneOp->name());
            return timeZoneOp;
        }
        ++op;
    } /*while*/

    cpLog( LOG_DEBUG, "Unable to get timezone '%s'.", id.c_str() );
    return 0;
}  //getTimeZone


///
bool
CPLBuildStatus::isBuildClean() const
{
    return (myCompoundOperatorList.size() == 0);
}  //isBuildClean


///
void
CPLBuildStatus::showData() const
{
    cpLog( LOG_DEBUG, "lastNde=%d  sizeof List=%d",
           myLastNode, myCompoundOperatorList.size() );

    if (myLastOp != 0)
        cpLog( LOG_DEBUG, "lastOp=%s", myLastOp->name() );

}  //showData

