
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLFeatureBuilder_cxx_Version =
    "$Id: CPLFeatureBuilder.cxx,v 1.11 2002/03/19 21:39:40 jason Exp $";
#include "CPLFeatureBuilder.hxx"
#include "CPLInterpreter.hxx"
#include "CPLValidator.hxx"
#include "CPLParser.hxx"
#include <parser.h>

using namespace Vocal;


//////////////////SINGLETON IMPLEMENTATION//////////////
CPLFeatureBuilder* CPLFeatureBuilder::MyInstance = NULL;
////////////////////////////////////////////////////////


///
CPLFeatureBuilder*
CPLFeatureBuilder::Instance()
{
    if ( MyInstance == 0 )
    {
        MyInstance = new CPLFeatureBuilder;
    } /*if*/

    return MyInstance;

}  // Instance

///
CPLFeatureBuilder::CPLFeatureBuilder()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "new CPLFeatureBuilder" );
#endif
    if ( createCplDtd() == false) 
    {
        cpLog(LOG_ERR, "Failed to create cpl.dtd on local drive." ); 
    } /*if*/

} //CPLFeatureBuilder


///
CPLFeatureBuilder::~CPLFeatureBuilder()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~CPLFeatureBuilder" );
#endif
} //~CPLFeatureBuilder


///
Sptr < CPLStartStateHolder >
CPLFeatureBuilder::create( const string& cplScript )
{
    xmlNodePtr ptr;

    //Build DOM Tree
    ptr = CPLParser::Instance()->buildTree( (string&)cplScript );
    if ( ptr == NULL) return 0;

    //Build CPL State Machine
    Sptr < CPLStartStateHolder > FtrStartState;
    FtrStartState = CPLInterpreter::Instance()->createFeatureFromCPL(ptr);

    //Clean-up DOM Tree because it will not be used anymore.
    CPLParser::Instance()->deleteTree();

    return FtrStartState;

}  //create


/**********************************************************************
 * IMPORTANT:  The cpl version to be used with this feature server is 
 * dependent on the cpl.dtd defined here.  To use the latest version
 * of cpl.dtd, you must put the new cpl.dtd here.  
   To convert a cpl.dtd to this code:
   1)  Add a \ in front of quotes
   2)  Add to end of each line:  end-quote, redirection, endl;
   3)  Add to beginning of each line:  oFile, redirection, start-quote
   4)  Replace '<< "" <<' with '<<'
 *********************************************************************/
bool 
CPLFeatureBuilder::createCplDtd() const
{
    string fileName;

    //Force a new cpl.dtd to be created to insure we have the latest dtd.
    char buffer[MAX_BUFFER_SIZE];
    if ( getcwd( buffer, MAX_BUFFER_SIZE) == NULL )
    {
        cpLog( LOG_ERR, "Problems reading current directory." );
        return false;
    } /*if*/
    else
    {
        //Create a cpl.dtd in the directory where the fs is running.
        fileName = buffer;           
        fileName += "/cpl.dtd";
    } /*else*/

    ofstream oFile( fileName.c_str(), ios::out );

    oFile << "<?xml version=\"1.0\" encoding=\"US-ASCII\" ?>" << endl;
    oFile << endl;
    oFile << "<!--" << endl;
    oFile << "    Draft DTD for CPL, corresponding to" << endl;
    oFile << "    draft-ietf-iptel-cpl-04." << endl;
    oFile << "-->" << endl;
    oFile << endl;
    oFile << "<!-- Nodes. -->" << endl;
    oFile << "<!-- Switch nodes -->" << endl;
    oFile << "<!ENTITY % Switch 'address-switch|string-switch|time-switch|" << endl;
    oFile << "                   priority-switch' >" << endl;
    oFile << endl;
    oFile << "<!-- Location nodes -->" << endl;
    oFile << "<!ENTITY % Location 'location|lookup|remove-location' >" << endl;
    oFile << endl;
    oFile << "<!-- Signalling action nodes -->" << endl;
    oFile << "<!ENTITY % SignallingAction 'proxy|redirect|vmredirect|reject' >" << endl;
    oFile << endl;
    oFile << "<!-- Other actions -->" << endl;
    oFile << "<!ENTITY % OtherAction 'mail|log' >" << endl;
    oFile << endl;
    oFile << "<!-- Links to subactions -->" << endl;
    oFile << "<!ENTITY % Sub 'sub' >" << endl;
    oFile << endl;
    oFile << "<!-- Nodes are one of the above four categories, or a subaction." << endl;
    oFile << "     This entity (macro) describes the contents of an output." << endl;
    oFile << "     Note that a node can be empty, implying default action. -->" << endl;
    oFile << "<!ENTITY % Node     '(%Location;|%Switch;|%SignallingAction;|" << endl;
    oFile << "                     %OtherAction;|%Sub;)?' >" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Switches: choices a CPL script can make. -->" << endl;
    oFile << endl;
    oFile << "<!-- All switches can have an 'otherwise' output. -->" << endl;
    oFile << "<!ELEMENT otherwise ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!-- All switches can have a 'not-present' output. -->" << endl;
    oFile << "<!ELEMENT not-present ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!-- Address-switch makes choices based on addresses. -->" << endl;
    oFile << "<!ELEMENT address-switch ( (address|not-present)+, otherwise? ) >" << endl;
    oFile << "<!-- <not-present> must appear at most once -->" << endl;
    oFile << "<!ATTLIST address-switch" << endl;
    oFile << "   field         CDATA    #REQUIRED" << endl;
    oFile << "   subfield      CDATA    #IMPLIED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT address ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!ATTLIST address" << endl;
    oFile << "   is            CDATA    #IMPLIED" << endl;
    oFile << "   contains      CDATA    #IMPLIED" << endl;
    oFile << "   subdomain-of  CDATA    #IMPLIED" << endl;
    oFile << "> <!-- Exactly one of these three attributes must appear -->" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- String-switch makes choices based on strings. -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT string-switch ( (string|not-present)+, otherwise? ) >" << endl;
    oFile << "<!-- <not-present> must appear at most once -->" << endl;
    oFile << "<!ATTLIST string-switch" << endl;
    oFile << "   field         CDATA    #REQUIRED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT string ( %Node; ) >" << endl;
    oFile << "<!ATTLIST string" << endl;
    oFile << "   is            CDATA    #IMPLIED" << endl;
    oFile << "   contains      CDATA    #IMPLIED" << endl;
    oFile << ">  <!-- Exactly one of these two attributes must appear -->" << endl;
    oFile << endl;
    oFile << "<!-- Time-switch makes choices based on the current time. -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT time-switch ( (time|not-present)+, otherwise? ) >" << endl;
    oFile << "<!ATTLIST time-switch" << endl;
    oFile << "   tzid          CDATA    #IMPLIED" << endl;
    oFile << "   tzurl         CDATA    #IMPLIED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT time ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!-- Exactly one of the two attributes \"dtend\" and \"duration\"" << endl;
    oFile << "     must occur. -->" << endl;
    oFile << "<!-- The value of \"freq\" is (daily|weekly|monthly|yearly).  It is" << endl;
    oFile << "        case-insensitive, so it is not given as a DTD switch. -->" << endl;
    oFile << "<!-- None of the attributes following freq are meaningful unless freq" << endl;
    oFile << "      appears. -->" << endl;
    oFile << "<!-- The value of \"wkst\" is (MO|TU|WE|TH|FR|SA|SU).  It is" << endl;
    oFile << "        case-insensitive, so it is not given as a DTD switch. -->" << endl;
    oFile << "<!ATTLIST time" << endl;
    oFile << "   dtstart       CDATA  #REQUIRED" << endl;
    oFile << "   dtend         CDATA  #IMPLIED" << endl;
    oFile << "   duration      CDATA  #IMPLIED" << endl;
    oFile << "   freq          CDATA  #IMPLIED" << endl;
    oFile << "   until         CDATA  #IMPLIED" << endl;
    oFile << "   interval      CDATA  \"1\"" << endl;
    oFile << "   byday         CDATA  #IMPLIED" << endl;
    oFile << "   bymonthday    CDATA  #IMPLIED" << endl;
    oFile << "   byyearday     CDATA  #IMPLIED" << endl;
    oFile << "   byweekno      CDATA  #IMPLIED" << endl;
    oFile << "   bymonth       CDATA  #IMPLIED" << endl;
    oFile << "   wkst          CDATA  \"MO\"" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Priority-switch makes choices based on message priority. -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT priority-switch ( (priority|not-present)+, otherwise? ) >" << endl;
    oFile << "<!-- <not-present> must appear at most once -->" << endl;
    oFile << endl;
    oFile << "<!ENTITY % PriorityVal '(emergency|urgent|normal|non-urgent)' >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT priority ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!-- Exactly one of these three attributes must appear -->" << endl;
    oFile << "<!ATTLIST priority" << endl;
    oFile << "   less          %PriorityVal;  #IMPLIED" << endl;
    oFile << "   greater       %PriorityVal;  #IMPLIED" << endl;
    oFile << "   equal         CDATA          #IMPLIED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Locations: ways to specify the location a subsequent action" << endl;
    oFile << "     (proxy, redirect) will attempt to contact. -->" << endl;
    oFile << endl;
    oFile << "<!ENTITY % Clear  'clear (yes|no) \"no\"' >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT location ( %Node; ) >" << endl;
    oFile << "<!ATTLIST location" << endl;
    oFile << "   url           CDATA    #REQUIRED" << endl;
    oFile << "   priority      CDATA    #IMPLIED" << endl;
    oFile << "   %Clear;" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT lookup ( success,notfound?,failure? ) >" << endl;
    oFile << "<!ATTLIST lookup" << endl;
    oFile << "  source         CDATA     #REQUIRED" << endl;
    oFile << "  timeout        CDATA     \"30\"" << endl;
    oFile << "  use            CDATA     #IMPLIED" << endl;
    oFile << "  ignore         CDATA     #IMPLIED" << endl;
    oFile << "  %Clear;" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT success  ( %Node; ) >" << endl;
    oFile << "<!ELEMENT notfound ( %Node; ) >" << endl;
    oFile << "<!ELEMENT failure ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT remove-location ( %Node; ) >" << endl;
    oFile << "<!ATTLIST remove-location" << endl;
    oFile << "   param         CDATA    #IMPLIED" << endl;
    oFile << "   value         CDATA    #IMPLIED" << endl;
    oFile << "   location      CDATA    #IMPLIED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Signalling Actions: call-signalling actions the script can" << endl;
    oFile << "     take. -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT proxy ( busy?,noanswer?,redirection?,failure?,default? ) >" << endl;
    oFile << endl;
    oFile << "<!-- The default value of timeout is \"20\" if the <noanswer> output" << endl;
    oFile << "     exists. -->" << endl;
    oFile << "<!ATTLIST proxy" << endl;
    oFile << "   timeout       CDATA    #IMPLIED" << endl;
    oFile << "   recurse       (yes|no) \"yes\"" << endl;
    oFile << "   ordering      CDATA    \"parallel\"" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT busy ( %Node; ) >" << endl;
    oFile << "<!ELEMENT noanswer ( %Node; ) >" << endl;
    oFile << "<!ELEMENT redirection ( %Node; ) >" << endl;
    oFile << "<!-- \"failure\" repeats from lookup, above. -->" << endl;
    oFile << "<!ELEMENT default ( %Node; ) >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT vmredirect EMPTY >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT redirect EMPTY >" << endl;
    oFile << "<!ATTLIST redirect" << endl;
    oFile << "   permanent     (yes|no) \"no\"" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Statuses we can return -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT reject EMPTY >" << endl;
    oFile << "<!-- The value of \"status\" is (busy|notfound|reject|error), or a SIP" << endl;
    oFile << "     4xx-6xx status. -->" << endl;
    oFile << "<!ATTLIST reject" << endl;
    oFile << "   status        CDATA    #REQUIRED" << endl;
    oFile << "   reason        CDATA    #IMPLIED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!-- Non-signalling actions: actions that don't affect the call -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT mail ( %Node; ) >" << endl;
    oFile << "<!ATTLIST mail" << endl;
    oFile << "   url           CDATA    #REQUIRED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << "<!ELEMENT log ( %Node; ) >" << endl;
    oFile << "<!ATTLIST log" << endl;
    oFile << "   name          CDATA    #IMPLIED" << endl;
    oFile << "   comment       CDATA    #IMPLIED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Calls to subactions. -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT sub EMPTY >" << endl;
    oFile << "<!ATTLIST sub" << endl;
    oFile << "   ref           IDREF    #REQUIRED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Ancillary data -->" << endl;
    oFile << endl;
    oFile << "<!ENTITY % Ancillary 'ancillary?' >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT ancillary EMPTY >" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Subactions -->" << endl;
    oFile << endl;
    oFile << "<!ENTITY % Subactions 'subaction*' >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT subaction ( %Node; )>" << endl;
    oFile << "<!ATTLIST subaction" << endl;
    oFile << "   id            ID       #REQUIRED" << endl;
    oFile << ">" << endl;
    oFile << endl;
    oFile << endl;
    oFile << "<!-- Top-level actions -->" << endl;
    oFile << endl;
    oFile << "<!ENTITY % TopLevelActions 'outgoing?,incoming?' >" << endl;
    oFile << endl;
    oFile << "<!ELEMENT outgoing ( %Node; )>" << endl;
    oFile << endl;
    oFile << "<!ELEMENT incoming ( %Node; )>" << endl;
    oFile << endl;
    oFile << "<!-- The top-level element of the script. -->" << endl;
    oFile << endl;
    oFile << "<!ELEMENT cpl  ( %Ancillary;,%Subactions;,%TopLevelActions; ) >" << endl;

    return true; 
} //createCplDtd


//End of File

