#ifndef CPLListElement_HXX
#define CPLListElement_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLListElementVersion =
    "$Id: CPLListElement.hxx,v 1.9 2002/03/19 21:39:40 jason Exp $";



#include "CPLOperator.hxx"
#include <string>

namespace Vocal
{


/**
  *  To be used in lists or vectors to hold a pointer to an operator. 
  */
class CPLListElement
{
    public:
        /// Default constructor
        CPLListElement();

        /**
         * Constructs the element to hold an operator and its type
         *@param id the ID of the element
         *@param op reference to the operator pointed by this element
         */
        CPLListElement( string& id, Sptr < CPLOperator > op );

        /// copy constructor
        CPLListElement( const CPLListElement& rhs );

        /// destructor
        ~CPLListElement();

        /// overloaded assignment operator
        CPLListElement& operator=( const CPLListElement& rhs );

        /// displays the name of operator this instance stores
        void showData() const;

        /**
         * initializes the element to hold an operator and its type
         *@param id the ID of the element
         *@param op reference to the operator pointed by this element
         */
        void setElement( string& id, Sptr < CPLOperator > op );

        /// initializes the ID of the element
        void setID( const string& id );

        /// initializes the element to point to given operator
        void setOp( Sptr < CPLOperator > op );

        /**
         * method to find the operator with the given ID
         *@param id the ID of operator being searched
         *@return reference to the operator if ID matches, else 0
         */
        Sptr < CPLOperator > findID( const string& id ) const;

        /// returns the reference of the operator pointed by this element
        Sptr < CPLOperator > getOp() const;

        /// compares the given ID with self's ID for equality
        bool operator==( string& rhs );

    private:
        /// stores the ID of the operator pointed by this element
        string myID;

        /// points to the operator
        Sptr < CPLOperator > myOp;
};
 
} // namespace Vocal

#endif


//End of File

