
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOffset_cxx_Version = "$Id$";

#include "CPLOffset.hxx"
#include "CPLOperator.hxx"

using namespace Vocal;

CPLOffset::CPLOffset()
{
    myUtcOffset = 0;
    myTimeOfDay = 0;
    myWeekDay = 0;
    myDate = 0;
    myMonth = 0;
    myYear = 0;
}


CPLOffset::~CPLOffset()
{
    myTimeOfDay = 0;
    myWeekDay = 0;
    myDate = 0;
    myMonth = 0;
    myYear = 0;
}


const Sptr < State > CPLOffset::process(const Sptr < SipProxyEvent > anEvent)
{
    return 0;
}


bool CPLOffset::setNextOperator(Sptr < Operator > anOperator, 
                                int nodeId /*def value*/)
{
    return true;
}


bool CPLOffset::setAttributes(const char* type, const char* value)
{
    istrstream str(value);

    if ( strcmp( type, "offset" ) == 0 )
        str >> myUtcOffset;
    else if ( strcmp( type, "abbr" ) == 0 )
        str >> myAbbr;
    else if ( strcmp( type, "year") == 0 )
        str >> *(myYear = new Pair < TimeField > ());
    else if ( strcmp( type, "month") == 0 )
        str >> *(myMonth = new Pair < TimeField > ());
    else if ( strcmp( type, "date") == 0 )
        str >> *(myDate = new Pair < TimeField > ());
    else if ( strcmp( type, "day") == 0 )
        str >> *(myWeekDay = new Pair < TimeField > ());
    else if ( strcmp( type, "timeofday") == 0 )
        str >> *(myTimeOfDay = new Pair < TimeField > ());
    else  //Unknown attribute name
        assert ( 0 );

    return true;
}


///
ostrstream & operator << (ostream & str, Pair < TimeField > & aPair)
{
    return operator << (dynamic_cast < ostrstream& > (str), aPair);
}


///
void CPLOffset::logData(void) const
{
    char buff[256];
    ostrstream oStr(buff, 256);

    if (myTimeOfDay != 0)
    {
        oStr << (*myTimeOfDay) << " Hrs:Min ";
    }
    else oStr << "0000 Hrs ";

    if (myWeekDay != 0)
    {
        oStr << "on day(s)[" << (*myWeekDay) << "] of week ";
    }
    else oStr << "on any day of week ";

    if (myDate != 0)
    {
        oStr << "on date(s)[" << (*myDate) << "] ";
    }
    else oStr << "on any dates ";

    if (myMonth != 0)
    {
        oStr << "of month(s)[" << (*myMonth) << "] ";
    }
    else oStr << "of any month ";

    if (myYear != 0)
    {
        operator<< (oStr, *myYear);
    }
    else oStr << "of any year";

    oStr << '\0';

    cpLog(LOG_DEBUG, "UTC offset [%05.4ld] @ %s", myUtcOffset, buff);
}


MyTime CPLOffset::getTime() const
{
    TimeField t;

    time(&t);
    t += (myUtcOffset % 100 + (myUtcOffset / 100) * 60) * 60;
    return (*gmtime(&t));
}


bool CPLOffset :: test()
{
    MyTime t;
    if (myYear != 0 && (myYear->lo > t.getYear() || myYear->hi < t.getYear()))
        return false;

    /**
     * current CPL specs (draft-ietf-iptel-cpl-01) are not satisfactory
     * on the offset effectiveness. hence, if its not a particular year,
     * then returning true...
     *
     * in this implementation, Daylight and Standard offsets are overloading
     * this method, checking the daylight flag of struct tm.
     *
     * this may be changed later, making comparisions in time ranges
     * that are "split" (eg. "Jan<---PST--->Apr<---PDT--->Oct<---PST--->Dec")
     * but the present specs do not define sufficient information to
     * split these ranges...
     */

    return true;
}


