
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpDfltAction_cxx_Version =
    "$Id: CPLOpDfltAction.cxx,v 1.11 2002/03/19 21:39:40 jason Exp $";
#include "CPLOpDfltAction.hxx"

using namespace Vocal;

///Constructor
CPLOpDfltAction::CPLOpDfltAction()
{
    //Unnecessary to initialize member variables since smart
    //pointers always initialize to zero. 
       
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "New [%x]", this );
#endif
}  //c'tor


///Destructor
CPLOpDfltAction::~CPLOpDfltAction()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~[%x]", this );
#endif
}  //destructor


///
const Sptr < State >
CPLOpDfltAction::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "process()" );

    //Look at the locations in the location set.  If the set is empty,
    //reject the call.  Otherwise, attempt to redirect the call.
    //Note:  Redirecting to oneself may cause a loop.
    if ( (getStateMachineData(event)->getLocations()->empty()) == true )
        return myRejectCallOp->process( event );
    else
        return myRedirectCallOp->process( event );

} //process


///
bool
CPLOpDfltAction::setAttributes( const char*, const char* )
{
    //Not a CPL tag, so there should not be any attributes
    return true;

}  //setAttributes


///
bool
CPLOpDfltAction::setNextOperator( Sptr < Operator > op, int prevNode )
{
    //This class is different in that it operators cannot be added
    //using setNextOperator.  To add operators to this class,
    //call setRejectOperator() to add operators to the reject output
    //or call setRedirectOperator() to add operators to the
    //redirect output.
    return true;

}  //setNextOperator


///
bool
CPLOpDfltAction::setRejectOperator( Sptr < Operator > op )
{
    if ( op == 0 )
    {
        cpLog( LOG_ERR, "op is NULL." );
        assert( op != 0 );
    }
    myRejectCallOp = op;
    return true;
}  //setRejectOperator


///
bool
CPLOpDfltAction::setRedirectOperator( Sptr < Operator > op )
{
    if ( op == 0 )
    {
        cpLog( LOG_ERR, "op is NULL." );
        assert( op != 0 );
    }
    myRedirectCallOp = op;
    return true;
} //setRedirectOperator


///
const char* const
CPLOpDfltAction::name() const
{
    return "CPLOpDfltAction";
} //name


///Prints debug information about class
void
CPLOpDfltAction::logData() const
{
    cpLog( LOG_DEBUG, "%s: Redirect[%x] Reject[%x]",
           name(),
           myRedirectCallOp.getPtr(),
           myRejectCallOp.getPtr() );

} //logData

///
void
CPLOpDfltAction::doCpLog() const
{
    logData();

    if ( myRedirectCallOp != 0 )
    {
        cpLog( LOG_DEBUG, "|||DEFAULT ACTION.1|||" );
        myRedirectCallOp->doCpLog();
    } /*if*/

    if ( myRejectCallOp != 0 )
    {
        cpLog( LOG_DEBUG, "|||DEFAULT ACTION.2|||" );
        myRejectCallOp->doCpLog();
    } /*if*/

} //doCpLog

//End of File

