#ifndef CPLOPDFLTACTION_HXX
#define CPLOPDFLTACTION_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpDfltActionVersion =
    "$Id: CPLOpDfltAction.hxx,v 1.12 2002/03/19 21:39:40 jason Exp $";



#include "CPLOperator.hxx"

namespace Vocal
{


/**
 * Executed when a CPL output is an empty node instead of another CPL tag.
 *   
 **/
class CPLOpDfltAction: public CPLOperator
{
    public:
        /**@name Initialization & Deallocation Functions */
        //@{
        ///Constructor
        CPLOpDfltAction();

        ///Destructor
        ~CPLOpDfltAction();
        //@}

        /// state m/c execution method
        const Sptr < State > process( const Sptr < SipProxyEvent > event );

        /**@name CPLOperator implementation methods*/
        //@{
        /** Save CPL tag attributes
          *@param attrName Attribute's name
          *@param attrValue Value of the attribute 
          *@return True if successful setting attributes */
        bool setAttributes( const char*, const char* );

        /**Return the class name.
           @return Pointer to character string */
        const char* const name() const;

        ///Display information regarding the class' member variables.
        void logData() const;

       /**Displays class data information including calling doCpLog of any of
         * its operators and states that it contains.  */
        void doCpLog() const;

        /**See setRejectOperator or setRedirectOperator. 
           @param op Pointer to the operator to add
           @param prevNode ID of the last CPL tag.
           @see setRejectOperator
           @see setRedirectOperator
           @return true if function did not have any errors setting next
           operator. 
         **/
        bool setNextOperator( Sptr < Operator > op, int prevNode );
        //@}

        /// explicitly set the reject operator as next operator
        bool setRejectOperator( Sptr < Operator > op );

        /// explicitly set the redirect operator as the next operator
        bool setRedirectOperator( Sptr < Operator > op );

    private:
        /// reference to reject operator
        Sptr < Operator > myRejectCallOp;
           
        /// reference to the redirect operator
        Sptr < Operator > myRedirectCallOp;
};

 
} // namespace Vocal

#endif



//End of file

