#ifndef CPLOpFork_Hxx
#define CPLOpFork_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpFork_hxx_Version =
    "$Id: CPLOpFork.hxx,v 1.11 2002/03/19 21:39:41 jason Exp $";





#include "CPLOperator.hxx"
#include "CPLUrlData.hxx"
#include "TimerEvent.hxx"
#include "SipEvent.hxx"
#include "SipMsg.hxx"
#include "StatusMsg.hxx"
#include "InviteMsg.hxx"
namespace Vocal
{


/**
 * Implements the proxying for sequential & parallel <proxy>. 
 */

class CPLOpFork : public CPLOperator
{
        /**@name Initialization & Deallocation Functions */
        //@{
    public:
        ///Constructor
        CPLOpFork();
        ///Destructor
        ~CPLOpFork();
        //@}

        /**@name State Machine Building Functions */
        //@{
        ///Append an operator to this class
        bool setNextOperator( Sptr < Operator > op, int prevNode );
        ///Initialize the Attributes member variables
        bool setAttributes( const char* attrName, const char* attrValue);
        //@}

        /**@name State Machine Execution Functions */
        //@{
        ///
        const Sptr < State > process( Sptr < SipProxyEvent > event);
        //@}

        /**@name Debugging Functions */
        //@{
        ///cpLogs values of member variables
        void logData() const;
        ///Returns the name of the class
        const char* const name() const;
        ///makes a detailed log...
        void doCpLog() const;
        //@}

    private:
        /**@name Output functions */
        //@{
        /**
         * method to handle timer expiration
         *@param tEvent reference to the timer event
         *@return next state after executing NoAnswere o/p (if present) else 0
         */
        const Sptr < State > noAnswer(Sptr < TimerEvent > tEvent);

        /**
         * method to handle BUSY best response
         *@param sEvent reference to the Sip Event causing this action
         *@return next state after executing Busy o/p (if present) else 0
         */
        const Sptr < State > busy(Sptr < SipEvent > sEvent);

        /**
         * method to handle FAILURE best response
         *@param sEvent reference to the Sip Event causing this action
         *@return next state after executing Failure o/p (if present) else 0
         */
        const Sptr < State > failure(Sptr < SipEvent > sEvent);

        /**
         * method to handle SUCCESS best response
         *@param sEvent reference to the Sip Event causing this action
         *@return next state after executing Success o/p (if present) else 0
         */
        const Sptr < State > success(Sptr < SipEvent > sEvent);

        /**
         * method to handle REDIRECT best response
         *@param sEvent reference to the Sip Event causing this action
         *@return next state after executing Redirect o/p (if present) else 0
         */
        const Sptr < State > redirect(Sptr < SipEvent > sEvent);

        /**
         * method to abort the proxy action
         *@param sEvent reference to the Sip Event causing this action
         *@return state returned by end action
         */
        const Sptr < State > abort(Sptr < SipEvent > sEvent);

        /**
         * method to terminate the proxy action
         *@param event reference to the Sip Event causing this action
         *@return 0 - causing current state to execute CPLOpEnd
         */
        const Sptr < State > end(Sptr < SipProxyEvent > event);

        /**
         * method to wait until an o/p decision is taken
         *@param event reference to the current Event containing call info
         *@return reference to the current state from call info
         */
        const Sptr < State > wait(Sptr < SipProxyEvent > event);
        //@}

        /**@name processing functions */
        //@{

        /**
         * method to do a recursion on 3xx responses
         *@param data reference to the location set
         *@param msg reference to the 3xx response
         */
        void doRecursion(Sptr < CPLUrlData > data, Sptr < StatusMsg > msg);

        /**
         * method to get the original incoming INVITE request corresponding to
         * the given response - used to identify relevent Sip messages
         *@param sipEvent reference to the Sip event containing Call leg msgs
         *@param msg reference to the response to match
         *@return Invite request corresponding to msg
         */
        Sptr < InviteMsg > getOriginal(Sptr <SipEvent> sipEvent,
                                       Sptr <StatusMsg> ) const;

        /**
         * equality test for INVITEs (RFC 2543: sec 12.3.4)
         *@param msg1 reference to first Invite message
         *@param msg2 reference to second Invite message
         *@return bool result of the test
         */
        bool matchInvites(Sptr < InviteMsg > msg1, Sptr < InviteMsg > msg2);

        /**
         * match for a response and an INVITE (RFC 2543: sec 12.3.6)
         *@param res reference to the response to match
         *@param req reference to the request to match
         *@return bool result of the test
         */
        bool match(Sptr < StatusMsg > res, Sptr < InviteMsg > req);

        /**
         * method to do a parallel spawn of Invites
         *@param sEvent reference to the current Sip event
         *@param urls reference to the location set
         */
        void doParallel(Sptr < SipEvent > sEvent, Sptr < CPLUrlData > urls);

        /**
         * method to do a sequential spawn of Invites
         *@param sEvent reference to the current Sip event
         *@param urls reference to the location set
         */
        void doSequential(Sptr <SipEvent> sipEvent,
                          Sptr <CPLUrlData> locationSet );

        /**
         * method to stop the timer, if set
         *@param event reference to the current
         */
        void stopTimer(Sptr < SipProxyEvent > event);

        /**
         * method to fork new requests to the given location set
         *@param sEvent reference to the current Sip Event
         *@param urls reference to the location set
         */
        void fork(Sptr < SipEvent > sEvent, CPLUrlData& urls);

        /**
         * method to cancels all the outstanding requests
         *@param event reference to the current event
         */
        void cancel(Sptr < SipProxyEvent > event);

        /**
         * method to acknowledge responses and return status code
         *@param sEvent sip event
         *@param msg sip message
         *@return status code if message is a response to a proxied request, else 0
         */
        int ack(Sptr < SipEvent > sEvent, Sptr < StatusMsg > msg);

        /**
         * method to proxy the intermediate messages not handled by forking proxy
         *@param sEvent reference to the current Sip event
         *@return next state or 0
         */
        const Sptr < State > proxy(Sptr < SipEvent > sEvent);

        // cleans up the callinfo state
        void cleanUp(Sptr < SipProxyEvent > anEvent );

        // checks the VIA before accepting a response
        bool viaCheck(Sptr < StatusMsg > response);

        /** Check incoming requests for a loop.
         * @return true if there's a loop.  Otherwise
         * false is returned.
         **/
        bool isLoop(Sptr < SipCommand > aCommand );

        // processes the Sip messages
        const Sptr < State > processMsg(Sptr <SipEvent> aSipEvent);

        // processes incoming responses calculates the best o/p action
        const Sptr < State > processResponse(Sptr < SipEvent > sipEvent,
                                             Sptr < StatusMsg > response );

        // takes care of the incoming Sip requests
        const Sptr < State > processRequest(Sptr <SipEvent> sipEvent, 
                                            Sptr <SipCommand> command);
        //@}

        /**@name properties*/
        //@{

        /** stores the ordering option from CPL script
          * parallel=0, sequential=1, firstOnly=2
         **/
        enum {parallel = 0, sequential, firstOnly} ordering;

        ///Constant
        static const int defaultTimeout = 20; 

        /// stores the time out limit from CPL script
        int myTimeOutVal;
        /// stores the recursion option from CPL script
        bool myRecurse;
        //@}

        /**@name data structure */
        //@{
        // iterator for outRequests
        typedef vector < Sptr < InviteMsg > > ::iterator OutIterator;

        // reference to the No answer operator in state machine
        Sptr < Operator > myNoAnswerOp;

        // reference to the Busy operator in state machine
        Sptr < Operator > myBusyOp;

        // reference to the Failure operator in state machine
        Sptr < Operator > myFailureOp;

        // reference to the CPLOpProxy to take care of intermediate messages
        Sptr < Operator > myProxyOp;
        //@}
};
 
} // namespace Vocal

#endif

