
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpFwdCall_cxx_Version =
    "$Id: CPLOpFwdCall.cxx,v 1.41 2002/04/05 00:21:47 bko Exp $";
#include "CallInfo.hxx"
#include "CPLOpFwdCall.hxx"
#include "State.hxx"
#include "TimerEvent.hxx"
#include "SipContact.hxx"
#include "FsUsers.hxx"
#include "CPLUrlData.hxx"
#include "SystemInfo.hxx"
#include "FsConfigFile.hxx"

using namespace Vocal;


///
CPLOpFwdCall::CPLOpFwdCall( ):
    myTimeout(defaultTimeOut),     
    myRecurse(true),
    myOrdering(contactLocationsParallel)
{
    //Have 1 copy of the current executable's host and port.
    string hostString( theSystem.gethostAddress() );
    myFSAddress.setHostName( hostString );
    myFSAddress.setPort( sipPort );

#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "CPLOpFwdCall[%x]", this );
#endif

}    //Constructor


///
CPLOpFwdCall::~CPLOpFwdCall()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~CPLOpFwdCall[%x]", this );
#endif
}    // CPLOpFwdCall::~CPLOpFwdCall


///
const char* const
CPLOpFwdCall::name() const
{
    return "CPLOpFwdCall";
}    // CPLOpFwdCall::name


///
bool
CPLOpFwdCall::setAttributes( const char* attrName, const char* attrValue)
{
    if (strcmp( attrName, "timeout" ) == 0)
    {
        myTimeout = atoi( attrValue );
    } /*if timeout*/
    else if (strcmp( attrName, "recurse" ) == 0)
    {
        if ( strcmp ( attrValue, "no" ) == 0 )
            myRecurse = false;
        else
            myRecurse = true;
    } /*if recurse*/
    else if (strcmp( attrName, "ordering" ) == 0)
    {
        if ( strcmp( attrValue, "first-only" ) == 0)
            myOrdering = contactFirstLocationOnly;
        else if ( strcmp( attrValue, "parallel" ) == 0)
            myOrdering = contactLocationsParallel;
        else
            myOrdering = contactLocationsSequentially;
    } /*if ordering*/

    return true;

}  //setAttributes


///
bool
CPLOpFwdCall::setNextOperator( Sptr < Operator > op, int prevNode)
{
    //Besides the error handling operator, CPLOpFwdCall should not have
    //any operators to process.  It points to the
    //WaitForAnswer state.  Operators must be added to the state.
    return ( myNextState->setNextOperator( op , prevNode ) );
}  //setNextOperator


///
bool
CPLOpFwdCall::setErrorHandlingOp( Sptr < Operator > op )
{
    //Operator pushed to the back of the list.
    addOperator( op );
    return true;
}  //setErrorHandlingOp


///
void
CPLOpFwdCall::resetTimeout()
{
    myTimeout = 0;
}  //resetTimeout


///
void
CPLOpFwdCall::logData() const
{
    string order;
    switch (myOrdering)
    {
        case contactFirstLocationOnly:
        {
            order = "first-only";
        }
        break;
        case contactLocationsSequentially:
        {
            order = "sequential";
        }
        break;
        case contactLocationsParallel:
        {
            order = "parallel";
        }
        break;
        default:
        {
            cpLog( LOG_ERR, "Undefined attribute." );  
            assert(0);  //Attribute undefined
        }
    } /*switch*/


    string recurseStr;
    if (myRecurse == true)
        recurseStr = "yes";
    else
        recurseStr = "no";

    cpLog(LOG_DEBUG, "%s-TIMEOUT(%d s) RECURSE(%s) ORDERING(%s)",
          name(), myTimeout, recurseStr.c_str(), order.c_str() );
} //logData


///
const Sptr < State >
CPLOpFwdCall::process( Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "process()" );

    if ( proxyToNextLocation( event ) == true )
    {
        if ( myTimeout > 0 )
            setTimer( event, myTimeout );

        return myNextState;
    } /*if*/
    else
    {
        //Do the Error Handling Operator
        return CPLOperator::process( event );
    } /*else*/
}    // CPLOpFwdCall::process


///
bool
CPLOpFwdCall::proxyToNextLocation( Sptr < SipProxyEvent > event )
{
    Sptr <SipUrl> sipUrl = new SipUrl;  

    //Get original INVITE and make a copy of it.
    Sptr < InviteMsg > inviteMessage;
    inviteMessage = getInvite( event );

    Sptr < CPLUrlData > locationSet = getStateMachineData(event)->getLocations();

    unsigned int totalLocations;
    totalLocations = locationSet->size();

    if (totalLocations > 0 )
    {
        bool results;
        results = locationSet->top( *sipUrl );
        assert( results != false );
    } /*if*/
    else
    {
        //Send a "404 NOT FOUND".
        cpLog( LOG_DEBUG, "No locations defined." );
        Sptr < StatusMsg > statusMsg = new StatusMsg( *inviteMessage, 404 );
        sendReply( event, *statusMsg );
        return false;
    } //else

    //Create BaseUrl pointer 
    Sptr<BaseUrl> baseUrl = sipUrl; 

    SipRequestLine requestLine( SIP_INVITE, baseUrl );
    inviteMessage->setRequestLine( requestLine );

    // Add Via header
    addVia( event, inviteMessage );

    // Add Record-Route header
    addRecordRoute( inviteMessage );

    // Add Call Control Diversion header ONLY if sending to a
    // different USER. 
    if ( inviteMessage->getRequestLine().getUrl()->getUserValue() !=
         inviteMessage->getTo().getUser() )
    {
        addCallControlDiversion( event, inviteMessage, baseUrl );
    }  /* if */

    cpLog( LOG_DEBUG, "Send to: '%s'",
           inviteMessage->getRequestLine().encode().logData() ); 

    // Change proxyStatus in CallInfo
    getStateMachineData(event)->setProxyStatus( FsStateMachnData::Redirected );

    // Forward message to destination
    sendAsync( event, *inviteMessage);

    return true;
}    // CPLOpFwdCall::proxyToNextLocation


///
void CPLOpFwdCall::addRecordRoute( Sptr < InviteMsg > inviteMsg )
{
    //Add record route ONLY if self is not already in record
    //route list.
    InviteMsg inviteCopy( *inviteMsg );

    int numRecordRoute;
    numRecordRoute = inviteCopy.getNumRecordRoute();

    inviteMsg->flushrecordrouteList();

    SipRecordRoute recordRte;

    Sptr <SipUrl> sipUrl;

    for (int idx = (numRecordRoute - 1); idx >= 0; idx-- )
    {
        recordRte = inviteCopy.getRecordRoute(idx);

        //Get Host & Port from Record Route and save to Network Address
        int rrPortNum;

        sipUrl.dynamicCast( recordRte.getUrl() );

        rrPortNum = sipUrl->getPort().convertInt();
        LocalScopeAllocator lo;
        string rrHost( sipUrl->getHost().getData(lo) );
        NetworkAddress rrAddr( rrHost, rrPortNum );

        if (rrAddr == myFSAddress )
        {
#if 0 //Change to 1 if needed for debugging
            cpLog( LOG_DEBUG, "Record Route Match: %s:%d",
                   rrAddr.getIpName().c_str(), rrAddr.getPort() );
#endif
            break;
        }
        else
        {
#if 0 //Change to 1 if needed for debugging
            cpLog( LOG_DEBUG, "No Record Route Match:  %s:%d vs %s:%d",
                   rrAddr.getIpName().c_str(), rrAddr.getPort(),
                   myFSAddress.getIpName().c_str(), myFSAddress.getPort() );
#endif
            inviteMsg->setRecordRoute( recordRte , 0 );
        }
    } /*for*/

    //Go ahead and add self to record route list.
    SipRecordRoute route;
    Sptr<BaseUrl> baseUrl = inviteMsg->getRequestLine().getUrl();
    sipUrl.dynamicCast( baseUrl );
    OpAddRecordRoute( route, *sipUrl, sipPort );

    inviteMsg->setRecordRoute( route, 0 );

}  //addRecordRoute


///
void
CPLOpFwdCall::addVia( Sptr < SipProxyEvent > event, Sptr < InviteMsg > inviteMsg )
{
    // Get last VIA's address
    LocalScopeAllocator lo;
    string viaHost( inviteMsg->getVia(0).getHost().getData(lo) );
    int viaPort = inviteMsg->getVia(0).getPort();
    NetworkAddress viaAddr( viaHost, viaPort );

    //Remove last VIA only if it belongs to FS's host.  A new VIA
    //with a new branch number will replace this old VIA. 
    if ( viaAddr == myFSAddress )
        inviteMsg->removeVia( 0 );
    else
    {
        cpLog( LOG_DEBUG, "Not removing last VIA at %s:%d.",
               viaAddr.getIpName().c_str(),
               viaAddr.getPort() );
    } /*else*/


    //Build the Via and append to the INVITE message.
    SipVia via;
    OpAddVia( via, sipPort );
    via.setBranch(inviteMsg->computeBranch2() + 
		  getStateMachineData(event)->getNextBranch());

    inviteMsg->setVia( via, 0 );

} //addVia


//End of file

