#ifndef CPLOpFwdCall_Hxx
#define CPLOpFwdCall_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpFwdCallVersion =
    "$Id: CPLOpFwdCall.hxx,v 1.17 2002/03/19 21:39:41 jason Exp $";



#include "CPLOperator.hxx"
#include <InviteMsg.hxx>
#include <SipEvent.hxx>

namespace Vocal
{


/**
 *  Contacts the destination and set a timer to wait for response
 */
class CPLOpFwdCall : public CPLOperator
{
    public:
        /**@name Initialization & Deallocation Functions */
        //@{
        ///
        CPLOpFwdCall();
        ///
        ~CPLOpFwdCall();
        //@}

        /**@name State Machine Building Functions */
        //@{
        ///Append an operator to this class
        bool setNextOperator( Sptr < Operator > op, int prevNode );
        ///Set an operator to handle errors if Proxy cannot proceed
        bool setErrorHandlingOp( Sptr < Operator > op );
        ///Initialize the Attributes member variables
        bool setAttributes( const char* attrName, const char* attrValue);
        ///Reset the timer, so timing won't occur
        void resetTimeout();
        //@}

        /**@name State Machine Execution Functions */
        //@{
        ///
        const Sptr < State > process( Sptr < SipProxyEvent > event);
        //@}

        /**@name Debugging Functions */
        //@{
        ///cpLogs values of member variables
        void logData() const;
        ///Returns the name of the class
        const char* const name() const;
        //@}

        ///
        typedef enum
        {
            contactFirstLocationOnly = 0,
            contactLocationsSequentially,
            contactLocationsParallel
        } contactLocationMethod;

        /**@name Constants */
        //@{
        ///The default timeout (in seconds)
        static const int defaultTimeOut = 20; 
        //@} 
          
    private:
        ///The current Feature Server's address/port.
        NetworkAddress myFSAddress;

        /**@name CPL Attributes */
        //@{
        ///
        unsigned int myTimeout;
        ///
        bool myRecurse;
        ///
        contactLocationMethod myOrdering;
        //@}

        /**@name State Machine Execution Functions */
        //@{
        ///
        bool proxyToNextLocation( Sptr < SipProxyEvent > event );
        ///
        void addRecordRoute( Sptr < InviteMsg > inviteMsg );
        ///
        void addVia( Sptr < SipProxyEvent > event, Sptr < InviteMsg > inviteMsg );
        //@}

};
 
} // namespace Vocal

#endif

