
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpLookup_cxx_Version =
    "$Id$";
#include "CPLOpLookup.hxx"
#include "FsConfigFile.hxx"
#include "CallInfo.hxx"
#include "TimerEvent.hxx"
#include "CallContainer.hxx"
#include "RoundRobinServers.hxx" 
#include "SystemInfo.hxx"
#include "CPLOpLocation.hxx"
using namespace Vocal;

///Constructor
CPLOpLookup::CPLOpLookup():
    myTimeout(defaultTimeout)
{
#ifdef MEM_TEST
    cpLog ( LOG_DEBUG, "new %s[%x]", name(), this );
#endif
}  //c'tor

///Destructor
CPLOpLookup::~CPLOpLookup()
{
#ifdef MEM_TEST
    cpLog ( LOG_DEBUG, "~%s[%x]", name(), this );
#endif
}  //d'tor


///Append an operator to this class
bool
CPLOpLookup::setNextOperator( Sptr < Operator > op, int prevNode )
{
    if ( myNextState != 0 )
        return (myNextState->setNextOperator( op, prevNode ));
    else
    {
        cpLog( LOG_ERR, "Expecting a state to be defined." );
        assert( 0 );
    } /*else*/

    return false;
}  //setNextOperator


///Initialize the Attributes member variables
bool
CPLOpLookup::setAttributes( const char* attrName, const char* attrValue)
{
    if (strcmp( attrName, "timeout" ) == 0 )
    {
        myTimeout = atoi( attrValue );
    } //timeout
    else if ( strcmp ( attrName, "source" ) == 0 )
    {
        if ( attrValue != 0 )
        {
            mySource = attrValue;

            //Add hostname to URL if it's missing it.
            if (validateUrl( mySource ) == false )
            {
                cpLog( LOG_ERR, "Invalid 'url'." );
                return false;
            };

            if (validateSource( mySource ) == false )
            {
                cpLog( LOG_ERR, "Invalid 'source' value." );
                return false;
            } /*if*/
        } /*if*/

    }  //source
    else if ( strcmp ( attrName, "use" ) == 0 )
    {
        if ( attrValue != 0 )
            myUseFields = attrValue;
    }  //use
    else if ( strcmp ( attrName, "ignore" ) == 0 )
    {
        if ( attrValue != 0 )
            myIgnoreFields = attrValue;
    }  //ignore
    else if ( strcmp ( attrName, "clear" ) == 0 )
    {
        /**
         Ignore since this attribute value is
         saved in CPLOpAckLookup. 
         **/

    }  /*clear*/

    else
    {
        cpLog( LOG_ERR, "Unknown attribute: '%s'.", attrName );
        return false;
    } //else

    return true;
}  //setAttributes


///Validate source attribute value
bool
CPLOpLookup::validateSource( const string& str ) const
{
    //Allow sip URLs
    if ( str.find( "sip:" ) == 0 )
        return true;

    //Allow 'registration'
    if ( str.find( "registration" ) == 0 )
        return true;

    return false;
}  //validateSource


///
const Sptr < State >
CPLOpLookup::process( Sptr < SipProxyEvent > event)
{
    Sptr < InviteMsg > inviteMsg;
    inviteMsg = getInvite( event );

    if ( inviteMsg == 0 )
    {
        cpLog(LOG_ERR, "No INVITE found." );
        return 0;
    } /*if*/

    //Send the message to the redirect server to get next
    //contact.
    if ( queryServer( event, inviteMsg ) == false )
        return 0;

    //Start the timer.  Redirect Server must respond
    //before time-out.
    if ( myTimeout > 0 )
        setTimer( event, myTimeout );
    else
        setTimer( event, 1 );

    return myNextState;
}  //process


///Ask Redirect Server for next Contact
bool
CPLOpLookup::queryServer( Sptr < SipProxyEvent > sipProxyEvent,
                          Sptr < InviteMsg > msg )
{
    //Create a new copy of Invite instead of modifying original
    Sptr < InviteMsg > inviteMsg = new InviteMsg( *msg );

    //Build a Request for Redirect Server.
    Sptr<BaseUrl> redirectBaseUrl;

    if ( mySource.find( "registration" ) != string::npos )
    {
        //Query for next location for original-destination's user only.
        redirectBaseUrl = inviteMsg->getRequestLine().getUrl();

    } /* if mySource.find(registration) */
    else
    {
        redirectBaseUrl = new SipUrl(mySource); 
    } /*else*/

    //Get original request line
    SipRequestLine requestLine( inviteMsg->getRequestLine() );

    //Create a SipUrl pointer from a BaseUrl pointer. 
    Sptr<SipUrl> redirectSipUrl; 
    redirectSipUrl.dynamicCast( redirectBaseUrl );

    //Build new request URL
    RoundRobinServers& rServers = RoundRobinServers::instance( SERVER_RS );
    Data lHost;
    int lPort;
    rServers.getHostNPort(inviteMsg->getFrom().getUrl()->encode(), lHost, lPort);
    redirectSipUrl->setHost(lHost); 
    redirectSipUrl->setPort(lPort);

    //Use the new URL in the request line
    requestLine.setUrl( redirectBaseUrl );
    inviteMsg->setRequestLine( requestLine );

    SipFrom origFrom = inviteMsg->getFrom();
    string featCode = getStateMachineData(sipProxyEvent)->getFeatureCode();
    if ((featCode.length() > 0) && 
        (strcmp(featCode.c_str(), DisableCallerID) == 0) )
    {
        /**
          Change the 'From' field to *67@FS_IPaddress,
          so the RS can lookup its database for the *67 user
          and return the correct response. 
         **/
        SipFrom from = inviteMsg->getFrom();
        Sptr<SipUrl> sipFromUrl;
        sipFromUrl.dynamicCast( from.getUrl() ); 

        sipFromUrl->setUserValue( Data( DisableCallerID ) );
        Data theFSAddress ( theSystem.gethostAddress() );
        sipFromUrl->setHost( theFSAddress );
        sipFromUrl->setPort( sipPort );
        //sipFromUrl->setUrl(sipFromUrl);
        inviteMsg->setFrom(from);

        Sptr < CallContainer > callcontainer = sipProxyEvent->getCallContainer();
        Sptr < CallInfo > call = sipProxyEvent->getCallInfo();
        assert (call != 0 );

        Sptr < SipEvent > sipEvent;
        sipEvent.dynamicCast( sipProxyEvent );
        if (sipEvent != 0)
        {
            getStateMachineData( sipEvent )->setOrigFrom(origFrom);
            SipFrom sFrom = getStateMachineData( sipEvent )->getOrigFrom();
            cpLog(LOG_DEBUG, "the saved From is: %s", sFrom.encode().logData());
            sipEvent->getSipMsg()->setFrom(from);
            sipEvent->setSipMsg(sipEvent->getSipMsg());  //reset CallLeg
            // add a new entry in CallContainer
            callcontainer->addCall(sipEvent->getSipCallLeg(), call);
        }
    }

    //Determine if we need to clean-up VIA list.
    checkViaList( inviteMsg );

    // Add new Via header
    SipVia via;
    OpAddVia( via, sipPort );


    //Set the branch number in Via and save in CallInfo
    via.setBranch(  inviteMsg->computeBranch2() +
		    getStateMachineData(sipProxyEvent)->getNextBranch() );

    //Set the Via
    inviteMsg->setVia( via, 0);

    //Useful for debugging
    redirectSipUrl.dynamicCast( redirectBaseUrl );
    cpLog( LOG_DEBUG, "Querying for '%s@%s:%s%s'",
           redirectBaseUrl->getUserValue().logData(),
           redirectSipUrl->getHost().logData(),
           redirectSipUrl->getPort().logData(),
           redirectSipUrl->getUrlParam().logData() );

    // Send message to destination
    sendAsync( sipProxyEvent, *inviteMsg );

    return true;

} //queryServer


///
void
CPLOpLookup::checkViaList( Sptr < InviteMsg > inviteMsg )
{
    //Create Network Address of current FS host
    NetworkAddress FSAddress;
    FSAddress.setHostName( string(theSystem.gethostAddress()) );
    FSAddress.setPort( sipPort );

    //Find Network Address from last VIA's URL.
    LocalScopeAllocator lo;
    string viaHost( inviteMsg->getVia(0).getHost().getData(lo) );
    int viaPort = inviteMsg->getVia(0).getPort();
    NetworkAddress ViaAddress( viaHost, viaPort);

    //Remove the last VIA if the VIA belongs to FS's host
    if ( FSAddress == ViaAddress)
    {
        inviteMsg->removeVia( 0 );
#if 0   //Change to 1 if needed for debugging
        cpLog( LOG_DEBUG, "Removed VIA: %s:%d.",
               ViaAddress.getIpName().c_str(),
               ViaAddress.getPort() );
#endif
    } /*if*/
#if 0   //Change to 1 if needed for debugging
    else
    {
        cpLog( LOG_DEBUG, "FS (%s:%d) != VIA (%s:%d)",
               FSAddress.getIpName().c_str(),
               FSAddress.getPort(),
               ViaAddress.getIpName().c_str(),
               ViaAddress.getPort() );
    } /*else*/
#endif

} //checkViaList


///
///Outputs the values of the member variables
void
CPLOpLookup::logData() const
{
    cpLog( LOG_DEBUG, "%s-TIMEOUT(%d s) SOURCE(%s) USE(%s) IGNORE(%s)",
           name(), myTimeout, mySource.c_str(),
           myUseFields.c_str(), myIgnoreFields.c_str() );

}  //logData


///Returns the name of the class
const char* const
CPLOpLookup::name() const
{
    return "CPLOpLookup";
}  //name


//End of file

