
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpMail_cxx_Version =
    "$Id$";
#include "CPLOpMail.hxx"
#include "MailThread.hxx"
#include "MailThreadMgr.hxx"
#include "SipSubject.hxx"

using namespace Vocal;


CPLOpMail::CPLOpMail()
{
}



CPLOpMail::~CPLOpMail()
{
    /// no op...
}


bool 
CPLOpMail::setAttributes(const char* type, const char* value)
{
    if ( strcmp( type, "url" ) == 0 )
        myUrl = value;
    else  //Unknown attribute name
        assert ( 0 );

    return true;
}


const Sptr < State > 
CPLOpMail::process(const Sptr < SipProxyEvent > anEvent)
{
    Mail aMail;
    aMail.mailTo = myUrl;
    cpLog(LOG_DEBUG, ":process()");
    aMail.msg = getMsg(anEvent);
    cpLog(LOG_DEBUG, "Message [%s]", aMail.msg.c_str());
    MailThreadMgr::thread()->mail(aMail);
    cpLog(LOG_DEBUG, "Mailed...");
    return CPLOperator::process(anEvent);
}


bool 
CPLOpMail::setNextOperator(Sptr < Operator > anOperator, int nodeId /*default value*/)
{
    addOperator(anOperator);
    return true;
}


const char* const 
CPLOpMail::name() const
{
    return "CPLOpMail";
}


void 
CPLOpMail::logData(void) const
{
    cpLog(LOG_DEBUG, "[%x]: mailto <%s>...", this, myUrl.c_str());
}


string 
CPLOpMail::getMsg(const Sptr < SipProxyEvent > & anEvent) const
{
    string msg;

    Sptr < SipMsg > sm = getSipMsg(anEvent);
    Sptr < SipCommand > sc = getSipCommand(anEvent);
    LocalScopeAllocator lo;

    if (sm == 0)
    {
        msg = "Sip message info not available\r\n";
    }
    else
    {
        // senders name ...
        Sptr<SipUrl> sipUrl;
        sipUrl.dynamicCast( sm->getFrom().getUrl() );

        msg = msg + "CALL FROM: " + sipUrl->getUserValue().getData(lo) + '@';
        msg = msg + sipUrl->getHost().getData(lo) + "\r\n";
    }

    if (sc == 0)
    {
        msg += "Sip command info unavailable.\r\n";
    }
    else
    {
        // request type...
        msg += "REQUEST TYPE: ";
        msg += sc->getRequestLine().getMethod().getData(lo);
        msg += "\r\n";

        // subject ...
        msg += "SUBJECT: ";
        msg += sc->getSubject().get().getData(lo); 
        msg += "\r\n";
    } /*else*/

    return msg;
} //getMsg() 


//End of File
