#ifndef CPLOpNull_Hxx
#define CPLOpNull_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CPLOpNullVersion =
    "$Id: CPLOpNull.hxx,v 1.11 2002/03/19 21:39:41 jason Exp $";




#include "CPLOperator.hxx"
#include <string>

namespace Vocal
{

/**
  *  Generic operator that calls any operators in its list. 
  */
class CPLOpNull: public CPLOperator
{
    public:
        /// default constructor
        CPLOpNull();

        /**
         * constructor with debugging info
         *@param debugInfo debugging info as char string
         */
        CPLOpNull( char* debugInfo );

        /// destructor
        ~CPLOpNull();

        /**
         * method to process the event
         *@param event reference to the new event to be processed
         *@return reference to next state or 0 if no transition
         */
        const Sptr < State > process( const Sptr < SipProxyEvent > event );

        /**
         * method to store the attributes
         *@param attrName name of the attribute
         *@param attrValue value of the attribute
         *@return true if valid operation else false
         */
        bool setAttributes( const char* attrName, const char* attrValue );

        /**
         * method to set the next operator in state m/c
         *@param op reference to the next operator
         *@param prevNode name of the prevNode in the CPL script
         *@return true if legal else false
         */
        bool setNextOperator( Sptr < Operator > op, int prevNode );

        /// method to return the class name
        const char* const name() const;

    protected:
        /// attribute to store any debugging info
        string myDebugName;

        /// method to display attribute value
        void logData() const;
};

 
} // namespace Vocal

#endif

