
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpPrioritySwitch_cxx_Version =
    "$Id$";
#include "CPLOpPrioritySwitch.hxx"
#include "CPLOpPriority.hxx"
#include "SipPriority.hxx"

using namespace Vocal;

CPLOpPrioritySwitch::CPLOpPrioritySwitch()
{
    // no op!
}



CPLOpPrioritySwitch::~CPLOpPrioritySwitch()
{
    //no-op!!!
}



bool CPLOpPrioritySwitch::setAttributes(const char* type, const char* value)
{
    return true;  //This CPL tag does not have attributes 
}



const Sptr < State > CPLOpPrioritySwitch::process(const Sptr < SipProxyEvent > anEvent)
{
    Sptr < State > retVal;
    Data str;

    if (getPriority(anEvent, str) == false) return 0;

    cpLog(LOG_DEBUG, ": Switch on \"%s\"...", str.logData());

    CPLOperator::myOpIterator curr = myOperators.begin();
    CPLOperator::myOpIterator end = myOperators.end();

    //assuming same as CPLOpAddressSwitch that default
    //operators are already in operator list...
    while (curr != end)
    {
        if (strcmp((*curr)->name(), "CPLOpPriority") == 0)
        {
            if (((CPLOpPriority *)curr->getPtr())->compare(str) == true)
            {
                retVal = (*curr)->process(anEvent);
                if (retVal != 0)
                    return retVal;
            }
        }
        else
        {
            cpLog(LOG_DEBUG, " Executing non <priority>");
            retVal = (*curr)->process(anEvent);
            if (retVal != 0)
                return retVal;
        }
        curr++;
    }

    //no change in state...
    return 0;
}


bool CPLOpPrioritySwitch::setNextOperator(Sptr < Operator > anOperator, int nodeId /*default value*/)
{
    if (nodeId == CPLNodeNotpresent)
    {
        /**
         * every message has a priority, hence not present is never executed...
         */ 
        return true;
    }

    CPLOperator::myOpIterator curr = myOperators.begin();
    CPLOperator::myOpIterator end = myOperators.end();

    //assuming same as CPLOpAddressSwitch that default
    //operators are already in operator list...
    while (curr != end)
    {
        if (strcmp((*curr)->name(), "CPLOpProxy") == 0)
        {
            myOperators.insert(curr, anOperator);
            return true;
        }
        curr++;
    }

    //default operator not found...
    assert(0);

    return false;
}


const char* const CPLOpPrioritySwitch::name() const
{
    return "CPLOpPrioritySwitch";
}


void CPLOpPrioritySwitch::doCpLog(void) const
{
    logData();

    CPLOperator::myOpConstIterator curr = myOperators.begin();
    CPLOperator::myOpConstIterator end = myOperators.end();

    while (curr != end)
    {
        cpLog(LOG_DEBUG, "%s[%x]...",
              (*curr)->name(), curr->getPtr());
        if (strcmp((*curr)->name(), "CPLOpProxy") != 0)
        {
            (*curr)->doCpLog();
        }
        curr++;
    }
}


void CPLOpPrioritySwitch::logData(void) const
{
    cpLog(LOG_DEBUG, ":@[%x] # of Operators = %d...", this,
          myOperators.size());
}


bool 
CPLOpPrioritySwitch::getPriority(Sptr < SipProxyEvent > anEvent, Data& data) const
{
    Sptr < SipCommand > aComm = getSipCommand(anEvent);
    if (aComm == 0) return false;

    data = aComm->getPriority().get();
    return true;
}

