
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpStart_cxx_Version =
    "$Id$";
#include "CPLOpStart.hxx"
#include "SipEvent.hxx"
#include "CallInfo.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"

using namespace Vocal;

///Constructor
CPLOpStart::CPLOpStart()
{
#ifdef MEM_TEST
    cpLog ( LOG_DEBUG, "new %s[%x]", name(), this );
#endif
}  //c'tor


///Destructor
CPLOpStart::~CPLOpStart()
{
#ifdef MEM_TEST
    cpLog ( LOG_DEBUG, "~%s[%x]", name(), this );
#endif
}  //d'tor


///Append an operator to this class
bool
CPLOpStart::setNextOperator( Sptr < Operator > op, int prevNode )
{
    addOperator( op );
    return true;
}  //setNextOperator


///Initialize the Attributes member variables
bool
CPLOpStart::setAttributes( const char* attrName, const char* attrValue)
{
    //This is not a CPL tag.  There are never attributes to set. 
    return true;
}  //setAttributes


///
const Sptr < State >
CPLOpStart::process( Sptr < SipProxyEvent > event)
{
    //Find SipEvent
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    assert( sipEvent != 0 );

    //Find Invite message
    Sptr < InviteMsg > invite;
    invite.dynamicCast( sipEvent->getSipMsg() );
    assert ( invite != 0 );

    //Save Invite message in CallInfo in case we need to refer back
    //to it later.
    sipEvent->getCallInfo()->setSipCommand( invite );

    //Initialize proxy Status
    getStateMachineData(event)->setProxyStatus( FsStateMachnData::NoProxy );

    //Send a 100 to source
    StatusMsg statusMsg( *invite, 100 );
    sipEvent->getSipStack()->sendReply( statusMsg );

    //Do next operator
    return ( CPLOperator::process( event ) );
}  //process


///Outputs the values of the member variables
void
CPLOpStart::logData() const
{
    cpLog( LOG_DEBUG, "%s[%x]", name(), this );
}  //logData


///Returns the name of the class
const char* const
CPLOpStart::name() const
{
    return "CPLOpStart";
}  //name


//End of file

