
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpString_cxx_Version =
    "$Id$";
#include "CPLOpString.hxx"

using namespace Vocal;

///
CPLOpString::CPLOpString():
    myField( CPLOpString::FieldInvalid )
{
    //myStr initialization defaults to ""
       
}  //Constructor


///
CPLOpString::~CPLOpString()
{
    //no-op!!!
}  //Destructor


///
bool
CPLOpString::setAttributes(const char* type, const char* value)
{
    myStr = value; 
    toCaseLess(myStr);

    if ( strcmp( type, "is" ) == 0 )
    {
        myField = CPLOpString::FieldIs;
    } /*if*/
    else if ( strcmp( type, "contains" ) == 0 )
    {
        myField = CPLOpString::FieldContains;
    } /*else-if*/
    else  //Unknown attribute name
    {
        assert ( 0 );
    } /*else*/

    return true;

}  //setAttributes


///
const Sptr < State >
CPLOpString::process(const Sptr < SipProxyEvent > anEvent)
{
    return CPLOperator::process(anEvent);
}  //process


///
bool
CPLOpString::setNextOperator(Sptr < Operator > anOperator, int nodeId /*def value*/)
{
    addOperator(anOperator);
    return true;
}  //setNextOperator


///
const char* const
CPLOpString::name() const
{
    return "CPLOpString";
}  //name


///
void
CPLOpString::logData(void) const
{
    switch (myField)
    {
        case CPLOpString::FieldIs:
        {
            cpLog(LOG_DEBUG, "CPLOpString-IS (%s)", myStr.c_str());
        } /*case*/
        break;

        case CPLOpString::FieldContains:
        {
            cpLog(LOG_DEBUG, "CPLOpString-Contains (%s)", myStr.c_str());
        } /*case*/
        break;

        default:
        {
            //attribute value not set...
            cpLog(LOG_DEBUG, "%s[%x]-FIELD (OTHERWISE)",
                  name(), this);
        } /*default*/
    } /*switch*/
}  //logData


/** CPL draft requires comparing strings "locale-insensitive caseless."
 * For simplicity, this implementation only does a caseless ASCII 
 * compare.
 */
bool
CPLOpString::compare(Data& data)
{
    if ( myStr.size() == 0 )
        return true;
    LocalScopeAllocator lo;
    string testStr(data.getData(lo));
    toCaseLess(testStr);
    cpLog(LOG_DEBUG, "comparing \"%s\" with \"%s\"...",
          testStr.c_str(), myStr.c_str());
    switch (myField)
    {
        case CPLOpString::FieldIs:
        { 
            return (testStr == myStr ? true : false);
        } /*case*/
        break; 

        case CPLOpString::FieldContains:
        {
            return (testStr.find(myStr) != string::npos ? true : false);
        } /*case*/
        break; 

        case CPLOpString::FieldInvalid:
        {
            return (true);  //assuming otherwise...
        } /*case*/
        break; 

        default:
          ;  //No default cases expected
    } /*switch*/

    return false;
}  //compare


///
void
CPLOpString::toCaseLess(string& aStr)
{
    char * p = (char*) aStr.c_str();

    for (; *p; p++) 
    {
        (*p) = toupper(*p);
    } /*for*/

}  //toCaseLess

//End of file


