
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpTransition_cxx_Version =
    "$Id: CPLOpTransition.cxx,v 1.11 2002/03/19 21:39:41 jason Exp $";
#include "CPLOpTransition.hxx"
#include "Data.hxx"
#include "SipProxyEvent.hxx"
#include "CallInfo.hxx"
#include "CPLState.hxx"
#include <string.h>
#include <stdlib.h>
#include <string>

using namespace Vocal;

///Constructor
CPLOpTransition::CPLOpTransition()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "new CPLOpTransition[%x]", this );
#endif
}  //c'tor


///Destructor
CPLOpTransition::~CPLOpTransition()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~CPLOpTransition[%x]", this );
#endif
}  //d'tor


///Append an operator to this class
bool
CPLOpTransition::setNextOperator( Sptr < Operator > op, int prevNode )
{
    myNextOp = op;
    return true;
}  //setNextOperator


///Initialize the Attributes member variables
bool
CPLOpTransition::setAttributes( const char* attrName, const char* attrValue)
{
    return false;
}  //setAttributes


///
const Sptr < State >
CPLOpTransition::process( Sptr < SipProxyEvent > event)
{
    //  make transition to the next state...
    event->getCallInfo()->setState(myNextState);
    return myNextState->process(event);
}  //process


///Outputs the values of the member variables
void
CPLOpTransition::logData() const
{
    cpLog(LOG_DEBUG, "transits to state[%s], next operator [%s] ...",
          myNextState->name(), myNextOp->name());
}  //logData


///Returns the name of the class
const char* const CPLOpTransition::name() const
{
    return "CPLOpTransition";
}  //name




//End of file

