
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLParser_cxx_Version =
    "$Id: CPLParser.cxx,v 1.16 2002/03/19 21:39:41 jason Exp $";
#include "CPLParser.hxx"
#include "cpLog.h"
#include <parserInternals.h>

using namespace Vocal;



//Singleton implementation
CPLParser* CPLParser::myInstance = NULL;
//Flag to be set to TRUE if parsing errors occur
bool CPLParser::problemsFound = false;

//LIBXML global for deciding on whether to validate document
//with dtd or not.
extern int xmlDoValidityCheckingDefaultValue;


///
CPLParser::CPLParser()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "CPLParser c'tor" );
#endif
}  //CPLParser


///
CPLParser*
CPLParser::Instance()
{
    if ( myInstance == 0 )
    {
        myInstance = new CPLParser;
    } /*if*/

    return myInstance;
}  //Instance


///
CPLParser::~CPLParser()
{
    deleteTree();
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "CPLParser d'tor" );
#endif
}  //Destructor


///
xmlNodePtr
CPLParser::buildTree( string& script )
{
    if (script.length() == 0) return NULL;

    //LIBXML crashes if string is not null-terminated.
    script[ script.length() ] = 0;

    xmlDoValidityCheckingDefaultValue = 1;

    myTree = NULL;
    problemsFound = false;

    xmlParserCtxtPtr ctxt;

    ctxt = xmlCreateMemoryParserCtxt( (char*)script.data(),
                                      script.length() );
    ctxt->vctxt.warning = CPLValidityWarning;
    ctxt->vctxt.error = CPLValidityError;

    xmlParseDocument( ctxt );

    int ret = ctxt->wellFormed;
    myTree = ctxt->myDoc;

    xmlFreeParserCtxt( ctxt );

    //Clear in case other libxml users do not want validation.
    xmlDoValidityCheckingDefaultValue = 0;

    if ( ( !ret ) || ( problemsFound == true ) )
    {
        xmlFreeDoc( myTree );
        myTree = NULL;
        cpLog( LOG_ERR, "Cannot process CPL Script." );
        return NULL;
    } /*if*/

    return ( xmlDocGetRootElement( myTree ) );

}  //buildTree


///
void
CPLParser::deleteTree()
{
    if ( myTree != NULL )
        xmlFreeDoc( myTree );


}  //deleteTree

//////////////////////////////////////////////////
// CALLBACKS WHEN LIBXML ENCOUNTERS PROBLEMS    //
//////////////////////////////////////////////////

///
void
Vocal::CPLValidityWarning(void *ctx, const char* msg, ...)
{
    va_list args;

    va_start( args, msg );
    vCpLog( LOG_WARNING, __FILE__, __LINE__, msg, args );
    va_end(args);

    CPLParser::problemsFound = true;
}  // CPLValidityWarning


///
void
Vocal::CPLValidityError(void *ctx, const char* msg, ...)
{
    va_list args;

    va_start( args, msg );
    vCpLog( LOG_ERR, __FILE__, __LINE__, msg, args );
    va_end(args);

    CPLParser::problemsFound = true;
}  // CPLValidityError


//End of file

