
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLTimeRange_cxx_Version =
    "$Id$";
#include "CPLTimeRange.hxx"
#include "Range.hxx"
#include "MyTime.hxx"
#include <string.h>
#include <stdio.h>

using namespace Vocal;


CPLTimeRange :: CPLTimeRange()
{
    myTimeOfDay = 0;
    myWeekDay = 0;
    myDate = 0;
    myMonth = 0;
    myYear = 0;
}  //Constructor


///
CPLTimeRange::CPLTimeRange(const CPLTimeRange & aCTR)
{
    *this = aCTR;
}  //Copy Constructor

///
CPLTimeRange :: ~CPLTimeRange()
{
    myTimeOfDay = 0;
    myWeekDay = 0;
    myDate = 0;
    myMonth = 0;
    myYear = 0;
}  //Destructor


///
CPLTimeRange&
CPLTimeRange::operator=(const CPLTimeRange & aCTR)
{
    if (aCTR.myTimeOfDay != 0)
    {
        this->myTimeOfDay = new Range < TimeField > (*(aCTR.myTimeOfDay));
    }
    else this->myTimeOfDay = 0;

    if (aCTR.myWeekDay != 0)
    {
        this->myWeekDay = new Range < TimeField > (*(aCTR.myWeekDay));
    }
    else this->myWeekDay = 0;

    if (aCTR.myDate != 0)
    {
        this->myDate = new Range < TimeField > (*(aCTR.myDate));
    }
    else this->myDate = 0;

    if (aCTR.myMonth != 0)
    {
        this->myMonth = new Range < TimeField > (*(aCTR.myMonth));
    }
    else this->myMonth = 0;

    if (aCTR.myYear != 0)
    {
        this->myYear = new Range < TimeField > (*(aCTR.myYear));
    }
    else this->myYear = 0;

    return *this;
}  //Assignment operator


///
char * const
CPLTimeRange::toString() const
{
    static char str[256];
    sprintf(str, "Time Range is");
    if (myTimeOfDay != 0)
    {
        sprintf(str, "%s time[%s]", str, this->myTimeOfDay->toString());
    }
    else sprintf(str, "%s any time", str);

    if (myWeekDay != 0)
    {
        sprintf(str, "%s on weekdays[%s]", str, this->myWeekDay->toString());
    }
    else sprintf(str, "%s any day of week", str);

    if (myDate != 0)
    {
        sprintf(str, "%s on dates[%s]", str, this->myDate->toString());
    }
    else sprintf(str, "%s on any date", str);

    if (myMonth != 0)
    {
        sprintf(str, "%s of months[%s]", str, this->myMonth->toString());
    }
    else sprintf(str, "%s of any month", str);

    if (myYear != 0)
    {
        sprintf(str, "%s of years[%s].", str, this->myYear->toString());
    }
    else sprintf(str, "%s of any year", str);

    return str;
}  //toString


///
void
CPLTimeRange::set(Sptr < Range < TimeField > > & aField, const char * str)
{
    if (aField == 0)
    {
        aField = new Range < TimeField > (str);
    }
    else
    {
        aField->add(str);
    }
}  //set


///
void
CPLTimeRange::timeOfDay(const char * str)
{
    set(myTimeOfDay, str);
}  //timeOfDay


///
void
CPLTimeRange::weekDay(const char * str)
{
    set(myWeekDay, str);
}  //weekDay


///
void
CPLTimeRange::date(const char * str)
{
    set(myDate, str);
}  //date


///
void
CPLTimeRange::month(const char * str)
{
    set(myMonth, str);
}  //month


///
void
CPLTimeRange::year(const char * str)
{
    set(myYear, str);
}  //year


///
bool
CPLTimeRange::testTimeOfDay(const TimeField& aValue)
{
    return ((myTimeOfDay == 0 || *myTimeOfDay == aValue) ? true : false);
}  //testTimeOfDay


///
bool
CPLTimeRange::testWeekDay(const TimeField& aValue)
{
    return ((myWeekDay == 0 || *myWeekDay == aValue) ? true : false);
}  //testWeekDay


///
bool
CPLTimeRange::testDate(const TimeField& aValue)
{
    return ((myDate == 0 || *myDate == aValue) ? true : false);
}  //testDate


///
bool
CPLTimeRange::testMonth(const TimeField& aValue)
{
    return ((myMonth == 0 || *myMonth == aValue) ? true : false);
}  //testMonth


///
bool
CPLTimeRange::testYear(const TimeField& aValue)
{
    return ((myYear == 0 || *myYear == aValue) ? true : false);
}  //testYear


//End of file

