#ifndef CallProcessingLanguage_Hxx
#define CallProcessingLanguage_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CallProcessingLanguageVersion =
    "$Id: CallProcessingLanguage.hxx,v 1.17 2002/03/19 21:39:41 jason Exp $";


namespace Vocal
{

/**All CPL nodes sorted by node string length and then alphabetically.
*/
typedef enum
{
    CPLNodeCpl = 0,
    CPLNodeLog,
    CPLNodeSub,
    CPLNodeBusy,
    CPLNodeMail,
    CPLNodeTime,
    CPLNodeProxy,
    CPLNodeLookup,
    CPLNodeReject,
    CPLNodeString,
    CPLNodeAddress,
    CPLNodeDefault,
    CPLNodeFailure,
    CPLNodeSuccess,
    CPLNodeDaylight,
    CPLNodeIncoming,
    CPLNodeLocation,
    CPLNodeNoanswer,
    CPLNodeNotfound,
    CPLNodeOutgoing,
    CPLNodePriority,
    CPLNodeRedirect,
    CPLNodeVmRedirect,
    CPLNodeStandard,
    CPLNodeTimezone,
    CPLNodeOtherwise,
    CPLNodeSubaction,
    CPLNodeNotpresent,
    CPLNodeTimeswitch,
    CPLNodeRedirection,
    CPLNodeStringswitch,
    CPLNodeAddressswitch,
    CPLNodePriorityswitch,
    CPLNodeRemoveLocation,
    CPLNodeComments,
    CPLNodeUndefined,
    CPLNodeEndofNodes
} CPLNode;

///New Constants that are used in CPLInterpreter to differentiate outputs
//from a <proxy> that has order="parallel" or "sequential."
typedef enum
{
    CPLNodeBusyExt = 973,
    CPLNodeFailureExt,
    CPLNodeNoanswerExt,
    CPLNodeRedirectionExt,
    CPLNodeDefaultExt
} CPLNodeExtensions;

/* Array of CPL tags.  The defined tags must MATCH the tags
 * in the cpl.dtd
 */
extern const char* CPLNodeStr [];

///Array of <location>'s attributes
extern const char* LocationAttributesTable [];

///Total Number of attributes defined by <location>
extern const unsigned int LocationAttributesTableSize;

///Array of <remove-location>'s attributes
extern const char* RemoveLocationAttributesTable [];

///Total Number of attributes defined by <remove-location>
extern const unsigned int RemoveLocationAttributesTableSize;

///Array of <lookup>'s attributes
extern const char* LookupAttributesTable [];

///Total Number of attributes for <lookup>
extern const unsigned int LookupAttrTableSize;

///Array of <proxy>'s attributes
extern const char* ProxyAttributesTable [];

///Total Number of attributes for <proxy>
extern const unsigned int ProxyAttributesTableSize;

///Array of <string-switch>'s attributes
extern const char* StringSwitchAttrTable [];

///Total Number of attributes for <string-switch>
extern const unsigned int StringSwitchAttrTableSize;

///Array of <priority-switch>'s attributes
extern const char* PrioritySwitchAttrTable [];

///Total Number of attributes for <priority-switch>
extern const unsigned int PrioritySwitchAttrTableSize;

///Array of <priority>'s attributes
extern const char* PriorityAttrTable [];

///Total Number of attributes for <priority>
extern const unsigned int PriorityAttrTableSize;

///Array of <mail>'s attributes
extern const char* MailAttrTable [];

///Total Number of attributes for <mail>
extern const unsigned int MailAttrTableSize;

///Array of <log>'s attributes
extern const char* LogAttrTable [];

///Total Number of attributes for <log>
extern const unsigned int LogAttrTableSize;

///Array of <standard>'s attributes
extern const unsigned int StandardAttrTableSize;

///Total number of attributes for <standard>
extern const char* StandardAttrTable [];

///Total number of attributes for <daylight>
extern const unsigned int DaylightAttrTableSize;

///Array of <daylight>'s attributes
extern const char* DaylightAttrTable [];

///Total number of attributes for <timezone>
extern const unsigned int TimeZoneAttrTableSize;

///Array of <timezone>'s attributes
extern const char* TimeZoneAttrTable [];

///Array of <string>'s attributes
extern const char* StringAttrTable [];

///Total number of attributes for <string>
extern const unsigned int StringAttrTableSize;

///Array of <time>'s attributes
extern const char * TimeAttrTable[];

///Total number of attributes for <time>
extern const unsigned int TimeAttrTableSize;

///Array of <address-switch>'s attributes
extern const char* AddrSwitchAttrTable [];

///Total number of attributes for <address-switch>
extern const unsigned int AddrSwitchAttrTableSize;

///Array of <address>'s attributes
extern const char* AddressAttrTable [];

///Total number of attributes for <address>
extern const unsigned int AddressAttrTableSize;

///Array of <reject>'s attributes
extern const char* RejectAttrTable [];

///Total number of attributes for <reject>
extern const unsigned int RejectAttrTableSize;
 
}

#endif




