#ifndef _RANGE__HXX
#define _RANGE__HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const Range_hxx_Version =
    "$Id$";





#include <vector>
#include <strstream>
#include "global.h"

template < class T > class Pair;
template < class T > class Range;

template < class T > istrstream& operator>>(istrstream &, Pair < T > &);
template < class T > ostrstream& operator<<(ostrstream &, Pair < T > &);
template < class T > bool operator==(const T&, const Range < T > &);

/**
 * The container for a hi - lo pair in a range...
 */
template < class T > class Pair
{
    public:
        T hi;
        T lo;

        Pair();
        Pair(const Pair&);
        friend istrstream& operator>> < T > (istrstream &, Pair < T > &);
        friend ostrstream& operator<<<T > (ostrstream &, Pair < T > &);
};

/**
 * A container class for maintaining the list of ranges in time switch
 * and the timezone tags...
 */
template < class T > class Range
{
    public:

        ///Constructor
        Range()
        {}

        ///Constructor

        Range(const Range&);

        ///Constructor
        Range(const Pair < T > &);

        ///Constructor
        Range(const char *);

        ///Destructor
        ~Range();

        ///Add pair to range
        Range& add(const Pair < T > &);

        ///Add pair to range
        Range& add(const char *);

        ///Assignment operator
        Range& operator=(const Range&);

        ///Convert a time range to a string
        char * const toString();

        ///Check for equality
        bool operator==(const T&) const;

    private:
        ///Data type to hold time ranges
        vector < Pair < T > * > myRangeList;

        ///Deletes all ranges from myRangeList
        void flush();
};

//Compilers require template headers to include the source code.
#include "Range.cxx"
 
#endif

