
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const VmPortContainer_cxx_Version =
    "$Id: VmPortContainer.cxx,v 1.14 2002/03/19 21:39:41 jason Exp $";
#include "VmPortContainer.hxx"
#include "cpLog.h"

using namespace Vocal;



//////////////////SINGLETON IMPLEMENTATION//////////////
VmPortContainer* VmPortContainer::myVmPortContainerInstance = NULL;
////////////////////////////////////////////////////////


///
VmPortContainer*
VmPortContainer::instance()
{
    if (myVmPortContainerInstance == 0 )
    {
        myVmPortContainerInstance = new VmPortContainer;
    } /*if*/

    return myVmPortContainerInstance;

}  // Instance


///
VmPortContainer::VmPortContainer()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "VmPortContainer" );
#endif
   myNumberOfConfiguredPorts = 0;
}  //VmPortContainer


///
VmPortContainer::~VmPortContainer()
{
    clearAll();
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~VmPortContainer" );
#endif
}  //~VmPortContainer


///
deque< vmPort >
VmPortContainer::getListOfVmPorts() const
{
    return myPortContainer;
}  //getListOfVmPorts


///
void VmPortContainer::setNumberOfConfiguredPorts( const int port)
{
    myNumberOfConfiguredPorts = port;
}


const int VmPortContainer::getNumberOfConfiguredPorts() const
{
    return myNumberOfConfiguredPorts;
}


///
int
VmPortContainer::getNumberOfVmPorts() const
{
    return myPortContainer.size();
}  //getNumberofPorts


/// GetPort: just get any available port
/// (for instance: last element in list)
bool
VmPortContainer::getVmPort(vmPort& returnValue ) 
{
    vmPort availableVmPort;

    if (!myPortContainer.empty() )
    {
        availableVmPort = myPortContainer.front();
        myPortContainer.pop_front();
        returnValue = availableVmPort;
        cpLog(LOG_DEBUG, "Returning Vm host:%s, port:%d", returnValue.host.c_str(), returnValue.port);
        return (true);
    }
    else
    {
        cpLog( LOG_DEBUG, "No Vm port available" );
        return (false);
    }
} //getPort


///
int
VmPortContainer::appendVmPort(vmPort voiceMailPort)
{
    myPortContainer.push_back(voiceMailPort);
    return true;
}  //appendVmPort


///
int
VmPortContainer::clearAll()
{
    cpLog(LOG_DEBUG, " Clearing the VmContainer");
    while ( !myPortContainer.empty() )
        myPortContainer.pop_front();
    return true;
}  //clearAll


///
void
VmPortContainer::logData()
{
    deque < vmPort > ::iterator it = myPortContainer.begin();
    deque < vmPort > ::iterator itEnd = myPortContainer.end();

    while (it != itEnd)
    {
        cpLog(LOG_DEBUG, "Server:%s", (*it).host.c_str());
        cpLog( LOG_DEBUG, "Port:'%d'", (*it).port );
        ++it;
    } /*while*/
} //logData

//End of File

