/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.singleton;

import java.lang.*;
import java.util.*;
import com.vovida.singleton.*;

/**
 * VCSingletonFactory contains all of the singletons in the system 
 * and makes the instance objects available as required while
 * also ensuring that only one instance exists for each
 * singleton class
 */
public final class VCSingletonFactory
{
    private static Hashtable singletons;
    private static VCSingletonFactory singletonFactory = null;
    
    /**
     * Constructor
     */
    private VCSingletonFactory() 
    {
	singletons = new Hashtable(10);
    }

    /**
     * Create the instance of the required singleton class 
     * or locate it and then return the instance object
     */
    public static Object create(String className)
    {
	if (singletonFactory == null)
	{
	    singletonFactory = new VCSingletonFactory();
	}
	if (className == "VCSingletonFactory")
	    return singletonFactory;
    Object instance = null;
	String classKey = "class " + className;

	if (singletons.containsKey(classKey))
 	{
	    instance = singletons.get(classKey);
	}
	else
	{
	    try
	    {
		instance = Class.forName(className).newInstance();
		singletons.put(classKey, instance);
	    }
	    catch (Exception e)
	    {
		System.out.println("Problem in SingletonFactory");
		e.printStackTrace();
	    }
	}
	return instance;
    }

    /**
     * If the instance object for the singleton class exists
     * in the singletonfactory then remove it In the case
     * of singletonfactory just remove the singletonfactory
     */
    public static void remove(String className)
    {
	if (singletonFactory == null)
        {
	    return;
	}
	String classKey = "class " + className;
	if (singletons.containsKey(classKey))
	{
	    singletons.remove(classKey);
	}
    }
}
