/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony;

import java.net.*;

/**
 * Class VCIncomingMsgHandler is seperate thread which 
 * reads blocking on the incoming port which receives all
 * requests and notifications from the associated JTapi server
 * When the message is read a VCEvent instance is created 
 * the type of which depends on the type of message received:
 * Request, Response, Notification
 * This event is then placed in the incoming FIFO queue for
 * received messages and can be taken from there by a seperate thread
 */

class VCIncomingMsgHandler extends Thread implements VCMsgTypes
{
    VCFifo infifo;
    String portnumber = "";
    boolean keepRunning = true;
    boolean readTimeout = false;
    DatagramSocket readSocket;

    /**
     * Constructor
     */
    public VCIncomingMsgHandler(VCFifo iq, String p)
    {
	this.infifo = iq;
	this.portnumber = p;
    }

    /**
     * Continue to monitor the incoming port for msgs from the jtapi server
     */
    public void run()
    {
 	try
	{
	    int port = Integer.parseInt(portnumber);
	    readSocket = new DatagramSocket(port);
	    DatagramPacket readPacket;
	    byte[] buffer;
            while (keepRunning == true)
	    {
		buffer = new byte[PACKET_BUFFER_SIZE];
		readPacket = new DatagramPacket(buffer, buffer.length);
		try
		{
		    readSocket.receive(readPacket);
		}
		catch (java.io.InterruptedIOException e)
		{
		    e.printStackTrace();
		}
	        if (readPacket.getLength() > 0)
	        {
	            System.out.println("packet size : " + readPacket.getLength());
	            String message = new String(buffer, 0, readPacket.getLength());
                    // find type of msg received
		    System.out.println(message);
                    VCEvent queueEvent = getEvent(message);
                    if (queueEvent != null)
                    {
	                infifo.push(queueEvent);
                    }
	        }
	        else
	        {
		    System.out.println("NO DATA IN PACKET");
	        }
	    }
	    readSocket.close();
	    infifo.release();
	}
        catch (java.io.InterruptedIOException e)
        {
            System.out.println("IO Interrupted");

        }
        catch (SocketException e)
        {
	    if (keepRunning == true)
	    {
	    	e.printStackTrace();
	    }
        }
        catch (UnknownHostException e)
        {
	    e.printStackTrace();
        }
        catch (java.io.IOException e)
        {
	    e.printStackTrace();
	    System.out.println(e.getMessage());
        }
        catch (NumberFormatException e)
        {
	    e.printStackTrace();
        }
    }

    /**
     * Discontiue monitoring the incoming port by setting keepRunning false
     */
    public void setKeep(boolean b)
    {
	keepRunning = b;
	if (keepRunning == false)
	{
 	    if (readSocket != null)
	    {
	        readSocket.close();
	    }
	}
    }

    /**
     * Given the msg from jtapi server create the appropriate event object
     */
    private VCEvent getEvent(String str)
    {
        VCEvent evt = null;
        int evtType = VCEvent.getEvtTypeForMsg(str);
        switch (evtType)
        {
            case VCEvent.REQUESTEVENT :
            {
                evt = new VCRequestEvent(str);
            }
            break;
            case VCEvent.RESPONSEEVENT :
            {
                evt = new VCResponseEvent(str);
            }
            break;
            case VCEvent.NOTIFICATIONEVENT :
            {
                evt = new VCNotificationEvent(str);
            }
            break;
            default :
            {
            }
        }
        return evt;
    }

}


