/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony;

import javax.telephony.*;
import javax.telephony.capabilities.*;
import javax.telephony.callcontrol.*;
import com.vovida.telephony.events.*;
import com.vovida.telephony.capabilities.*;
import com.vovida.telephony.callcontrol.*;
import com.vovida.telephony.callcontrol.capabilities.*;
import com.vovida.singleton.*;
import java.util.*;


/*************************************************************/
/* VCProvider is the Vocal Client Provider Class which      */
/* implements the Provider interface of the JTapi Spec.      */
/*************************************************************/

/**
 * VCProvider is the Vocal Client Provider Class which
 * implements the Provider interface of the JTapi Spec
 * Refer To JTapi Provider Interface for more details
 */

public class VCProvider implements Provider
{

    static protected Provider defaultProvider = null;
    static private final String DEFAULT_PROVIDER_NAME = "VOVIDA";
    static protected Hashtable validProviders = null;
    static protected Hashtable validLocalAddresses = null;
    static protected Hashtable validLocalTerminals = null;
    static protected VCFifo inFifo = null;
    static protected VCFifo outFifo = null;
    static protected VCIncomingMsgHandler inMsgHandler = null;
    static protected VCOutgoingMsgHandler outMsgHandler = null;
    static protected VCEventHandler evtHandler = null;
    static protected VCIdentity idCreator = null;
    static protected VCEventTimer evtTimer = null;

    static private final int UNDEFINED = 0;
    static private boolean provShutdown = true;
    static private String INPORT = "9999";
    static private String HOST = "heisenberg";
    static private String PORT = "9998";

    final static private String IDCLASS = "com.vovida.singleton.VCIdentity";

    /**
     * assume provider needs to be created unless a provider is
     * found with a state which is not shutdown
     */
    static protected void createValidProviders()
        throws IllegalArgumentException
    {
	provShutdown = true;
	System.out.println("Provider - createValidProviders");
	if (validProviders != null)
	{
	    if (defaultProvider != null)
	    {
	        if (defaultProvider.getState() != SHUTDOWN)
	        {
		    provShutdown = false;
		}
	    }
	}
	if (provShutdown == true)
	{
	    validProviders = new Hashtable( 1 );
	    defaultProvider = new VCProvider(DEFAULT_PROVIDER_NAME);
	    validProviders.put(defaultProvider, DEFAULT_PROVIDER_NAME);
	    idCreator = (VCIdentity)VCSingletonFactory.create(IDCLASS);
	    inFifo = new VCFifo();
	    outFifo = new VCFifo();
	}
    }

    /**
     * Stop all threads fromrunning for clean shutdown
     */
    static private void stopThreads()
    {
	outMsgHandler.setKeep(false);
	evtHandler.setKeep(false);
        inMsgHandler.setKeep(false);
	evtTimer.setKeep(false);
	outFifo.release();
	inFifo.release();
    }

    /**
     * start threads running for startup
     */
    static public void startThreads()
    {
	inFifo.startQueue();
	outFifo.startQueue();
        inMsgHandler = new VCIncomingMsgHandler(inFifo, INPORT);
        inMsgHandler.start();
        outMsgHandler = new VCOutgoingMsgHandler(outFifo, HOST, PORT);
        outMsgHandler.start();
        evtHandler = new VCEventHandler(inFifo, getDefaultProvider());
        evtHandler.start();
        evtTimer = new VCEventTimer(inFifo);
        evtTimer.start();
    }

    /**
     * Host, Port of Server and Incoming Port of Client are set
     */
    static public void setHostandPorts(String hostStr, String portStr, String inPortStr)
    {
	HOST = hostStr;
	PORT = portStr;
	INPORT = inPortStr;
    }

    /**
     * Get incoming port of jtapi client
     */
    static public String getInPort()
    {
	return INPORT;
    }
	

    /**
     * Create the terminals and addresses of this provider domain
     */
    static public void setProviderDomain(String addr, String term)
    {
	createValidLocalAddresses(addr);
	createValidLocalTerminals(term);
	associateAddressesWithTerminals(addr, term);
    }

    /**
     * Clear the addresses and terminals of the provider domain
     */
    static public void clearProviderDomain()
    {
	if (validLocalAddresses != null)
	    validLocalAddresses = null;
	if (validLocalTerminals != null)
	    validLocalTerminals = null;
    }

    /**
     * Create the one valid local address for this domain
     */
    static protected void createValidLocalAddresses(String addr)
    {
	if ( validLocalAddresses == null )
	{
	    validLocalAddresses = new Hashtable();
	    validLocalAddresses.put(addr, new VCCallControlAddress(defaultProvider, true, addr));
	}
    }

    /**
     * Create the one valid local terminal for this domain
     */
    static protected void createValidLocalTerminals(String term)
    {
	if (validLocalTerminals == null)
  	{
	    validLocalTerminals = new Hashtable();
	    validLocalTerminals.put(term, new VCCallControlTerminal(defaultProvider, true, term));
	}
    }

    /**
     * associate address with terminal
     */
    static protected void associateAddressesWithTerminals(String addr, String term)
    {
	if ((validLocalAddresses != null) && (validLocalTerminals != null))
	{
	    VCCallControlTerminal localPhone = (VCCallControlTerminal)validLocalTerminals.get(term);
	    VCCallControlAddress localNumber = (VCCallControlAddress)validLocalAddresses.get(addr);
	    localPhone.addAddress(localNumber);
	    localNumber.addTerminal(localPhone);
	}
    }

    /**
     * Check if str is valid key to Local Address
     */
    static public boolean isLocalAddress(String str)
    {
	if (validLocalAddresses.containsKey(str))
	    return true;
	else
	    return false;
    }

    /**
     * Get Address object associated with str
     */
    static protected Address getLocalAddress(String str)
    {
	return (CallControlAddress)validLocalAddresses.get(str);
    }

    /**
     * Check if str is valid key to Local Terminal
     */
    static public boolean isLocalTerminal(String str)
    {
	if (validLocalTerminals.containsKey(str))
	    return true;
	else
	    return false;
    }

    /**
     * Get Terminal object associated with str
     */
    static protected Terminal getLocalTerminal(String str)
    {
	return (Terminal)validLocalTerminals.get(str);
    }

    /**
     * Get default Provider
     */
    static public Provider getDefaultProvider()
    {
	return defaultProvider;
    }

    /**
     * Get Incoming Fifo
     */
    static VCFifo getInFifo()
    {
	return inFifo;
    }

    /**
     * Get Outgoing Fifo
     */
    static VCFifo getOutFifo()
    {
	return outFifo;
    }

    /**
     * Get instance of singleton VCIdentity
     */
    static VCIdentity getIdCreator()
    {
	return idCreator;
    }


/**********************************************************/
/* Instance of provider for JTapi                         */
/*********************************************************/


    private String name;
    protected Vector calls;
    protected Vector addresses;
    protected Vector terminals;
    protected Vector providerObservers;
    protected ProviderEventNotifier provEvtNotifier;
    protected int state = UNDEFINED;
    protected VCStaticProviderCapabilities staticProviderCapabilities;
    protected VCDynamicProviderCapabilities dynamicProviderCapabilities;
    protected VCStaticCallCapabilities callCapabilities;
    protected VCStaticAddressCapabilities addressCapabilities;
    protected VCStaticTerminalCapabilities terminalCapabilities;
    protected VCStaticConnectionCapabilities connectionCapabilities;
    protected VCStaticTerminalConnectionCapabilities terminalConnectionCapabilities;
    protected Hashtable pendingRequests = null;
    protected Hashtable callList = null;
    protected Hashtable addressList = null;

    protected VCProvider(String name)
    {
	this.name = name;
	calls = new Vector();
	terminals = new Vector();
	addresses = new Vector();
	providerObservers = new Vector();
	provEvtNotifier = new ProviderEventNotifier();
	staticProviderCapabilities = new VCStaticProviderCapabilities();
	callCapabilities = new VCStaticCallCapabilities();
	addressCapabilities = new VCStaticAddressCapabilities();
	terminalCapabilities = new VCStaticTerminalCapabilities();
	connectionCapabilities = new VCStaticConnectionCapabilities();
	terminalConnectionCapabilities = new VCStaticTerminalConnectionCapabilities();
	pendingRequests = new Hashtable(20);
        callList = new Hashtable(20);
	addressList = new Hashtable(20);
        VCRequestUtil.setProvider(this);
    }

    private VCProvider() { }


/***************************************************************/
/* JTapi Implementation                                        */
/***************************************************************/

    // return state of the provider instance
    /**
     * Refer To JTapi Provider Interface
     */
    public int getState()
    {
	return state;
    }

    // return the name of the provider
    /**
     * Refer To JTapi Provider Interface
     */
    public String getName()
    {
	return name;
    }


    // return the calls in the provider domain 
    // as soon as a call becomes INVALID then it is removed
    // from the calls Vector - and is no longer included
    // as a call in the provider domain
    /**
     * Refer To JTapi Provider Interface
     */
    public Call[] getCalls() throws ResourceUnavailableException
    {
	if (inService())
	{
	    Call[] callsArray = new Call[calls.size()];
	    calls.copyInto(callsArray);
	    return callsArray;
	}
	throw new ResourceUnavailableException(ResourceUnavailableException.ORIGINATOR_UNAVAILABLE);
    }


    // return an address object from the provider domain 
    // with an address name of the given string
    /**
     * Refer To JTapi Provider Interface
     */
    public Address getAddress(String str) throws InvalidArgumentException
    {
        Address addr = null;
	if (VCProvider.isLocalAddress(str))
	{
	    addr = VCProvider.getLocalAddress(str);
	}
	if (addr != null)
	{
	    addAddressToList(addr, str);
	    return addr;
	}
	else
	{
            throw new InvalidArgumentException();
	}
    }


    // return all addresses in the provider domain 
    /**
     * Refer To JTapi Provider Interface
     */
    public Address[] getAddresses() throws ResourceUnavailableException
    {
	if (inService())
	{
	    Vector localAddresses = new Vector();
	    for(int i=0; i < addresses.size(); i++)
	    {
		VCAddress addr = (VCAddress)addresses.elementAt(i);
		if(addr.isLocal())
		{
		    localAddresses.addElement(addr);
		}
            }
	    Address[] addressArray = new Address[localAddresses.size()];
	    localAddresses.copyInto(addressArray);
	    return addressArray;
	}
	throw new ResourceUnavailableException(ResourceUnavailableException.ORIGINATOR_UNAVAILABLE);
    }


    // return all terminals in the provider domain
    /**
     * Refer To JTapi Provider Interface
     */
    public Terminal[] getTerminals() throws ResourceUnavailableException
    {
	if (inService())
	{
            Vector localTerminals = new Vector();
            for (int i=0; i < terminals.size(); i++)
            {
                VCTerminal term = (VCTerminal)terminals.elementAt(i);
                if (term.isLocal())
                    localTerminals.addElement(term);
            }
	    Terminal[] terminalsArray = new Terminal[localTerminals.size()];
	    localTerminals.copyInto(terminalsArray);
	    return terminalsArray;
	}
        throw new ResourceUnavailableException(ResourceUnavailableException.ORIGINATOR_UNAVAILABLE);
    }


    // return the terminal object from the provider domain
    // matching the given terminal name
    /**
     * Refer To JTapi Provider Interface
     */
    public Terminal getTerminal(String str) throws InvalidArgumentException
    {
        Terminal term = null;
	if(VCProvider.isLocalTerminal(str))
	{
	    term = VCProvider.getLocalTerminal(str);
	}
        else
        {
            term = new VCCallControlTerminal(defaultProvider, false, str);
        }
        if (term != null)
        {
            addTerminalToList(term);
            return term;
        }
        else
        {
	    throw new InvalidArgumentException();
        }
    }


    // shutdown the provider 
    // set state to SHUTDOWN
    /**
     * Refer To JTapi Provider Interface
     */
    public void shutdown()
    {
	try
	{
	    setState(SHUTDOWN);
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}
    }


    // create a call - a CallControl call is created as this
    // provider supports the CallControl package
    /**
     * Refer To JTapi Provider Interface
     */
    public Call createCall()
	throws ResourceUnavailableException, InvalidStateException,
	PrivilegeViolationException, MethodNotSupportedException
    {
	if (inService())
	{
	    Call newCall = new VCCallControlCall(this);
            String callId = ((VCCall)newCall).getCallId();
            addCallToList(newCall, callId);
	    return newCall;
	}
	throw new ResourceUnavailableException(ResourceUnavailableException.ORIGINATOR_UNAVAILABLE);
    }


    // add an observer to the provider
    // the observer should be informed of state changes in the provider
    /**
     * Refer To JTapi Provider Interface
     */
    public void addObserver(ProviderObserver observer)
	throws ResourceUnavailableException, MethodNotSupportedException
    {
	providerObservers.addElement(observer);
    }


    // return all the observers of this provider
    /**
     * Refer To JTapi Provider Interface
     */
    public ProviderObserver[] getObservers()
    {
	ProviderObserver[] providerObserversArray = new
			ProviderObserver[providerObservers.size()];
	providerObservers.copyInto(providerObserversArray);
	return providerObserversArray;
    }


    // remove the given observer from the provider
    /**
     *  Refer To JTapi Provider Interface
     */
    public void removeObserver(ProviderObserver observer)
    {
	providerObservers.removeElement(observer);
    }


    // return the dynamic capabilities of the provider
    /**
     * Refer To JTapi Provider Interface
     */
    public ProviderCapabilities getCapabilities()
    {
	return dynamicProviderCapabilities;
    }

    // return the static address capabilities
    /**
     * Refer To JTapi Provider Interface
     */
    public AddressCapabilities getAddressCapabilities()
    {
	return addressCapabilities;
    }

    // return the static connection capabilities
    /**
     * Refer To JTapi Provider Interface
     */
    public ConnectionCapabilities getConnectionCapabilities()
    {
 	return connectionCapabilities;
    }

    // return the static terminalconnection capabilities
    /**
     * Refer To JTapi Provider Interface
     */
    public TerminalConnectionCapabilities getTerminalConnectionCapabilities()
    {
	return terminalConnectionCapabilities;
    }

    // return the static call capabilities
    /**
     * Refer To JTapi Provider Interface
     */
    public CallCapabilities getCallCapabilities()
    {
	return callCapabilities;
    }

    // return the static provider capabilities
    /**
     * Refer To JTapi Provider Interface
     */
    public ProviderCapabilities getProviderCapabilities()
    {
	return staticProviderCapabilities;
    }

    // return the static terminal capabilities
    /**
     * Refer To JTapi Provider Interface
     */
    public TerminalCapabilities getTerminalCapabilities()
    {
	return terminalCapabilities;
    }

// The following set of functions are now depracated

    /**
     * @deprecated
     */
    public CallCapabilities getCallCapabilities(Terminal terminal,
	Address address) throws InvalidArgumentException, PlatformException
    {
	return null;
    }

    /**
     * @deprecated
     */
    public ConnectionCapabilities getConnectionCapabilities(Terminal terminal,
	Address address) throws InvalidArgumentException, PlatformException
    {
	return null;
    }

    /**
     * @deprecated
     */
    public ProviderCapabilities getProviderCapabilities(Terminal term)
        throws InvalidArgumentException, PlatformException
    {
        return null;
    }

    /**
     * @deprecated
     */
    public AddressCapabilities getAddressCapabilities(Terminal term)
        throws InvalidArgumentException, PlatformException
    {
	return null;
    }

    /**
     * @deprecated
     */
    public TerminalConnectionCapabilities
	getTerminalConnectionCapabilities(Terminal terminal)
	throws InvalidArgumentException, PlatformException
    {
	return null;
    }

    /**
     * @deprecated
     */
    public TerminalCapabilities getTerminalCapabilities(Terminal terminal)
	throws InvalidArgumentException, PlatformException
    {
	return null;
    }

/*************************************************************/
/* Additional Functionality                                  */
/*************************************************************/

    /**
     * set the State of the provider, instantiating relevant events
     * and informing the observers 
     * checks are made to ensure valid state transition
     */
    public void setState(int newState) throws InvalidStateException
    {
	if (state == newState)
	    return;
	if (newState == IN_SERVICE)
	{
	    if (state != SHUTDOWN)
	    {
	        provEvtNotifier.notifyObserversOfEvent(new VCProvInServiceEv(this));    
	    }
	    else
	    {
		throw (new InvalidStateException(this,
                        InvalidStateException.PROVIDER_OBJECT, state));
	    }
	}
	else if (newState == OUT_OF_SERVICE)
	{
	    if (state == IN_SERVICE)
	    {
		provEvtNotifier.notifyObserversOfEvent(new VCProvOutOfServiceEv(this));
	    }
	    else
	    {
		throw (new InvalidStateException(this,
			InvalidStateException.PROVIDER_OBJECT, state));
	    }
	}
	else if (newState == SHUTDOWN)
	{
	    if (state == OUT_OF_SERVICE)
	    {
		provEvtNotifier.notifyObserversOfEvent(new VCProvShutdownEv(this));
	    }
	    else
	    {
		throw (new InvalidStateException(this,
                        InvalidStateException.PROVIDER_OBJECT, state));
	    }
	}

	this.state = newState;
    }


    /**
     * start the provider by setting the state in Service and
     * and starting the auxilliary threads
     */
    public void startup()
    {
	try
	{
	    setState(IN_SERVICE);
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}    
  	startThreads();
    }

    /**
     * Stop the provider by setting the state Out Of Service and
     * stop the auxilliary threads
     */
   
    public void setOOS()
    {
	try
        {
	    setState(OUT_OF_SERVICE);
	}
	catch (InvalidStateException e)
	{
            e.printStackTrace();
        }
	stopThreads();
    }


    /**
     * return indication of whether the provider is in service
     */
    public boolean inService()
    {
	if (state == IN_SERVICE)
	    return true;
	else
	    return false;
    }

 
    /**
     * return indication if address name is local to provider domain
     */
    public boolean isAddressLocal(String str)
    {
	Address addr = null;
	try
        {
            addr = getAddress(str);
	    if (((VCAddress)addr).isLocal() == true)
	    {
		return true;
	    }
		
        }
	catch (InvalidArgumentException e)
	{
	    return false;
	}
	return false;
    }
	
    /**
     * return an address for the passed string
     * first search for the address in the provider domain
     * if the address is not local to the provider domain 
     * then search the list of active addresses which contains
     * the active (local and remote) addresses 
     * if the address does not exist in this list then the address must
     * be a new remote address and so a new CallControl Address is
     * created
     */

    public Address getTheAddress(String str) throws InvalidArgumentException
    {
	Address addr = null;
	Object addrObj = null;
	try
	{
	    addr = getAddress(str);
	}
	catch (InvalidArgumentException e)
	{
	    addrObj = addressList.get(str);
	    if (addrObj != null)
	    {
	        addr = (Address)addrObj;
	    }
	    if (addr == null)
	    {
	        addr = new VCCallControlAddress(this, false, str);
	        if (addr != null)
	        {
		    addAddressToList(addr, str);
	        }
	    }
	}
	if (addr != null)
	{
	    return addr;
	}
	else
	{
	    throw new InvalidArgumentException();
	}
    }


    /**
     * add an address to the current list of active addresses
     */
    public void addAddressToList(Address addr, String str)
    {
	if (addresses.contains(addr) == false)
	{
	    addresses.addElement(addr);
	    addressList.put(str, addr);
	}
    }


    /**
     * remove an address from the list of current addresses
     */
    public void removeAddressFromList(Address addr)
    {
	addresses.removeElement(addr);
	if (addressList.contains(addr))
	{
	    String key = addr.getName();
	    addressList.remove(key);
	}
    }

    /**
     * add terminal to list of current terminals
     */
    public void addTerminalToList(Terminal term)
    {
	terminals.addElement(term);
    }

    /**
     * remove terminal from list of current terminals
     */
    public void removeTerminalFromList(Terminal term)
    {
	terminals.removeElement(term);
    }

    /**
     * add a call to the list of current calls
     * duplicated list to a vector for easier manipulation
     */
    public void addCallToList(Call call, String callId)
    {
        calls.addElement(call);
        callList.put(callId, call);
    }

    /**
     * remove the call from list of current calls
     * remove from both list and vector
     */
    public void removeCallFromList(Call call)
    {
        if (calls.contains(call))
        {
            calls.removeElement(call);
        }
        if (callList.contains(call))
        {
	    String key = ((VCCall)call).getCallId();
            callList.remove(key);
        }
    }


    /**
     * find a call in the list given the callid of the call
     */
    public Call findCall(String key)
    {
	Call call = null;
	Object callObj = callList.get(key);
	if (callObj != null)
	{
	    call = (Call)callObj;
	}
	return call;
    }

    /**
     * Add a request to the request queue, with the Message ID as key
     */
    public void addRequest(String key, VCRequest req)
    {
	System.out.println("Number of Pending Requests " + pendingRequests.size());
	pendingRequests.put(key, req);
    }

    /**
     * Find a request in the queue given the key, which is the Message ID
     */
    public VCRequest findRequest(String key)
    {
	VCRequest req;
	Object obj = pendingRequests.get(key);
	if (obj != null)
	{
	    req = (VCRequest)obj;
	}
	else
	{
	    req = null;
	}
	return req;
    }

    /**
     * find a request in the list of requests based on the Msg Id
     * remove the request from the list before return the reference
     */
    public VCRequest getRequest(String key)
    {
	VCRequest req;
	req = findRequest(key);
	if (req != null)
	{
	    pendingRequests.remove(key);
	}
	return req;
    }

    /**
     * Find requests which have a timer set to an earlier time than
     * the current time 
     * The key is the ID of the message
     * If the request is deemed to be expired then it is removed
     * from the list and an Expirytimerexception is thrown
     */

    public void checkForExpiredRequests() throws ExpiryTimerException
    {
	if (pendingRequests.isEmpty())
	{
	    return;
	}
	Date now = new Date();
	long timeInMilliSecs = now.getTime();
	for (Enumeration e = pendingRequests.keys(); e.hasMoreElements(); )
	{
	    String key = (String)e.nextElement();
	    VCRequest req = (VCRequest)pendingRequests.get(key);
	    if (timeInMilliSecs > req.getExpiryTime())
	    {
		pendingRequests.remove(key);
		throw (new ExpiryTimerException(req));
	    }
	}
    }

}

