/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony;

import javax.telephony.*;
import com.vovida.telephony.callcontrol.*;
import javax.telephony.callcontrol.*;

// Class contains static functions to be called
// by JTapi when a message needs to be sent to
// the server. The functions create a VCRequest object
// and store all the relevant data for the message to
// the request object.

/**
 * Class contains static functions to be called
 * by JTapi when a message needs to be sent to
 * the server The functions create a VCRequest object
 * and store all the relevant data for the message to
 * the request object
 */

public class VCRequestUtil implements VCMsgTypes
{

    static private Provider provider = null;

// Send a connect request to the server.
// Data: callId, address name for connection 1
//       address name for connection 2

    /**
     * Send a connect request to the server
     */
    static boolean callConnect(String orig, String dest, Call call)
    {
	    if (providerOK() == false)
	    {
	        return false;
	    }
	    VCRequest req = new VCRequest(provider);
	    System.out.println("Call Connect request");
        req.setReqCall(call);
        String[] addrs = new String[2];
        addrs[0] = orig;
        addrs[1] = dest;
        req.setReqAddrs(addrs);
	    return (req.sendRequest(CONNECT, false));
    }

// Send a disconnect request to the server.
// Data: callId, address name for disconnecting connection

    /**
     * Send a disconnect request to the server
     */
    static boolean callDisconnect(Connection conn)
    {
        if (providerOK() == false)
        {
            return false;
        }
        VCRequest req = new VCRequest(provider);
        System.out.println("Call DisConnect request");
        req.setReqCall(conn.getCall());
	    String[] addrs = new String[1];
	    addrs[0] = conn.getAddress().getName();
	    req.setReqAddrs(addrs);
	    return (req.sendRequest(DISCONNECT, true));
    }

// Send an answer request to the server.
// Data: callId, address name for answering connection,
//       terminal name for answering terminal

    /**
     * Send an answer request to the server
     */
    static boolean answer(String terminal, Connection conn)
    {
        if (providerOK() == false)
	    {
	        return false;
	    }
        VCRequest req = new VCRequest(provider);
	    System.out.println("Call Answer request");
        req.setReqCall(conn.getCall());
        String[] addrs = new String[1];
        addrs[0] = conn.getAddress().getName();
        req.setReqAddrs(addrs);
        req.setReqTerm(terminal);
        return(req.sendRequest(ANSWER, true));
    }

// Send a transfer request to the server.
// Data: callId, address name for transferring connection
//       address name for destination

    /**
     * Send a transfer request to the server
     */
    public static boolean transfer(Call call, 
				    TerminalConnection tc, String dest)
    {
	    if (providerOK() == false)
        {
            return false;
        }
	    VCRequest req = new VCRequest(provider);
        System.out.println("Call Transfer request");
	    req.setReqCall(call);
	    String[] addrs = new String[2];
	    addrs[0] = tc.getConnection().getAddress().getName();
	    addrs[1] = dest;
	    req.setReqAddrs(addrs);
	    return (req.sendRequest(TRANSFER, true));
    }

// Send a callcreate response to the server.
// Data: callId, msgId

    /**
     * Send a callcreate response to the server
     */
    static boolean callCreateResp(Call call, String id, String src, String dest)
    {
	    if (providerOK() == false)
        {
            return false;
        }
	    VCRequest req = new VCRequest(provider);
	    System.out.println("Call Create response");
	    req.setReqCall(call);
	    req.setReqId(id);
        String[] addrs = new String[2];
        addrs[0] = src;
        addrs[1] = dest;
        req.setReqAddrs(addrs);
	    req.sendResponse(CALLCREATERESP);
	    return true;
    }

// Send a register request to the server.
// Data: address name for proposed caller, 
// username and password for user login

    /**
     * Send a register request to the server
     */
    static public boolean register(String name, String password, String caller)
    {
	    if (providerOK() == false)
        {
            return false;
        }
	    VCRequest req = new VCRequest(provider);
	    System.out.println("Register request");
	    String[] addrs = new String[1];
	    addrs[0] = caller;
	    req.setReqAddrs(addrs);
	    req.setReqUser(name);
	    req.setReqPassWord(password);
	    req.setReqPort(VCProvider.getInPort());
	    return (req.sendRequest(REGISTER, true));
    }
	
    /**
     * Send a chkRegister request to the server
     */
    static public boolean chkRegister()
    {
	    if (providerOK() == false)
        {
            return false;
        }
	    VCRequest req = new VCRequest(provider);
	    System.out.println("Chkregister request");
	    return (req.sendRequest(CHKREGISTER, true));
    }
	

    /**
     * Set the reference to the Provider object
     */
    static public void setProvider(Provider prov)
    {
	    provider = prov;
    }

    /**
     * Check that the Provider is operational
     */
    private static boolean providerOK()
    {
        if (provider != null)
        {
            if (provider.getState() == provider.IN_SERVICE)
            {
                return true;
            }
        }
        return false;
    }

}
