/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony.callcontrol;

import javax.telephony.*;
import javax.telephony.callcontrol.*;
import com.vovida.telephony.*;
import com.vovida.telephony.callcontrol.events.*;


/*************************************************************/
/* VCCallControlConnection is the Vocal Client Call Control */
/* Connection Class which  is an implementation              */
/* of the Call Control Connection interface from the         */
/* JTapi Spec.                                               */
/*************************************************************/

/**
 * Implementation of the Jtapi CallControlConnection Interface
 *
 */
public class VCCallControlConnection extends VCConnection implements CallControlConnection
{
    int callControlState;
    boolean isCaller;


    /**
     * Constructor
     */
    public VCCallControlConnection(Address addr, Call call, boolean isCaller)
    {
	super(addr, call);
	try
	{
	    setCallControlState(CallControlConnection.IDLE);
            this.isCaller = isCaller;
	}
	catch (InvalidStateException e)
	{
	    e.printStackTrace();
	}
    }


/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/

    // return the CallControl state of this connection
    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public int getCallControlState()
    {
	return callControlState;
    }


    // accept the call which has been offered on this connection
    
    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public void accept() throws InvalidStateException,
	MethodNotSupportedException, PrivilegeViolationException,
	ResourceUnavailableException
    {
	isProviderOK();
	if (this.getCallControlState() != CallControlConnection.OFFERED)
	{
            InvalidStateException e = new InvalidStateException(this,
                    InvalidStateException.CONNECTION_OBJECT,
                    this.getCallControlState());
            throw e;
	}
        //send request ctrlconn accept to server
        setCallControlState(CallControlConnection.ALERTING);
    }



    // reject the call offered on this connection

    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public void reject() throws InvalidStateException,
	MethodNotSupportedException, PrivilegeViolationException,
	ResourceUnavailableException
    {
	isProviderOK();
        if (this.getCallControlState() != CallControlConnection.OFFERED)
	{
            InvalidStateException e = new InvalidStateException(this,
                InvalidStateException.CONNECTION_OBJECT,
                this.getCallControlState());
            throw e;
        }
	// Need to request ctrlconn reject
	setCallControlState(CallControlConnection.DISCONNECTED);
    }



    // redirect a call which is offered or alerting on this connection
    // to another destination - similar to transfer but the call needs
    // not to be answered - this connection is disconnected and the new
    // connection returned from this fn, however if the new connection
    // represents an address outside the domain then null is returned.

    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public Connection redirect(String destinationAddress)
	throws InvalidStateException, InvalidPartyException,
	    MethodNotSupportedException, PrivilegeViolationException,
	    ResourceUnavailableException
    {
	try
	{
	    VCCallControlTerminalConnection termConn;
	    isProviderOK();
	    if ((this.getCallControlState() != CallControlConnection.OFFERED)
    	        && (this.getCallControlState() != CallControlConnection.ALERTING))
	    {
	        throw (new InvalidStateException(this,
				InvalidStateException.CONNECTION_OBJECT,
				this.getCallControlState()));
	    }
	    // perform redirect operation
	    setCallControlState(CallControlConnection.DISCONNECTED);
	    TerminalConnection tc[] = this.getTerminalConnections();
	    for (int i=0; i < tc.length; i++)
	    {
	        termConn = (VCCallControlTerminalConnection)tc[i];
	        termConn.setCallControlState(CallControlTerminalConnection.DROPPED);
	    }
	    VCProvider prov = (VCProvider)getCall().getProvider();
	    if (prov.isAddressLocal(destinationAddress))
	    {
	        Address addr = prov.getAddress(destinationAddress);
	        Connection conn = new VCCallControlConnection(addr, getCall(), true);
	        return conn;
	    }
	    return null;
	}
	catch (InvalidArgumentException e)
	{
	    return null;
	}
    }


    // add a string to the currently dialed string in order for the
    // call to be allowed to be completed - the inference is that the 
    // previous dialling has been incomplete

    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public void addToAddress(String additionalAddress)
	throws InvalidStateException, MethodNotSupportedException,
	    PrivilegeViolationException, ResourceUnavailableException
    {
	isProviderOK();
	if ((this.getCallControlState() != CallControlConnection.DIALING) &&
	    (this.getCallControlState() != CallControlConnection.INITIATED))
	{
	    throw (new InvalidStateException(this,
				InvalidStateException.CONNECTION_OBJECT,
				this.getCallControlState()));
	}
	// send the request to the server
	// check the result of request to determine next state
	int result = 0;

	if (result == VCMsgTypes.ADDRESSING_COMPLETE)
	{
	    setCallControlState(CallControlConnection.ESTABLISHED);
	}
	else if (result == VCMsgTypes.ADDRESSING_INCOMPLETE)
	{
	    setCallControlState(CallControlConnection.DIALING);
	}
    }


    // park a call at another address
    // the other address must be in the domain
    // a new connection is created for the call to represent the 
    // destination address
    // this conection is set to DISCONNECTED state

    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public Connection park(String destinationAddress)
	throws InvalidStateException, MethodNotSupportedException,
	    PrivilegeViolationException, InvalidPartyException,
	    ResourceUnavailableException
    {
	try
	{
	    isProviderOK();
	    if (getCallControlState() != CallControlConnection.ESTABLISHED)
	    {
	        throw (new InvalidStateException(this,
				InvalidStateException.CONNECTION_OBJECT,
				getCallControlState()));
	    }
	    VCProvider prov = (VCProvider)getCall().getProvider();
	    if (prov.isAddressLocal(destinationAddress) != true)
	    {
	        throw (new InvalidPartyException(InvalidPartyException.DESTINATION_PARTY));
	    }
	    // send the request to the server
	    Address addr = prov.getAddress(destinationAddress);
	    VCCallControlConnection conn = new VCCallControlConnection(addr, getCall(), true);
	    conn.setCallControlState(CallControlConnection.QUEUED);
	    TerminalConnection[] tcs = getTerminalConnections();
	    if (tcs != null)
	    {
	        for (int i=0; i < tcs.length; i++)
	        {
		    ((VCCallControlTerminalConnection)tcs[i]).setCallControlState
				        (CallControlTerminalConnection.DROPPED);
	        }
	    }
	    setCallControlState(CallControlConnection.DISCONNECTED);
	    return conn;
	}
	catch (InvalidArgumentException e)
	{
	    throw (new InvalidPartyException(InvalidPartyException.DESTINATION_PARTY));
	}
    }


    // disconnect the connection - this fn is overridden so that
    // the CallControl state can be set when this fn is called for
    // a CallControlConnection object

    /**
     * Refer To JTapi CallControlConnection Interface
     */
    public void disconnect() throws PrivilegeViolationException,
        ResourceUnavailableException, MethodNotSupportedException,
        InvalidStateException
    {
	super.disconnect();
	if (getState() == Connection.FAILED)
	    setCallControlState(CallControlConnection.FAILED);
	if (getState() == Connection.DISCONNECTED)
	{
	    setCallControlState(CallControlConnection.DISCONNECTED);
	    TerminalConnection[] tcSet = getTerminalConnections();
	    VCCallControlTerminalConnection tc;
	    if (tcSet != null)
	    {
	        for (int i=0; i < tcSet.length; i++)
	        {
		    tc = (VCCallControlTerminalConnection)tcSet[i];
		    tc.setCallControlState(CallControlTerminalConnection.DROPPED);
	        }
	    }
	}
    }
			

/**************************************************************/
/* Additional Functionality                                   */
/**************************************************************/


    // set the CallControl state of the CallControlConnection.
    // if the state has already reached the newState then
    // then there is no need to set it or send an event so return.
    // the previous state is checked to ensure valid state transition.
    // NOTE: the corresponding Connection state is set.

    /**
     * set the CallControl state of the CallControlConnection
     * If the state has already reached the newState then
     * there is no need to set it or send an event so return
     * The previous state is checked to ensure valid state transition
     */
    public void setCallControlState(int newState) throws InvalidStateException
    {
	System.out.println("CallControl Connection Set State");
	if (newState != CallControlConnection.IDLE)
	{
	    if (callControlState == newState)
	    {
		return;
	    }
	}
	int state = callControlState;
        // INCLUDE SOME EVT HANDLING
        switch (newState)
        {
            case CallControlConnection.IDLE:
                {
                }
                break;
            case CallControlConnection.OFFERED:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.UNKNOWN)) 
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnOfferedEv(this));
			setState(Connection.INPROGRESS);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
			        InvalidStateException.CONNECTION_OBJECT, state));
		    }
                }
                break;
            case CallControlConnection.QUEUED:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.OFFERED) ||
			(state == CallControlConnection.UNKNOWN))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnQueuedEv(this));
			setState(Connection.INPROGRESS);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
		    }
                }
                break;
            case CallControlConnection.ALERTING:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.OFFERED) || 
			(state == CallControlConnection.QUEUED) || 
			(state == CallControlConnection.UNKNOWN))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnAlertingEv(this));
			setState(Connection.ALERTING);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.INITIATED:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.UNKNOWN))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnInitiatedEv(this));
			setState(Connection.CONNECTED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.DIALING:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.INITIATED) ||
			(state == CallControlConnection.UNKNOWN))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnDialingEv(this));
			setState(Connection.CONNECTED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.NETWORK_REACHED:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.UNKNOWN))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnNetworkReachedEv(this));
			setState(Connection.CONNECTED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.NETWORK_ALERTING:
                {
		    if ((state == CallControlConnection.IDLE) || 
			(state == CallControlConnection.UNKNOWN) ||
			(state == CallControlConnection.NETWORK_REACHED))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnNetworkAlertingEv(this));
			setState(Connection.CONNECTED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.ESTABLISHED:
                {
		    if ((state != CallControlConnection.FAILED) && 
			(state != CallControlConnection.DISCONNECTED))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnEstablishedEv(this));
			setState(Connection.CONNECTED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.DISCONNECTED:
                {
		    if (state != CallControlConnection.IDLE)
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnDisconnectedEv(this));
			setState(Connection.DISCONNECTED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.FAILED:
                {
		    if (state != CallControlConnection.DISCONNECTED)
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnFailedEv(this));
			setState(Connection.FAILED);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
                break;
            case CallControlConnection.UNKNOWN:
                {
		    if ((state != CallControlConnection.DISCONNECTED) &&
			(state != CallControlConnection.FAILED))
		    {
                        callEvtNotifier.notifyObserversOfEvent(
                                        new VCCallCtlConnUnknownEv(this));
			setState(Connection.UNKNOWN);
		    }
		    else
		    {
			throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
                    }
                }
	    default :
		{
		    throw (new InvalidStateException(this,
                                InvalidStateException.CONNECTION_OBJECT, state));
		}
        }
	callControlState = newState;

    }


}
