/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.vocalpad;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.text.*;
import javax.swing.event.*;
import java.awt.event.*;
import javax.swing.border.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;
import javax.telephony.privatedata.*;
import com.vovida.telephony.*;
import com.vovida.telephony.callcontrol.*;
import com.vovida.telephony.privatedata.*;

/**
 * Japp can be invoked form the command line or as an Applet
 * It is a sample application which utilizes the Jtapi
 * implementation to communicate with the Vocal Jtapi server
 */

public class Japp extends JApplet implements Runnable, PadValues
{
    JRootPane rootP;
    Container contents;
    Dimension dim;

    private Thread jThread = null;
    private boolean keepRunning = true;
    private boolean callInProgress = false;
    private static boolean isStandAlone = false;
    private final static String DEFAULT_HOST = "ada";
    private final static String DEFAULT_PORT = "9998";
    private final static String DEFAULT_INPORT = "9999";

    private JPanel phonePanel, regPanel, regButtonPanel, mainPanel, padPanel, buttonPanel, dispPanel, regHeadPanel, phoneHeadPanel;
    private JButton dialButton, hangUpButton, exitButton, saveCallerButton;
    private JButton[] digitButtons = new JButton[12];
    private String[] digitValues = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "*", "0", "#"};
    private JButton chgHomeButton, retCallButton, answerButton, transferButton;
    private JButton regButton, regCancelButton, regExitButton;
    private JTextField destField, callerField;
    private JTextField status1_1, status1_2, status1_3, status1_4;
    public JTextField status2_1, status2_2;
    private JTextField status3_1;
    private JTextArea statusArea;
    private JTextField userId;
    private JPasswordField passWord;
    private JLabel homeLabel, dispCaller;
    private final String DIGITS = "0123456789";
 
    private PrivateData pd;
    private JtapiPeer jtapiPeer;
    private Provider provider;
    private VCProvider vcprovider;
    private ProviderObserver provObserver;
    private CallObserver callObserver;
    private Address origAddress;
    private Address homeAddress;
    private Terminal origTerminal;
    private TerminalConnection homeTerminalConnection;
    private Call call;
    private Connection[] connectionsArray;
    private String dest, orig, destStr, callerStr, userStr, passStr;
    private Vector allCallers;
    private int state = IDLE;
    private Japp vocalPadApp;
    appFrame vocalPad;

    /**
     * Constructor
     */
    public Japp()
    {
        vocalPadApp = this;
    }

    /**
     * Can be called as an application with server host, server port
     * and client port as parameters This will then instantiate the
     * applet class and appears as it were running as an applet
     */
    public static void main(String args[])
    {
        isStandAlone = true;
        if (args.length > 0)
        {
            if (args.length == 3)
            {
                String host = args[0];
                String outport = args[1];
                String inport = args[2];
                VCProvider.setHostandPorts(host, outport, inport);
            }
            else
            {
                System.out.println("Command requires 3 parameters - svr host - svr port - client port");
                return;
            }
        }
        Japp app = new Japp();
        app.init();
        app.start();
        Dimension d = new Dimension(400, 400);
        JFrame f = new JFrame("VocalPad Application");
        f.setSize(d);
        f.getContentPane().add(app);
        f.pack();
        f.setVisible(true);
    }

    /**
     * Inits the applet class reading parameters from html file
     * if necessary - gets the provider and initiates the observers
     */
    public void init()
    {
        if (isStandAlone == false)
        {
            String host, port, inport;
            host = "";
            port = "";
            inport = "";
            try
            {
                host = this.getParameter("host", DEFAULT_HOST);
            }
            catch(Exception e) 
            {
                String str = "Faulty host definition in HTML page";
                popUpError(str);
            }
            try
            {
                port = this.getParameter("port", DEFAULT_PORT);
            }
            catch(Exception e) 
            {
                String str = "Faulty port definition in HTML page";
                popUpError(str);
            }
            try
            {
                inport = this.getParameter("inport", DEFAULT_INPORT);
            }
            catch(Exception e) 
            {
                String str = "Faulty In Port definition in HTML page";
                popUpError(str);
            }
                System.out.println("Setting" + host + " +" + port + "+" + inport);
                VCProvider.setHostandPorts(host, port, inport);
        }
        try
        {
            jtapiPeer = JtapiPeerFactory.getJtapiPeer("com.vovida.telephony.VCJtapiPeer");
            provider = jtapiPeer.getProvider(null);
            vcprovider = (VCProvider)provider;
            provObserver = new VCProviderObserver();
            provider.addObserver(provObserver);
            vcprovider.setState(Provider.IN_SERVICE);
        }
        catch (JtapiPeerUnavailableException e)
        {
            String str = "Peer access not available";
            popUpError(str);
        }
        catch (Exception e)
        {
            String str = "Provider access not permitted by Peer";
            popUpError(str);
        }
        pd = new VCPrivateData();
        pd.setPrivateData(vcprovider);
        callObserver = new VCCallControlCallObserver();
        vocalPad = new appFrame();
        getContentPane().add(vocalPad);
        ((VCCallObserver)callObserver).setApp(vocalPadApp);
        allCallers = new Vector();
        state = IDLE;
    }

    /**
     * Call Provider to start Jtapi threads
     * If running as applet need to check if user is registered
     * already - if so go straight to VocalPad display
     */
    public void start()
    {
        vcprovider.startup();
        if (isStandAlone == false)
        {
            Vector v = new Vector();
            Integer n = new Integer(CHKREGISTER);
            v.addElement(n);
            Address addr = (Address)pd.sendPrivateData(v);
            if (addr == null)
            {
                // need to register as new user
                vocalPad.reRegister();
            }
            else
            {
                // no need to register straight to phone
                // Server should supply current status
                String caller = addr.getName();
                vocalPad.dispPhone(caller);

            }
        }
    
        jThread = new Thread(vocalPadApp);
        jThread.start();
        validate();
    }

    /**
     * Exit from the VocalPad
     */
    public void stop()
    {
        if (isStandAlone == true)
        {
            System.exit(0);
            return;
        }
        vcprovider.setOOS();
        jThread.stop();
        keepRunning = false;
    }

    /**
     * Shutdown the Jtapi 
     */
    public void destroy()
    {
        System.out.println("Destroy Called");
        provider.shutdown();
    }

    /**
     * Provides timing for the VocalPad
     */
    public void run()
    {
        Thread t = Thread.currentThread();
        while (keepRunning)
        {
            DateFormat shortTime = DateFormat.getTimeInstance(DateFormat.SHORT);
            status1_4.setText((shortTime.format(new Date())).toString());
            try
            {
                t.sleep(5000);
            }
            catch (InterruptedException e)
            {
                String str = "VocalPad Timer not executing successfully";
                popUpError(str);
            }
        }
    }

    /**
     * Get parameter from HTML page
     */
    public String getParameter(String key, String def)
    {
      if (getParameter(key) != null)
      {
        return getParameter(key);
      }
      else
      {
        System.out.println("Failed to get parameter with key: "
        + key);
        return def;
      }
    }

    /**
     * Pops up an Error screen
     */
    private void popUpError(String str)
    {
        JOptionPane.showMessageDialog(this, str, "Info Window", JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Set the state of the VocalPad app
     */
    private void setState(int newState)
    {
        state = newState;
    }

    /**
     * Get the state of the VocalPad App
     */
    public int getState()
    {
        return state;
    }

    /**
     * Get the Address of the Originator
     */
    public Address getOrigAddress()
    {
        return origAddress;
    }

    /**
     * Get the Terminal of the Originator
     */
    public Terminal getOrigTerminal()
    {
        return origTerminal;
    }

    /**
     * Set the TerminalConnection for the Terminal represented by
     * this VocalPad
     */
    public void setHomeTC(TerminalConnection tc)
    {
        homeTerminalConnection = tc;
    }

    /**
     * Get the TerminalConnection for the Terminal represented by
     * this VocalPad
     */
    public TerminalConnection getHomeTC()
    {
        return homeTerminalConnection;
    }

    /**
     * AppFrame is the Panel in which the VocalPad screens are located
     */
    class appFrame extends JPanel implements ActionListener
    {
        Color buttons, fields, msgColor;
        Border raisedBevel, loweredBevel, empty;
        Font buttonFont;

    /**
     * Constructor creates all of the screens which are used by the VocalPad
     */
    public appFrame()
    {
//    super("VocalPad Application");
        super();
//    buttons = new Color(220, 170, 170);
//    fields = new Color(170, 220, 170);
//            buttons = new Color(195, 152, 146);
//            fields = new Color(201, 213, 140);
        buttons = new Color(130, 27, 63);
        fields = new Color(251, 236, 163);
        buttonFont = new Font("buttonsfont", Font.ITALIC, 10);
        raisedBevel = BorderFactory.createRaisedBevelBorder();
        loweredBevel = BorderFactory.createLoweredBevelBorder();
        empty = BorderFactory.createEmptyBorder();
        dim = new Dimension(500, 350);
        setSize(dim);



        createRegPanel();

    
        createRegButtonPanel();

        createButtonPanel();

//    createHeadPanel();
        createDispPanel();

        createPadPanel();


        setActionButtons();

        createPhonePanel();

        status2_1.setText(READY);

        createMainPanel();


        add(mainPanel);
        setVisible(true);

    }

    /**
     * Detects actions as performed on the VocalPad screens
     * particularly when buttons are selected
     */
        public void actionPerformed(ActionEvent ev)
        {
            System.out.println("Action Performed on VocalPad");
            String action = ev.getActionCommand();
            if (action.compareTo("dest") == 0)
            {
                System.out.println("Destination Action");
                destField.paste();
            }
            else if (action.compareTo("Dial") == 0)
            {
                destStr = destField.getText();
                if (destStr.length()  == 0)
                {
                    popUpMessage("A valid Destination is required");
                    return;
                }
                makeCall(destStr);
            }
            else if (action.compareTo("Hang Up") == 0)
            {
                System.out.println("Hang Up Action");
                try
                {
                    String homeName = origAddress.getName();
                    Connection conn = ((VCCall)call).findConnection(homeName);
                    System.out.println(conn);
                    if (conn != null)
                    {
                        conn.disconnect();
                        setPadToIdle();
                    }
                }
                catch (Exception e)
                {
                    popUpMessage("HangUp Exception - All Calls Cleared");
                }
            }
            else if (action.compareTo("Exit") == 0)
            {
                status3_1.setText("Program Terminated..");
                stop();
            }
            else if (action.compareTo("Chg Home#") == 0)
            {
                System.out.println("Chg Home Action");
                if (phonePanel == mainPanel.getComponent(0))
                {
                    mainPanel.remove(phonePanel);
                    mainPanel.remove(buttonPanel);
                    mainPanel.add(regPanel, 0);
                    mainPanel.add(regButtonPanel, 1);
                    setVisible(true);
//    contents.repaint();
                    repaint();
                    validate();
                }
            }
            else if (action.compareTo("Answer") == 0)
            {
                System.out.println("Answer Action");
                try
                {
                    if (homeTerminalConnection != null)
                    {
                        homeTerminalConnection.answer();
                        setPadToAnswered();
                    }
                    else
                    {
                        popUpMessage("No Active Call To Answer");
                    }
                }
                catch (Exception e)
                {
                    popUpMessage("Fault occurred - All Calls Cleared");
                }


            }  
            else if (action.compareTo("Return Call") == 0)
            {
                System.out.println("Return Call Action");
                if (allCallers.size() == 0)
                {
                    popUpMessage("No calls received");
                }
                else if (allCallers.size() > 0)
                {
                    Object[] callerArray = new Object[allCallers.size()];
                    allCallers.copyInto(callerArray);
                    Object obj = JOptionPane.showInputDialog(this,
                                "Select a destination", "Return Call",
                                JOptionPane.QUESTION_MESSAGE, null,
                                callerArray, String.valueOf(callerArray[0]));
    
                    if (obj != null)
                    {
                        String dest = String.valueOf(obj);
                        makeCall(dest);
                    }
                }
            }

            else if (action.compareTo("Transfer") == 0)
            {
                System.out.println("Transfer Action");
                String transferDest = null;
                Connection newConn;
                try
                {
                    if (homeTerminalConnection != null)
                    {
                        if (call != null)
                        {
                            Object obj = JOptionPane.showInputDialog(this, 
                                            "Transfer To", "Transfer",
                                            JOptionPane.QUESTION_MESSAGE, null,
                                            null, "");
                            if (obj != null)
                            {
                                transferDest = String.valueOf(obj);
                            }
                            else
                            {
                                return;
                            }
                            ((CallControlCall)call).setTransferController
                                                    (homeTerminalConnection);
                            state = TRANSFERCALL;
                            newConn = ((CallControlCall)call).transfer(transferDest);
                            Connection[] conns = call.getConnections();
                            if (conns == null)
                            {
                                if (call.getState() == Call.INVALID)
                                {
                                    setPadToIdle();
                                }
                            }
                            else
                            {
                                System.out.println("No Connections Exist");
                            }
    
                        }
                    }
                    else
                    {
                        popUpMessage("Call not Active");
                    }
                }
                catch (Exception e)
                {
                    popUpMessage("Transfer Exception - All Calls Cleared");
                }
            }
            else if (action.compareTo("Register") == 0)
            {
                callerStr = callerField.getText();
                if (callerStr.length() == 0)
                {
                    popUpMessage("Caller Data Required ");
                    return;
                }
                userStr = userId.getText();
                if (userStr.length() == 0)
                {
                    popUpMessage("User Data Required");
                    return;
                }
                passStr = passWord.getText();
                if (passStr.length() == 0)
                {
                    popUpMessage("PassWord Required");
                    return;
                }
                Vector v = new Vector();
                Integer n = new Integer(REGISTER);
                v.addElement(n);
                v.addElement(userStr);
                v.addElement(passStr);
                v.addElement(callerStr);
                Address addr = (Address)pd.sendPrivateData(v);
                if (addr == null)
                {
                    popUpMessage("Registration Failed");
                    return;
                }
                else
                {
                    dispPhone(callerStr);
                }
            }
            else if (action.compareTo("Cancel") == 0)
            {
                if (regPanel == mainPanel.getComponent(0))
                {
                    mainPanel.remove(regPanel);
                    mainPanel.remove(regButtonPanel);
                    mainPanel.add(phonePanel, 0);
                    mainPanel.add(buttonPanel, 1);
                    setVisible(true);
//    contents.repaint();
                    repaint();
                    validate();
                }
            }

            // check if action is one of 1, 2, 3, 4, 5, 6, 7, 8, 9 or 0
            else if (DIGITS.indexOf(action) != -1)   
            {
                if (action.length() == 1)
                {
                    String x = destField.getText();
                    x += action;
                    destField.setText(x);
                }
            }
        }

    /**
     * Makes a call request to the Jtapi Passes the required
     * data to the Jtapi and sets the application to a state
     * which is suitable for making a call
     */
        private void makeCall(String dest)
        {
            if ((origTerminal == null) || (origAddress == null))
            {
                popUpMessage("Home # must be set");
                return;
            }

            status3_1.setText(dest);
            try
            {
                call = null;
                connectionsArray = null;
                call = (CallControlCall)provider.createCall();
                call.addObserver(callObserver);
                System.out.println("Making a call");
                connectionsArray = call.connect(origTerminal,
                                    origAddress, dest);
                if (connectionsArray != null)
                {
                    hangUpButton.setEnabled(false);
                    dialButton.setEnabled(false);
                    retCallButton.setEnabled(false);
                    state = MAKECALL;
                }

            }
            catch (InvalidArgumentException e)
            {
                popUpMessage("Invalid Caller or Destination");
                status3_1.setText("");
            }
            catch (InvalidPartyException e)
            {
                popUpMessage("Invalid Caller data..Reset Home #");
            }
            catch (Exception e)
            {
                popUpMessage("Make Call Exception - All Calls Cleared");
            }
        }

    /**
     * Displays the panels required for reRegistration
     */
        private void reRegister()
        {
            if (phonePanel == mainPanel.getComponent(0))
            {
                mainPanel.remove(phonePanel);
                mainPanel.remove(buttonPanel);
                mainPanel.add(regPanel, 0);
                mainPanel.add(regButtonPanel, 1);
                setVisible(true);
//              contents.repaint();
                repaint();
                validate();
            }
        }

    /**
     * Displays the panels of the VocalPad phone
     */
        private void dispPhone(String caller)
        {
            try
            {
                origAddress = provider.getAddress(caller);
                origTerminal = provider.getTerminal(caller);
                Terminal[] ts = origAddress.getTerminals();

                origAddress.addCallObserver(callObserver);

                dispCaller.setText(caller);
                System.out.println("register");
                regCancelButton.setEnabled(true);

                if (regPanel == mainPanel.getComponent(0))
                {
                    mainPanel.remove(regPanel);
                    mainPanel.remove(regButtonPanel);
                    mainPanel.add(phonePanel, 0);
                    mainPanel.add(buttonPanel, 1);
                    setVisible(true);
                    repaint();
                    validate();
                }
            }
            catch (InvalidArgumentException e)
            {
                popUpMessage("Home # Not Available in Provider");
            }
            catch (ResourceUnavailableException e)
            {
                popUpMessage("Cannot add an Observer to the Caller");
            }
            catch (MethodNotSupportedException e)
            {
                popUpMessage("Requested Method Not Supported");	
            }
        }

    /**
     * Pops up a message screen
     */
        private void popUpMessage(String str)
        {
            JOptionPane.showMessageDialog(this, str, "Info Window", JOptionPane.INFORMATION_MESSAGE);
        }


    
    /**
     * Sets the VocalPad to the correct state for an active call
     */
        public void setPadToActiveCall(Call call)
        {
            status2_1.setText(ACTIVE);
            status2_2.setText(INPROGRESS);
            dialButton.setEnabled(false);
            retCallButton.setEnabled(false);
            answerButton.setEnabled(false);
            hangUpButton.setEnabled(false);
            transferButton.setEnabled(false);
        }
    
    /**
     * Sets the VocalPad to the correct state for receiving call
     */
        public void setPadToRecvCall()
        {
            System.out.println("VocalPad Receiving Call");
            state = ANSWERCALL;
        }

    /**
     * Sets the VocalPad to the correct state for a failure case
     */
        public void setPadToFail()
        {
            popUpMessage("Connection Failure - All Calls Cleared");
            setPadToIdle();
        }

    /**
     * Sets the VocalPad to the correct state for Busy
     */
        public void setPadToBusy()
        {
            status2_2.setText(BUSY);
            dialButton.setEnabled(false);
            retCallButton.setEnabled(false);
            answerButton.setEnabled(false);
            hangUpButton.setEnabled(true);
            transferButton.setEnabled(false);
        }

    /**
     * Sets the VocalPad to the correct state for Call In Progress
     */
        public void setPadToInProgress(Call newCall, String calledName)
        {
            if (call == null)
            {
                call = newCall;
            }
            state = MAKECALL;
            status2_2.setText(CALLING);
            status3_1.setText(calledName);
            dialButton.setEnabled(false);
            retCallButton.setEnabled(false);
            transferButton.setEnabled(false);
            answerButton.setEnabled(false);
            hangUpButton.setEnabled(true);
        }

    /**
     * Sets the VocalPad to the correct state for Call Ringing
     */
        public void setPadToRing(Call newCall, String callingName)
        {
            state = ANSWERCALL;
            call = newCall;
            status3_1.setText(callingName);
            allCallers.addElement(callingName);
            status2_2.setText(RINGING);
            dialButton.setEnabled(false);
            retCallButton.setEnabled(false);
            answerButton.setEnabled(true);
        }

    /**
     * Sets the VocalPad to the correct state for Call Answered
     */
        public void setPadToAnswered()
        {
            state = TALKING;
            status2_1.setText(ACTIVE);
            status2_2.setText(CONNECTED);
            dialButton.setEnabled(false);
            answerButton.setEnabled(false);
            transferButton.setEnabled(true);
            hangUpButton.setEnabled(true);
        }

    /**
     * Sets the VocalPad to clear
     */
        public void setPadToIdle()
        {
            state = IDLE;
            status2_1.setText(READY);
            status2_2.setText("");
            status3_1.setText("");
            dialButton.setEnabled(true);
            answerButton.setEnabled(false);
            transferButton.setEnabled(false);
            hangUpButton.setEnabled(false);
            homeTerminalConnection = null;
            call = null;
            connectionsArray = null;
        }


    /**
     * Create the panel for VocalPad registration
     */
        private void createRegPanel()
        {
            GridBagLayout regGbl = new GridBagLayout();
            GridBagConstraints regConstraints = new GridBagConstraints();

            regPanel = new JPanel();
            regPanel.setLayout(regGbl);

            regConstraints.gridx = 0;
            regConstraints.gridy = 0;
            regConstraints.gridwidth = 2;
            regConstraints.anchor = GridBagConstraints.NORTH;
            regConstraints.insets = new Insets(20, 0, 90, 0);
            regHeadPanel = new HeadPanel();
            regGbl.setConstraints(regHeadPanel, regConstraints);

            regConstraints.gridx = 0;
            regConstraints.gridy = 1;
            regConstraints.gridwidth = 1;
            regConstraints.insets = new Insets(15, 30, 15, 3);
            JLabel userIdLabel = new JLabel("UserID :");
            regGbl.setConstraints(userIdLabel, regConstraints);

            regConstraints.gridx = GridBagConstraints.RELATIVE;
            userId = new JTextField(10);
            regConstraints.insets = new Insets(15, 3, 15, 30);
            regGbl.setConstraints(userId, regConstraints);

            regConstraints.gridx = 0;
            regConstraints.gridy = 2;
            JLabel passWordLabel = new JLabel("PassWord :");
            regConstraints.insets = new Insets(15, 30, 15, 3);
            regGbl.setConstraints(passWordLabel, regConstraints);

            regConstraints.gridx = GridBagConstraints.RELATIVE;
            passWord = new JPasswordField(10);
            regConstraints.insets = new Insets(15, 3, 15, 30);
            regGbl.setConstraints(passWord, regConstraints);

            regConstraints.gridx = 0;
            regConstraints.gridy = 3;
            regConstraints.insets = new Insets(15, 30, 90, 3);
            JLabel homeNumberLabel = new JLabel("Home # :");
            regGbl.setConstraints(homeNumberLabel, regConstraints);

            regConstraints.gridx = GridBagConstraints.RELATIVE;
            regConstraints.insets = new Insets(15, 3, 90, 30);
            callerField = new JTextField(10);
            regGbl.setConstraints(callerField, regConstraints);

            regPanel.add(regHeadPanel);
            regPanel.add(userIdLabel);
            regPanel.add(userId);
            regPanel.add(passWordLabel);
            regPanel.add(passWord);
            regPanel.add(homeNumberLabel);
            regPanel.add(callerField);

            regPanel.setBackground(fields);
}

    /**
     * Create the button panel used with VocalPad registration
     */
        private void createRegButtonPanel()
        {
            GridBagLayout regButtonGbl = new GridBagLayout();
            GridBagConstraints regButtonConstraints = new GridBagConstraints();

            regButtonPanel = new JPanel();
            regButtonPanel.setLayout(regButtonGbl);


            regButtonConstraints.gridx = 0;
            regButtonConstraints.gridy = 0;
            regButtonConstraints.weighty = 1;
            regButtonConstraints.insets = new Insets(4, 27, 4, 27);

            regButton = new JButton ( "Register" );
            regButton.setFont(buttonFont);
            regButtonGbl.setConstraints(regButton, regButtonConstraints);

            regButtonConstraints.gridx = 0;
            regButtonConstraints.gridy = 1;
            regButtonConstraints.weighty = 1;

            regCancelButton = new JButton( "Cancel" );
            regCancelButton.setFont(buttonFont);
            regButtonGbl.setConstraints(regCancelButton, regButtonConstraints);


            regButtonConstraints.gridx = 0;
            regButtonConstraints.gridy = 2;
            regButtonConstraints.weighty = 1;

            regExitButton = new JButton( "Exit" );
            regExitButton.setFont(buttonFont);
            regButtonGbl.setConstraints(regExitButton, regButtonConstraints);

            regButtonPanel.add(regButton);
            regButtonPanel.add(regCancelButton);
            regButtonPanel.add(regExitButton);
            regCancelButton.setEnabled(false);

            regButtonPanel.setBackground(buttons);
            regButtonPanel.setForeground(Color.red);
}

    /**
     * Create the VocalPad button panel
     */
        private void createButtonPanel()
        {
            GridBagLayout buttonGbl = new GridBagLayout();
            GridBagConstraints buttonConstraints = new GridBagConstraints();

            buttonPanel = new JPanel();
            buttonPanel.setLayout(buttonGbl);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 0;
            buttonConstraints.weighty = 1;
            buttonConstraints.insets = new Insets(4, 20, 4, 20);
            chgHomeButton = new JButton( "Chg Home#" );
            chgHomeButton.setFont(buttonFont);
            buttonGbl.setConstraints(chgHomeButton, buttonConstraints);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 1;
            buttonConstraints.weighty = 1;
            retCallButton = new JButton( "Return Call" );
            retCallButton.setFont(buttonFont);
            buttonGbl.setConstraints(retCallButton, buttonConstraints);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 2;
            buttonConstraints.weighty = 1;
            answerButton = new JButton( "Answer" );
            answerButton.setFont(buttonFont);
            buttonGbl.setConstraints(answerButton, buttonConstraints);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 3;
            buttonConstraints.weighty = 1;
            transferButton = new JButton( "Transfer" );
            transferButton.setFont(buttonFont);
            buttonGbl.setConstraints(transferButton, buttonConstraints);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 4;
            buttonConstraints.weighty = 1;
            dialButton = new JButton( "Dial" );
            dialButton.setFont(buttonFont);
            buttonGbl.setConstraints(dialButton, buttonConstraints);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 5;
            buttonConstraints.weighty = 1;
            hangUpButton = new JButton("Hang Up");
            hangUpButton.setFont(buttonFont);
            buttonGbl.setConstraints(hangUpButton, buttonConstraints);

            buttonConstraints.gridx = 0;
            buttonConstraints.gridy = 6;
            buttonConstraints.weighty = 1;
            exitButton = new JButton("Exit");
            exitButton.setFont(buttonFont);
            buttonGbl.setConstraints(exitButton, buttonConstraints);

            buttonPanel.add(chgHomeButton);
            buttonPanel.add(retCallButton);
            buttonPanel.add(answerButton);
            buttonPanel.add(transferButton);
            buttonPanel.add(dialButton);
            buttonPanel.add(hangUpButton);
            buttonPanel.add(exitButton);

            buttonPanel.setBackground(buttons);
            buttonPanel.setForeground(Color.red);
        }

    /**
     * Create the Display panel of the VocalPad phone
     */
        private void createDispPanel()
        {
            GridBagLayout dispGbl = new GridBagLayout();
            GridBagConstraints dispConstraints = new GridBagConstraints();


            dispPanel = new JPanel();
            dispPanel.setLayout(dispGbl);

            dispPanel.setBorder(loweredBevel);

            dispConstraints.gridx = 0;
            dispConstraints.gridy = 0;
            dispConstraints.gridwidth = 1;
            dispConstraints.gridheight = 1;
            dispConstraints.fill = GridBagConstraints.BOTH;
            status1_1 = new JTextField(5);
            status1_1.setBackground(Color.gray);
            status1_1.setBorder(empty);
            status1_1.setEditable(false);
            dispGbl.setConstraints(status1_1, dispConstraints);

            dispConstraints.gridx = 1;
            status1_2 = new JTextField(5);
            status1_2.setBackground(Color.gray);
            status1_2.setBorder(empty);
            status1_2.setEditable(false);
            dispGbl.setConstraints(status1_2, dispConstraints);

            dispConstraints.gridx = 2;
            status1_3 = new JTextField(5);
            status1_3.setBackground(Color.gray);
            status1_3.setBorder(empty);
            status1_3.setEditable(false);
            dispGbl.setConstraints(status1_3, dispConstraints);

            dispConstraints.gridx = 3;
            status1_4 = new JTextField(5);
            status1_4.setBackground(Color.gray);
            status1_4.setBorder(empty);
            status1_4.setEditable(false);
            dispGbl.setConstraints(status1_4, dispConstraints);

            dispConstraints.gridx = 0;
            dispConstraints.gridy = 1;
            dispConstraints.gridwidth = 2;
            dispConstraints.gridheight = 1;
            dispConstraints.fill = GridBagConstraints.BOTH;
            status2_1 = new JTextField(10);
            status2_1.setBackground(Color.gray);
            status2_1.setBorder(empty);
            status2_1.setEditable(false);
            dispGbl.setConstraints(status2_1, dispConstraints);

            dispConstraints.gridx = 2;
            dispConstraints.fill = GridBagConstraints.BOTH;
            status2_2 = new JTextField(10);
            status2_2.setBackground(Color.gray);
            status2_2.setBorder(empty);
            status2_2.setEditable(false);
            dispGbl.setConstraints(status2_2, dispConstraints);

            dispConstraints.gridx = 0;
            dispConstraints.gridy = 2;
            dispConstraints.gridwidth = 4;
            dispConstraints.gridheight = 1;
            dispConstraints.fill = GridBagConstraints.BOTH;
            status3_1 = new JTextField(20);
            status3_1.setBackground(Color.gray);
            status3_1.setBorder(empty);
            status3_1.setEditable(false);
            dispGbl.setConstraints(status3_1, dispConstraints);

            dispPanel.add(status1_1);
            dispPanel.add(status1_2);
            dispPanel.add(status1_3);
            dispPanel.add(status1_4);
            dispPanel.add(status2_1);
            dispPanel.add(status2_2);
            dispPanel.add(status3_1);

            dispPanel.setBackground(fields);
        }

    /**
     * Create the number pad of the VocalPad phone
     */
        private void createPadPanel()
        {
            GridBagLayout padGbl = new GridBagLayout();
            GridBagConstraints padConstraints = new GridBagConstraints();


            padPanel = new JPanel();
            padPanel.setLayout(padGbl);

//          padPanel.setBorder(raisedBevel);

            for (int i=0; i < digitButtons.length; i++)
            {
                padConstraints.gridx = (i%3);
                padConstraints.gridy = (i/3);
                padConstraints.gridheight = 1;
                padConstraints.gridwidth = 1;
                padConstraints.insets = new Insets(2, 2, 2, 2);
                padConstraints.fill = GridBagConstraints.BOTH;
                digitButtons[i] = new JButton(digitValues[i]);
                padGbl.setConstraints(digitButtons[i], padConstraints);
            }

            padConstraints.gridx = 0;
            padConstraints.gridy = 4;
            padConstraints.gridwidth = 3;
            padConstraints.gridheight = 1;
            padConstraints.insets = new Insets(5, 0, 5, 0);
            destField = new JTextField(10);
            padGbl.setConstraints(destField, padConstraints);
            destField.setBackground(Color.black);
            destField.setForeground(Color.white);
            destField.setCaretColor(Color.white);

            for (int i=0; i < digitButtons.length; i++)
            {
                padPanel.add(digitButtons[i]);
            }
            padPanel.add(destField);

            destField.addActionListener(this);
            destField.setActionCommand("dest");
    
            padPanel.setBackground(fields);
}


    /**
     * Create the panel for the entire VocalPad phone including
     * display and number pad
     */
        private void createPhonePanel()
        {
            GridBagLayout phoneGbl = new GridBagLayout();
            GridBagConstraints phoneConstraints = new GridBagConstraints();


            phonePanel = new JPanel();
            phonePanel.setLayout(phoneGbl);
//          phonePanel.setBackground(Color.gray);

            phoneConstraints.gridx = 0;
            phoneConstraints.gridy = 0;
            phoneConstraints.gridheight = 1;
            phoneConstraints.gridwidth = 4;
            phoneConstraints.anchor = GridBagConstraints.NORTH;
            phoneConstraints.insets = new Insets(20, 20, 60, 0);
            phoneHeadPanel = new HeadPanel();
            phoneGbl.setConstraints(phoneHeadPanel, phoneConstraints);

            phoneConstraints.gridx = 0;
            phoneConstraints.gridy = 1;
            phoneConstraints.gridheight = 1;
            phoneConstraints.gridwidth = 2;
            phoneConstraints.anchor = GridBagConstraints.NORTHEAST;
            phoneConstraints.insets = new Insets(0, 21, 30, 1);
            JLabel dispCallerLabel = new JLabel("Home#");
            phoneGbl.setConstraints(dispCallerLabel, phoneConstraints); 

            phoneConstraints.gridx = 2;
            phoneConstraints.gridy = 1;
            phoneConstraints.gridheight = 1;
            phoneConstraints.gridwidth = 2;
            phoneConstraints.anchor = GridBagConstraints.NORTHWEST;
            phoneConstraints.insets = new Insets(0, 1, 30, 21);
            dispCaller = new JLabel("");
            dispCaller.setBorder(loweredBevel);
            phoneGbl.setConstraints(dispCaller, phoneConstraints);

            phoneConstraints.gridx = 0;
            phoneConstraints.gridy = 2;
            phoneConstraints.gridheight = 3;
            phoneConstraints.gridwidth = 4;
            phoneConstraints.anchor = GridBagConstraints.CENTER;
            phoneConstraints.insets = new Insets(0, 20, 0, 20);
            phoneGbl.setConstraints(dispPanel, phoneConstraints);

            phoneConstraints.gridx = 0;
            phoneConstraints.gridy = 5;
            phoneConstraints.gridheight = 3;
            phoneConstraints.gridwidth = 4;
            phoneConstraints.insets = new Insets(0, 20, 20, 20);
            phoneGbl.setConstraints(padPanel, phoneConstraints);

            phonePanel.add(phoneHeadPanel);
            phonePanel.add(dispCallerLabel);
            phonePanel.add(dispCaller);
            phonePanel.add(dispPanel);
            phonePanel.add(padPanel);

            phonePanel.setBackground(fields);
        }

    /**
     * Create main panel with registration panel and registration
     * button panel
     */
        private void createMainPanel()
        {
            mainPanel = new JPanel();
            BoxLayout main2Gbl = new BoxLayout(mainPanel, BoxLayout.X_AXIS);
            mainPanel.setLayout(main2Gbl);

            mainPanel.add(regPanel);
            mainPanel.add(regButtonPanel);
        }

    /**
     * Add an Action Listener for each button and set the buttons as
     * expected by default
     */
        private void setActionButtons()
        {
            for (int i=0; i < digitButtons.length; i++)
            {
                digitButtons[i].addActionListener(this);
            }
            regButton.addActionListener(this);
            regCancelButton.addActionListener(this);
            regExitButton.addActionListener(this);
            chgHomeButton.addActionListener(this);
            dialButton.addActionListener(this);
            answerButton.addActionListener(this);
            transferButton.addActionListener(this);
            retCallButton.addActionListener(this);
            hangUpButton.addActionListener(this);
            exitButton.addActionListener(this);

            if (isStandAlone == false)
            {
                exitButton.setEnabled(false);
                regExitButton.setEnabled(false);
            }
            transferButton.setEnabled(false);
            answerButton.setEnabled(false);
            hangUpButton.setEnabled(false);
            dialButton.setEnabled(true);
            retCallButton.setEnabled(true);
        }
    }

}
