#ifndef JSCALL_HXX
#define JSCALL_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const JSCall_hxx_Version =
    "$Id$";


#include "Sptr.hxx"
#include "CallInfo.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"
#include "ByeMsg.hxx"
#include "JSRequest.hxx"
#include "JSProvider.hxx"
#include "NetworkAddress.h"

class JSRequest;
class JSProvider;

/** Class for call objects within JTAPI server
 */
class JSCall : public CallInfo
{
    public:
        ///
        JSCall();

        ///
        JSCall(const string& user);

        ///
        virtual ~JSCall();

        ///
        void setOriginator( const bool yorn );

        ///
        bool isOriginator();

        ///
        void setCallee( const SipUrl url );

        ///
        SipUrl getCallee();

        ///
        void setCaller( const SipUrl url );

        ///
        SipUrl getCaller();

        /// JTAPI Server's SDP, needed to act as an UA
        void setLocalSdp( const Sptr < SipSdp > sdp )
        {
            localSdp = sdp;
        }

	///
        Sptr < SipSdp > getLocalSdp()
        {
            return localSdp;
        }

        /// Caller's SDP
        void setFromSdp( const Sptr < SipSdp > sdp )
        {
            fromSdp = sdp;
        }

	///
        Sptr < SipSdp > getFromSdp()
        {
            return fromSdp;
        }

        /// Callee's SDP
        void setToSdp( const Sptr < SipSdp > sdp )
        {
            toSdp = sdp;
        }

	///
        Sptr < SipSdp > getToSdp()
        {
            return toSdp;
        }

        /// SIP Leg between JSD and caller
        void setControlInvite( Sptr < InviteMsg > invite )
        {
            controlInvite = invite;
        }

	///
        Sptr < InviteMsg > getControlInvite()
        {
            return controlInvite;
        }

        /// SIP leg between caller and JSN
        void setFromInvite( Sptr < InviteMsg > invite )
        {
            fromInvite = invite;
        }

	///
        Sptr < InviteMsg > getFromInvite()
        {
            return fromInvite;
        }

        /// SIP leg between JSN and callee
        void setToInvite( Sptr < InviteMsg > invite )
        {
            toInvite = invite;
        }

	///
        Sptr < InviteMsg > getToInvite()
        {
            return toInvite;
        }

        /// Invite message sent to Redirect Server
        void setRSInvite( Sptr < InviteMsg > invite )
        {
            rsInvite = invite;
        }

	///
        Sptr < InviteMsg > getRSInvite()
        {
            return rsInvite;
        }

        ///
        void setProvider(JSProvider* provider)
        {
            itsProvider = provider;
        }

	///
        JSProvider* getProvider()
        {
            return itsProvider;
        }

        ///
        void setRequest(JSRequest* request);

	///
        JSRequest* getRequest()
        {
            return itsRequest;
        }

        ///
        void setCallID(const string& callid)
        {
            itsCallID = callid;
        }

	///
        string& getCallID()
        {
            return itsCallID;
        }

        /// 
        void setSender(const NetworkAddress& sender)
        {
            itsSender = sender;
        }

	///
        NetworkAddress& getSender()
        {
            return itsSender;
        }

        ///
        void setReceiver(const NetworkAddress& receiver)
        {
            itsReceiver = receiver;
        }

	///
        NetworkAddress& getReceiver()
        {
            return itsReceiver;
        }

        ///
        void setSendClientCallID(const string& callid)
        {
            itsSendClientCallID = callid;
        }

	///
        string& getSendClientCallID()
        {
            return itsSendClientCallID;
        }

        ///
        void setRecvClientCallID(const string& callid)
        {
            itsRecvClientCallID = callid;
        }

	///
        string& getRecvClientCallID()
        {
            return itsRecvClientCallID;
        }

        ///
        void setFromUser(Data& user)
        {
            itsFromUser = user;
        }

	///
        Data& getFromUser()
        {
            return itsFromUser;
        }

	///
        void setToUser(Data& user)
        {
            itsToUser = user;
        }

	///
        Data& getToUser()
        {
            return itsToUser;
        }

        ///
        void setControlCallID(SipCallId& callid)
        {
            controlSipCallId = callid;
        }

	///
        SipCallId& getControlCallID()
        {
            return controlSipCallId;
        }

	///
        void setFromCallID(SipCallId& callid)
        {
            fromSipCallId = callid;
        }

	///
        SipCallId& getFromCallID()
        {
            return fromSipCallId;
        }

	///
        void setToCallID(SipCallId& callid)
        {
            toSipCallId = callid;
        }

	///
        SipCallId& getToCallID()
        {
            return toSipCallId;
        }

        ///
        void setControl200(StatusMsg& status)
        {
            controlStatusMsg = status;
        }

	///
        StatusMsg& getControl200()
        {
            return controlStatusMsg;
        }

        ///
        void setTo200(StatusMsg& status)
        {
            toStatusMsg = status;
        }

	///
        StatusMsg& getTo200()
        {
            return toStatusMsg;
        }

        ///
        void setFromBye(Sptr < ByeMsg > bye)
        {
            fromBye = bye;
        }

	///
        Sptr < ByeMsg > getFromBye()
        {
            return fromBye;
        }

        ///
        void setToBye(Sptr < ByeMsg > bye)
        {
            toBye = bye;
        }

	///
        Sptr < ByeMsg > getToBye()
        {
            return toBye;
        }

        ///
        void setControlled(bool control)
        {
            controlled = control;
        }

	///
        bool getControlled()
        {
            return controlled;
        }

        ///
        void setCallerAddr(string addr)
        {
            callerAddr = addr;
        }

	///
        string getCallerAddr()
        {
            return callerAddr;
        }

        ///
        void setCalleeAddr(string addr)
        {
            calleeAddr = addr;
        }

	///
        string getCalleeAddr()
        {
            return calleeAddr;
        }

    protected:

    private:

        bool originator;
        SipUrl callee;
        SipUrl caller;
        Sptr < SipSdp > localSdp;
        Sptr < SipSdp > fromSdp;
        Sptr < SipSdp > toSdp;

        Sptr < InviteMsg > controlInvite;
        Sptr < InviteMsg > fromInvite;
        Sptr < InviteMsg > toInvite;
        Sptr < InviteMsg > rsInvite;
        JSProvider* itsProvider;

        JSRequest* itsRequest;
        Data itsFromUser; 	// This is the key for the call container
        Data itsToUser; 	// This is the key for the call container
        string itsCallID;
        NetworkAddress itsSender;
        SipCallId controlSipCallId;
        SipCallId fromSipCallId;
        SipCallId toSipCallId;
        StatusMsg controlStatusMsg;
        StatusMsg toStatusMsg;
        NetworkAddress itsReceiver;
        string itsSendClientCallID;
        string itsRecvClientCallID;

        string callerAddr;
        string calleeAddr;

        Sptr < ByeMsg > fromBye;
        Sptr < ByeMsg > toBye;

        bool controlled;
};

#endif
