
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpAckFromCaller_cxx_Version =
    "$Id$";
#include "JSOpAckFromCaller.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "CallInfo.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"
#include "SipProxyAuthorization.hxx"


///
JSOpAckFromCaller::JSOpAckFromCaller()
{}


///

JSOpAckFromCaller::~JSOpAckFromCaller()
{}


///

const char* const
JSOpAckFromCaller::name() const
{
    return "JSOpAckFromCaller";
}


///
const Sptr < State >
JSOpAckFromCaller::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive ACK message
    // 2. Chech if received as JSN or JSD
    // 3. If JSD, then the ACK is from JSN for toINVITE, do nothing
    //		, this is handled in JSOpACK of "Connected" state
    // 4. If JSN, then the ACK is from caller for fromINVITE, need to
    //		create a new ACK for toINVITE and send it to callee
    // 5. Change state to "Connected"

    cpLog( LOG_DEBUG, "JSOpAckFromCaller operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0);

    if ( SIP_ACK != sipMsg->getType() )
    {
        return 0;
    }

    // Do we need to save the ACK to call object?
    Sptr < AckMsg > ack;
    ack.dynamicCast(sipMsg);
    assert( ack != 0);

    // First check if the port is empty which means default 5060
    Sptr <BaseUrl> baseUrl;
    baseUrl = ack->getRequestLine().getUrl();
    assert( baseUrl.getPtr() != 0 );
    assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

    Sptr <SipUrl> sipUrl;
    assert( sipUrl.dynamicCast( baseUrl ) != 0 );

    Data viaPort = sipUrl->getPort();
    if ( viaPort.length() == 0 )
    {
        viaPort = 5060;
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0);

    JSProvider* provider = call->getProvider();

    // Only handle ACK received as JSN
    if ( viaPort != provider->getConfig()->getJSNSipPort() )
    {
        return 0;
    }

    // send a connconnected notification for Caller and Callee
    // need udpStack, dest NetworkAddress and request
    // to send the notification
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    NetworkAddress dest = call->getSender();
    JSRequest* req = call->getRequest();
    JSNotification notification(clientUdp, dest);

    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpAckFromCaller : no notifications sent" );
    }
    else
    {

        notification.setType(CONNCONNECTED);
        notification.setCall(req->getClientCallID());
        notification.setAddr(req->getCallerAddr());
        notification.send();

        notification.setType(CONNCONNECTED);
        notification.setCall(req->getClientCallID());
        notification.setAddr(req->getCalleeAddr());
        notification.send();
    }

    AckMsg newAck( call->getTo200() );

    // Add authorization
    newAck.setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    //AckMsg newAckMsg( newAck , AckMsg::ackMsgForward );
    cpLog(LOG_DEBUG, "JSOpAckFromCaller: Send new ACK to callee");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", newAck.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipEvent->getSipStack()->sendAsync( newAck );

    // Change state to Connected
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateConnected" );
}
