
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpBusyDisconnectReq_cxx_Version =
    "$Id: JSOpBusyDisconnectReq.cxx,v 1.15 2002/02/25 05:51:16 sprajpat Exp $";
#include "JSOpBusyDisconnectReq.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
#include "SipContact.hxx"
#include "JSStateDisconnectReqd.hxx"
//#include "SipTransceiver.hxx"


///
JSOpBusyDisconnectReq::JSOpBusyDisconnectReq()
{}


///

JSOpBusyDisconnectReq::~JSOpBusyDisconnectReq()
{}


///

const char* const
JSOpBusyDisconnectReq::name() const
{
    return "JSOpBusyDisconnectReq";
}


///
const Sptr < State >
JSOpBusyDisconnectReq::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive disconnect request from the vocalpad
    // 2. Send BYE to caller
    // 3. Change state to disconnect requested where 200 will
    // 		be handled
    cpLog( LOG_DEBUG, "JSOpBusyDisconnectReq operation" );

    Sptr < JSReqEvent > reqEvent;
    reqEvent.dynamicCast( event );

    if ( reqEvent == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );
    JSProvider* provider = call->getProvider();
    JSRequest* request = call->getRequest();

    if ( request->getMsgType() != 102 /* disconnect */)
    {
        // Only handle disconnect request
        return 0;
    }

    int port = provider->getConfig()->getJSNSipPort().convertInt();

    // Create and send new BYE to caller
    Sptr < InviteMsg > fromInvite = call->getFromInvite();
    ByeMsg newCallerBye(*fromInvite);  // Use the constructor with SipCommand

    // Clear via list
    newCallerBye.flushViaList();

    // Add via
    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( theSystem.gethostAddress() ) );
    sipVia.setPort( port = provider->getConfig()->getJSNSipPort().convertInt() );
    sipVia.setTransport( "UDP" );
    newCallerBye.setVia( sipVia, 0 );

    cpLog(LOG_DEBUG, "JSOpBusyDisconnectReq: new BYE to caller:");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", newCallerBye.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    provider->getJSNSipStack()->sendAsync( newCallerBye );

    // Change state to disconnect requested
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateDisconnectReqd" );
}
