
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpBye_cxx_Version =
    "$Id$";
#include "JSOpBye.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
//kevin
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpBye::JSOpBye()
{}


///

JSOpBye::~JSOpBye()
{}


///

const char* const
JSOpBye::name() const
{
    return "JSOpBye";
}


///
const Sptr < State >
JSOpBye::process( const Sptr < SipProxyEvent > event )
{
    // Here is the logic of handling BYE message:
    // 1. Check if this is a controlled call
    // 2. If NO, check which direction the message is from, call or callee,
    // 		we don't have to worry about the case of the callee use the
    // 		vocalpad to disconnect the call. Check how JSProvider
    // 		handle the disconnect request from vocalpad.
    // 		Checking direction by using the SIP call id.
    // 		2.1 If from callee, means callee on-hook rather than use vocalpad
    // 			2.1.1 Notify vocalpad that user on-hook
    // 			2.1.2 Create ans send new BYE to caller
    // 			2.1.3 Change state to "disconnected"
    // 		2.2 If from caller,
    // 			2.2.1 Create ans send a new BYE to callee
    // 			2.2.2 Change state to "disconnected"
    // 3. If YES, check which on which port the message is received, JSD or JSN
    // 4. JSD,
    // 		4.1 Add via to the message
    // 		4.2 Send async, Sip stack will know where to send
    // 5. JSN, check which direction the message is from by using SIP call id
    // 		5.1 If from caller
    // 			5.1.1 Create and send new BYE to  callee
    // 			5.1.2 Change state to "disconnected"
    // 		5.2 If from callee
    // 			5.1.1 Create and send new BYE to  caller
    // 			5.1.2 Change state to "disconnected"

    cpLog( LOG_DEBUG, "JSOpBye operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0);

    Sptr < ByeMsg > bye;
    bye.dynamicCast(sipMsg);
    if ( bye == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );
    JSProvider* provider = call->getProvider();

    if ( !(call->getControlled()) ) // Non-controlled call, i.e. NJS-JS
    {
        cpLog(LOG_DEBUG, "JSOpBye: receive message for uncontrolled call");
        SipCallId callId = sipMsg->getCallId();

        if ( (callId != call->getToCallID()) &&
                (callId != call->getFromCallID()) )
        {
            cpLog(LOG_ERR, "JSOpBye: SIP call id doesnt' match tp or from");
            cpLog(LOG_ERR, "JSOpBye: message call id %s", callId.getData().logData() );
            cpLog(LOG_ERR, "JSOpBye: from call id %s", call->getFromCallID().getData().logData() );
            cpLog(LOG_ERR, "JSOpBye: to call id %s", call->getFromCallID().getData().logData() );
            // Change state to unknown to terminate this call
            Sptr < JSStateMachine > stateMachine;
            stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
            assert( stateMachine != 0 );

            return stateMachine->findState( "JSStateUnknown" );
        }

        // Send notification to vocalpad
        Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
        NetworkAddress dest = call->getReceiver();
        JSRequest* req = call->getRequest();
        JSNotification notification(clientUdp, dest);

        if (dest.getPort() == -1)
        {
            cpLog( LOG_DEBUG, "JSOpBye : no notification sent" );
        }
        else
        {
            notification.setType(CONNDISCONNECTED);
            notification.setCall(req->getClientCallID());
            notification.setAddr(req->getCalleeAddr());
            notification.send();

            notification.setType(CONNDISCONNECTED);
            notification.setCall(req->getClientCallID());
            notification.setAddr(req->getCallerAddr());
            notification.send();
        }

        if ( callId == call->getToCallID() )
        {
            // Notify vocalpad user hangup from the set
            // Save the "to" BYE to the call object, used by 200 later
            cpLog(LOG_DEBUG, "JSOpBye: receive bye from CALLEE");
            call->setToBye( bye );


            // Create and send new BYE to caller
            Sptr < InviteMsg > fromInvite = call->getFromInvite();
            ByeMsg newBye(*fromInvite);  // Use the constructor with SipCommand

            newBye.flushViaList();
            SipVia sipVia;
            sipVia.setprotoVersion( "2.0" );
            sipVia.setHost( Data( theSystem.gethostAddress() ) );
            sipVia.setPort( provider->getConfig()->getJSDSipPort().convertInt() );
            sipVia.setTransport( "UDP" );
            newBye.setVia( sipVia );
            // Add contact
            Sptr <SipUrl> myUrl = new SipUrl;
            myUrl->setUserValue( provider->getConfig()->getUserName() );
            myUrl->setHost( Data( theSystem.gethostAddress() ) );
            myUrl->setPort( provider->getConfig()->getJSDSipPort() );

            SipContact me;
            me.setUrl( myUrl );
            newBye.setNumContact( 0 );    // Clear
            newBye.setContact( me );

            cpLog(LOG_DEBUG, "JSOpBye: new BYE to caller:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, "%s", newBye.encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendAsync( newBye );
        }
        else if ( callId == call->getFromCallID() )
        {
            cpLog(LOG_DEBUG, "JSOpBye: receive bye from CALLER");
            // Save the "from" BYE to the call object, used by 200 later
            call->setFromBye( bye );

            StatusMsg to200 = call->getTo200();

            // Use the constructor with StatusMsg
            // Create and send new BYE to callee
            ByeMsg newBye( to200 );

            newBye.flushViaList();
            SipVia sipVia;
            sipVia.setprotoVersion( "2.0" );
            sipVia.setHost( Data( theSystem.gethostAddress() ) );
            sipVia.setPort( provider->getConfig()->getJSDSipPort().convertInt() );
            sipVia.setTransport( "UDP" );
            newBye.setVia( sipVia );
            // Add contact
            Sptr <SipUrl> myUrl = new SipUrl;
            myUrl->setUserValue( provider->getConfig()->getUserName() );
            myUrl->setHost( Data( theSystem.gethostAddress() ) );
            myUrl->setPort( provider->getConfig()->getJSDSipPort() );

            SipContact me;
            me.setUrl( myUrl );
            newBye.setNumContact( 0 );    // Clear
            newBye.setContact( me );

            cpLog(LOG_DEBUG, "JSOpBye: new BYE to callee:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, "%s", newBye.encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendAsync( newBye );
        }

    }
    else // Controlled call, i.e. JS-JS
    {
        Sptr <BaseUrl> baseUrl;
        baseUrl = bye->getRequestLine().getUrl();
        assert( baseUrl.getPtr() != 0 );
        assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

        Sptr <SipUrl> sipUrl;
        sipUrl.dynamicCast( baseUrl );
        assert( sipUrl != 0 );

        Data viaPort = sipUrl->getPort();
        if ( viaPort.length() == 0 )
            viaPort = 5060;

        cpLog(LOG_DEBUG, "Message received on port: %s", viaPort.logData());

        if ( (viaPort != provider->getConfig()->getJSDSipPort()) &&
                (viaPort != provider->getConfig()->getJSNSipPort()) )
        {
            cpLog(LOG_ERR, "JSOpBye: BYE received on unknow port");
            // Change state to unknown to terminate this call
            Sptr < JSStateMachine > stateMachine;
            stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
            assert( stateMachine != 0 );

            return stateMachine->findState( "JSStateUnknown" );
        }

        if ( viaPort == provider->getConfig()->getJSDSipPort() )
        {
            ByeMsg newBye( *bye, ByeMsg::byeMsgForward );

            // Send notification to vocalpad
            Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
            NetworkAddress dest = call->getReceiver();
            JSRequest* req = call->getRequest();
            JSNotification notification(clientUdp, dest);

            if (dest.getPort() == -1)
            {
                cpLog( LOG_DEBUG, "JSOpBye : no notification sent" );
            }
            else
            {
                notification.setType(CONNDISCONNECTED);
                notification.setCall(req->getRecvClientCallID());
                notification.setAddr(req->getCalleeAddr());
                notification.send();

                notification.setType(CONNDISCONNECTED);
                notification.setCall(req->getRecvClientCallID());
                notification.setAddr(req->getCallerAddr());
                notification.send();
            }


            // Add via
            SipVia sipVia;
            sipVia.setprotoVersion( "2.0" );
            sipVia.setHost( Data( theSystem.gethostAddress() ) );
            sipVia.setPort( provider->getConfig()->getJSDSipPort().convertInt() );
            sipVia.setTransport( Data("UDP") );
            newBye.setVia( sipVia, 0 );

            cpLog(LOG_DEBUG, "JSOpBye: JSD forward BYE to next hop:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, newBye.encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendAsync( newBye );
            return 0;
        }
        else // JSN
        {
            SipCallId callId = sipMsg->getCallId();

            if ( (callId != call->getToCallID()) &&
                    callId != call->getFromCallID() )
            {
                cpLog(LOG_ERR, "JSOpBye: SIP call id doesnt' match tp or from");
                // Change state to unknown to terminate this call
                Sptr < JSStateMachine > stateMachine;
                stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
                assert( stateMachine != 0 );

                return stateMachine->findState( "JSStateUnknown" );
            }

            Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
            NetworkAddress dest = call->getSender();
            JSRequest* req = call->getRequest();
            JSNotification notification(clientUdp, dest);

            if (dest.getPort() == -1)
            {
                cpLog( LOG_DEBUG, "JSOpBye : no notification sent" );
            }
            else
            {
                notification.setType(CONNDISCONNECTED);
                notification.setCall(req->getClientCallID());
                notification.setAddr(req->getCalleeAddr());
                notification.send();

                notification.setType(CONNDISCONNECTED);
                notification.setCall(req->getClientCallID());
                notification.setAddr(req->getCallerAddr());
                notification.send();
            }


            if ( callId == call->getToCallID() )
            {
                call->setToBye( bye );

                // Create and send new BYE to caller
                Sptr < InviteMsg > fromInvite = call->getFromInvite();

                cpLog(LOG_DEBUG, "from INVITE:");
                cpLog(LOG_DEBUG, "***************************************");
                cpLog(LOG_DEBUG, "%s", fromInvite->encode().logData());
                cpLog(LOG_DEBUG, "***************************************");
                // Use the constructor with SipCommand
                ByeMsg newBye(*fromInvite);

                newBye.flushViaList();
                //Add via
                SipVia sipVia;
                sipVia.setprotoVersion( "2.0" );
                sipVia.setHost( Data( theSystem.gethostAddress() ) );
                sipVia.setPort( provider->getConfig()->getJSNSipPort().convertInt() );
                sipVia.setTransport( "UDP" );

                newBye.setVia( sipVia, 0 );

                cpLog(LOG_DEBUG, "JSOpBye: new BYE to caller as JSN:");
                cpLog(LOG_DEBUG, "***************************************");
                cpLog(LOG_DEBUG, "%s", newBye.encode().logData());
                cpLog(LOG_DEBUG, "***************************************");

                sipEvent->getSipStack()->sendAsync( newBye );
            }
            else
            {
                call->setFromBye( bye );

                // Create and send new BYE to callee
                StatusMsg to200 = call->getTo200();

                // Use the constructor with StatusMsg
                ByeMsg newBye(to200);

                newBye.flushViaList();
                //Add via
                SipVia sipVia;
                sipVia.setprotoVersion( "2.0" );
                sipVia.setHost( Data( theSystem.gethostAddress() ) );
                sipVia.setPort( provider->getConfig()->getJSNSipPort().convertInt() );
                sipVia.setTransport( "UDP" );
                newBye.setVia( sipVia, 0 );

                cpLog(LOG_DEBUG, "JSOpBye: new BYE to callee as JSN:");
                cpLog(LOG_DEBUG, "***************************************");
                cpLog(LOG_DEBUG, "%s", newBye.encode().logData());
                cpLog(LOG_DEBUG, "***************************************");
                sipEvent->getSipStack()->sendAsync( newBye );
            }
        }
    }

    // Change state to disconnected
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateDisconnected" );
}
