
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpExit_cxx_Version =
    "$Id$";
#include "JSOpExit.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"


///
JSOpExit::JSOpExit()
{}


///

JSOpExit::~JSOpExit()
{}


///

const char* const
JSOpExit::name() const
{
    return "JSOpExit";
}


///
const Sptr < State >
JSOpExit::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive ACK, make sure received as JSD
    // 2. Forward the message to callee

    cpLog( LOG_DEBUG, "JSOpExit operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        cpLog(LOG_ERR, "JSOpExit: Can't come in operator without an Sip event");
        assert( sipEvent == 0 );
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    // Delete the call from user & sip maps
    JSProvider* provider = call->getProvider();

    Sptr < JSUserCallContainer > userMap = provider->getUserCallContainer();
    Sptr < JSSipCallContainer > sipMap = provider->getSipCallContainer();

    userMap->deleteCall( call->getFromUser() );
    userMap->deleteCall( call->getToUser() );

    sipMap->deleteCall( call->getFromCallID() );
    sipMap->deleteCall( call->getToCallID() );

    // Delete the call object
    call = 0;

    // Dump the content of the maps
    if ( userMap->size() > 0 )
        userMap->dump();
    else
        cpLog(LOG_DEBUG, "User map is empty");

    if ( sipMap->size() > 0 )
        sipMap->dump();
    else
        cpLog(LOG_DEBUG, "SIP map is empty");
    return 0;
}
