
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpStartCallFromSet_cxx_Version =
    "$Id: JSOpStartCallFromSet.cxx,v 1.18 2002/02/25 05:51:17 sprajpat Exp $";
#include "JSOpStartCallFromSet.hxx"
#include "SipEvent.hxx"
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "InviteMsg.hxx"
#include "JSSipCallContainer.hxx"
#include "JSProvider.hxx"
#include "SipVia.hxx"
#include "SipProxyAuthorization.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"


///
JSOpStartCallFromSet::JSOpStartCallFromSet()
{}


///

JSOpStartCallFromSet::~JSOpStartCallFromSet()
{}


///

const char* const
JSOpStartCallFromSet::name() const
{
    return "JSOpStartCallFromSet";
}


///
const Sptr < State >
JSOpStartCallFromSet::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive INIVTE from caller's set
    // 2. Send a CallCreate request to the JTapi client
    // 3. If can't sent notification to vocalpad, the call go on via steps 4, 5
    //		else, remain in "idle" state
    // 4. Create new INVITE and send it to RS
    // 5. Change state to In Progress

    cpLog( LOG_DEBUG, "JSOpStartCallFromSet operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < InviteMsg > invite;
    invite.dynamicCast( sipMsg );
    if ( invite == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );
    JSProvider* provider = call->getProvider();

    // Check if received on JSN or JSD
    // JSN means caller calling from set directly rather than vocalpad
    // JSD means an NJS caller calling
    Sptr <BaseUrl> baseUrl;
    baseUrl = invite->getRequestLine().getUrl();
    assert( baseUrl.getPtr() != 0 );
    assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

    Sptr <SipUrl> sipUrl;
    assert( sipUrl.dynamicCast( baseUrl ) != 0 );

    Data viaPort = sipUrl->getPort();

    if ( viaPort.length() == 0 )
    {
        // No port number sepcified on the requestline which means
        // SIP default port 5060, but JTAPI server doesn't listening
        // on 5060. we shouldn't receive this message.
        // Change state to unknown which will get rid of the call object
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    if ( (viaPort != provider->getConfig()->getJSDSipPort()) &&
            (viaPort != provider->getConfig()->getJSNSipPort()) )
    {
        cpLog(LOG_ERR, "JSOpBye: BYE received on unknow port %s:", viaPort.logData());
        return 0;
    }

    if ( viaPort == provider->getConfig()->getJSDSipPort() ) // JSD
    {
        // Invite received on JSD
        // Handled in JSOpCalled
        return 0;
    }

    // INVITE received on JSN, i.e. user making call from set not vocalpad

    // Set controlled flag to true
    call->setControlled( true );

    // Save INVITE
    call->setFromInvite( new InviteMsg( *invite ) );

    Sptr<SipSdp> sdp;
    sdp.dynamicCast ( sipMsg->getContentData(0) );

    if (sdp == 0)
    {
        cpLog( LOG_ERR, "No remote SDP or invalid SDP in invite" );
        StatusMsg errorStatusMsg ( *invite, 404 );  // 404 Not Found
        errorStatusMsg.removeVia( 0 );
        sipEvent->getSipStack()->sendReply( errorStatusMsg );
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    call->setFromSdp( new SipSdp( *sdp ) );

    // send CALLCREATE to Jtapi Client
    Sptr < JSUserRegContainer > addrMap;
    addrMap.dynamicCast( provider->getUserRegContainer() );
    assert( addrMap != 0 );

    Sptr <BaseUrl> toBaseUrl = invite->getTo().getUrl();
    Sptr <BaseUrl> fromBaseUrl = invite->getFrom().getUrl();
    assert( toBaseUrl.getPtr() != 0 );
    assert( fromBaseUrl.getPtr() != 0 );
    assert( toBaseUrl->getType() == SIP_URL );
    assert( fromBaseUrl->getType() == SIP_URL );

    Data toUser = toBaseUrl->getUserValue();
    Data fromUser = fromBaseUrl->getUserValue();

    NetworkAddress* netAddr_ptr = addrMap->findAddr( fromUser );
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    if ( netAddr_ptr != 0 )
    {
        // 1. Send CALLCREATE to vocalpad
        // 2. Add call to sip map
        // 3. Stay in state "idle"
        call->setSender( *netAddr_ptr );
        LocalScopeAllocator lo;
        string callid( invite->getCallId().getCallIdData().getData(lo) );
        JSNotification notification(clientUdp, *netAddr_ptr);
        notification.setType(CALLCREATE);
        notification.setId( callid );
        notification.setAddr( fromUser.getData(lo) );
        notification.setAddr( toUser.getData(lo) );
        notification.send();
        cpLog(LOG_DEBUG, "Send CALLCREATE notif to client");
        cpLog(LOG_DEBUG, "%s", notification.getNotificationString().c_str());

        SipCallId fromId;
        fromId = invite->getCallId();
        call->setFromCallID( fromId );
        Sptr < JSSipCallContainer > sipMap = provider->getSipCallContainer();
        sipMap->addCall( fromId, call );
        return 0;
    }

    cpLog(LOG_WARNING, "From user not registered");  // i.e. not running vocalpad
    // We won't notify vocalpad but the call should go on
    // I.e. here is the condition of this call
    // 1. caller has JTAPI server configured in his calling contact list
    // 2. caller making call from set
    // 3. caller is not running his vocalpad application
    // So, we will let the call go on by
    // 1. Sned new INVITE to callee
    // 2. Change state to "in progress"

    // Create new INVITE to send to callee
    InviteMsg toInvite( toBaseUrl,
                        provider->getConfig()->getJSNSipPort().convertInt() );

    // Add both call ids to the call object
    SipCallId fromId, toId;
    fromId = invite->getCallId();
    toId = toInvite.getCallId();
    call->setFromCallID( fromId );
    call->setToCallID( toId );
    // Add SIP call id of both "from" and "to" legs to sipmap
    Sptr < JSSipCallContainer > sipMap = provider->getSipCallContainer();
    sipMap->addCall( fromId, call );
    sipMap->addCall( toId, call );

    // Change "from" header to the caller, so he'll pay the call
    SipFrom from = invite->getFrom();
    toInvite.setFrom( from );

    // Change toInvite's requestline for RS
    SipRequestLine rsRequestLine = toInvite.getRequestLine();
    Sptr <BaseUrl> rsBaseUrl = rsRequestLine.getUrl();
    assert( rsBaseUrl.getPtr() != 0 );
    assert( rsBaseUrl->getType() == SIP_URL );
    Sptr <SipUrl> rsUrl;
    assert( rsUrl.dynamicCast( rsBaseUrl ) != 0 );

    rsUrl->setHost( provider->getConfig()->getRedirectHost() );
    rsUrl->setPort( provider->getConfig()->getRedirectPort() );
    rsUrl->setUserParam( "phone" );
    rsRequestLine.setUrl( rsUrl );
    toInvite.setRequestLine( rsRequestLine );

    // Add contact
    Sptr <SipUrl> myUrl = new SipUrl;
    myUrl->setUserValue( provider->getConfig()->getUserName() );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    myUrl->setPort( provider->getConfig()->getJSNSipPort() );

    SipContact me;
    me.setUrl( myUrl );
    toInvite.setNumContact( 0 );    // Clear
    toInvite.setContact( me );

    // Add authorization
    //SipProxyAuthorization authorization("VovidaClassXswitch");
    //toInvite.setProxyAuthorization( authorization );
    //toInvite.setAuthBasic( provider->getConfig()->getProxyAuthKey() );
    toInvite.setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    // Change Via
    toInvite.flushViaList();
    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( theSystem.gethostAddress() ) );
    sipVia.setPort( provider->getConfig()->getJSNSipPort().convertInt() );
    sipVia.setTransport( Data("UDP") );
    sipVia.setBranch( REDIRECT_JSN );
    toInvite.setVia( sipVia );

    // Add SDP from the "from" invite to this invite
    // Get remote SDP from message
    toInvite.flushContentData();
    // toInvite.flushContentType();
    toInvite.setContentData(&(*sdp));

    call->setToInvite( new InviteMsg( toInvite ) );
    call->setRSInvite( new InviteMsg( toInvite ) );
    // Since we still JSN here, use the sip stack in the event
    cpLog(LOG_DEBUG, "JSOpStartCallFromSet: Send new INVITE to callee");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", toInvite.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipEvent->getSipStack()->sendAsync( toInvite );

    // Change state to In Progress
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateInProgress" );
}
