#ifndef JSPROVIDER_HXX
#define JSPROVIDER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const JSProvider_hxx_Version =
    "$Id$";



#include <cassert>
#include <list>
#include "Sptr.hxx"
#include "JSUserCallContainer.hxx"
#include "JSSipCallContainer.hxx"
#include "SipEvent.hxx"
#include "JSReqEvent.hxx"
#include "CallInfo.hxx"
#include "InviteMsg.hxx"
#include "AckMsg.hxx"
#include "JSCall.hxx"
#include "ProvisionInterface.hxx"
#include "JSStateMachine.hxx"
#include "JSUserRegContainer.hxx"
#include "SipTransceiverFilter.hxx"

/**
 *
 */

class JSCall;
class JSUserCallContainer;
class JSSipCallContainer;
class JSUserRegContainer;

class JSConfig
{
    public:
        JSConfig()
        {
            JSNsipPort = 15060;
            JSDsipPort = 25060;
            udpPort	= 9999;
            rtpPort	= 23456;
            userName	= "js";
        }

        ~JSConfig();

        ///
        void setJSNSipPort(Data port)
        {
            JSNsipPort = port;
        }
        Data getJSNSipPort()
        {
            return JSNsipPort;
        }
        ///
        void setJSDSipPort(Data port)
        {
            JSDsipPort = port;
        }
        Data getJSDSipPort()
        {
            return JSDsipPort;
        }
        ///
        void setUdpPort(Data port)
        {
            udpPort = port;
        }
        Data getUdpPort()
        {
            return udpPort;
        }
        ///
        void setRtpPort(unsigned short port)
        {
            rtpPort = port;
        }
        unsigned short getRtpPort()
        {
            return rtpPort;
        }
        ///
        void setpServerPort(unsigned short port)
        {
            pServerPort = port;
        }
        unsigned short getpServerPort()
        {
            return pServerPort;
        }
        void setpServerHost(string host)
        {
            pServerHost = host;
        }
        string getpServerHost()
        {
            return pServerHost;
        }

        ///
        void setRedirectHost(Data host)
        {
            redirectHost = host;
        }
        Data getRedirectHost()
        {
            return redirectHost;
        }
        void setRedirectPort(Data port)
        {
            redirectPort = port;
        }
        Data getRedirectPort()
        {
            return redirectPort;
        }

        ///
        void setUserName(Data user)
        {
            userName = user;
        }
        Data getUserName()
        {
            return userName;
        }

        ///
        void setBasicCookie(Data cookie)
        {
            basicCookie = cookie;
        }
        Data getBasicCookie()
        {
            return basicCookie;
        }
        ///
        void setProxyAuthKey(Data cookie)
        {
            basicCookie = cookie;
        }
        Data getProxyAuthKey()
        {
            return basicCookie;
        }
    private:
        Data JSNsipPort;
        Data JSDsipPort;
        Data udpPort;
        unsigned short rtpPort;
        unsigned short pServerPort;
        string pServerHost;
        Data redirectHost;
        Data redirectPort;
        Data userName;
        Data basicCookie;

};

/**Functinally similar to Base framework's Builder class, but we use
 * Event instead of SipProxyEvent, so can't inherite from the Builder class.
 * The name "Provider" is from JTAPI terminology; this is the class that
 * provides platform's (in this case, Vovida's softswitch) functionality to
 * JTAPI applications that are using Vovida's JTAPI implementation.
 * Provider object sits in the worker thread, receive messages, events,
 * finds corresponding call objects and pass the messages received to the
 * call object and invokes that call object's state machine to porcess
 * messages received.
 */
class JSProvider
{
    public:
        ///
        JSProvider();
        virtual ~JSProvider();

        JSProvider(const JSProvider& rhs);
        JSProvider& operator=(const JSProvider& rhs);

        ///
        void setConfig(JSConfig*config)
        {
            itsConfig = config;
        }
        JSConfig* getConfig()
        {
            return itsConfig;
        }

        /// Base framework Builder part
        //    virtual void process( const Sptr<SipProxyEvent> nextEvent );
        virtual void process( const Sptr < SipProxyEvent > nextEvent );

        void setUserCallContainer(const Sptr < JSUserCallContainer > userCallContainer);
        Sptr < JSUserCallContainer > getUserCallContainer();
        void setSipCallContainer(const Sptr < JSSipCallContainer > sipCallContainer);
        Sptr < JSSipCallContainer > getSipCallContainer();
        void setUserRegContainer(const Sptr < JSUserRegContainer > regContainer);
        Sptr < JSUserRegContainer > getUserRegContainer();


        ///
        void setJSNSipStack(const Sptr < SipTransceiverFilter > sipStack);
        void setJSDSipStack(const Sptr < SipTransceiverFilter > sipStack);
        Sptr < SipTransceiverFilter > getJSNSipStack()
        {
            return itsJSNSipStack;
        }
        Sptr < SipTransceiverFilter > getJSDSipStack()
        {
            return itsJSDSipStack;
        }
        void setClientUdpStack(const Sptr < UdpStack > udpStack);
        Sptr < UdpStack > getClientUdpStack()
        {
            return itsClientUdpStack;
        }


        ///
        bool isExist(string callID);

        /// TEMP
        /*
            void setToCallId(SipCallId& callId) { toCallId = callId; }
            SipCallId getToCallId() { return toCallId; }
            void setFromCallId(SipCallId& callId) { fromCallId = callId; }
            SipCallId getFromCallId() { return fromCallId; }
            void setFrom200(StatusMsg& fromStatusMsg) { from200 = fromStatusMsg; }
            StatusMsg getFrom200() { return from200; }
            void setTo200(StatusMsg& toStatusMsg) { to200 = toStatusMsg; }
            StatusMsg getTo200() { return to200; }
            void setFromUrl(Data& url) { fromUrl = url; }
            Data getFromUrl() { return fromUrl; }
            void setToUrl(Data& url) { toUrl = url; }
            Data getToUrl() { return toUrl; }
        */

    private:

        ///
        JSConfig* itsConfig;

        /// Base Framework Builder part
        Sptr < JSUserCallContainer > itsUserCallContainer;
        Sptr < JSSipCallContainer > itsSipCallContainer;
        Sptr < JSUserRegContainer > itsUserRegContainer;

        Sptr < JSStateMachine > itsStateMachine;
        ///
        Sptr < SipTransceiverFilter > itsJSNSipStack;
        Sptr < SipTransceiverFilter > itsJSDSipStack;
        Sptr < UdpStack > itsClientUdpStack;

        void copyRhsToThis( const JSProvider& rhs);

        /// TEMP
        SipCallId toCallId;
        SipCallId fromCallId;
        StatusMsg from200;
        StatusMsg to200;
        Data fromUrl;
        Data toUrl;

        /// Provisioning Server Stuff
        ///
        ProvisionInterface* pif ;
        string JtapiServerName;
        string pServerHost;
        int pServerPort;
        vector < VRedirectServerData > redirectServerUrls;
};

#endif
