
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSProxy_cxx_Version =
    "$Id$";
#include "JSProxy.hxx"

///
JSProxy::JSProxy( const Sptr < JSProvider > provider, unsigned short JSNSipPort, unsigned short JSDSipPort)
{
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: JSProxy c'tor started!");
    itsJSNSipStack = new SipTransceiverFilter("JSCalling", JSNSipPort);
    itsJSDSipStack = new SipTransceiverFilter("JSCalled", JSDSipPort);
    itsClientUdpStack = new UdpStack(0, provider->getConfig()->getUdpPort().convertInt());
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: UDP STACK STARTED");

    itsUserCallContainer = new JSUserCallContainer;
    itsSipCallContainer = new JSSipCallContainer;
    itsUserRegContainer = new JSUserRegContainer;
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: CALL CONTAINER STARTED");

    itsProvider = provider;
    itsProvider->setUserCallContainer( itsUserCallContainer );
    itsProvider->setSipCallContainer( itsSipCallContainer );
    itsProvider->setUserRegContainer( itsUserRegContainer );
    itsProvider->setJSNSipStack( itsJSNSipStack );
    itsProvider->setJSDSipStack( itsJSDSipStack );
    itsProvider->setClientUdpStack( itsClientUdpStack );

    itsCallProcessingQueue = new Fifo < Sptr < SipProxyEvent > > ;
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: FIFO QUEUE STARTED");

    itsWorkerThread = new JSWorkerThread(itsCallProcessingQueue, itsProvider);
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: WORKER THREAD STARTED");

    itsJSNSipThread = new SipThread(itsJSNSipStack, itsCallProcessingQueue);
    itsJSDSipThread = new SipThread(itsJSDSipStack, itsCallProcessingQueue);
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: SIP THREAD STARTED");

    itsCommThread = new CommThread(itsClientUdpStack, itsCallProcessingQueue);
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: COMM THREAD STARTED");

    itsJSNHeartbeatTxThread = new HeartbeatTxThread( provider->getConfig()->getJSNSipPort().convertInt() );
    itsJSDHeartbeatTxThread = new HeartbeatTxThread( provider->getConfig()->getJSDSipPort().convertInt() );
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: HERATBEAT THREAD STARTED");

    // Heart beat thread c'tor could hung for misconfiguration
    cpLog(LOG_DEBUG, "JSProxy::JSProxy: JSProxy c'tor completed!");
}

///
void
JSProxy::copyRhsToThis( const JSProxy& rhs )
{
    itsUserCallContainer = rhs.itsUserCallContainer;
    itsSipCallContainer = rhs.itsSipCallContainer;
    itsProvider = rhs.itsProvider;
    itsCallProcessingQueue = rhs.itsCallProcessingQueue;
    itsJSNSipStack = rhs.itsJSNSipStack;
    itsJSDSipStack = rhs.itsJSDSipStack;
    itsWorkerThread = rhs.itsWorkerThread;
    itsJSNSipThread = rhs.itsJSNSipThread;
    itsJSDSipThread = rhs.itsJSDSipThread;
    itsClientUdpStack = rhs.itsClientUdpStack;
    itsJSNUdpStack = rhs.itsJSNUdpStack;
    itsJSDUdpStack = rhs.itsJSDUdpStack;
    itsCommThread = rhs.itsCommThread;
}

///
JSProxy::JSProxy( const JSProxy& rhs )
{
    if ( this != &rhs)
        copyRhsToThis( rhs );
}

///
JSProxy&
JSProxy::operator=(const JSProxy& rhs )
{
    if ( this != &rhs)
        copyRhsToThis( rhs );

    return *this;
}

///
void
JSProxy::run()
{
    if (itsWorkerThread != 0) itsWorkerThread->run();
    if (itsJSNSipThread != 0) itsJSNSipThread->run();
    if (itsJSDSipThread != 0) itsJSDSipThread->run();
    if (itsCommThread != 0) itsCommThread->run();
    if (itsJSNHeartbeatTxThread != 0) itsJSNHeartbeatTxThread->run();
    if (itsJSDHeartbeatTxThread != 0) itsJSDHeartbeatTxThread->run();
}

///
void
JSProxy::join()
{
    if (itsWorkerThread != 0) itsWorkerThread->join();
    if (itsJSNSipThread != 0) itsJSNSipThread->join();
    if (itsJSDSipThread != 0) itsJSDSipThread->join();
    if (itsCommThread != 0) itsCommThread->join();
    if (itsJSNHeartbeatTxThread != 0) itsJSNHeartbeatTxThread->join();
    if (itsJSDHeartbeatTxThread != 0) itsJSDHeartbeatTxThread->join();
}

///
JSProxy::~JSProxy()
{}

// End of File
