/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VmCallContainer_cxx_Version =
    "$Id: VmCallContainer.cxx,v 1.11 2002/03/20 05:44:58 bko Exp $";

#include "VmCallContainer.hxx"
#include "LockHelper.hxx"
#include "VmCallInfo.hxx"
#include "CallInfo.hxx"

using namespace Vocal;

///
VmCallContainer::VmCallContainer()
{ 
}


///

VmCallContainer::~VmCallContainer()
{
}

/// Overload base function to instantiate VmCallInfo objects

Sptr < CallInfo >
VmCallContainer::getCall( Sptr < SipCallLeg > newOrExistingCallLeg )
{
    assert ( newOrExistingCallLeg != 0 );

    TableIter iter;
    Sptr < CallInfo > newCall = 0;
    Sptr < VmCallInfo > newVmCall = 0;

    LockHelper lockHelper(myMutex);

    {
        iter = myCallInfo.find( newOrExistingCallLeg );

        if ( iter != myCallInfo.end() )
        {
            newCall = iter->second;
        }
        else
        {
            newVmCall = new VmCallInfo;
            newCall = newVmCall;
            myCallInfo[newOrExistingCallLeg] = newCall;
        }
    }

    assert ( newCall != 0 );

    return newCall;
}




/// Overload base function to put back the uavm port in the
/// list of available ports when the call is cleared
void
VmCallContainer::removeCall ( Sptr < SipCallLeg > existingCall )
{
    assert ( existingCall != 0 );

    // Free the voicemail port before freeing the call information
    Sptr < CallInfo > callInfoPtr = getCall( existingCall);
    Sptr < VmCallInfo > vmCallInfo;
    vmCallInfo.dynamicCast(callInfoPtr);
    VmPortContainer::instance()->appendVmPort(vmCallInfo->getVmPort());

    // Remove the Call Info from the map
    TableIter iter;

    LockHelper lockHelper(myMutex);

    iter = myCallInfo.find( existingCall );

    if ( iter != myCallInfo.end() )
    {
        myCallInfo.erase ( iter );
    }
    else
    {
        cpLog( LOG_DEBUG, "callInfo not found" );
    }
}


// End of File
