/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VmProvisioning_cxx_Version =
    "$Id: VmProvisioning.cxx,v 1.18 2002/03/20 05:44:58 bko Exp $";

#include "VmProvisioning.hxx"
#include "VException.hxx"
#include "FsConfigFile.hxx"
#include <vector>
#include "Data.hxx"
#include <cpLog.h>
#include <iostream.h>  //ifstream testing
#include <fstream.h>  //ifstream testing
#include "CPLStartStateHolder.hxx"
#include "SystemInfo.hxx"

#include "VmPortContainer.hxx"
#include "State.hxx"
#include "CPLFeatureBuilder.hxx"
//#include <cerrno>

using namespace Vocal;

//////////////SINGLETON IMPLEMENTATION//////////////
VmProvisioning* VmProvisioning::myInstance = NULL;
////////////////////////////////////////////////////

//////////////CONSTANTS/////////////////////////////
static int PortNumber = 0;
static string FSfilename("");
////////////////////////////////////////////////////


///
VmProvisioning::VmProvisioning()
{
    // Feature Server local port
    extern int sipPort;

    //Verify PortNumber and Feature Server name are
    //defined.  These are needed to use Provisioning
    //Server's interface.
    PortNumber = sipPort;
    cpLog( LOG_DEBUG, "FS Port Number=%d", PortNumber );

    FeatureServerName = theSystem.gethostAddress();

    // Initialize FSfilename to <IP>:<port>
    char buf[126];
    sprintf(buf, "%s:%d", FeatureServerName.c_str(), PortNumber);
    FSfilename = buf;

    if (FeatureServerName.length() == 0)
    {
        cpLog( LOG_ERR, "Feature server name could not be retrieved" );
        assert( FeatureServerName.length() > 0 );
    }
    else
        cpLog( LOG_DEBUG, "FS Name=%s", FeatureServerName.c_str() );

    try
    {
        //Retrieve pointer to Provision Interface singleton
        pif = &ProvisionInterface::instance(pServerHost.c_str(), pServerPort, pServerAltHost.c_str(), pServerAltPort);

        pif ->registerForUpdate( updateUavmServers, FSfilename, ProvisionInterface::VFEATURE);
    } /*try*/
    catch (VException& e)
    {
        cpLog( LOG_ERR, "Network exception: %s", e.getDescription().c_str() ); 
        cpLog( LOG_ERR, "Provisioning Server problems.  Is PS running?" );
    } /*catch*/

    updateUavmServers("", FSfilename, true);
}  // VmProvisioning




///
VmProvisioning*
VmProvisioning::Instance()
{
    if ( myInstance == 0 )
    {
        myInstance = new VmProvisioning;
    } /*if*/

    return myInstance;

}  // Instance


///
VmProvisioning::~VmProvisioning()
{}  // VmProvisioning


///

void
VmProvisioning::initialize()
{
    cpLog(LOG_DEBUG, "initialize()");

    // Create the vm.cpl script (always the same)
    createVmCplScript();

} // initialize



///
void
VmProvisioning::createVmCplScript()
{
    string cplScript;

    // Create vm.cpl
    cplScript = "<?xml version=\"1.0\" ?> <!DOCTYPE cpl SYSTEM \"cpl.dtd\"> <cpl> <incoming> <vmredirect/> </incoming> </cpl> " ;

    //Parse the script, store any user data, and create
    //the feature.
    if (!createFeatureFromCPL( cplScript) )
        cpLog( LOG_ERR, "Could not create feature for '%s'.",
               cplScript.c_str() );

}  //createVmCplScript



///
bool
VmProvisioning::createFeatureFromCPL( const string& cplScript)
{

    //Interpret the CPL script to feature and store user
    //data with the feature.
    Sptr < CPLStartStateHolder > featureStartState;
    featureStartState = CPLFeatureBuilder::Instance()->create(cplScript);

    if (featureStartState == 0)
        return false;

    Sptr < FsFeature > newFeature = new FsFeature;

    newFeature->addState( featureStartState->getIncoming() );
    VmFeature = newFeature;
    return (true);

} //createFeatureFromCPL


Sptr < FsFeature >
VmProvisioning::getVmFeature()
{
    return VmFeature;
}


//////////////////////////////////
// Callback //////////////////////
//////////////////////////////////

// The content of the file is passed by the callback function but not needed
// as we just re-parse the entire file
void updateUavmServers(const string& data, const string& fileName, bool delFlag)
{
    VFeatureServerData featureServerData;

    cpLog( LOG_DEBUG, "updateUavmServers triggered");
    // Empty the Uavm ports list
    VmPortContainer::instance()->clearAll();

    try
    {
      //Retrieve pointer to Provision Interface singleton
      ProvisionInterface& pif = ProvisionInterface::instance(pServerHost.c_str(), pServerPort, pServerAltHost.c_str(), pServerAltPort);
      pif.getFeatureData(fileName, featureServerData);

      vector < VSuavmServerData > suavmServerList;
      featureServerData.getVMList(suavmServerList );

      vector < VSuavmServerData > ::iterator suavmList = suavmServerList.begin();
      vector < VSuavmServerData > ::iterator listEnd = suavmServerList.end();
      while ( suavmList != listEnd )
      {
          cpLog(LOG_DEBUG, "Suavm Server: %s", suavmList->getHost().c_str() );
          if (addVmPorts(suavmList->getHost(),
                         suavmList->getFirstPort(),
                         suavmList->getLastPort()))
              cpLog( LOG_DEBUG, "vm ports added to vm ports pool") ;
          ++suavmList;
      } /*while*/
    } /* try */
    catch (VException& e)
    {
        cpLog( LOG_ERR, "Network exception: %s", e.getDescription().c_str() ); 
        cpLog( LOG_ERR, "Provisioning Server problems.  Is PS running?" );
    } /*catch*/  

} // updateUavmServers


/// addVmPorts: 
int addVmPorts(const string& vmHost, int firstPort, int lastPort)
{
    cpLog(LOG_DEBUG, "addVmPort");

    if (vmHost.size() == 0)
    {
        cpLog( LOG_ERR, "No VM server configured. Exit " );
        return false;
    }
    else
        if (firstPort == 0)
        {
            cpLog( LOG_ERR, "No first VM ports configured. Exit " );
            return false;
        }
        else if (lastPort == 0)
        {
            cpLog( LOG_ERR, "No last VM port configured. Exit" );
            return false;
        }
        else
        {
            vmPort vmPortStruct;

            vmPortStruct.host = vmHost;

            int port;

            // save the total number of configured ports
            port = (lastPort - firstPort) + 1;
            VmPortContainer::instance()->setNumberOfConfiguredPorts(port);

            for (port = firstPort; port <= lastPort ; port++)
            {
                vmPortStruct.port = port;
                VmPortContainer::instance()->appendVmPort(vmPortStruct);
            }
            VmPortContainer::instance()->logData();
            return true;
        }

}

// End of File
