#ifndef VmProvisioning_hxx
#define VmProvisioning_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const FsProvisioningVersion =
    "$Id: VmProvisioning.hxx,v 1.12 2002/03/20 05:44:58 bko Exp $";

#include <string>
#include <Sptr.hxx>
#include "ProvisionInterface.hxx"
#include "FsFeature.hxx"

/// Retrieve the configuration information (uavm hosts and ports) from the provisioning interface and populate the uavm container (callback function)
void updateUavmServers(const string& data, const string& fileName, bool);

/// From the configuration information, populate the container with all the free ports available (callback function)
int addVmPorts(const string& vmHost, int firstPort, int lastPort);


/**
 * This class is the interface to contact provisioning Server for data.
 * Note: it can't derive from FsProvisioning as default constructor does
 * not match Vm purposes.
 */
class VmProvisioning
{
    public:
        /// type of features
        typedef enum Features { FeaturesIncoming, FeaturesOutgoing };

        /// method to access singleton instance
        static VmProvisioning* Instance();

        /// destructor
        ~VmProvisioning();

        /// initialization method
        void initialize();

        /// method to access VM feature
        Sptr < Vocal::FsFeature > getVmFeature();

    protected:

        /**method to creat feature from CPL scripts
         *@param cplScript the CPL cript
         *@return bool result if feature constructed successfully
         */
        bool createFeatureFromCPL( const string& cplScript);

        /** Creates the vm.cpl script for fsvm: as it is always the same
	 *  script there is no need to retrieve it from provisioning.
         */        
        void createVmCplScript();

        /**method to read a file from provisioning server
         *@param fileName name of the file to read
         *@return read string from file
         */
        string readFile(const string& fileName);

    private:
        /// private constructor for singleton implementation
        VmProvisioning();

        /// reference to the singleton instance
        static VmProvisioning* myInstance;

        /// reference to the provisioning interface
        ProvisionInterface* pif ;

        /// name of the feature server
        string FeatureServerName;

        /// reference to the VM feature
        Sptr < Vocal::FsFeature > VmFeature;

};

#endif

//End of File
