#ifndef _MRSHL_CONF_PARTICIPANT_INFO_HXX_
#define _MRSHL_CONF_PARTICIPANT_INFO_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MrshlConfParticipantInfo_hxx_Version =
    "$Id: MrshlConfParticipantInfo.hxx,v 1.6 2002/03/19 21:39:47 jason Exp $";



#include "BasicProxy.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"


namespace Vocal
{

/** MrshlConfParticipantInfo is a class used to store/access data on
 *  an a participant in a conference
 */
class MrshlConfParticipantInfo
{
    public:

        /// Constructor initializes data members
        MrshlConfParticipantInfo();

        /**
         * Copy constructor
         * @param rhs the info object to copy
         */
        MrshlConfParticipantInfo( const MrshlConfParticipantInfo& rhs );

        /**
         * Assignment Operator
         * @param rhs the info object to assign
         * @return *this
         */
        const MrshlConfParticipantInfo& operator=( 
	    const MrshlConfParticipantInfo& rhs );

        /// Destructor Method
        virtual ~MrshlConfParticipantInfo();

        /// Accessor methods
        ///
        Data getUser() const
        {
            return myUser ;
        }
        ///
        InviteMsg getInviteMsg() const
        {
            return myInviteMsg ;
        }
        ///
        StatusMsg getStatusMsg() const
        {
            return myStatusMsg ;
        }

        /// Member assignment methods
        ///
        void setUser( const Data& newUser )
        {
            myUser = newUser ;
        }
        ///
        void setInviteMsg( const InviteMsg& msg )
        {
            myInviteMsg = msg ;
        }
        ///
        void setStatusMsg( const StatusMsg& msg )
        {
            myStatusMsg = msg ;
        }


    private:

        /**
         * Copy method copies class members
         * @param rhs the entry to copy
         * @return void
         */
        void copyRhsToThis( const MrshlConfParticipantInfo& rhs );

        /// participant's user data from from: message field
        Data myUser;

        /// saved messages used to terminate call
        InviteMsg myInviteMsg;
        StatusMsg myStatusMsg;
};
 
}

#endif //  _MRSHL_CONF_PARTICIPANT_INFO_HXX_
