#ifndef MRSHL_OP_STATUS_HXX
#define MRSHL_OP_STATUS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const MrshlOpStatus_hxx_Version =
    "$Id: MrshlOpStatus.hxx,v 1.29 2002/03/19 21:39:47 jason Exp $";


#include "Operator.hxx"
#include "State.hxx"

#include "SipEvent.hxx"
#include "StatusMsg.hxx"

namespace Vocal
{

class SipContact;

/// MrshlOpStatus, deals with the status message 
class MrshlOpStatus : public Vocal::Operator
{
    public:
        /**
         * Default constructor
         */
        MrshlOpStatus();

        /**
         * Destructor
         */
        virtual ~MrshlOpStatus();

        /**
         * Returns class name string
         * @return char pointer
         */
        virtual const char* const name() const;

        /**
         * Handles the sip message
         * @param event the SipEvent that contains the Sip Message
	 * @return Sptr to State
         */
        virtual const Sptr <Vocal::State>
        process( const Sptr < Vocal::SipProxyEvent > event );

        /**
         * @param sipEvent event through which the status msg is to be sent
	 * @param statusMsg status message to be forwarded
         */
        static void forwardBasedOnVia( Sptr < Vocal::SipEvent > sipEvent,
                                       Sptr < StatusMsg > statusMsg );

        /** takes a Contact list and a flag indicating whether or not to
         ** retrieve the first Contact;
         ** returns a ptr to the appropriate Contact or 0 if not found
	 *
	 * @param contactList Contact List from which to retrieve self
	 * @param getFristContact boolean flag, true to retrieve the first
	 * @return SipContact of self in list
         */
        static Sptr<SipContact> getContactFromList( const SipContactList& contactList,
						    bool getFirstContact = true );

    private:
        /**
         * @param sipEvent event contains the INVITE message
	 * @param statusMsg status message to be set in ACK
         */
        void ackBasedOnCommand( Sptr < Vocal::SipEvent > sipEvent,
				Sptr < StatusMsg > statusMsg );

        /**
         * @param sipEvent event contains the InviteMsg
	 * @statusMsg status message contains caller info
         */
        void process180183( Sptr < Vocal::SipEvent > sipEvent,
                            Sptr < StatusMsg > statusMsg );

        /**
         * @param sipEvent event to be forwarded
	 * @param status status message to be examined and processed
         */
        void process200( Sptr < Vocal::SipEvent > sipEvent,
                         Sptr < StatusMsg > statusMsg );
        /**
         * @param sipEvent event to be acked or forwardde
	 * @statusMsg status message contains caller info
         */
        void process302( Sptr < Vocal::SipEvent > sipEvent,
                         Sptr < StatusMsg > statusMsg );

        /**
         * @param sipEvent event to be acked or forwarded
	 * @statusMsg status message contains caller info
         */
        void processDefault( Sptr < Vocal::SipEvent > sipEvent,
                         Sptr < StatusMsg > statusMsg );    
};
 
}

#endif //_MRSHL_OP_STATUS_HXX_
