#ifndef GROUP_HXX
#define GROUP_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "Data.hxx"
#include "RSCommon.h"

static const char* const GroupVersion = "$Id: Group.hxx,v 1.9 2001/02/03 01:29:01 maivu Exp $";

/**
 *  A group defines a collection of servers. The servers are
 *  all of the same type which is stored in this object along
 *  with a name for the group.
 */
class Group
{
    public:
        /**
         * Constructor
         * @param  type  Type of servers contained in this group.
         * @param  name  Unique identifier for this group.
         */
        Group( const ServerType type, const Data& name);

        /**
         * Copy Constructor
         * @param  rhs  Object from which to copy
         */
        Group( const Group& rhs );

        /**
         * Operator overload for "="
         * @param  rhs  Object from which to copy
         * @return  Reference to this object with new data
         */
        Group& operator=( const Group& rhs );

        /// Destructor
        virtual ~Group();

        /**
         * Obtain the type of servers belonging to this group
         * @return  Server type
         */
        const ServerType getType() const;

        /**
         * Obtain the unique identifier for this group
         * @return  Unique identifier for group
         */
        const Data getName() const;

    protected:
        /**
         * Store type of servers belonging to this group
         * @param  type  Type of server
         */
        void setType (const ServerType type);

        /**
         * Store this group's unique identifier
         * @param  name  Unique identifier for group
         */
        void setName (const Data& name);

        /// Type of servers belonging to this group
        ServerType itsType;

        /// Unique identifier for this group
        Data itsName;

    private:
        /**
         * Copy member data from given object into this object.
         * @param  rhs  Object from which to copy
         */
        void copyRhsToThis( const Group& rhs );
};

#endif //GROUP_HXX
