
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const RedundancyEvent_cxx_Version =
    "$Id$";
#include "RedundancyEvent.hxx"

using namespace Vocal;

/**
 * Constructor
 * @param  sender  Address of server initiating the message.
 * @param  info  Contains message type
 * @param  eventId  Unique identifier
 * @param  outputFifo 
 */
RedundancyEvent::RedundancyEvent( const Sptr < Data > sender, const InfoType info,
                                  const FifoEventId eventId, const Sptr < Fifo < Sptr < SipProxyEvent > > > outputFifo )
{
    assert ( outputFifo != 0 );

    itsSender = new Data;
    *itsSender = *sender;
    itsInfo = info;
    itsEventId = eventId;
    myFifo = outputFifo;
}

/**
 * Obtain the sender information.
 * @return  Smart pointer to this object's sender data.
 */
const Sptr < Data >
RedundancyEvent::getSender() const
{
    if (0 == itsSender)
    {
        cpLog(LOG_ERR, "Sender == 0");
    }
    return itsSender;
}


/**
 * Obtain the info data.
 * @return  The info portion of this object.
 */
const InfoType
RedundancyEvent::getInfo() const
{
    return itsInfo;
}

/**
 * Store the sender data into this object.
 * @param  sender  Address of server initiating the message.
 */
void
RedundancyEvent::setSender( const Sptr < Data > sender )
{
    if (0 != sender)
      *itsSender = *sender;
}


/**
 * Store the info passed in into this object.
 * @param  info  Typically a message type.
 */
void
RedundancyEvent::setInfo( const InfoType info )
{
    itsInfo = info;
}


/**
 * Store the passed in event id.
 * @param  eventId  Unique identifier
 */
void
RedundancyEvent::setEventId( const FifoEventId eventId )
{
    itsEventId = eventId;
}


/**
 * Obtain the event id data
 * @return  This object's event id
 */
FifoEventId
RedundancyEvent::getEventId( ) const
{
    return itsEventId;
}


/**
 * Destructor
 */
RedundancyEvent::~RedundancyEvent()
{}

// End of File
