
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const RedundancyRxThread_cxx_Version =
    "$Id: RedundancyRxThread.cxx,v 1.18 2002/03/19 21:39:49 jason Exp $";

#include "cpLog.h"
#include "RedundancyRxThread.hxx"
#include "RedundancyEvent.hxx"
#include "RSCommon.h"
#include "SystemInfo.hxx"

using namespace Vocal;

const unsigned short int BUFFERSIZE = 256;

/**
 * Constructor
 * @param  redundancyFifo  Fifo containing redundancy events.
 * @param  redundancyPort  Port on which redundancy messages are received.
 */
RedundancyRxThread::RedundancyRxThread( const Sptr < Fifo < Sptr < SipProxyEvent > > > redundancyFifo,
                                        unsigned short redundancyPort /* HAS DEFAULT */ )
        : itsRedundancyFifo(redundancyFifo)
{
    assert ( itsRedundancyFifo != 0 );

    itsUdpStack = new UdpStack(0, redundancyPort, redundancyPort, sendrecv);
    assert( itsUdpStack != 0);
}

/**
 * Destructor 
 */
RedundancyRxThread::~RedundancyRxThread()
{ }

/**
 * Process events from the udp stack.
 */
void
RedundancyRxThread::thread()
{
    int bytesRead = 0;
    char buf[BUFFERSIZE];
    NetworkAddress sender;

    while (true)
    {
        try
        {
            bytesRead = itsUdpStack->receiveFrom(buf, BUFFERSIZE, &sender);

            redundancyPortData* portData = (redundancyPortData*) & buf;
              
            if (0 == portData)
            {
                cpLog(LOG_ERR, "Bad redundancy port data ");
                continue;
            }

            Sptr < Data > serverAddress = new Data( sender.getIpName() );
            *serverAddress += ":";
            *serverAddress += ntohs(portData->port);

            if ( static_cast <u_int16_t> (RedundancyData) == ntohs(portData->dataType) )
            {
                Sptr < RedundancyEvent > redundancyEvent = new RedundancyEvent( serverAddress,
                        (InfoType)ntohs(portData->info),
                        (FifoEventId)ntohl(portData->misc),
                        itsRedundancyFifo);

                redundancyEvent->postEvent(redundancyEvent);
            }
            else
                // Provisioning may be notifying us of a change in the redundancy port.
                if ( static_cast <u_int16_t> (MaintenanceData) == ntohs(portData->dataType) )
                {
                    unsigned short redundancyPort( ntohl(portData->misc) );
                    cpLog( LOG_DEBUG, "Replacing Udp Stack with new redundancy port [%d]", redundancyPort);
                    itsUdpStack = new UdpStack(0, redundancyPort, redundancyPort, sendrecv);
                }
                else
                    cpLog( LOG_ERR, "unrecognized message received on redundancy port");

        }
        catch ( ... )
        {
            cpLog( LOG_ERR, "Exception from udpstack for RedundancyRx" );
        }
    }

}
// End of File
