
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include <sys/time.h>
#include <cstdlib>
#include <fstream>
#include <stdio.h>
#include <string.h>

#include "UdpStack.hxx"
#include "NetworkAddress.h"
#include "Data.hxx"

#define TIMEOUT 10
string ua1phone("<UA1PHONE>");
string ua2phone("<UA2PHONE>");
string ua1ip("<UA1IP>");
string ua2ip("<UA2IP>");
string ua1port("<UA1PORT>");
string ua2port("<UA2PORT>");
string ms1ip("<MS1IP>");
string ms2ip("<MS2IP>");
string ms1port("<MS1PORT>");
string ms2port("<MS2PORT>");
string rsip("<RSIP>");
string rsport("<RSPORT>");


string uaInvite(
"INVITE sip:<UA1PHONE>@<MS1IP>:<MS1PORT>;user=phone  SIP/2.0
Via: SIP/2.0/UDP <UA1IP>:<UA1PORT>
From:<UA1PHONE><sip:<UA1PHONE>@<UA1IP>:<UA1PORT>;user=phone>
To: <UA2PHONE><sip:<UA2PHONE>@<UA2IP>:<UA2PORT>;user=phone>
Call-ID: #@<UA1IP>
CSeq: 100 INVITE
Route: <sip:<MS2IP>:<MS2PORT>;maddr=<MS2IP>> , <sip:<UA2IP>:<UA2PORT>;maddr=<UA2IP>mv >
Subject: VovidaINVITE
Contact: <sip:<UA1PHONE>@<UA1IP>:<UA1PORT>>
Content-Type: application/sdp
Content-Length: 170
v=0
o=- 541997401 541997401 IN IP4 172.19.175.117
s=VOVIDA Session
c=IN IP4 172.19.175.117
t=3191924312 0
m=audio 12345 RTP/AVP 0
a=rtpmap:0 PCMU/8000
a=ptime:20

");

string uaInvite200 (
    "SIP/2.0 200 OK
");

string uaAck (
"ACK sip:<UA1PHONE>@<MS1IP>:<MS1PORT>;user=phone  SIP/2.0
Via: SIP/2.0/UDP <UA1IP>:<UA1PORT>
From: <UA1PHONE><sip:<UA1PHONE>@<UA1IP>:<UA1PORT>;user=phone>
To: <UA2PHONE><sip:<UA2PHONE>@<UA2IP>:<UA2PORT>;user=phone>
Call-ID: #@<UA1IP>
CSeq: 100 ACK
Route: <sip:<MS2IP>:<MS2PORT>;maddr=<MS2IP>>,<sip:<UA2IP>:<UA2PORT>;maddr=<UA2IP>>
Content-Length: 0
"); 

string uaBye (
"BYE sip:<UA1PHONE>@<MS1IP>:<MS1PORT>;user=phone  SIP/2.0
Via: SIP/2.0/UDP  <UA1IP>:<UA1PORT>
From:<UA1PHONE><sip:<UA1PHONE>@<UA1IP>:<UA1PORT>;user=phone>
To: <UA2PHONE><sip:<UA2PHONE>@<MS1IP>:<MS1PORT>;user=phone>
Call-ID: #@<UA1IP>
Route: <sip:<MS2IP>:<MS2PORT>;maddr=<MS2IP>>,<sip:<UA2IP>:<UA2PORT>;maddr=<UA2IP>>
CSeq: 101 BYE
Content-Length: 0
"); 

NetworkAddress msAddress;
string ua1Str, ua2Str, rsStr, ms1Str, ms2Str;
string serverType;
UdpStack txStack, rxStack;
bool debugOn = false;
const int bufSize = 1024;

string
replaceString(const string &baseMsg, const string &oldStr, const string &newStr)
{
    const int baseSeqPos = baseMsg.find(oldStr);
    if (baseSeqPos == -1) {
	return baseMsg;
    }
    string tmpStr(baseMsg.substr(0, baseSeqPos) + newStr + baseMsg.substr(baseSeqPos + oldStr.length()));

    return tmpStr;
}

string
replaceAllStrings(const string &baseMsg, const string &oldStr, const string &newStr)
{
    string tmpStr1, tmpStr2 = baseMsg;
    while (tmpStr1 != tmpStr2)
    {
	tmpStr1 = tmpStr2;
	tmpStr2 = replaceString(tmpStr2, oldStr, newStr);
    }
    return tmpStr2;
}

int
replaceAllMessages(const string &oldStr, const string &newStr)
{
    uaInvite = replaceAllStrings(uaInvite, oldStr, newStr);
    uaAck = replaceAllStrings(uaAck, oldStr, newStr);
    uaBye = replaceAllStrings(uaBye, oldStr, newStr );
    return 1;
}


// This function reads the input file and changes the sip messages accordingly.
// the format of the input file is:
// UA1 <phoneNumber@ip:port>
// MS1 <phoneNumber@ip:port>
// MS2 <ip:port>
// UA2 <ip:port>
// RS  <ip:port>
//
int
readInputFile(string fileName)
{
    char aLine[1024];

    ifstream inFile(fileName.c_str());
    if (!inFile) {
	cout << "can not open input file "<<fileName<<endl;
	return 0;
    }
    memset(aLine, 0, sizeof(aLine));
    while(!inFile.eof() && inFile.getline(aLine, sizeof(aLine), '\n'))
    {
	string tmpStr(aLine);
	if (tmpStr.find("UA1") == 0 )
	{
	    ua1Str = tmpStr.substr(4);
	}
	if (tmpStr.find("UA2") == 0)
	{
	    ua2Str = tmpStr.substr(4);
	}
	if (tmpStr.find("MS1") == 0)
	{
	    ms1Str = tmpStr.substr(4);
	}
	if (tmpStr.find("MS2") == 0 )
	{
	    ms2Str = tmpStr.substr(4);
	}
	if (tmpStr.find("RS") == 0)
	{
	    rsStr = tmpStr.substr(3);
	}

    }
	// replace all string in all messages.
    const int ua1Pos1 = ua1Str.find("@");
    const int ua1Pos2 = ua1Str.find(":");
    string strUa1Port = ua1Str.substr(ua1Pos2 + 1);
    replaceAllMessages(ua1phone, ua1Str.substr(0, ua1Pos1));
    replaceAllMessages(ua1ip, ua1Str.substr(ua1Pos1 + 1, ua1Pos2 - ua1Pos1 - 1) );
    replaceAllMessages(ua1port, strUa1Port);

    const int ua2Pos1 = ua2Str.find("@");
    const int ua2Pos2 = ua2Str.find(":");
    string strUa2Port = ua2Str.substr(ua2Pos2 + 1);
    replaceAllMessages(ua2phone, ua2Str.substr(0, ua2Pos1));
    replaceAllMessages(ua2ip, ua2Str.substr(ua2Pos1 + 1, ua2Pos2 - ua2Pos1 - 1) );
    replaceAllMessages(ua2port, strUa2Port);

    const int ms1Pos = ms1Str.find(":");
    string strMs1Port = ms1Str.substr(ms1Pos + 1);
    replaceAllMessages(ms1ip, ms1Str.substr(0, ms1Pos));
    replaceAllMessages(ms1port, strMs1Port);

    const int ms2Pos = ms2Str.find(":");
    string strMs2Port = ms1Str.substr(ms2Pos + 1);
    replaceAllMessages(ms2ip, ms2Str.substr(0, ms2Pos));
    replaceAllMessages(ms2port, strMs2Port);

    const int rsPos = rsStr.find(":");
    string strRsPort = ms1Str.substr(rsPos + 1);
    replaceAllMessages(rsip, rsStr.substr(0, rsPos));
    replaceAllMessages(rsport, strRsPort);

    int intTmpPort;
    if (serverType == "UA1") {
	intTmpPort = atoi(strUa1Port.c_str());
	msAddress.setHostName( ms1Str);
//	msAddress.setPort( atoi(strMs1Port.c_str()));
    } else if (serverType == "UA2") {
	intTmpPort = atoi(strUa2Port.c_str());
	msAddress.setHostName( ms2Str);
//	msAddress.setPort( atoi(strMs2Port.c_str()));
    } else if (serverType == "RS") {
	intTmpPort = atoi(strRsPort.c_str());
    } else if (serverType == "MS") {
	intTmpPort = atoi(strMs2Port.c_str());
    } else {
	cout << "Error in parm #2" << endl;
	return 0;
    }
    rxStack.setLocal(intTmpPort, intTmpPort);

//     cout << endl << " ********* uaInvite ********* " <<endl << uaInvite << endl;
//     cout << endl << " ********* uaAck ********* " <<endl << uaAck << endl;
//     cout << endl << " ********* uaBye ********* " <<endl <<uaBye  << endl;

}

string
addCId(const string &baseMsg, const string &cSeq)
{
    const int baseSeqPos = baseMsg.find("#");
    string tmpStr(baseMsg.substr(0, baseSeqPos) + cSeq + baseMsg.substr(baseSeqPos + 1));
    return tmpStr;
}

string
findCId(const string &baseMsg)
{
    const int pos1 = baseMsg.find("Call-ID:") + 9;
    const int pos2 = baseMsg.find("@", pos1);
    return baseMsg.substr(pos1, pos2-pos1);
}

string
addBranch(const string &baseMsg, const string &branch)
{
    string tmpStr = baseMsg;
    const int tsP1 =  tmpStr.find("branch=") + 8;
    tmpStr = tmpStr.substr(0, tsP1 - 1) + branch + tmpStr.substr(tsP1);
    return tmpStr;
}
string
findBranch(const string &rxString)
{
    const int rxBrStart = rxString.find("branch=") + 7;
    const int rxBrLen = rxString.find( "Via:", rxBrStart) - rxBrStart;
    string branch = rxString.substr(rxBrStart, rxBrLen - 1);
    return branch;
}

int
ua1Thread(int numOfMessages)
{
    char rxBuff[bufSize];
    int size;
    div_t tmpDiv;
    NetworkAddress tmpAddress;

    struct timeval tvStart, tvEnd;
    struct timezone tz;

    tz.tz_minuteswest = 0;
    tz.tz_dsttime = 0;
    timerclear(&tvStart);
    if (gettimeofday(&tvStart,&tz) > 0 ) {
	perror("gettimeofday 1 failed:");
	return 0;
    }

    for ( int i = 1; i <= numOfMessages; i++ )
    {
	Data iData(tvStart.tv_sec+i);
	string iStr = iData;
	//different INVITES, different TransactionIds.
	string tmpInvite = addCId(uaInvite, iStr);
	if (debugOn) {
	    cout << "*********** Sending invite  ********"  <<endl<< tmpInvite << endl;
	}

	txStack.transmitTo (tmpInvite.c_str(), tmpInvite.length(), &msAddress);

	memset(rxBuff, 0, sizeof(rxBuff));
	size = rxStack.receiveTimeout(rxBuff, bufSize, &tmpAddress, TIMEOUT,0 );
	if (size <= 0) {
	    cout << endl << "   Receive TimeOut:  ENDING TEST !!!!!" << endl;
	    return 0;
	}

	string rxString(rxBuff, size);
	// while the message isn't the ack to the bye we send out
	while (!(rxString.find("SIP/2.0 200 OK") == 0 && rxString.find("CSeq: 101 BYE") != (ulong)-1) ) {
	    if (rxString.find("SIP/2.0 200 OK") == 0) {
		if (debugOn) {
		    cout << "*********** recieved 200 for Invite *******"  <<endl<< rxString << endl;
		}
		string cseq = findCId(rxString);
		string tmpStr = addCId(uaAck, cseq);
		if (debugOn) {
		    cout << " ********* Sending ack ********"  <<endl<< tmpStr << endl;
		}
		txStack.transmitTo (tmpStr.c_str(), tmpStr.length(), &msAddress);
		
		tmpStr = addCId(uaBye, cseq);
		if (debugOn) {
		    cout << " ********* Sending bye ********"  <<endl<< tmpStr << endl;
		}
		txStack.transmitTo (tmpStr.c_str(), tmpStr.length(), &msAddress);

		memset(rxBuff, 0, sizeof(rxBuff));
		size = rxStack.receiveTimeout(rxBuff, bufSize, &tmpAddress, TIMEOUT,0 );
		if (size <= 0) {
		    cout << endl << "   Receive TimeOut waiting for 200:  ENDING TEST !!!!!" << endl;
		    return 0;
		}
		rxString.assign(rxBuff);
	    } else {
		if (debugOn) {
		    cout << "*********** recieved non-200  *******"  <<endl<< rxString << endl;
		}
		memset(rxBuff, 0, sizeof(rxBuff));
		size = rxStack.receiveTimeout(rxBuff, bufSize, &tmpAddress, TIMEOUT,0 );
		if (size <= 0) {
		    cout << endl << "   Receive TimeOut waiting for 200:  ENDING TEST !!!!!" << endl;
		    return 0;
		}
		rxString.assign(rxBuff);
	    }

	}
	if (debugOn) {
	    cout << "*********** recieved 200 for Bye *******"  <<endl<< rxString << endl;
	    cout << "***********  TRANSACTION "<< i << " COMPLETE ********" <<endl;
	    
	}
	tmpDiv = div(i, 25);
	if (tmpDiv.rem == 0) {
	    cout << "processed "<< i << " messages"<< endl;
	}
	    
    }//end for.

    timerclear(&tvEnd);
    if (gettimeofday(&tvEnd,&tz) > 0 ) {
	perror("gettimeofday 2 failed:");
	return 0;
    }

    cout << "total real time: " << tvEnd.tv_sec - tvStart.tv_sec << " seconds" << endl;
    
    return 0;
}

int
ua2Thread()
{
    char rxBuff[bufSize];
    int size;
    string rxString;

    while (true) {

        memset(rxBuff, 0, sizeof(rxBuff));
	size = rxStack.receive(rxBuff, bufSize);
	rxString.assign(rxBuff);

	if (rxString.find("INVITE sip:") == 0 ||rxString.find("BYE sip:") == 0 ) {
	    if (debugOn) {
		cout << "*********** recieved Invite / Bye *******"  <<endl<< rxString << endl;
	    }
		// send 200		
	    int pos1 = rxString.find("Via:");
	    int pos2 = rxString.find("Content-Length:") + 16;
	    string tmpStr = uaInvite200 +  rxString.substr(pos1, pos2-pos1) + " 0";
	    if (debugOn) {
		cout << "********* Sending 200 ********" << endl << tmpStr << endl;
	    }
	    txStack.transmitTo (tmpStr.c_str(), tmpStr.length(), &msAddress);
	} else {
	    if (debugOn) {
		cout << "*********** recieved Other Msg *******"  <<endl<< rxString << endl;
	    } 
	}
    }
    return 1;
}


///
/// This test program is to test a single MS with 2 driver UAs
///         UA1-->MS-->UA2
int
main( int argc, char* argv[] )
{
    if ( argc < 3)
    {
        cerr << "Usage: "
        << argv[0]
        << " <inputFileName>"
	     << " < server type UA1|UA2>"
        << " <#messages> "
	<< " [ DEBUG ] "
        << endl;
    return ( -1 );
    }

try {

    int numOfMessages = 1;
    string inputFile(argv[1]);
    serverType.assign(argv[2]);
    numOfMessages = atoi( argv[3] );
    
    // Optional command line arguments
    if ( argc >= 5 )
    {
	debugOn = true;
    } else {
	debugOn = false;
    }

    if (!readInputFile(inputFile))
	return 0;

    txStack.setDestination(&msAddress);

    if (serverType == "UA1") {
	ua1Thread(numOfMessages);
    } else if (serverType == "UA2") {
	ua2Thread();
    } else {
	cout << "Error in parm #2" << endl;
	exit (-1);
    }
}
catch(VException& e)
{
    cerr << "Caught exception :" << e.getDescription().c_str() << endl;
}
catch(...)
{
    cerr << "Caught exception :" << endl;
}
    return 0;
}
