

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CmuSnmp_cxx_Version =
    "$Id: CmuSnmp.cxx,v 1.12 2002/04/16 00:54:41 bko Exp $";

//#include <iostream>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <algorithm>
#include "CmuSnmp.hxx"

#include "cpLog.h"

// // This class was put in at the last minute because of the shortcommings of SNMP++
// // so it isn't a completed item.   TODO:

CmuSnmp::CmuSnmp(const char * inTarget, const char * comm )
{
    // remove leading blanks from inTarget
    string tmpStr = inTarget;
    target = tmpStr.substr(tmpStr.find_first_not_of(" "));
    target_community = comm;

}

CmuSnmp::~CmuSnmp()
{}



// This function is a wrapper for the cmu snmpLib function for get
// currently this function only does int and string oid return values

// input target = ip address of destination
// input comm = community string of destination
// input oid = oid we are looking for
// output resultType = type of data returned
// output result = returned value
// returns false on failure; true on success


bool
CmuSnmp::get(vector < ulong > inOid,
             snmpTypeT &resultType,
             Data &result)
{
    struct snmp_pdu *pdu;
    struct snmp_pdu *response;
    int status, name_length = inOid.size();
    bool retVal;

    oid *name = (oid *)calloc(name_length, sizeof(oid));
    if (!name)
    {
        cpLog(LOG_ERR, "failure to allocate memory");
        return false;
    }

    for (int i = 0; i < name_length; i++)
        name[i] = inOid[i];

    struct snmp_session *ss = getSnmpSession();
    pdu = snmp_pdu_create(SNMP_MSG_GET);
    snmp_add_null_var(pdu, name, name_length);
    status = snmp_synch_response(ss, pdu, &response);

    if (status == STAT_SUCCESS)
    {
        if (response && response->errstat == SNMP_ERR_NOERROR)
        {
            if (response->variables)
            {
                switch (response->variables->type)
                {
                    case ASN_BOOLEAN:
                    case ASN_INTEGER:
                    case ASN_UNSIGNED:
                    resultType = snmpInt;
                    result = (int) * response->variables->val.integer;
                    break;
                    case ASN_BIT_STR:
                    case ASN_OCTET_STR:
                    {
                        char*  tmpStr = new char[strlen((char*)response->variables->val.string) + 1];
                        strncpy(tmpStr, (char *)response->variables->val.string, response->variables->val_len);
                        resultType = snmpString;
                        result = tmpStr;
                        delete [] tmpStr;
                    }
                    break;
                    case ASN_OBJECT_ID :
                    resultType = snmpOid;
                    result = "ASN_OBJECT_ID Not Implemented in CmuSnmp.cxx";
                    break;
                    default :
                    resultType = snmpUnknown;
                    result = "UNKNOWN: Not Implemented in CmuSnmp.cxx";
                }
                retVal = true;
            }
            else
            {
                retVal = false;
            }
        }
        else
        {
            string oidStr = oidToStr(inOid);
            cpLog(LOG_ERR, "Snmp Get Failure. %s. %s %s ",
                  snmp_errstring(response->errstat),
                  getTarget(),
                  oidStr.c_str() );
            retVal = false;
        }
    }
    else
    {
        string oidStr = oidToStr(inOid);
        cpLog(LOG_ERR, "Snmp Get Failed to send request.  %s %s ",
              getTarget(),
              oidStr.c_str() );
        retVal = false;
    }
    snmp_close(ss);
    SOCK_CLEANUP;
    if (name) free (name);
    return retVal;
}

//nextOid is expected to be empty
bool
CmuSnmp::getNext(vector < ulong > inOid,
                 vector < ulong > &nextOid,
                 snmpTypeT &resultType,
                 Data &result)
{
    struct snmp_pdu *pdu;
    struct snmp_pdu *response;
    int status, name_length = inOid.size(), i;
    bool retVal;

    if (!nextOid.empty())
    {
        cpLog(LOG_ERR, "nextOid vector is not empty");
        return false;
    }

    oid *name = (oid *)calloc(name_length, sizeof(oid));
    if (!name)
    {
        cpLog(LOG_ERR, "failure to allocate memory");
        return false;
    }

    for (int i = 0; i < name_length; i++)
        name[i] = inOid[i];

    struct snmp_session *ss = getSnmpSession();
    pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
    snmp_add_null_var(pdu, name, name_length);
    status = snmp_synch_response(ss, pdu, &response);

    if (status == STAT_SUCCESS)
    {
        if (response && response->errstat == SNMP_ERR_NOERROR)
        {
            if (response->variables)
            {
                switch (response->variables->type)
                {
                    case ASN_BOOLEAN:
                    case ASN_INTEGER:
                    case ASN_UNSIGNED:
                    resultType = snmpInt;
                    result = (int) * response->variables->val.integer;
                    break;
                    case ASN_BIT_STR:
                    case ASN_OCTET_STR:
                    {
		        char*  tmpStr = (char *)calloc(response->variables->val_len +1, 1);
			//char*  tmpStr = new char[strlen((char*)response->variables->val.string) + 1];
                        strncpy(tmpStr, (char *)response->variables->val.string, response->variables->val_len);
                        resultType = snmpString;
                        result = tmpStr;
                        free (tmpStr);
                    }
                    break;
                    case ASN_OBJECT_ID :
                    resultType = snmpOid;
                    result = "ASN_OBJECT_ID Not Implemented in CmuSnmp.cxx";
                    break;
                    default :
                    resultType = snmpUnknown;
                    result = "UNKNOWN: Not Implemented in CmuSnmp.cxx";
                }
                retVal = true;
                for (i = 0; i < (int)response->variables->name_length; i++)
                    nextOid.push_back(*(response->variables->name + i));
            }
            else
            {
                retVal = false;
            }
        }
        else
        {
            string oidStr = oidToStr(inOid);
            cpLog(LOG_ERR, "Snmp Get Failure. %s. %s %s ",
                  snmp_errstring(response->errstat),
                  getTarget(),
                  oidStr.c_str() );
            retVal = false;
        }
    }
    else
    {
        string oidStr = oidToStr(inOid);
        cpLog(LOG_ERR, "Snmp Get Failed to send request.  %s %s ",
              getTarget(),
              oidStr.c_str() );
        retVal = false;
    }
    snmp_close(ss);
    SOCK_CLEANUP;
    if (name) free (name);
    return retVal;
}

//       The type is a single character, one of:
//               i  INTEGER
//               s  STRING
//               x  HEX STRING
//               d  DECIMAL STRING
//               n  NULLOBJ
//               o  OBJID
//               t  TIMETICKS
//               a  IPADDRESS


bool 
CmuSnmp::set(vector < ulong > inOid,
    char type,
    Data &value,
    snmpTypeT &resultType,
    Data &result,
    int timeout,
    int retries)
{
    vector < ulong > inOid2;
    char type2 = 0;
    Data value2;
    return CmuSnmp::set(inOid,  type,  value, 
	       inOid2, type2, value2,
	       resultType, result,
	       timeout, retries);
}

bool 
CmuSnmp::set(vector < ulong > inOid,
    char type,
    Data &value,
    vector < ulong > inOid2,
    char type2,
    Data &value2,
    snmpTypeT &resultType,
    Data &result,
    int timeout,
    int retries)
{
    struct snmp_pdu *pdu;
    struct snmp_pdu *response;
    int status, name_length = inOid.size();
    bool retVal;

    oid *name = (oid *)calloc(name_length, sizeof(oid));
    if (!name)
    {
        cpLog(LOG_ERR, "failure to allocate memory");
        return false;
    }

    for (int i = 0; i < name_length; i++)
        name[i] = inOid[i];

    struct snmp_session *ss = getSnmpSession(timeout, retries);
    pdu = snmp_pdu_create(SNMP_MSG_SET);
    LocalScopeAllocator lo;
    snmp_add_var(pdu, name, name_length, type, value.getData(lo));
    if (type != 0) {
	// a second set variable
	int name_len2 = inOid2.size();
	oid *name2 = (oid *)calloc(name_len2, sizeof(oid));
	for (int i = 0; i < name_len2; i++)
	    name2[i] = inOid2[i];
	snmp_add_var(pdu, name2, name_len2, type2, value2.getData(lo));
    }

    status = snmp_synch_response(ss, pdu, &response);

    if (status == STAT_SUCCESS)
    {
        if (response && response->errstat == SNMP_ERR_NOERROR)
        {
            if (response->variables)
            {
                switch (response->variables->type)
                {
                    case ASN_BOOLEAN:
                    case ASN_INTEGER:
                    case ASN_UNSIGNED:
                    resultType = snmpInt;
                    result = (int) * response->variables->val.integer;
                    break;
                    case ASN_BIT_STR:
                    case ASN_OCTET_STR:
                    case ASN_APPLICATION:
                    case ASN_UNIVERSAL:
                    {
                        /*
                        char tmpStr[1024];
                        memset(tmpStr, 0, sizeof(tmpStr));
                        strncpy(tmpStr, (char *)response->variables->val.string, response->variables->val_len);
                        resultType = snmpString;
                        result = tmpStr;
                        */

                        char*  tmpStr = new char[strlen((char*)response->variables->val.string) + 1];
                        strncpy(tmpStr, (char *)response->variables->val.string, response->variables->val_len);
                        resultType = snmpString;
                        result = tmpStr;
                        delete [] tmpStr;
                    }
                    break;
                    case ASN_OBJECT_ID :
                    resultType = snmpOid;
                    result = "ASN_OBJECT_ID Not Implemented in CmuSnmp.cxx";
                    break;
                    default :
                    resultType = snmpUnknown;
                    result = "UNKNOWN: Not Implemented in CmuSnmp.cxx";
                }
                retVal = true;
            }
            else
            {
                retVal = false;
            }
        }
        else
        {
            string oidStr = oidToStr(inOid);
            cpLog(LOG_ERR, "Snmp Get Failure. %s. %s %s ",
                  snmp_errstring(response->errstat),
                  getTarget(),
                  oidStr.c_str() );
            retVal = false;
        }
    }
    else
    {
        string oidStr = oidToStr(inOid);
        cpLog(LOG_ERR, "Snmp Get Failed to send request.  %s %s ",
              getTarget(),
              oidStr.c_str() );
        retVal = false;
    }
    snmp_close(ss);
    SOCK_CLEANUP;
    if (name) free (name);
    return retVal;
}

struct snmp_session *
CmuSnmp::getSnmpSession(int timeout,  int retries )
{
    struct snmp_session session, *ss;

    SOCK_STARTUP;

    memset(&session , 0, sizeof(session));
    session.version = SNMP_VERSION_2c;
    session.retries = retries;
    session.timeout = timeout;
    session.peername = (char *)target.c_str();
    session.community = (u_char *)target_community.c_str();
    session.community_len = target_community.size();
    session.remote_port = 161;

    ss = snmp_open(&session);
    if (ss == NULL)
    {
        /* diagnose snmp_open errors with the input struct snmp_session pointer */
        cpLog(LOG_ERR, "Failed to allocate in snmpGet");
    }
    return ss;
}

//similar to strncmp
//returns an integer less than, equal to, or greater than zero if the first  n
//bytes of oid1  is found, respectively, to be less than, to match, or be greater than oid2.
int
CmuSnmp::oidncmp (const vector < ulong > oid1, const vector < ulong > oid2, const int n)
{
    int len = min(min(oid1.size(), oid2.size()), (unsigned)n);
    for (int i = 0; i < len; i++)
    {
        if (oid1[i] != oid2[i]) return (oid1[i] < oid2[i] ? -1 : 1);
    }
    return 0;
}

string
CmuSnmp::oidToStr(vector < ulong > inOid)
{
    string retStr;
    retStr.resize(inOid.size() * 2);
    for (int i = 0; i < (int)inOid.size(); i++)
    {
        sprintf(&retStr[i*2], ".");
        sprintf(&retStr[i*2 + 1], "%lu", inOid[i]);
    }
    return retStr;
}

