/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtCapMgrRcvdSDP0_cxx_Version = 
"$Id: EvtCapMgrRcvdSDP0.cxx,v 1.5 2001/05/23 20:50:47 vzubarev Exp $";

#include "EvtCapMgrRcvdSDP0.hxx"

using namespace CapabilityTranslator;
using namespace CapabilityManager;

EvtCapMgrRcvdSDP0::
EvtCapMgrRcvdSDP0(SM_CapManager* sm)
    : EvtUserCapMgr(sm)
{
}


void EvtCapMgrRcvdSDP0::inSDP(FeatureTCS &f_tcs)
{
	VLog log;
	VDEBUG(log) << "EvtCapMgrRcvdSDP0::inSDP(FeatureTCS &f_tcs)" << VDEBUG_END(log);
	
	f_tcs.setState(&PatternTCS::inIdle);
}

void EvtCapMgrRcvdSDP0::inSent(FeatureTCS &f_tcs)
{
	VLog log;
	VDEBUG(log) << "EvtCapMgrRcvdSDP0::inSent(FeatureTCS &f_tcs)" << VDEBUG_END(log);
	
	// TODO:
	// here we should generate
	// TCS 0 message 
	
	TCS msgTCS;
	
	f_tcs.getSM()->getEM()->startCEOTransferReq(msgTCS);
	
	f_tcs.setState(&PatternTCS::inOpen);
}

void EvtCapMgrRcvdSDP0::inOther(FeatureTCS &f_tcs)
{
    VLog log;
    VDEBUG(log) << "EvtCapMgrRcvdSDP0::inOther(FeatureTCS &f_tcs) (ignored)" << VDEBUG_END(log);

    // Ignore
    if(f_tcs.getState() != &PatternTCS::inOpen)
    {
         VWARN(log) << "Got EvtCapMgrRcvdSDP0 in wrong state for a call "
               << f_tcs.getSM() << VWARN_END(log);
    };
}

void EvtCapMgrRcvdSDP0::inSDP(FeatureSDP &f_sdp)
{
	VLog log;
	VDEBUG(log) << "EvtCapMgrRcvdSDP0::inSDP(FeatureSDP &f_sdp)" << VDEBUG_END(log);
	
	f_sdp.setState(&PatternSDP::inIdle);
}

void EvtCapMgrRcvdSDP0::inIntersect(FeatureSDP &f_sdp)
{
	
	// TODO:
	// at this point TCS0 is supposed to be generated
	// FeatureTCS is taking care of it
	
	// here we should generate SDP as a successfull response to
	// SDP0 received
	
    // sending out SDP
    // Note: generating SDP only after getting confirmation
    // from media session manager with valid information
    // needed to construct SDP (c= addresses and RTP ports)
    
    
  SDP msgSDP;
  
  const string	fn("EvtCapMgrRcvdSDP0::inIntersect");
  VLog	    	log(fn);
  
/*  SdpConnection connection;
  IPAddress ipAddress;
  std::string connAddr;*/
  Capability::intSDP packetSize;
  int capNumber;
  int i = 0, descrSize = 0;
  int pTypeCap = 0;
  int clockRate = 0;
  
  VDEBUG(log) << fn << ": BEGIN -------------------- " << VDEBUG_END(log);
  
  /*
  try
  {
    ipAddress = sDescriptor.transportIDH323.getRTPAddr(0, 0);
  }
  catch (XlatorException &mS)
  {
    VERR(log) << fn << ": Error with getting RTPAddr from IDH323 : " << mS.getDescription() << VERR_END(log);
    throw XlatorException("Error with getting RTPAddr from IDH323", __FILE__, __LINE__);
  }
  
  VDEBUG(log) << fn << ": Here is ipAddress : " << ipAddress.getIPAsString() << VDEBUG_END(log);
  
  connAddr = ipAddress.getIPAsString();
  connection.setAddressType(AddressTypeIPV4);
  connection.setUnicast(connAddr.c_str());
  msgSDP.setConnection(connection);

  extern string	    SipToH323CallSigGW_HostAddress;

  VDEBUG(log) << fn << ": Here is the IP address : " << SipToH323CallSigGW_HostAddress << VDEBUG_END(log);
  msgSDP.setAddress(SipToH323CallSigGW_HostAddress.c_str());*/
  
  msgSDP.setSessionName("VOVIDA session");
  
  // at this point we assume that we use only first (0) capability descriptor
  // from CapabilityDescrVec
  
  descrSize = f_sdp.getSM()->capSetIntersect.capRecv.getCapDescrSize(0);

  for(i = 0; i < descrSize; i++)
  {
    SdpMedia *media = new SdpMedia;
    //ipAddress = sDescriptor.transportIDH323.getRTPAddr(0, i);
    //media->setPort(ipAddress.getPort());
  
    MediaAttributes *mediaAttrib = new MediaAttributes();
    ValueAttribute *attrib = new ValueAttribute();
  
    capNumber = f_sdp.getSM()->capSetIntersect.capRecv.getCapability(0, i, 0);
  
    H323_CapTable::CapTable_I pI = f_sdp.getSM()->capSetIntersect.h323CapTable.getTableIter_pt(capNumber);
    
    // TODO:
    // commented out fromOLCtoSDPcapTable was defined in old CapMap
    
    //H323_CapTable::CapTable_I pI = fromOLCtoSDPcapTable.getTableIter_pt(capNumber);
    
    if(pI == NULL)
    {
	pI = f_sdp.getSM()->capSetIntersect.h323CapTable.getTableIter_pt(capNumber);
        
	if(pI == NULL)
        {
              VCRIT(log) << fn << ": Did not find the entry in CapTable by number gotten from capDescriptor..." << VCRIT_END(log);
              throw XlatorException("Did not find the entry in CapTable by number gotten from capDescriptor",
                             __FILE__, __LINE__);
	}
    }
    
    (*pI)->getParamSendMsg(packetSize);

    clockRate = (*pI)->getClockRate()/1000;
    packetSize = packetSize/clockRate;

    attrib->setAttribute("ptime");
    attrib->setValue(Data(packetSize));
  
    SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();
    pTypeCap = (*pI)->getCapabilityNumber();
    
    
    if(pTypeCap == 1)pTypeCap = 0;
    
    rtpMapAttrib->setPayloadType(pTypeCap);
  
    GlobalCapTable::GlobalCapTable_p pG;
    try
    {
      pG = f_sdp.getSM()->globalCapTable.getTableIter_pt((*pI)->getCapabilityNumber());
    }
    catch (XlatorException)
    {
      VCRIT(log) << fn << ": Did not find the entry in GlobalCapTable..." << VCRIT_END(log);
      throw XlatorException("Did not find the entry in GlobalCapTable", __FILE__, __LINE__);
    }
  
    media->clearFormatList();
    media->addFormat(pTypeCap);

    rtpMapAttrib->setEncodingName(pG->nameCapIANA);
    
    rtpMapAttrib->setClockRate(pG->clockrateCap);
  
    mediaAttrib->addValueAttribute(attrib);
    mediaAttrib->addmap(rtpMapAttrib);
  
    media->setMediaAttributes(mediaAttrib);
  
    msgSDP.addMedia(media);
  }

  f_sdp.getSM()->getEM()->startSendSDP(msgSDP, false);

  VDEBUG(log) << fn << ": END -------------------- " << VDEBUG_END(log);

  f_sdp.setState(&PatternSDP::inWaitForCfrmTCS0);
}

void EvtCapMgrRcvdSDP0::inOther(FeatureSDP &f_sdp)
{
    VLog log;
    VDEBUG(log) << "EvtCapMgrRcvdSDP0::inOther(FeatureSDP &f_sdp) (ignored)" << VDEBUG_END(log);

    // Ignore
         VWARN(log) << "Got EvtCapMgrRcvdSDP0 in wrong state for a call "
               << f_sdp.getSM() << VWARN_END(log);
}

#ifndef DISABLE_WRITER

ostream& EvtCapMgrRcvdSDP0::
writeTo(ostream& out) const
{
    out << "EvtCapMgrRcvdSDP0";
    return out;
}

#endif
 
