
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const G722Capability_cxx_Version = 
"$Id: G722Capability.cxx,v 1.2 2001/05/22 18:12:37 vzubarev Exp $";
#include "ptlib.h"
#include "asner.h"
#include "G722Capability.hxx"

using namespace CapabilityTranslator;

G722Capability::G722Capability(Speed s)
  : AudioCapability(G722_SMPLS_PER_FRAME, 
                    G722_FRMS_PER_PACKET, 
		    G722_CLCK_RATE), speed(s) {}


AudioCapability * G722Capability::clone() const
{
  return new G722Capability(speed);
}


const unsigned G722Capability::g722SubType[3] = 
  { H245_AudioCapability::e_g722_64k, 
    H245_AudioCapability::e_g722_56k, 
    H245_AudioCapability::e_g722_48k };



unsigned G722Capability::getSubType() const
{
  return g722SubType[speed];
}



bool G722Capability::getParamSendMsg(H245_AudioCapability & cap,
                                       unsigned packetSize) const
{
  cap.SetTag(g722SubType[speed]);

  PASN_Integer & value = cap;
  value = packetSize;
  return true;
}


bool G722Capability::setParamRecvMsg(const H245_AudioCapability & cap,
                                        unsigned & packetSize)
{
  switch (cap.GetTag()) {
    case H245_AudioCapability::e_g722_64k :
      speed = G722Capability::At64k;
      break;

    case H245_AudioCapability::e_g722_56k :
      speed = G722Capability::At56k;
      break;

    case H245_AudioCapability::e_g722_48k :
      speed = G722Capability::At48k;
      break;

    default :
      return false;
  }

  const PASN_Integer & value = cap;

  // Get the maximum number of milliseconds
  packetSize = value;
  return true;
}
