/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DefaultRASPolicyBuilder_cxx_Version = 
    "$Id: DefaultRASPolicyBuilder.cxx,v 1.10 2001/10/13 01:15:53 icahoon Exp $";


#include "DefaultRASPolicyBuilder.hxx"
#include "DefaultDiscoveryPolicy.hxx"
#include "DefaultRegistrationPolicy.hxx"
#include "DefaultAdmissionPolicy.hxx"
//#include "DefaultLocationPolicy.hxx"
#include "LocationPolicy.hxx"
#include "DefaultResourcesAvailablePolicy.hxx"


using Vocal::H323GK::DefaultRASPolicyBuilder;
using Vocal::H323GK::DefaultDiscoveryPolicy;
using Vocal::H323GK::DefaultRegistrationPolicy;
using Vocal::H323GK::DefaultAdmissionPolicy;
using Vocal::H323GK::DiscoveryPolicy;
using Vocal::H323GK::RegistrationPolicy;
using Vocal::H323GK::AdmissionPolicy;
using Vocal::H323GK::LocationPolicy;
using Vocal::H323GK::ResourcesAvailablePolicy;
using Vocal::H323GK::RASGKProtocol;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfoMap;
using Vocal::Services::Service;


DefaultRASPolicyBuilder::DefaultRASPolicyBuilder()
{
}


DefaultRASPolicyBuilder::~DefaultRASPolicyBuilder()
{
}


DiscoveryPolicy *   	
DefaultRASPolicyBuilder::buildDiscoveryPolicy(
    Service  	    	    &	service,
    RASGKProtocol   	    & 	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    return ( new DefaultDiscoveryPolicy(
    	    	    	    	service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );
}


RegistrationPolicy *	
DefaultRASPolicyBuilder::buildRegistrationPolicy(
    Service  	    	    &	service,
    RASGKProtocol   	    & 	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    return ( new DefaultRegistrationPolicy(
    	    	    	    	service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );
}


AdmissionPolicy *	
DefaultRASPolicyBuilder::buildAdmissionPolicy(
    Service  	    	    &	service,
    RASGKProtocol   	    & 	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    return ( new DefaultAdmissionPolicy(
                                service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );
}


LocationPolicy *	
DefaultRASPolicyBuilder::buildLocationPolicy(
    Service  	    	    &	service,
    RASGKProtocol   	    & 	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    /*
    return ( new DefaultLocationPolicy(
                                service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );
    */
    return ( new LocationPolicy(service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );

}


ResourcesAvailablePolicy * 
DefaultRASPolicyBuilder::buildResourcesAvailablePolicy(
    Service  	    	    &	service,
    RASGKProtocol   	    & 	rasGkProtocol,
    GatekeeperInfo  	    &	gatekeeperInfo,
    EndpointInfoMap 	    &	endpointInfoMap
)
{
    return ( new DefaultResourcesAvailablePolicy(
                                service,
    	    	    	    	rasGkProtocol, 
    	    	    	    	gatekeeperInfo,
    	    	    	    	endpointInfoMap) );
}
