#if !defined(VOCAL_RAS_MCAST_GK_PROTOCOL_HXX)
#define VOCAL_RAS_MCAST_GK_PROTOCOL_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RASMCastGKProtocol_hxx_Version = 
    "$Id: RASMCastGKProtocol.hxx,v 1.10 2001/10/13 01:15:53 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include "Protocol.hxx"
#include "Sptr.hxx"
#include "UDPSocket.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Gatekeeper<br><br>
 */
namespace H323GK
{


using Vocal::Transport::IPAddress;
using Vocal::Transport::UDPSocket;
using Vocal::IO::FileDescriptor;


class DiscoveryPolicy;
class LocationPolicy;


/** The protocol is used to receive and decode mulitcase RAS 
 *  messages.<br><br>
 *
 *  @see    Vocal::Transport::Protocol
 *  @see    Vocal::Transport::IPAddress
 */
class RASMCastGKProtocol : public Vocal::Transport::Protocol
{

    public:


    	/** Create given the multicast address. The port will be set
	 *  to the well port 1718.
	 */
	RASMCastGKProtocol(IPAddress);


    	/** Virtual destructor
	 */
	virtual ~RASMCastGKProtocol();


    	/** Set the discovery policy. When incoming multicast discovery
	 *  messages arrive, they will be forwarded to the discovery policy.
	 */
    	void	    	    	    setDiscoveryPolicy(DiscoveryPolicy &);


    	/** Set the location policy. When incoming multicast location
	 *  messages arrive, they will be forwarded to the location policy.
	 */
    	void	    	    	    setLocationPolicy(LocationPolicy &);


    	/** Access the FileDescriptor associated with this protocol.
    	 */
    	const FileDescriptor &	    getFileDescriptor() const;


    	/** Protocol callback. When an incoming message arrives, this protocol
	 *  will decode the RAS message and forward it to the appropriate
	 *  policy.
	 */
    	void	    	    	    onIncomingAvailable();

    private:

    	DiscoveryPolicy     	*   myDiscoveryPolicy;
        LocationPolicy          *   myLocationPolicy;
    	Sptr<UDPSocket>     	    myRasMulticastSocket;
	PPER_Stream 	    	    myPerStream;
	size_t	    	    	    myPerStreamSize;
};


} // namespace H323GK
} // namespace Vocal


#endif // !defined(VOCAL_RAS_MCAST_GK_PROTOCOL_HXX)
