#if !defined(VOCAL_ENDPOINT_INFO_MAP_HXX)
#define VOCAL_ENDPOINT_INFO_MAP_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EndpointInfoMap_hxx_Version = 
    "$Id: EndpointInfoMap.hxx,v 1.14 2001/05/29 09:00:12 icahoon Exp $";


#include <map>
#include <set>
#include <string>
#include "Sptr.hxx"
#include "EndpointInfo.hxx"
#include "IPAddress.hxx"
#include "Writer.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Object Information<br><br>
 */
namespace SipH323Info
{


struct IPCompare
{
    bool operator()(const IPAddress & left, const IPAddress & right) const;
};


struct SptrEptInfoCompare
{
    bool operator()(Sptr<EndpointInfo> left, Sptr<EndpointInfo> right) const;
};


typedef map<string, Sptr<EndpointInfo> >    	        EndpointIdMap;

typedef set<Sptr<EndpointInfo>, SptrEptInfoCompare>     EndpointInfoSet;

typedef map<Alias, Sptr<EndpointInfo> >    	        AliasMap;

typedef map<IPAddress, Sptr<EndpointInfo>, IPCompare>   CallSignalAddressMap;
                    
typedef map<string, EndpointInfoSet>                    PrefixMap;


/** Table of registered endpoints.<br><br>
 *
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::Transport::IPAddress
 *  @see    Sptr
 */
class EndpointInfoMap : public Vocal::IO::Writer
{

    public:


    	/** Create an endpoint info map.
	 */
    	EndpointInfoMap();


    	/** Virtual destructor
	 */
	virtual ~EndpointInfoMap();


    	/** Add and endpoint to the map.
	 */
    	void    insert(Sptr<EndpointInfo>);


    	/** Remove the given endpoint from the map.
	 */
    	void	erase(Sptr<EndpointInfo>);


    	/** Find an endpoint, given the endpoint identifier.
	 *  Zero returned if not found.
	 */
	Sptr<EndpointInfo>  findByEndpointId(const string &);


    	/** Find an endpoint, given an array of h.225 aliases.
	 *  Zero returned if not found.
	 */
	Sptr<EndpointInfo>  findByAlias(const H225_ArrayOf_AliasAddress &);


    	/** Find an endpoint, given the call signaling address of the endpoint.
	 *  Zero returned if not found.
	 */
	Sptr<EndpointInfo>  findByCallSignalAddress(const IPAddress &);


        /** Find the gateways that supports the given prefix.
         */
        EndpointInfoSet     findByPrefix(const string &, string & matchedPrefix);
        

    	/** Remove all endpoints from the table.
	 */
	void	clear();


    	/** Write the table to an ostream. Useful for debugging.
	 */
	ostream &   writeTo(ostream &) const;


    	/** A handle to the native endpoint table.
	 */
    	EndpointIdMap &     endpointIdMap();
		

    private:

    	EndpointIdMap	    	myEndpointIdMap;
	AliasMap    	    	myAliasMap;
	CallSignalAddressMap    myCallSignalAddressMap;
        PrefixMap               myPrefixMap;
};


} // namespace SipH323Info
} // namespace Vocal


#endif // !defined(VOCAL_ENDPOINT_INFO_MAP_HXX)
