/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipOpCancelIn_cxx_Version = 
  "$Id: H323SipOpCancelIn.cxx,v 1.13 2002/02/25 05:57:16 sprajpat Exp $";


#include "H323SipOpCancelIn.hxx"
#include "CancelMsg.hxx"
#include "StatusMsg.hxx"
#include "SipProxyInfo.hxx"
#include "VLog.hxx"
#include "CallTrace.hxx"
#include "IncomingSipEvent.hxx"

using Vocal::SipH323Proxy::H323SipOpCancelIn;
using Vocal::SipH323Proxy::H323SipOpIn;
using Vocal::SipH323Info::SipProxyInfo;
using Vocal::SipEvents::IncomingSipEvent;
using Vocal::Logging::VLog;
    

H323SipOpCancelIn::H323SipOpCancelIn(SipProxyInfo &  sipProxyInfo)
    :	H323SipOpIn(sipProxyInfo)
{
}


H323SipOpCancelIn::~H323SipOpCancelIn() 
{
}

   
const char * const 
H323SipOpCancelIn::name() const 
{
    return ( "H323SipOpCancelIn" );
}


const Sptr<State>
H323SipOpCancelIn::process(const Sptr<SipProxyEvent> event)
{
    const string    fn("H323SipOpCancelIn::process");
    VLog    	    log(fn);

    Sptr<SipEvent> sipevent;

    if ( sipevent.dynamicCast(event) == 0 )
    {
        return ( PROXY_CONTINUE );
    }

    Sptr<SipMsg> sipMsg = sipevent->getSipMsg();
    Sptr<CancelMsg> cancelMsg;
    cancelMsg.dynamicCast(sipMsg);

    if ( cancelMsg == 0 )
    {
        return ( PROXY_CONTINUE );
    }

    // return 200 to sender
    //
    StatusMsg statusMsg(*cancelMsg, 200);
    sipevent->getSipStack()->sendReply( statusMsg );

    if ( fifo_ != 0 || serviceFifo_ != 0 )
    {
	CALLTRACE(log, CT_IN, sipProxyInfo_.sipRemoteAddr(), CT_SIP, "CANCEL")
	    << "{\n" 
	    << cancelMsg->encode().logData()
	    << "}" << CALLTRACE_END(log);

        fifo_->add(new IncomingSipEvent(sipevent, *serviceFifo_));
    }
    else
    {
    	VWARN(log)  << fn << ": Received sip msg from SipStack "
                    << "before initialization. \n"
		    << VWARN_END(log);
    }

    return ( PROXY_CONTINUE );
}
