#if !defined(H323_SIP_OP_OUT_DOT_H)
#define H323_SIP_OP_OUT_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipOpOut_hxx_Version = 
    "$Id: H323SipOpOut.hxx,v 1.12 2001/02/14 03:45:58 icahoon Exp $";


#include "Operator.hxx"
#include "FifoBase.h"
#include "Sptr.hxx"
#include "SipTransceiver.hxx"
#include "SipEvent.hxx"


// Forward declaration
//
namespace Vocal {
    namespace SipH323Info {
    	class SipProxyInfo;
    }
}


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** Sip to H.323 Proxy<br><br>
 */
namespace SipH323Proxy
{


using Vocal::SipH323Info::SipProxyInfo;


/** The base class of all outgoing message operators.<br><br>
 *
 *  @see    Vocal::SipH323Info::SipProxyInfo
 */
class H323SipOpOut : public Operator
{

    protected:
    
    	/** Create give the configuration information for the sip proxy.
    	 */
    	H323SipOpOut(SipProxyInfo &);
	

    public:
	

    	/** Virtual destructor
	 */
    	virtual     ~H323SipOpOut();


    	/** Set the sip transceiver.
	 */
    	void	    sipStack(Sptr<SipTransceiver>);


    protected:


    	/** Place the address of the remote sip entity in the request line.
	 */
    	void	    updateRequestLine(SipCommand &);


    	/** Add our address to the via list.
	 */
    	void	    updateVia(SipMsg &);


    	/** Change the To: host address to the remote sip entity.
	 */
    	void	    updateTo(SipMsg &);


    	/** Change the From: host address to the remote sip entity.
	 */
    	void	    updateFrom(SipMsg &);


    	/** Add the contact field to the message. Based on the address
	 *  of the remote entity.
	 */
    	void	    addContact(SipMsg &, bool basedOnTo);


        /** Retrieve the sip event from the sip proxy event.
         */
        Sptr<SipEvent>  getSipEvent(Sptr<SipProxyEvent>);	    
        

    	/** Sip proxy configuration information available to all outgoing
	 *  operators.
	 */
    	SipProxyInfo	    	    &	sipProxyInfo_;


    	/** Sip stack is available to all outgoing operators.
	 */
	Sptr<SipTransceiver> 	    	sipStack_;


    private:
    
        // Suppress copying.
	//
    	H323SipOpOut(const H323SipOpOut &);
        H323SipOpOut & operator=(const H323SipOpOut &);
};


} // namespace SipH323Proxy
} // namespace Vocal


#endif // H323_SIP_OP_OUT_DOT_H
