/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipProxyService_cxx_Version = 
    "$Id: H323SipProxyService.cxx,v 1.13 2001/10/26 21:47:54 icahoon Exp $";


#include "H323SipProxyService.hxx"
#include "SipProxyInfo.hxx"
#include "H323SipProxy.hxx"
#include "H323SipBuilder.hxx"
#include "OutgoingSipEventExch.hxx"
#include "VLog.hxx"


using Vocal::SipH323Proxy::H323SipProxyService;
using Vocal::SipH323Proxy::H323SipProxy;
using Vocal::SipH323Proxy::OutgoingSipEventExch;
using Vocal::Services::ControlEvent;
using Vocal::SipH323Info::SipProxyInfo;
using Vocal::Logging::VLog;
using Vocal::ReturnCode;


H323SipProxyService::H323SipProxyService(
    SipProxyInfo	    	&   sipProxyInfo,
    const char      	    	*   appName
)
    :	FifoService("H323SipProxyService"),
    	sipProxyInfo_(sipProxyInfo),
	sipProxy_(0),
        outSipEventExch_(0),
    	started_(false)
{
    const string    	fn("H323SipProxyService::H323SipProxyService");
    VLog    	    	log(fn);

    H323SipBuilder * h323SipBuilder 
	= new H323SipBuilder(sipProxyInfo_);

    u_int16_t 	sipPort = sipProxyInfo_.sipProxy().getPort();

    sipProxy_ = new H323SipProxy(h323SipBuilder, sipPort, appName);

    h323SipBuilder->sipStack(sipProxy_->getSipStack());
    h323SipBuilder->serviceFifo(getFifo());

    outSipEventExch_ = new OutgoingSipEventExch(
                                *this, 
                                *(sipProxy_->getCallProcessingQueue()));
        
    VDEBUG(log) << fn << ": created. port = " << sipPort << VDEBUG_END(log);
}


H323SipProxyService::~H323SipProxyService()
{
    const string    	fn("H323SipProxyService::~H323SipProxyService");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": destroyed." << VDEBUG_END(log);

    delete sipProxy_;
}


ReturnCode
H323SipProxyService::onStart()
{
    const string    	fn("H323SipProxyService::onStart");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": starting." << VDEBUG_END(log);
    
    ReturnCode	rc = SUCCESS;
    
    if ( !started_ )
    {
	try
	{
            sipProxy_->run();
	}
	catch ( ... )
	{
    	    rc = !SUCCESS;
	}
    }

    return ( rc );
}


ReturnCode	    
H323SipProxyService::onShutdown()
{
    const string    	fn("H323SipProxyService::onShutdown");
    VLog    	    	log(fn);
    
    VDEBUG(log) << fn << ": shutting down." << VDEBUG_END(log);
    
    if ( sipProxy_ )
    {
    	sipProxy_->shutdown();
    	sipProxy_->join();
    }

    return ( SUCCESS );
}


void	    
H323SipProxyService::connectToCallTranslator(
    FifoBase< Sptr<Event> > 	&   callTransFifo
)
{
    assert( sipProxy_ != 0 );
    sipProxy_->callTranslatorFifo(callTransFifo);
}
