/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323SipState_cxx_Version = 
    "$Id: H323SipState.cxx,v 1.13 2001/02/14 03:45:58 icahoon Exp $";


#include "H323SipState.hxx"
#include "CallInfo.hxx"
#include "CallContainer.hxx"


using Vocal::SipH323Proxy::H323SipState;
using Vocal::SipH323Info::SipProxyInfo;
using Vocal::SipH323Proxy::H323SipRegistrationMap;
using Vocal::SipH323Proxy::H323SipOpAckIn;
using Vocal::SipH323Proxy::H323SipOpAckOut;
using Vocal::SipH323Proxy::H323SipOpByeIn;
using Vocal::SipH323Proxy::H323SipOpByeOut;
using Vocal::SipH323Proxy::H323SipOpCancelIn;
using Vocal::SipH323Proxy::H323SipOpCancelOut;
using Vocal::SipH323Proxy::H323SipOpInviteIn;
using Vocal::SipH323Proxy::H323SipOpInviteOut;
using Vocal::SipH323Proxy::H323SipOpRegisterIn;
using Vocal::SipH323Proxy::H323SipOpRegisterOut;
using Vocal::SipH323Proxy::H323SipOpStatusIn;
using Vocal::SipH323Proxy::H323SipOpStatusOut;


H323SipState::H323SipState(
    SipProxyInfo    	    	&   sipProxyInfo
)
    :	sipProxyInfo_(sipProxyInfo),
        me_(0)
{
    createOperators();
}

 
H323SipState::~H323SipState() 
{
}

    
void	    	    	
H323SipState::sipStack(
    Sptr<SipTransceiver>     	sipstack
)
{
    assert( opAckOut_ != 0 );
    opAckOut_->sipStack(sipstack);
    
    assert ( opByeOut_ != 0 );
    opByeOut_->sipStack(sipstack);
    
    assert ( opCancelOut_ != 0 );
    opCancelOut_->sipStack(sipstack);
    
    assert ( opInviteOut_ != 0 );
    opInviteOut_->sipStack(sipstack);
    
    assert ( opRegisterOut_ != 0 );
    opRegisterOut_->sipStack(sipstack);
    
    assert ( opStatusOut_ != 0 );
    opStatusOut_->sipStack(sipstack);
}


void	    	    	
H323SipState::callTranslatorFifo(
    FifoBase< Sptr<Event> > &	callTransFifo
)
{
    assert( opAckIn_ != 0 );
    opAckIn_->callTranslatorFifo(callTransFifo);

    assert( opByeIn_ != 0 );
    opByeIn_->callTranslatorFifo(callTransFifo);

    assert( opCancelIn_ != 0 );
    opCancelIn_->callTranslatorFifo(callTransFifo);

    assert( opInviteIn_ != 0 );
    opInviteIn_->callTranslatorFifo(callTransFifo);

    assert( opRegisterIn_ != 0 );
    opRegisterIn_->callTranslatorFifo(callTransFifo);

    assert( opStatusIn_ != 0 );
    opStatusIn_->callTranslatorFifo(callTransFifo);
}


void	    	    	
H323SipState::serviceFifo(
    FifoBase< Sptr<Event> > &	servFifo
)
{
    assert( opAckIn_ != 0 );
    opAckIn_->serviceFifo(servFifo);

    assert( opByeIn_ != 0 );
    opByeIn_->serviceFifo(servFifo);

    assert( opCancelIn_ != 0 );
    opCancelIn_->serviceFifo(servFifo);

    assert( opInviteIn_ != 0 );
    opInviteIn_->serviceFifo(servFifo);

    assert( opRegisterIn_ != 0 );
    opRegisterIn_->serviceFifo(servFifo);

    assert( opStatusIn_ != 0 );
    opStatusIn_->serviceFifo(servFifo);
}


Sptr < State > 
H323SipState::process(const Sptr < SipProxyEvent > event)
{
    assert( event != 0 );
    assert( me_ != 0 );
    
    if ( event->getCallInfo() == 0 )
    {   
        event->setCallInfo(new CallInfo, new CallContainer);
        event->getCallInfo()->setState(me_);
    }
    
    return ( State::process(event) );
}


const char* const 
H323SipState::name() const 
{
    return ( "H323SipState" );
}


void
H323SipState::createOperators()
{
    opAckIn_    = new H323SipOpAckIn(sipProxyInfo_);
    addOperator(opAckIn_);

    opAckOut_    = new H323SipOpAckOut(sipProxyInfo_);
    addOperator(opAckOut_);
    
    opByeIn_    = new H323SipOpByeIn(sipProxyInfo_);
    addOperator(opByeIn_);

    opByeOut_	= new H323SipOpByeOut(sipProxyInfo_);
    addOperator(opByeOut_);
    
    opCancelIn_ = new H323SipOpCancelIn(sipProxyInfo_);
    addOperator(opCancelIn_);

    opCancelOut_ = new H323SipOpCancelOut(sipProxyInfo_);
    addOperator(opCancelOut_);
    
    opInviteIn_ = new H323SipOpInviteIn(sipProxyInfo_);
    addOperator(opInviteIn_);

    opInviteOut_ = new H323SipOpInviteOut(sipProxyInfo_);
    addOperator(opInviteOut_);
    
    opRegisterIn_ = new H323SipOpRegisterIn(sipProxyInfo_);
    addOperator(opRegisterIn_);

    opRegisterOut_ = new H323SipOpRegisterOut(sipProxyInfo_, registrationMap_);
    addOperator(opRegisterOut_);
    
    opStatusIn_   = new H323SipOpStatusIn(registrationMap_, 
                                          sipProxyInfo_);
    addOperator(opStatusIn_);
 
    opStatusOut_   = new H323SipOpStatusOut(sipProxyInfo_);
    addOperator(opStatusOut_);
}


void    
H323SipState::me(Sptr<State> moi)
{
    me_ = moi;
}
