#!/bin/sh
#
# $Id$
#
# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal\@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.
#
# $Log$
# Revision 1.5  2000/08/12 00:14:37  icahoon
# added license
#
# Revision 1.4  2000/08/03 22:10:14  icahoon
# update for two provisioning servers
#
# Revision 1.3  2000/07/24 22:34:35  icahoon
# moved to vocal
#
# Revision 1.2  2000/07/05 22:58:43  hyu
# Merging BRANCH_CURD into HEAD.
#
# Revision 1.1.2.1  2000/06/20 23:29:51  icahoon
# added html interface for allinone server
#
#

# disable filename globbing
set -f

ARGS=""
CMDLINE=0

if [ "$1" = "-nc" ]
then

    shift
    ARGS="$*"
    CMDLINE=1
else

    FLAGS=""
    LOG_LEVEL=""
    PSERVER_HOST=""
    PSERVER_PORT="6005"
    PORT=""

    if [ -f config/flags ]
    then

	FLAGS=`cat config/flags`
	if [ "$FLAGS" != "" ]
	then
	    ARGS="$ARGS -${FLAGS}"
	fi
    fi

    if [ -f config/log_level ]
    then

	LOG_LEVEL=`cat config/log_level`
	if [ "$LOG_LEVEL" != "" ]
	then
	    ARGS="$ARGS -v${LOG_LEVEL}"
	fi
    fi

    if [ -f config/pserver_host ]
    then
    
	if [ -f config/pserver_port ]
	then
	    PSERVER_HOST=`cat config/pserver_host`
	    PSERVER_PORT=`cat config/pserver_port`
	    if [ "$PSERVER_HOST" != "" -a "$PSERVER_PORT" != "" ]
	    then
		ARGS="$ARGS ${PSERVER_HOST}:${PSERVER_PORT} ${PSERVER_HOST}:6010"
	    fi
	fi
    fi

    if [ -f config/port ]
    then

	PORT=`cat config/port`
	if [ "$PORT" != "" ]
	then
	    ARGS="$ARGS $PORT"
	fi
    fi

    echo Content-type: text/html
    echo

    cat << EOF
<html>

<!-- #include virtual="/vocal/common/copyright"-->

<head> 
	<title>Feature Server Start</title> 
	<link rel="stylesheet" href="/vocal/common/style.css" type="text/css">
</head>

<body>
<table border="0" cellspacing="0" cellpadding="10" width="100%">
    <tr>
	<td colspan="4" valign="bottom" align="left" bgcolor="#D2C8B6">
	<h3>Feature Server Start</h3>
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
    	    &nbsp;
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
<pre>
EOF

fi

RS_TOP=`top b n 1 | grep " rs" | head -1`
RS_TOP=`echo $RS_TOP`
RS_PROCESS=${RS_TOP%% *}

if [ "$RS_PROCESS" = "" ]
then
    echo Must start redirect server first.
else

    TOP=`top b n 1 | grep " fs" | head -1`
    TOP=`echo $TOP`
    PROCESS=${TOP%% *}

    if [ "$PROCESS" != "" ]
    then
    	echo Already started...
    else
	echo Starting...

    	FS_LOG_FILE=fs.`date -u +"%Y-%m-%d.%T"`.log.txt
    	FS_CURRENT_LOG_FILE=current.txt

	echo "./fs $ARGS > log/$FS_LOG_FILE 2>&1 &"
	./fs $ARGS > log/$FS_LOG_FILE 2>&1 &

	cd log
	/bin/ln -sf $FS_LOG_FILE $FS_CURRENT_LOG_FILE 
	cd ..
    fi
fi

if [ $CMDLINE -eq 0 ]
then

    echo '<hr><br>'
    echo '<ul>'
    echo '<br>'
    top b n 1 | head -n 9 -
    top b n 1 | grep " fs"
    echo '</ul>'
    cat <<EOF
</pre>		
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
	    &nbsp;
	</td>
    </tr>

</table>
</body>
</html>
EOF

fi


