#!/bin/sh
#
# $Id: start.cgi,v 1.2 2000/08/12 00:15:50 icahoon Exp $
#
# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal\@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.
#
# $Log: start.cgi,v $
# Revision 1.2  2000/08/12 00:15:50  icahoon
# added license
#
# Revision 1.1  2000/07/28 07:00:03  icahoon
# first version
#
#

# disable filename globbing
set -f

APP="ua"

ARGS="-q"
CMDLINE=0

if [ "$1" = "-nc" ]
then

    shift
    ARGS="$*"
    CMDLINE=1
else

    LOCAL_IP=`ifconfig eth0 | grep inet`
    LOCAL_IP=${LOCAL_IP#*:}
    LOCAL_IP=${LOCAL_IP%% *}
    
    PHONE="6388"
    DISPLAY="UserAgent"
    LOG_LEVEL="LOG_DEBUG"
    PORT=6388
    REMOTE_IP="127.0.0.1"
    REMOTE_PORT="5070"
    REMOTE=${REMOTE_IP}:${REMOTE_PORT}
    REMOTE_TYPE="SIP_Proxy"
    REGISTER="True"

    if [ -f config/phone ]
    then
	TEMP_PHONE=`cat config/phone`
	if [ "$TEMP_PHONE" != "" ]
	then
	    PHONE=$TEMP_PHONE
	fi
    fi

    if [ -f config/display ]
    then
	TEMP_DISPLAY=`cat config/display`
	if [ "$TEMP_DISPLAY" != "" ]
	then
	    DISPLAY=$TEMP_DISPLAY
	fi
    fi

    if [ -f config/log_level ]
    then

	LOG_LEVEL=`cat config/log_level`
	if [ "$LOG_LEVEL" != "" ]
	then
	    ARGS="$ARGS -v${LOG_LEVEL}"
	fi
    fi

    if [ -f config/port ]
    then
	TEMP_PORT=`cat config/port`
	if [ "$TEMP_PORT" != "" ]
	then
	    PORT=$TEMP_PORT
	fi
    fi

    if [ -r config/remote ]
    then
	TEMP_REMOTE=`cat config/remote`
	if [ "$TEMP_REMOTE" != "" ]
	then
	    REMOTE=$TEMP_REMOTE
	fi
    fi

    if [ -r config/remote_type ]
    then
	REMOTE_TYPE=`cat config/remote_type`
	if [ "$REMOTE_TYPE" != "SIP_Proxy" ]
	then
	    REGISTER="False"
	fi
    fi

    # Make config file.
    #
    cat ${APP}.cfg  | sed -e "s/__phone__/$PHONE/g" \
    		    | sed -e "s/__display__/$DISPLAY/g" \
		    | sed -e "s/__port__/$PORT/g" \
		    | sed -e "s/__remote__/$REMOTE/g" \
		    | sed -e "s/__local_ip__/$LOCAL_IP/g" \
		    | sed -e "s/__register__/$REGISTER/g" \
		    > $PHONE.cfg

    ARGS="$ARGS -f $PHONE.cfg"
    		
    echo Content-type: text/html
    echo

    cat << EOF
<html>

<!-- #include virtual="/vocal/common/copyright"-->

<head> 
	<title>User Agent Start</title> 
	<link rel="stylesheet" href="/vocal/common/style.css" type="text/css">
</head>

<body>
<table border="0" cellspacing="0" cellpadding="10" width="100%">
    <tr>
	<td colspan="4" valign="bottom" align="left" bgcolor="#D2C8B6">
	<h3>User Agent Start</h3>
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
    	    &nbsp;
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
<pre>
EOF

fi

TOP=`top b n 1 | grep " $APP" | head -1`
TOP=`echo $TOP`
PROCESS=${TOP%% *}

if [ "$PROCESS" != "" ]
then
    echo Already started...
else
    echo Starting...

    LOG_FILE=$APP.`date -u +"%Y-%m-%d.%T"`.log.txt
    CURRENT_LOG_FILE=current.txt

    echo "./$APP $ARGS > log/$LOG_FILE 2>&1 &"
    ./$APP $ARGS > log/$LOG_FILE 2>&1 &

    cd log
    /bin/ln -sf $LOG_FILE $CURRENT_LOG_FILE 
    cd ..

    echo '<ul>'
    echo "phone number:         $PHONE"
    echo "display name:         $DISPLAY"
    echo "sip port:             $PORT"
    echo "sip remote:           $REMOTE"
    echo "local ip:             $LOCAL_IP"
    echo "register:             $REGISTER"
    echo '</ul>'

fi

if [ $CMDLINE -eq 0 ]
then

    echo '<hr><br>'
    echo '<ul>'
    echo '<br>'
    top b n 1 | head -n 9 -
    top b n 1 | grep " $APP"
    echo '</ul>'
    cat <<EOF
</pre>		
	</td>
    </tr>

    <tr>
	<td colspan="4" bgcolor="#FFFFFF">
	    &nbsp;
	</td>
    </tr>

</table>
</body>
</html>
EOF

fi


