#ifndef INCLUDED_Call_hxx
#define INCLUDED_Call_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Call_hxx_Version = 
"$Id: Call.hxx,v 1.18 2001/05/09 23:00:56 vzubarev Exp $";

#include "mte/Mediator.hxx"

#include "SMCall.hxx"
#include "ConnectionCallSig.hxx"
#include "ConnectionCallCtl.hxx"
#include "FeatureCallSig.hxx"
#include "FeatureMsd.hxx"
#include "FeatureRTD.hxx"
#include "FeatureCEO.hxx"
#include "FeatureCEI.hxx"
#include "FeatureCallCtlSession.hxx"
#include "FeatureOpenCC.hxx"
#include "FeatureAcceptCC.hxx"
#include "FeatureH245.hxx"
#include "RouterLCO.hxx"
#include "RouterLCI.hxx"
#include "EvtH245TCPOpened.hxx"
#include "EvtTCSRejected.hxx"
#include "EvtTCSConfirmed.hxx"
#include "EvtMSDStarted.hxx"
#include "EvtMSDFailed.hxx"
#include "EvtMSDConfirmed.hxx"
#include "SesQ931.hxx"

namespace Vocal
{
    namespace Services {
    	class PollFifoService;
    }
    namespace Transport {
        class TPKTClientSocket;
        class IPAddress;
    }
    namespace SipH323Info {
        class RouterInfo;
    }
    class Q931ProtocolBase;
}

namespace router
{
    using Vocal::Services::PollFifoService;
    using Vocal::Transport::TPKTClientSocket;
    using Vocal::Transport::IPAddress;
    using Vocal::SipH323Info::RouterInfo;
    using Vocal::Q931ProtocolBase;
    
    class H245Protocol;
    class H245ServerProtocol;
    class SesH245;
    class RouterCallSig;
    class RouterCallCtl;

    /// H.323 call
    class Call: public SMCall
    {
    public:
        /// Construct call
        Call(mte::Mediator& master);

        virtual ~Call();

        void init();

        /// Return object that owns this call
        virtual mte::Mediator& getMaster() throw();

        /// Return call signalling connection
        virtual ConnectionCallSig* getCallSigConnection() throw();

        /// Return call control connection
        virtual ConnectionCallCtl* getCallCtlConnection() throw();

        /// Return pointer to Call Signalling feature
        virtual FeatureCallSigBase* getCallSig() throw();
        virtual FeatureCallSig& getCallSigImp() throw();

        /// Return pointer to MS Determination feature
        virtual FeatureMsdBase* getMsd() throw();
        virtual FeatureMsd& getMsdImp() throw();

        /// Return pointer to Roundtrip Determination feature
        virtual FeatureRTDBase* getRTD() throw();
        virtual FeatureRTD& getRTDImp() throw();

        /// Return pointer to capability exchange (out) feature
        virtual FeatureCEOBase* getCEO() throw();
        virtual FeatureCEO& getCEOImp() throw();

        /// Return pointer to capability exchange (in) feature
        virtual FeatureCEIBase* getCEI() throw();
        virtual FeatureCEI& getCEIImp() throw();

        /// Return pointer to call sontrol session feature
        virtual FeatureCallCtlSessionBase* getCallCtlSession() throw();
        virtual FeatureCallCtlSession& getCallCtlSessionImp() throw();

        /// Return pointer to open ctl. channel feature
        virtual FeatureOpenCCBase* getOpenCC() throw();
        virtual FeatureOpenCC& getOpenCCImp() throw();

        /// Return pointer to accept ctl channel feature
        virtual FeatureAcceptCCBase* getAcceptCC() throw();
        virtual FeatureAcceptCC& getAcceptCCImp() throw();

        /// Return pointer to H245 feature
        virtual FeatureH245Base* getH245() throw();
        virtual FeatureH245& getH245Imp() throw();

        /// Return pointer to logical channel (out) router
        virtual RouterLCO* getRtLCO() throw();

        /// Return pointer to logical channel (in) router
        virtual RouterLCI* getRtLCI() throw();

        virtual void makeH245Protocol();

        virtual PollFifoService* getFifo() throw();

        virtual void setFifo(PollFifoService* fifo) throw();

        virtual void setSesH245(SesH245* ses) throw();

        virtual void setCallSigRouter(RouterCallSig* q931Router) throw();

        virtual void setCallCtlRouter(RouterCallCtl* h245Router) throw();

        virtual void setRouterInfo(RouterInfo* routerInfo) throw();

        virtual RouterInfo* getRouterInfo() throw();

        virtual void selectActiveProtocol() throw();

        virtual void selectPassiveProtocol() throw();

        /// Return address of the H.245 server socket (throws bad_cast)
        virtual const IPAddress& getH245Address() const;

        virtual TPKTClientSocket* getH245Socket();

        virtual IPAddress getCallSignalAddress() const;

        void setProtocol(Q931ProtocolBase* q931Protocol) throw ();

        virtual Q931ProtocolBase* getProtocol() throw ();

        virtual H245Protocol* getH245Protocol() throw ();

        void setQ931Src(SesQ931* q931EvtSource) throw ();

        virtual int getTerminateReason() const throw();

        virtual void setTerminateReason(int reason) throw();

        virtual void disable(H245Protocol* protocol);

        virtual bool isActiveH245AddressGreater();

        virtual void complete();

        virtual void terminate(
            int reason = H225_ReleaseCompleteReason::e_undefinedReason);

        virtual void close();

        virtual void startTimer(Sptr<mte::EvtTimer<TimerEvent> >);

        virtual void stopTimer(Sptr<mte::EvtTimer<TimerEvent> >);

        virtual SesH245* getSesH245() throw ();

        /// Call Complete callback event
        mte::EventPtr<CallbackEvent> callbackComplete_;

        /// Call Terminate callback event
        mte::EventPtr<CallbackEvent> callbackTerminate_;

        /// Call Close callback event
        mte::EventPtr<CallbackEvent> callbackClose_;

    private:
#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

        mte::Mediator& master_;
        ConnectionCallSig sigConnection_;
        ConnectionCallCtl ctlConnection_;
        FeatureCallSig callsig_;
        FeatureMsd msd_;
        FeatureRTD rtd_;
        FeatureCEO ceo_;
        FeatureCEI cei_;
        FeatureCallCtlSession callCtlSession_;
        FeatureOpenCC openCC_;
        FeatureAcceptCC acceptCC_;
        FeatureH245 h245_;
        RouterLCO rtLco_;
        RouterLCI rtLci_;
        PollFifoService* fifo_;
        H245Protocol* h245PassiveProtocol_;
        H245Protocol* h245ActiveProtocol_;
        H245Protocol* h245Protocol_;
        H245ServerProtocol* ccServerProtocol_;
        SesH245* sesH245_;
        RouterCallSig* routerCallSig_;
        RouterCallCtl* routerCallCtl_;
        Q931ProtocolBase* q931Protocol_;
        RouterInfo* routerInfo_;
        EvtH245TCPOpened evtH245TCPOpened_;
        SesQ931* q931EvtSource_;
        int terminateReason_;
        EvtTCSRejected evtTCSRejected_;
        EvtTCSConfirmed evtTCSConfirmed_;
        EvtMSDStarted evtMSDStarted_;
        EvtMSDFailed evtMSDFailed_;
        EvtMSDConfirmed evtMSDConfirmed_;
    };

}

#endif
