/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtCallProceeding_cxx_Version = 
    "$Id: EvtCallProceeding.cxx,v 1.14 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "RouterInfo.hxx"

#include "EvtCallProceeding.hxx"
#include "FeatureCallSig.hxx"
#include "CallSigUnexpectedException.hxx"
#include "EvtReceivedCCAddress.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtCallProceeding::
EvtCallProceeding(RouterCallSig* sm)
    : EvtCallSig(sm)
{
}


void EvtCallProceeding::
inInitiated(FeatureCallSig& callsig)
{
    const string    fn("EvtCallProceeding::inInitiated");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.setState(&PatternCallSig::inAck, "inAck");
    handleH245Address(callsig);
}


void EvtCallProceeding::
inAck(FeatureCallSig& callsig)
{
    const string    fn("EvtCallProceeding::inAck");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Reset timer
    callsig.setState(&PatternCallSig::inAck, "inAck");
    handleH245Address(callsig);
}


void EvtCallProceeding::
inWaitConnect(FeatureCallSig& callsig)
{
    const string    fn("EvtCallProceeding::inWaitConnect");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    handleH245Address(callsig);

    VINFO(log)  << fn << ": Got H323 Call Proceeding for call " 
                << callsig.getSM() << VINFO_END(log);
}


void EvtCallProceeding::
inConnect(FeatureCallSig& callsig)
{
    const string    fn("EvtCallProceeding::inConnect");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VINFO(log)  << fn << ": Got H323 Call Proceeding for call " 
                << callsig.getSM() << VINFO_END(log);
}


void EvtCallProceeding::
handleH245Address(FeatureCallSig& callsig)
{
    const string    fn("EvtCallProceeding::handleH245Address");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    if (getMessage().HasOptionalField(H225_CallProceeding_UUIE::e_fastStart))
    {
        callsig.onFastStart(getMessage().m_fastStart);
    }

    if (callsig.getSM()->getRouterInfo()->routerMediaControl())
    {
        if (getMessage().HasOptionalField(H225_Setup_UUIE::e_h245Address))
        {
            if(getMessage().m_h245Address.GetTag() ==
               H225_TransportAddress::e_ipAddress)
            {
                EvtReceivedCCAddress event(callsig.getSM());
                const H225_TransportAddress_ipAddress& ipAddress =
                    getMessage().m_h245Address;
                event.destAddress_.setIP(ipAddress.m_ip);
                event.destAddress_.setPort(ipAddress.m_port);

                event.start();
            }
        }
    }
}


void EvtCallProceeding::
inOther(FeatureCallSig& callsig)
{
    const string    fn("EvtCallProceeding::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.wrongEventSequenceInd_.start();
    throw CallSigUnexpectedException(
        "Got unexpected message: CallProceeding",
        __FILE__, __LINE__);
}


EvtQ931B* EvtCallProceeding::
clone_impl()
{
    return new EvtCallProceeding(*this);
}


EvtCallProceeding::TagType EvtCallProceeding::
getTag() const throw()
{
    // (depends from Q.931/H.225 parser)
    return H225Tag::e_callProceeding;
}


#ifndef DISABLE_WRITER

ostream& EvtCallProceeding::
writeTo(ostream& out) const
{
    out << "EvtCallProceeding";
    EvtCallSig::writeTo(out);
    return out;
}

#endif
