#ifndef INCLUDED_EvtH245Msg_hxx
#define INCLUDED_EvtH245Msg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtH245Msg_hxx_Version = 
"$Id: EvtH245Msg.hxx,v 1.12 2000/12/18 23:47:34 bko Exp $";

#include "mte/EventStatic.hxx"
#include "H245Defs.hxx"
#include "EvtH245.hxx"

namespace router
{

    /// Event containing H.245 message
    class BEvtH245Msg: public EvtH245
    {
    public:
        /// Copy H.245 message to the event
        void setMessage(const H245Message& message);

    protected:
        const H245Message& getTopMessage() const throw();

    private:
        /// Update call control ID (call-back)
        virtual void updateID() {};

        H245Message message_;
    };

    typedef mte::EventStatic<BEvtH245Msg> EvtH245B;

    /// Event containing specific type of H.245 message
    template<class Msg, class Tag>
    class EvtH245Msg: public EvtH245B
    {
    public:
        typedef Tag TagType;
        typedef EvtH245B BaseType;

        /// Get message type for the H.245 message
        TagType getType() const throw();

    protected:
        const Msg& getSelMessage() const throw();

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Get tag for the H.245 message (call-back)
        virtual TagType getTag() const throw() = 0;
    };

    //----------------------------
    // Inline and template methods
    //----------------------------

    inline void BEvtH245Msg::
    setMessage(const H245Message& message)
    {
        message_ = message;
        updateID();
    }

    inline const H245Message& BEvtH245Msg::
    getTopMessage() const throw()
    {
        return message_;
    }

    template<class Msg, class Tag>
    inline const Msg& EvtH245Msg<Msg, Tag>::
    getSelMessage() const throw()
    {
        return static_cast<const Msg&>(getTopMessage());
    }

    template<class Msg, class Tag>
    inline EvtH245Msg<Msg, Tag>::TagType EvtH245Msg<Msg, Tag>::
    getType() const throw()
    {
        return getTag();
    }

#ifndef DISABLE_WRITER
    template<class Msg, class Tag>
    ostream &EvtH245Msg<Msg, Tag>::
    writeTo(ostream& out) const
    {
        out << endl;
        return out;
    }
#endif

    typedef EvtH245Msg<H245Request, H245RequestTag> EvtH245Request;
    typedef EvtH245Msg<H245Response, H245ResponseTag> EvtH245Response;
    typedef EvtH245Msg<H245Command, H245CommandTag> EvtH245Command;
    typedef EvtH245Msg<H245Indication, H245IndicationTag> EvtH245Indication;
}

#endif
