/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtLCIEstablishResp_cxx_Version = 
    "$Id: EvtLCIEstablishResp.cxx,v 1.8 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "H323Consts.hxx"
#include "EvtLCIEstablishResp.hxx"
#include "FeatureLCI.hxx"
#include "SMCall.hxx"
#include "ConnectionCallCtl.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtLCIEstablishResp::
EvtLCIEstablishResp(RouterLCI* sm)
    : EvtUserLCI(sm)
{
}


void EvtLCIEstablishResp::
inWaitEstabl(FeatureLCI& lci)
{
    const string    fn("EvtLCIEstablishResp::inWaitEstabl");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // (depends from H.245 parser)

    lci.setState(&PatternLCI::inEstablished, "inEstablished");

    // Send OLC Ack message
    assert(lci.getSM()->getLCN());
    olcAck_.m().m_forwardLogicalChannelNumber = lci.getSM()->getLCN();
    lci.getSM()->getSMCall()->getCallCtlConnection()->send(olcAck_);
}


void EvtLCIEstablishResp::
inOther(FeatureLCI& lci)
{
    const string    fn("EvtLCIEstablishResp::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


#ifndef DISABLE_WRITER

ostream& EvtLCIEstablishResp::
writeTo(ostream& out) const
{
    out << "EvtLCIEstablishResp";
    return out;
}

#endif
