/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF 1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtMSDConfirmed_cxx_Version = 
    "$Id: EvtMSDConfirmed.cxx,v 1.3 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtMSDConfirmed.hxx"
#include "FeatureH245.hxx"
#include "SMCall.hxx"
#include "H323Consts.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtMSDConfirmed::
EvtMSDConfirmed(SMCall* sm)
    : EvtUserH245(sm)
{
}


void EvtMSDConfirmed::
inPaused(FeatureH245& h245)
{
    const string    fn("EvtMSDConfirmed::inPaused");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.msdStarted = false;
    h245.msdCompleted = true;
}


void EvtMSDConfirmed::
inTPaused(FeatureH245& h245)
{
    const string    fn("EvtMSDConfirmed::inTPaused");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.msdStarted = false;
    h245.msdCompleted = true;
}


void EvtMSDConfirmed::
inTCE(FeatureH245& h245)
{
    const string    fn("EvtMSDConfirmed::inTCE");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.msdStarted = false;
    h245.msdCompleted = true;

    if (h245.gotTCSAck && h245.gotTCS)
    {
	h245.setState(&PatternH245::inOpen, "inOpen");
    }
}


void EvtMSDConfirmed::
inOther(FeatureH245& h245)
{
    const string    fn("EvtMSDConfirmed::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    h245.msdStarted = false;
    h245.msdCompleted = true;

    VWARN(log) << fn << ": Unexpected EvtMSDConfirmed event for call "
               << h245.getSM() << VWARN_END(log);
}


#ifndef DISABLE_WRITER

ostream& EvtMSDConfirmed::
writeTo(ostream& out) const
{
    out << "EvtMSDConfirmed";
    return out;
}

#endif
