/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtMSDeterminationAck_cxx_Version = 
    "$Id: EvtMSDeterminationAck.cxx,v 1.13 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtMSDeterminationAck.hxx"
#include "FeatureMsd.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtMSDeterminationAck::
EvtMSDeterminationAck(RouterCallCtl* sm)
    : EvtCallCtlMsdResponse(sm)
{
}


void EvtMSDeterminationAck::
inIdle(FeatureMsd& msd)
{
    const string    fn("EvtMSDeterminationAck::inIdle");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VINFO(log) << fn << ": Got MSD Ack for call "
               << msd.getSM() << VINFO_END(log);
}


void EvtMSDeterminationAck::
inWaitOutgoing(FeatureMsd& msd)
{
    const string    fn("EvtMSDeterminationAck::inWaitOutgoing");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // (depends from H.245 parser)
    
    msd.sv_STATUS = ((getMessage().m_decision.GetTag() ==
                      H245_MasterSlaveDeterminationAck_decision::e_master) ?
                     FeatureMsd::SV_Master :
                     FeatureMsd::SV_Slave);

    // Send Ack message
    BH245MSDeterminationAck ack;
    bool isMaster = (msd.sv_STATUS == FeatureMsd::SV_Master);
    ack.m().m_decision.SetTag(
        !isMaster ?
        H245_MasterSlaveDeterminationAck_decision::e_master :
        H245_MasterSlaveDeterminationAck_decision::e_slave);
    msd.getSM()->getCallCtlConnection()->send(ack);

    msd.setState(&PatternMsd::inIdle, "inIdle");

    if (msd.callbackConfirm_.get())
    {
	msd.callbackConfirm_->isMaster_ = isMaster;
	msd.callbackConfirm_.start();
    }
}


void EvtMSDeterminationAck::
inWaitIncoming(FeatureMsd& msd)
{
    const string    fn("EvtMSDeterminationAck::inWaitIncoming");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // (depends from H.245 parser)
    
    FeatureMsd::SV_Status receivedStatus =
        ((getMessage().m_decision.GetTag() ==
          H245_MasterSlaveDeterminationAck_decision::e_master) ?
         FeatureMsd::SV_Master :
         FeatureMsd::SV_Slave);

    if ((receivedStatus == msd.sv_STATUS) &&
        (msd.sv_STATUS != FeatureMsd::SV_Indeterminate))
    {
        msd.setState(&PatternMsd::inIdle, "inIdle");

	if (msd.callbackConfirm_.get())
	{
	    msd.callbackConfirm_->isMaster_ =
		(receivedStatus == FeatureMsd::SV_Master);
	    msd.callbackConfirm_.start();
	}
    }
    else
    {
        VWARN(log)
            << "Inconsistent 'decision' value in"
            "master-slave determination procedure"
            << VWARN_END(log);
        msd.setState(&PatternMsd::inIdle, "inIdle");
        msd.callbackReject_.start();
    }
}


void EvtMSDeterminationAck::
inOther(FeatureMsd& msd)
{
    assert(false);
}


EvtH245B* EvtMSDeterminationAck::
clone_impl()
{
    return new EvtMSDeterminationAck(*this);
}


H245ResponseTag EvtMSDeterminationAck::
getTag() const throw()
{
    // (depends from H.245 parser)
    return H245Response::e_masterSlaveDeterminationAck;
}


#ifndef DISABLE_WRITER

ostream& EvtMSDeterminationAck::
writeTo(ostream& out) const
{
    out << "EvtMSDeterminationAck";
    EvtCallCtlMsdResponse::writeTo(out);
    return out;
}

#endif
