#ifndef INCLUDED_FeatureCallCtlSession_hxx
#define INCLUDED_FeatureCallCtlSession_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const FeatureCallCtlSession_hxx_Version = 
"$Id: FeatureCallCtlSession.hxx,v 1.7 2000/12/18 23:47:36 bko Exp $";

#include "Sptr.hxx"

#include "mte/EventPtr.hxx"
#include "mte/Feature.hxx"

#include "H323Consts.hxx"
#include "H323RouterTypes.hxx"
#include "PatternCallCtlSession.hxx"
#include "FeatureCallCtlSessionDefs.hxx"
#include "EvtCallCtlSessionTimer.hxx"
#include "EntryCallCtlSession.hxx"
#include "ExitCallCtlSession.hxx"

namespace router
{

    class SMCall;
    
    /// Call control session state variables
    class CallCtlSessionData
    {
    private:
        CallCtlSessionData()
            {}

        // These classes are allowed to access state variables:
        friend class FeatureCallCtlSession;
    };

    /// Feature for Call control session
    class FeatureCallCtlSession:
        public FeatureCallCtlSessionBase,
        public CallCtlSessionData
    {
    public:
        /// Construct feature and assign pointer to state machine
        FeatureCallCtlSession(SMCall* sm);
        
        virtual ~FeatureCallCtlSession();

        /// Return pointer to the state machine
        SMCall* getSM() const;

        /// Reset feature
        virtual void reset();

        /// Start timer
        void startTimer();

        /// Stop timer
        void stopTimer();

        // Callback events

        mte::EventPtr<CallbackEvent> ccConnected_;

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Process event (implementation)
        virtual void process_impl(PatternCallCtlSession& pattern);

#ifndef DISABLE_WRITER
        char* getStateName() const;
#endif
        
        SMCall* const sm_;
        Sptr<EvtCallCtlSessionTimer> timer_;
        EntryCallCtlSession entry_;
        ExitCallCtlSession exit_;
    };
    
    //----------------------------
    // Inline  methods
    //----------------------------

    inline SMCall* FeatureCallCtlSession::
    getSM() const
    {
        return sm_;
    }

}

#endif
