
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FeatureCallSig_cxx_Version = 
"$Id: FeatureCallSig.cxx,v 1.11 2001/10/19 21:37:34 icahoon Exp $";
#include "FeatureCallSig.hxx"
#include "SMCall.hxx"

using namespace router;


FeatureCallSig::
FeatureCallSig(SMCall* sm)
    : sm_(sm)
      , timer_(new EvtCallSigTimer(sm))
{
    setEntry(&entry_);
    setExit(&exit_);
    reset();
}


FeatureCallSig::
~FeatureCallSig()
{
    // Call exit methods for any state (inIdleRAS may be used also)
    setState(&PatternCallSig::inIdleNoRAS, "inIdleNoRAS");
}


void FeatureCallSig::
reset()
{
    // TODO: determine if RAS is to be used
    setStateStealth(&PatternCallSig::inIdleNoRAS, "inIdleNoRAS");
}


void FeatureCallSig::
stopAndReset()
{
    setState(&PatternCallSig::inIdleNoRAS, "inIdleNoRAS");
    reset();
}


void FeatureCallSig::
process_impl(PatternCallSig& pattern)
{
    (pattern.*(getState()))(*this);
}


void FeatureCallSig::
startTimer(mte::TimeInterval timeout)
{
    assert(this);

    timer_->setTimeout(timeout);

    if (timeout)
    {
        getSM()->startTimer(timer_);
    }
}


void FeatureCallSig::
stopTimer()
{
    assert(this);

    getSM()->stopTimer(timer_);
}


void FeatureCallSig::
onFastStart(const H225_ArrayOf_PASN_OctetString& source)
{
    if (source.GetSize() > 0)
    {
        // Start 'enter fs' event
        fsEnterInd_.start();
    }

    // For each attached OLC element:
    for (unsigned int i = 0; i < source.GetSize(); i ++)
    {
        // Extract OLC PDU from the source
        PASN_OctetString &rOLOCStream = source[i];

        // Prepare stream for decoding
        PPER_Stream strm(
            (const unsigned char*)rOLOCStream.GetPointer(),
            rOLOCStream.GetSize());

        // Make decoded OLC message
        if (fsOLCInd_.get() != 0)
        {
            fsOLCInd_->message_.Decode(strm);
        }

        // Start 'fs OLC' event
        fsOLCInd_.start();
    }

    if (source.GetSize() > 0)
    {
        // Start 'exit fs' event
        fsExitInd_.start();
    }
}


#ifndef DISABLE_WRITER

ostream& FeatureCallSig::
writeTo(ostream& out) const
{
    out << "FeatureCallSig:"
        << " state=" << getStateName();
    return out;
}


char* FeatureCallSig::
getStateName() const
{
    if (getState() == &PatternCallSig::inIdleRAS)
        return "IdleRAS";
    else if (getState() == &PatternCallSig::inIdleNoRAS)
        return "IdleNoRAS";
    else if (getState() == &PatternCallSig::inAdmitted)
        return "Admitted";
    else if (getState() == &PatternCallSig::inAlerting)
        return "Alerting";
    else if (getState() == &PatternCallSig::inSetup)
        return "Setup";
    else if (getState() == &PatternCallSig::inAdmission)
        return "Admission";
    else if (getState() == &PatternCallSig::inConnecting)
        return "Connecting";
    else if (getState() == &PatternCallSig::inInitiated)
        return "Initiated";
    else if (getState() == &PatternCallSig::inAck)
        return "Ack";
    else if (getState() == &PatternCallSig::inWaitConnect)
        return "WaitConnect";
    else if (getState() == &PatternCallSig::inConnect)
        return "Connect";
    else if (getState() == &PatternCallSig::inOther)
        return "Other";
    else
        return "UNKNOWN";
}

#endif
