#ifndef INCLUDED_FeatureCallSig_hxx
#define INCLUDED_FeatureCallSig_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const FeatureCallSig_hxx_Version = 
"$Id: FeatureCallSig.hxx,v 1.9 2001/03/09 06:54:42 icahoon Exp $";

#include "Sptr.hxx"

#include "mte/EventPtr.hxx"
#include "mte/Feature.hxx"

#include "H323Consts.hxx"
#include "H323RouterTypes.hxx"
#include "PatternCallSig.hxx"
#include "FeatureCallSigDefs.hxx"
#include "EvtCallSigTimer.hxx"
#include "EntryCallSig.hxx"
#include "ExitCallSig.hxx"
#include "EvtFSOLC.hxx"

namespace router
{

    class SMCall;
    
    /// Call Signalling state variables
    class CallSigData
    {
    private:
        CallSigData()
            {}

        // These classes are allowed to access state variables:
        friend class FeatureCallSig;
        friend class EvtCallInit;
        friend class EvtCallOffHook;
        friend class EvtTCPConnected;
        friend class EvtSetup;
        friend class EvtCallProceeding;
        friend class EvtAlerting;
        friend class EvtConnect;
    };

    /// Feature for Call Signalling
    class FeatureCallSig:
        public FeatureCallSigBase,
        public CallSigData
    {
    public:
        /// Construct feature and assign pointer to state machine
        FeatureCallSig(SMCall* sm);
        
        virtual ~FeatureCallSig();

        /// Return pointer to the state machine
        SMCall* getSM() const;

        /// Reset feature
        virtual void reset();

        /// Stop timer and reset feature
        virtual void stopAndReset();

        /// Start timer
        void startTimer(mte::TimeInterval timeout);

        /// Stop timer
        void stopTimer();

        void onFastStart(const H225_ArrayOf_PASN_OctetString& source);

        // Callback events

        mte::EventPtr<CallbackEvent> wrongEventSequenceInd_;
        mte::EventPtr<CallbackEvent> noAnswerInd_;
        mte::EventPtr<CallbackEvent> connectionEstablTimeoutInd_;
        mte::EventPtr<CallbackEvent> connectionAcceptTimeoutInd_;
        mte::EventPtr<CallbackEvent> userTimeoutInd_;
        mte::EventPtr<CallbackEvent> connectionTimeoutInd_;
        mte::EventPtr<CallbackEvent> rasTimeoutInd_;
        mte::EventPtr<CallbackEvent> callCleanupInd_;

        mte::EventPtr<CallbackEvent> fsEnterInd_;
        mte::EventPtr<EvtFSOLC> fsOLCInd_;
        mte::EventPtr<CallbackEvent> fsExitInd_;

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Process event (implementation)
        virtual void process_impl(PatternCallSig& pattern);

#ifndef DISABLE_WRITER
        char* getStateName() const;
#endif
        
        SMCall* const sm_;
        Sptr<EvtCallSigTimer> timer_;
        EntryCallSig entry_;
        ExitCallSig exit_;
    };
    
    //----------------------------
    // Inline  methods
    //----------------------------

    inline SMCall* FeatureCallSig::
    getSM() const
    {
        return sm_;
    }

}

#endif
