#ifndef INCLUDED_SMCall_hxx
#define INCLUDED_SMCall_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SMCall_hxx_Version = 
"$Id: SMCall.hxx,v 1.18 2001/04/03 07:51:15 icahoon Exp $";

#ifndef DISABLE_WRITER
#include "Writer.hxx"
#endif

#include "IPAddress.hxx"

#include "mte/Mediator.hxx"
#include "mte/Feature.hxx"

#include "PatternCallSig.hxx"
#include "PatternMsd.hxx"
#include "PatternRTD.hxx"
#include "PatternCEO.hxx"
#include "PatternCEI.hxx"
#include "PatternCallCtlSession.hxx"
#include "PatternOpenCC.hxx"
#include "PatternAcceptCC.hxx"
#include "FeatureCallSigDefs.hxx"
#include "FeatureMsdDefs.hxx"
#include "FeatureRTDDefs.hxx"
#include "FeatureCEODefs.hxx"
#include "FeatureCEIDefs.hxx"
#include "FeatureCallCtlSessionDefs.hxx"
#include "FeatureOpenCCDefs.hxx"
#include "FeatureAcceptCCDefs.hxx"
#include "FeatureH245Defs.hxx"
#include "Q931Defs.hxx"
#include "FifoBase.h"
#include "Sptr.hxx"
#include "Event.hxx"
#include "mte/EvtTimer.hxx"
#include "H323RouterTypes.hxx"

namespace Vocal
{
    namespace Services {
    	class PollFifoService;
    }
    namespace Transport {
    	class TPKTClientSocket;
    }
    namespace SipH323Info {
        class RouterInfo;
    }
    class Q931ProtocolBase;
}

namespace router
{
    using Vocal::Services::PollFifoService;
    using Vocal::Transport::TPKTClientSocket;
    using Vocal::Transport::IPAddress;
    using Vocal::SipH323Info::RouterInfo;
    using Vocal::Q931ProtocolBase;

    class ConnectionCallSig;
    class ConnectionCallCtl;
    class RouterLCO;
    class RouterLCI;
    class H245Protocol;
    class SesH245;

    /// State machine for H.323 call
    class SMCall
#ifndef DISABLE_WRITER
        : public Vocal::IO::Writer
#endif
    {
    public:
        /// Construct state machine for H.323 call
        SMCall();

        virtual ~SMCall();

        /// Return object that owns this call (call-back)
        virtual mte::Mediator& getMaster() throw() = 0;

        /// Return call signalling connection (call-back)
        virtual ConnectionCallSig* getCallSigConnection() throw() = 0;

        /// Return call control connection (call-back)
        virtual ConnectionCallCtl* getCallCtlConnection() throw() = 0;

        /// Return pointer to Call Signalling feature (call-back)
        virtual FeatureCallSigBase* getCallSig() throw() = 0;
        virtual FeatureCallSig& getCallSigImp() throw() = 0;

        /// Return pointer to MS Determination feature (call-back)
        virtual FeatureMsdBase* getMsd() throw() = 0;

        /// Return pointer to Roundtrip Determination feature (call-back)
        virtual FeatureRTDBase* getRTD() throw()
            = 0;

        /// Return pointer to capability exchange (out) feature (call-back)
        virtual FeatureCEOBase* getCEO() throw()
            = 0;

        /// Return pointer to capability exchange (in) feature (call-back)
        virtual FeatureCEIBase* getCEI() throw()
            = 0;

        /// Return pointer to call sontrol session feature (call-back)
        virtual FeatureCallCtlSessionBase* getCallCtlSession() throw()
            = 0;

        /// Return pointer to open ctl. channel feature (call-back)
        virtual FeatureOpenCCBase* getOpenCC() throw() = 0;

        /// Return pointer to accept ctl channel feature (call-back)
        virtual FeatureAcceptCCBase* getAcceptCC() throw() = 0;

        /// Return pointer to H245 feature (call-back)
        virtual FeatureH245Base* getH245() throw() = 0;

        /// Return pointer to logical channel (out) router (call-back)
        virtual RouterLCO* getRtLCO() throw() = 0;

        /// Return pointer to logical channel (in) router (call-back)
        virtual RouterLCI* getRtLCI() throw() = 0;

        virtual void makeH245Protocol() = 0;

        virtual PollFifoService* getFifo() throw() = 0;

        virtual int getTerminalType() const throw();

        virtual void setSiteInfo(EndpointType& info) const;

        virtual void selectActiveProtocol() throw() = 0;

        virtual void selectPassiveProtocol() throw() = 0;

        virtual const IPAddress& getH245Address() const = 0;

        virtual TPKTClientSocket* getH245Socket() = 0;;

        virtual IPAddress getCallSignalAddress() const = 0;

        virtual Q931ProtocolBase* getProtocol() throw () = 0;

        virtual H245Protocol* getH245Protocol() throw () = 0;

        virtual RouterInfo* getRouterInfo() throw() = 0;

        virtual int getTerminateReason() const throw() = 0;

        virtual void setTerminateReason(int reason) throw() = 0;

        virtual void disable(H245Protocol* protocol) = 0;

        virtual bool isActiveH245AddressGreater() = 0;

        virtual void complete() = 0;

        virtual void terminate(
            int reason = H225_ReleaseCompleteReason::e_undefinedReason) = 0;

        virtual void close() = 0;

        virtual void startTimer(Sptr<mte::EvtTimer<TimerEvent> >) = 0;

        virtual void stopTimer(Sptr<mte::EvtTimer<TimerEvent> >) = 0;

        virtual SesH245* getSesH245() throw () = 0;

    private:
#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif
    };

}

#endif
