
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const tCallSig_cxx_Version = 
"$Id: tCallSig.cxx,v 1.4 2000/12/18 23:48:21 bko Exp $";
#include <assert.h>
#include <iostream>

#include "VLog.hxx"

#include "Call.hxx"
#include "RouterCallSig.hxx"
#include "EvtCallInit.hxx"
#include "EvtTCPConnected.hxx"
#include "EvtCallProceeding.hxx"
#include "EvtAlerting.hxx"
#include "EvtConnect.hxx"
#include "SesQ931.hxx"
#include "RouterInfo.hxx"
#include "PollFifo.hxx"
#include "PollFifoService.hxx"

using namespace router;


int main()
{
    Vovida::VLog::init(LOG_DEBUG);
    Vovida::VLog::on(LOG_DEBUG);
    Vovida::VLog::on(LOG_WARNING);
    
    RouterCallSig router;
    mte::Mediator null;
    Call* call = new Call(null);
    RouterInfo ri(Vovida::IPAddress("22.22.22.22:22"), false, false);
    PollFifo< Sptr<Event> > fb;
    PollFifoService f(fb);
    call->setFifo(&f);
    call->setRouterInfo(&ri);
    call->setCallSigRouter(&router);
    call->init();
    SesQ931 ses;
    EvtCallInit init(call);
    EvtTCPConnected tcp(call);
    EvtCallProceeding* cp = new EvtCallProceeding(&router);
    ses.add(cp);
    EvtAlerting* alerting = new EvtAlerting(&router);
    ses.add(alerting);
    EvtConnect* connect = new EvtConnect(&router);
    ses.add(connect);

    BH225Connect msgConnect;
    BH225Alerting msgAlerting;
    BH225CallProceeding msgCallProceeding;

    assert(call->getCallSigConnection());

    cerr << "Initial state:" << endl;
    cerr << *call;
    
    init.start();
    cerr << "State after request:" << endl;
    cerr << *call;

    tcp.start();
    cerr << "State after TCP connected:" << endl;
    cerr << *call;

    ses.generate(msgCallProceeding, call->getCallSigConnection());
    cerr << "State after call proceeding:" << endl;
    cerr << *call;

    ses.generate(msgAlerting, call->getCallSigConnection());
    cerr << "State after alerting:" << endl;
    cerr << *call;

    ses.generate(msgConnect, call->getCallSigConnection());
    cerr << "State after connect:" << endl;
    cerr << *call;

    return 0;
}
