#ifndef INCLUDED_CallTranslatorEM_hxx
#define INCLUDED_CallTranslatorEM_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CallTranslatorEM_hxx_Version = 
"$Id: CallTranslatorEM.hxx,v 1.1 2001/05/09 23:15:42 vzubarev Exp $";

#include "EvtResolve.hxx"
#include "EvtSendFSOLC.hxx"
#include "EvtZeroTCS.hxx"
#include "EvtZeroSDP.hxx"
#include "EvtSendSDP.hxx"
#include "EvtCEOTransferReq.hxx"
#include "EvtLCOEstablishReq.hxx"
#include "EvtLCOReleaseReq.hxx"
#include "EvtLCIEstablishResp.hxx"
#include "EvtLCIReleaseReq.hxx"
#include "EvtRCORequest.hxx"
#include "EvtCreateInLC.hxx"
#include "EvtCreateOutLC.hxx"
#include "EvtSDPParseDone.hxx"
#include "EvtReceivedCCAddress.hxx"

namespace translator
{

    class CallTranslatorEM
    {
    public:
        CallTranslatorEM();

        virtual ~CallTranslatorEM();

        // Registration
        void add(EvtResolve* evtResolve);
        void add(router::EvtSendFSOLC* evtSendFSOLC);
        void add(EvtZeroTCS* evtZeroTCS);
        void add(EvtZeroSDP* evtZeroSDP);
        void add(EvtSendSDP* evtSendSDP);
        void add(router::EvtCEOTransferReq* evtCEOTransferReq);
        void add(router::EvtLCOEstablishReq* evtLCOEstablishReq);
        void add(router::EvtLCOReleaseReq* evtLCOReleaseReq);
        void add(router::EvtLCIEstablishResp* evtLCIEstablishResp);
        void add(router::EvtLCIReleaseReq* evtLCIReleaseReq);
        void add(router::EvtRCORequest* evtRCORequest);
        void add(router::EvtCreateInLC* evtCreateInLC);
        void add(router::EvtCreateOutLC* evtCreateOutLC);
        void add(EvtSDPParseDone* evtSDPParseDone);
        void add(router::EvtReceivedCCAddress* evtForceCCOpen);

        // Invokation
        void startResolve(std::string hostName);
        void startSendFSOLC(const router::H245OLC& olc);
        void startZeroTCS();
        void startZeroSDP();
        void startSendSDP(const SdpSession& sdp, bool change);
        void startCEOTransferReq(const router::H245TCS& tcs);
        void startLCOEstablishReq(const router::H245OLC& olc);
        void startLCOReleaseReq(const router::H245CLC& clc);
        void startLCIEstablishResp(const router::H245OLCAck& olcAck);
        void startLCIReleaseReq(const router::H245OLCReject& olcReject);
        void startRCORequest(const router::H245RCC& rcc);
        void startCreateInLC(int lcNumber);
        void startCreateOutLC(int lcNumber);
        void startSDPParseDone();
        void startForceCCOpen();

    protected:
        EvtResolve* evtResolve_;
        router::EvtSendFSOLC* evtSendFSOLC_;
        EvtZeroTCS* evtZeroTCS_;
        EvtZeroSDP* evtZeroSDP_;
        EvtSendSDP* evtSendSDP_;
        router::EvtCEOTransferReq* evtCEOTransferReq_;
        router::EvtLCOEstablishReq* evtLCOEstablishReq_;
        router::EvtLCOReleaseReq* evtLCOReleaseReq_;
        router::EvtLCIEstablishResp* evtLCIEstablishResp_;
        router::EvtLCIReleaseReq* evtLCIReleaseReq_;
        router::EvtRCORequest* evtRCORequest_;
        router::EvtCreateInLC* evtCreateInLC_;
        router::EvtCreateOutLC* evtCreateOutLC_;
        EvtSDPParseDone* evtSDPParseDone_;
        router::EvtReceivedCCAddress* evtForceCCOpen_;
    };
    
}

#endif
