#ifndef __CAPABMAP_H_
#define __CAPABMAP_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CapabilityMap_hxx_Version = 
"$Id: CapabilityMap.hxx,v 1.18 2002/03/01 22:51:07 bko Exp $";

#include <iostream>
#include <string>
#include <vector>
#include <iterator>
#include <map>

#include "Sdp2Session.hxx"
#include "ptlib.h"
#include "h245.h"
#include "GlobalCapTable.hxx"
#include "Capability.hxx"
#include "CapabilitySetSip.hxx"
#include "CapabilitySetH323.hxx"
#include "CapabilitySetIntersect.hxx"
#include "SessionDescriptorVec.hxx"
#include "XlatorException.hxx"
#include "H323_CapTable.hxx"

using namespace Translator;

class H245_TerminalCapabilitySet;
class H245_OpenLogicalChannel;
class H245_OpenLogicalChannelAck;

using Vocal::SDP::SdpSession;
using Vocal::SDP::SdpConnection;
using Vocal::SDP::SdpMedia;

namespace Translator
{
    typedef H245_TerminalCapabilitySet TCS;
    typedef H245_OpenLogicalChannel OLC;
    typedef H245_OpenLogicalChannelAck OLCAck;
    
    typedef Vocal::SDP::SdpSession SDP;

    ///
    class CapabilityMap
    {
    private:
        /// if true reject OLC with packetsize > packetsize we received from SDP
	bool strictModePackSize;
	
	/// The number of current CapabilityDescriptor from CapabilityDescrVec
	int curDescrN;
	
	/// The number of the last SessionID assigned
	int maxSesID;
	
	/// The number of the last logical channel opened
	int maxLogChN;
	
	/** Map between sessionID and the number of particular
	    AltCapSet in a current Descriptor Number (curDescrN)
	*/
	std::map<int, int> sessionAltCapMap;
	
	std::map<int, int>::const_iterator sA_I;
	
	/** Map between Logical Channel number of the particular OLC
	    and altCapSet for which we have sent an OLC
	    We will need it to map incoming OLCAck with the particular
	    altCapSet and SessionDescriptor
	*/
	std::map<int, int> altCapLogChMap;
	
	std::map<int, int>::const_iterator aL_I;
	
	bool master;
	
	/// indicate whether we received SDP yet or not
	bool receivedSDP;
	
	/// counter of a number of OLC messages sent
	int sentOLC;
	
	/// counter of a number of OLCAck messages received
	int receivedOLCAck;
	
	/// counter of a number of OLC messages received
	int receivedOLC;
	
	/// Number of capabilities we currently support
	enum { capNumber = 9 };
	
	// The constant to start assign sessionID with
	// if we are master and receive 0 from slave
	//
	//enum { sesConstID = 4 };

	/// capTable to store capabilities to form Outcoming SDP msg
	H323_CapTable fromOLCtoSDPcapTable;
	
	///
	GlobalCapTable globalCapTable;
	
	/** map between incoming CapTable numeration and our inner numeration
	    of capabilities (as assigned by payload types in GlobalCapTable)
	*/
	std::map<int, int> mapTable;
	
	///
	typedef std::map<int, Capability*> CapMap;
	
	/** map between the payload type (as assigned in GlobalCapTable)
	    and particular capability object
	    will be used to construct capability Table
	*/
	CapMap capabilityMap;
	
	///
	CapabilitySetSip capSetSip;
	
	///
	CapabilitySetH323 capSetH323;
	
	///
	CapabilitySetIntersect capSetIntersect;
	
	///
	SessionDescriptorVec sDescriptor;
	
	/** Media descriptor table, containing media descriptor numbers
	    (number of altCapSet which has already been associated with
	     sessionID in sessionAltCapMap)
	*/
	typedef std::vector<int> MediaDescrTable;

	///
	MediaDescrTable mediaDescrTable;
	
	/// Media descriptor table iterator type
	typedef MediaDescrTable::const_iterator MediaDescrTable_I;
	
	/// Media descriptor table iterator
	MediaDescrTable_I mIter;
	
	/// fill capabilityMap out
	void fillCapMap();
	
	/** if master generate new sID
	    in either case make an association
	    between sID and altCapN
	*/
	void generateSID(int &sID, int altCapN);
	
	/** find an altCapSetNumber by a pTypeCap
	    used when we receive OLC with particular cap
	*/
	int findAltCapSetN(int pTypeCap);
	
	/// find first Audio or Video Cap
	int findFirst(char cap, int &index) throw (XlatorException);
	
	/// find the next cap for which OLC has not been issued yet
	int findNext(int &index) throw (XlatorException);
	
	/// Initializer/accessor for media descriptor table iterator begin
	MediaDescrTable_I beginTableIter()
	    { mIter = mediaDescrTable.begin(); return mIter; }
	
	/// Accessor for media descriptor table iterator end
	MediaDescrTable_I endTableIter()
	    { mIter = mediaDescrTable.end(); return mIter; }

    public:
	/// Constructors
	CapabilityMap(bool mode = false);
	
	///
	virtual ~CapabilityMap();

	/// set myself (Translator) Master or Slave
	void setMaster(bool mI);
	
	/// Set capabilities, transport ID of Sip side (from SDP msg)
	void handleSDP(SDP &msgSDP) throw (XlatorException);
	
	/// Get capabilities of Sip side to make TCS msg
	void makeTCS(TCS &msgTCS) const;
	
	/// Set capabilities of H323 Side (from TCS msg)
	void handleTCS(const TCS &msgTCS) throw (XlatorException);
	
	/// Intersect Sip & H323 CapabilitySets
	void makeIntersect() throw (XlatorException);
	
	/// Get TransportIDs of Sip side & one of Intersected (common) caps (media) to make OLC msg
	void makeOLC(OLC &msgOLC) throw (XlatorException);

	/// Get TransportIDs of H323 side for a given media session to make SDP msg
	void makeSDP(SDP &msgSDP) throw (XlatorException);
	
	/** Set TransportIDs (get from OLCAck msg) of H323 side for a given media session
	    if returns false - OLCAck msg is bad - should issue CloseChannel for this channel
	*/
	bool handleOLCAck(const OLCAck&);
	
	/** Set TransportIDs (get from OLC msg) of H323 side for a given media session
	    & form OLCAck msg
	    if returns false - OLC msg is bad (for instance, not mediaChannel specified)
	*/
	bool handleOLC_makeOLCAck(const OLC&, OLCAck&, H245_OpenLogicalChannelReject_cause::Choices* RejCause = NULL);
	
	///
	bool isSDPreceived() const;
	
	///
	bool areAllOLCsent() const;
	
	///
	bool areAllOLCAckreceived() const;
	
	///
	bool areAllOLCreceived() const;

	// only for debugging purposes
	//
	void fillDebug();
    };
}

#endif
