/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CapabilitySetIntersect_cxx_Version = 
    "$Id: CapabilitySetIntersect.cxx,v 1.11 2001/01/15 21:54:22 vzubarev Exp $";


#include <algorithm>
#include "VLog.hxx"
#include "Assert.hxx"
#include "CapabilitySetIntersect.hxx"

#ifndef DEBUG
#define DEBUG 1
#endif

using namespace Translator;
using Vocal::Logging::VLog;

int CapabilitySetIntersect::getArrangement(int n, int m)
{
  long int  nFact = 0, mFact = 0;
  int i = 0;
  
  nFact = n;
  for(i = n-1; i > 1; i--)nFact = nFact * i;
  
  mFact = n - m;
  for(i = mFact -1; i > 1; i--)mFact = mFact * i;
  
  return nFact/mFact;
}

void CapabilitySetIntersect::makeIntersect(bool tableInt) throw (XlatorException)
{
  const string	fn("CapabilitySetIntersect::makeIntersect");
  VLog	log(fn);
  
  int descrVecSizeSip = 0, descrSizeSip = 0, i = 0, j = 0, h = 0;
  int descrVecSizeH = 0, descrSizeH = 0, y = 0;
  int descrSizeLarge = 0, descrSizeSmall = 0, iSm = 0, iLg = 0, curIntDescr = 0;
  int hTableSize = 0, sTableSize = 0, largeTableSize = 0, smallTableSize = 0;
  unsigned int z = 0;
  
  std::vector<int> permVec;
  bool sipIsLarge, iDone = false, found = false;

  
  VDEBUG(log) << fn << ": BEGIN ------------------ " << VDEBUG_END(log);
  
  if(tableInt)
  {
   // Intersection of Capability Tables
   // At this point we compare only packet size parameters
   // find the minimum we could support
  
   sTableSize = capSetSip.h323CapTable.getCapabilityTable().size();
   VDEBUG(log) << fn << ": Size of Sip capTable : " << sTableSize << VDEBUG_END(log);
  
   hTableSize = capSetH323.h323CapTable.getCapabilityTable().size();
   VDEBUG(log) << fn << ": Size of H323 capTable : " << hTableSize << VDEBUG_END(log);

   if((hTableSize == 0)&&(sTableSize == 0))
        throw XlatorException("Both Cap Tables are empty...", __FILE__, __LINE__);

   if(hTableSize == 0)
        throw XlatorException("H323 capTable is empty...", __FILE__, __LINE__);
  
   if(sTableSize == 0)
        throw XlatorException("Sip capTable is empty...", __FILE__, __LINE__);
  
  if(!iDone)
  {
    H323_CapTable::CapTable largeTable;
    H323_CapTable::CapTable smallTable;
    
    if(hTableSize > sTableSize)
    {
      largeTable = capSetH323.h323CapTable.getCapabilityTable();
      largeTableSize = hTableSize;
      
      smallTable = capSetSip.h323CapTable.getCapabilityTable();
      smallTableSize = sTableSize;
      sipIsLarge = false;
    }
    else
    {
      smallTable = capSetH323.h323CapTable.getCapabilityTable();
      largeTableSize = sTableSize;
      
      largeTable = capSetSip.h323CapTable.getCapabilityTable();
      smallTableSize = hTableSize;
      sipIsLarge = true;
    }
  
   VDEBUG(log) << fn << ": Right before capTable intersection and sipIsLarge : "	<< sipIsLarge
	      << ", largeTableSize :" << largeTableSize << VDEBUG_END(log);
  
   for(i = 0; i < largeTableSize; i++)
   {
    H323_CapTable::CapTable_I pI;
    
    if(!sipIsLarge)
    {
      pI = capSetSip.h323CapTable.getTableIter_pt(largeTable[i]->getCapabilityNumber());
    }
    else
    {
      VDEBUG(log) << fn << ": Right before getting pI, largeTable[" << i << "] : "
		  << largeTable[i]->getCapabilityNumber() << VDEBUG_END(log);
      pI = capSetH323.h323CapTable.getTableIter_pt(largeTable[i]->getCapabilityNumber());
      VDEBUG(log) << fn << ": Right after getting pI" << VDEBUG_END(log);
    }

    VDEBUG(log) << fn << ": Right before pI != NULL" << VDEBUG_END(log);
    
    if(pI != NULL)
    {
      VDEBUG(log) << fn << ": pI packetsize : " << (*pI)->getPacketSizeFrames() << VDEBUG_END(log);
      VDEBUG(log) << fn << ": largeTable[i] packetsize : " << largeTable[i]->getPacketSizeFrames() << VDEBUG_END(log);
      
      if((*pI)->getPacketSizeFrames() <= largeTable[i]->getPacketSizeFrames())
      {
        Capability *copy = (*pI)->clone();
	copy->assignCapabilityNumber((*pI)->getCapabilityNumber());
        copy->setPacketSizeFrames((*pI)->getPacketSizeFrames());

	h323CapTable.addCapTableEntry(copy);
	VDEBUG(log) << fn << ": Capability number : " << copy->getCapabilityNumber() << VDEBUG_END(log);
	VDEBUG(log) << fn << ": ... with packetSize: " << copy->getPacketSizeFrames() << VDEBUG_END(log);
	found = true;
      }
      else
      {
        Capability *copy = largeTable[i]->clone();
	copy->assignCapabilityNumber(largeTable[i]->getCapabilityNumber());
        copy->setPacketSizeFrames(largeTable[i]->getPacketSizeFrames());

	h323CapTable.addCapTableEntry(copy);
	VDEBUG(log) << fn << ": Capability number : " << copy->getCapabilityNumber() << VDEBUG_END(log);
	VDEBUG(log) << fn << ": ... with packetSize: " << copy->getPacketSizeFrames() << VDEBUG_END(log);
	found = true;
      }
    }
    else
    {
      VDEBUG(log) << fn << ": pI == NULL" << VDEBUG_END(log);
    }
   }
  }
  
  VDEBUG(log) << fn << ": Table intersection is done..." << VDEBUG_END(log);
  
  if(!found)
  {
    throw XlatorException("No common capabilities - not possible to intersect", __FILE__, __LINE__);
  }
    
 } // tableInt

  // Intersection of Capability Descriptor Vectors
  
  CapabilitySet::CapabilityDescrVec::AltCapSet altCapSetIntersect;
  CapabilitySet::CapabilityDescrVec::CapDescriptor capDescrIntersect; 

  capRecv.clear();
  descrVecSizeSip = capSetSip.capRecv.getCapDescrVecSize();
  descrVecSizeH = capSetH323.capRecv.getCapDescrVecSize();
  
  
  // first we find the capdescriptor with larger number of
  // alt cap sets, then for each permutation of this particular cap descriptor
  // intersect every altcapset from permutation with every
  // altcapset from smaller descriptor (in smaller descriptor we do NOT
  // trade places altcapsets)
  
  // sort all AltCapSets in both Sip & H323 CapDescriptors before intersection
  
  // capSetH323.capRecv.sortCapDescrVec();
  // capSetSip.capRecv.sortCapDescrVec();

  const CapabilitySet *capSetLarge;
  const CapabilitySet *capSetSmall;

  unsigned int w = 0;
  
  for(i = 0; i < descrVecSizeSip; i++)
  {
    // Every Sip CapDescriptor... (at this point only ONE SIP capability descriptor is
    // supported)
    
    descrSizeSip = capSetSip.capRecv.getCapDescrSize(i);

    for(j = 0; j < descrVecSizeH; j++)
    {
      // ...with every H323 CapDescriptor
      
      descrSizeH = capSetH323.capRecv.getCapDescrSize(j);

      // Get the size of the largest capability descriptor ...
      // e.g. Sip: {[] [] [] []}
      //      H323: {[] []}
	
      sipIsLarge = (descrSizeSip > descrSizeH ? true : false);
      descrSizeLarge = (sipIsLarge == true ? descrSizeSip : descrSizeH);
      descrSizeSmall = (sipIsLarge == true ? descrSizeH : descrSizeSip);
      
      if(sipIsLarge)
      {
        capSetLarge = &capSetSip;
	capSetSmall = &capSetH323;
      }
      else
      {
        capSetLarge = &capSetH323;
	capSetSmall = &capSetSip;
      }
      
      // fill out the permVec - the vector of indexes of the largest CapDescriptor
      // then get all permutations of one

      permVec.clear();

      for(h = 0; h < descrSizeLarge; h++)
      {
	permVec.push_back(h);
      }
      
      iLg = (sipIsLarge == true ? i : j);
      iSm = (sipIsLarge == true ? j : i);
	   
      // get all permutations of indexes of all AltCapSets in the largest CapDescriptor

      if(permVec.size() != 0)
      do
      {
        capDescrIntersect.clear();

	for(y = 0; y < descrSizeLarge; y++)
	{
           // for current permutation of the largest capDescriptor
	   // make intersection of two simultaneous cap sets (Sip & H323)
	   
	   if(y < descrSizeSmall)
	   {
             const CapabilitySet::CapabilityDescrVec::AltCapSet *altCapSip;
	     const CapabilitySet::CapabilityDescrVec::AltCapSet *altCapH323;
	     
	     if(sipIsLarge)
	     {
	       altCapSip = &capSetLarge->capRecv.getAltCapSet(iLg, permVec[y]);
	       altCapH323 = &capSetSmall->capRecv.getAltCapSet(iSm, y);
	     }
	     else
	     {
	       altCapH323 = &capSetLarge->capRecv.getAltCapSet(iLg, permVec[y]);
	       altCapSip = &capSetSmall->capRecv.getAltCapSet(iSm, y);
	     }
	     
	     altCapSetIntersect.clear();

	     // had to do our own intersect algorithm, since the one from STL works only
	     // with sorted containers, however in our case the original order in
	     // altCapSet is very important (H.323 standard)
	     
	     for(z = 0; z < altCapH323->size(); z++)
	     {
	       if(std::find(altCapSip->begin(), altCapSip->end(), (*altCapH323)[z]) != altCapSip->end())
	       {
	         altCapSetIntersect.push_back((*altCapH323)[z]);
	       }
	     }
	     
	     // see comment above
	     /*std::set_intersection(altCapLarge.begin(), altCapLarge.end(), 
	                           altCapSmall.begin(), altCapSmall.end(), 
				   inserter(altCapSetIntersect, altCapSetIntersect.begin()));
             */
	     
	     
	     if(altCapSetIntersect.size() != 0)
	     {
     	       try
	       {
		 capRecv.addAltCapSet(altCapSetIntersect, curIntDescr);
	       }
	       catch (XlatorException &mS)
	       {
	         VERR(log) << fn << ": Error with setting altCapSet" << VDEBUG_END(log);
		 assert(0);
	       }
	       
	       altCapH323 = &capRecv.getAltCapSet(curIntDescr, capRecv.getCapDescrSize(curIntDescr) - 1);
	       
	       VDEBUG(log) << fn << ": Int capSet :" << VDEBUG_END(log);
	       
	       for(w = 0; w < altCapH323->size(); w++)
	       {
	         VDEBUG(log) << fn << ":               " << (*altCapH323)[w] << VDEBUG_END(log);
	       }

	     }
	   }
	}
        if(capRecv.getCapDescrSize(curIntDescr) != 0)curIntDescr++;
      }
      while(std::next_permutation(permVec.begin(), permVec.end()));
    }
  }
   // check whether we have duplicate capability descriptors
   // e. g. : {[a1, a2] [v1, v2]} and
   //         {[a1, a2] [v1]}
   
   // do not use it at this point - need to test it
   // not a high priority though, since currently we are using
   // only FIRST capability descrptor from intersection, do not
   // see the case now, when we are using the rest of them

   /*int altSizeH = 0, y = 0, descrVecInter = 0, descrInter = 0, altSizeSip = 0;
   int descrVecSizeInt = 0, descrSizeInt = 0, altSizeInt = 0;
   bool both;
   
   descrVecSizeInt = capRecv.getCapDescrVecSize();
   for(i = 0; i < descrVecSizeInt; i ++)
   {
     descrSizeSip = capRecv.getCapDescrSize(i);
     
     for(j = i + 1; j < descrVecSizeInt; j++)
     {
       descrSizeH = capRecv.getCapDescrSize(j);
       
       descrSizeLarge = (descrSizeSip > descrSizeH ? descrSizeSip : descrSizeH);

      if(descrSizeSip == descrSizeH)both = true;
      else both = false;
      
      permVec.clear();
      if((descrSizeSip < descrSizeH)||both)
      {
       for(h = 0; h < descrSizeSip; h++)
       {
         const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapSmall = capRecv.getAltCapSet(i, h);

         for(y = 0; y < descrSizeH; y++)
	 {
	  if(permVec.size() != 0)
	   if(find(permVec.begin(), permVec.end(), y) != permVec.end())continue;
	   const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapLarge = capRecv.getAltCapSet(j, y);
	   
	   if(includes(altCapLarge.begin(), altCapLarge.end(), altCapSmall.begin(), altCapSmall.end())
	     permVec.push_back(y);
	 }
       }
       
       if(descrSizeSip == permVec.size())removeCapDescriptor(y);
      }   
	   
      permVec.clear();
      if(descrSizeSip > descrSizeH||both)
      {
       for(h = 0; h < descrSizeH; h++)
       {
         const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapSmall = capRecv.getAltCapSet(j, h);

         for(y = 0; y < descrSizeSip; y++)
	 {
	  if(permVec.size() != 0)
	   if(find(permVec.begin(), permVec.end(), y) != permVec.end())continue;
	   const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapLarge = capRecv.getAltCapSet(i, y);
	   
	   if(includes(altCapLarge.begin(), altCapLarge.end(), altCapSmall.begin(), altCapSmall.end())
	     permVec.push_back(y);
	 }
       }
       
       if(descrSizeSip == permVec.size())removeCapDescriptor(y);
      }   

   }


  }*/
  VDEBUG(log) << fn << ": END ------------------ " << VDEBUG_END(log);

}
