/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Contacts_cxx_Version = 
    "$Id: Contacts.cxx,v 1.4 2001/09/11 01:30:57 icahoon Exp $";


#include <assert.h>

#include "VLog.hxx"

#include "SipVia.hxx"

#include "Contacts.hxx"

using namespace translator;
using Vocal::Logging::VLog;


Contacts::
Contacts()
{
}

Contacts::
~Contacts()
{
}


Contact* Contacts::
findByBranch(const StatusMsg& msg) throw()
{
    VLog log;
    VDEBUG(log) << "Contacts::findByBranch" << VDEBUG_END(log);

    if (!size())
    {
	VWARN(log) << "Got SIP status but contact list is empty"
		   << VWARN_END(log);
	return 0;
    }

    if (msg.getNumVia())
    {
	SipVia via = msg.getVia();
	Data branchData = via.getBranch();
	if (branchData.length())
	{
	    int branch = branchData.convertInt();

	    if (branch < 1 || branch >= size())
	    {
		VWARN(log) << "Got SIP status with invalid branch"
			   << VWARN_END(log);
		// Assume original Invite
		return &(list_[0]);
	    }
	    else
	    {
		return &(list_[branch]);
	    }
	}
	else
	{
	    // No branch: assume original Invite
	    return &(list_[0]);
	}
    }
    else
    {
	VWARN(log) << "Got SIP status without Via" << VWARN_END(log);
	// Assume original Invite
	return &(list_[0]);
    }
}


Contact* Contacts::
findByUrl(const SipUrl& url) throw()
{
    for (int i = 0; i < size(); i ++)
    {
	if (url == list_[i])
	{
	    return &(list_[i]);
	}
    }

    return 0;
}


void Contacts::
clear() throw()
{
    list_.clear();
}


void Contacts::
add(const SipUrl& url)
{
    list_.push_back(Contact(url));
}


void Contacts::
update(Contact* contact, const StatusMsg& msg)
{
    VLog log;
    VDEBUG(log) << "Contacts::update" << VDEBUG_END(log);
    VDEBUG(log) << "New status = "
		<< msg.getStatusLine().getStatusCode()
    		<< VDEBUG_END(log);

    contact->setStatus(msg.getStatusLine().getStatusCode());
}


int Contacts::
getStatus() const throw()
{
    VLog log;
    VDEBUG(log) << "Contacts::getStatus" << VDEBUG_END(log);

    int status = 1000;

    for (int i = 0; i < size(); i ++)
    {
	VDEBUG(log) << "Contact.status = "
		    << list_[i].getStatus()
		    << VDEBUG_END(log);

	if (list_[i].getStatus() < status)
	{
	    status = list_[i].getStatus();
	}
    }

    return status;
}


int Contacts::
size() const throw()
{
    return list_.size();
}


int Contacts::
redirects() const throw()
{
    int redirects = 0;

    for (int i = 0; i < size(); i ++)
    {
	if (list_[i].getStatus() >= 300 && list_[i].getStatus() < 400)
	{
	    redirects ++;
	}
    }

    return redirects;
}
