/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTrSetup_cxx_Version = 
    "$Id: EvtTrSetup.cxx,v 1.29 2002/02/25 05:57:17 sprajpat Exp $";


#include <cassert>

#include "VLog.hxx"
#include "PollFifoService.hxx"

#include "H323AdmissionEvent.hxx"

#include "InviteMsg.hxx"

#include "RouterInfo.hxx"
#include "Address.hxx"

#include "TrConnectionRas.hxx"
#include "TSMMediator.hxx"
#include "EvtTrSetup.hxx"
#include "FeatureTrCallSig.hxx"
#include "TrConnectionSIP.hxx"
#include "TranslatorConsts.hxx"
#include "Contacts.hxx"

using namespace translator;
using Vocal::H323Events::H323AdmissionEvent;
using Vocal::Transport::IPAddress;
using Vocal::Logging::VLog;


EvtTrSetup::
EvtTrSetup(router::RouterCallSig* tr, TrSMCall* sm)
    : mte::EvtBind11<router::EvtSetup, EvtDirectTrCallSig>(tr, sm)
{
}


void EvtTrSetup::
inIdle(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrSetup::inIdle");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // (depends from H.323/SIP parser)

    // Translate 'Setup' data to 'Invite'
    Translator::Address destAddressTranslator, srcAddressTranslator;

    destAddressTranslator.clear();
    srcAddressTranslator.clear();

    // To: field
    if (getMessage().HasOptionalField(
        H225_Setup_UUIE::e_destinationAddress))
    {
        destAddressTranslator = getMessage().m_destinationAddress;
    }
    else if (getMessage().HasOptionalField(
        H225_Setup_UUIE::e_destCallSignalAddress))
    {
        if(getMessage().m_destCallSignalAddress.GetTag() ==
           H225_TransportAddress::e_ipAddress)
        {
            const H225_TransportAddress_ipAddress& ipAddress =
                getMessage().m_destCallSignalAddress;
            destAddressTranslator = ipAddress;
        }
    }
    
    if((destAddressTranslator.isEmpty())||(!destAddressTranslator.hasPhone()))
    {
        PString pString;
		
        if(getQ931().GetCalledPartyNumber(pString))
        {
            std::string calledNum = std::string((const char *)pString);
            destAddressTranslator.setPhone(calledNum);
        }
        else
        {
            // Setup message does not have destination address
            VWARN(log) << fn << ": Setup message does not have destination address"
                       << ", call can not be established"
                       << VWARN_END(log);

            trcallsig.getSM()->complete();
            return;
        }
    }

    if (getMessage().HasOptionalField(
        H225_Setup_UUIE::e_destCallSignalAddress))
    {
        if(getMessage().m_destCallSignalAddress.GetTag() ==
           H225_TransportAddress::e_ipAddress)
        {
            const H225_TransportAddress_ipAddress& ipAddress =
                getMessage().m_destCallSignalAddress;
            destAddressTranslator.setH225_Transport(ipAddress);
        }
    }

    destAddressTranslator.setTransport(IPAddress());
    VDEBUG(log) << fn << ": Setting 'To'" << VDEBUG_END(log);
    SipUrl toUrl = destAddressTranslator;
    VDEBUG(log) << fn << ": To***:" << (const char*)(toUrl.encode().logData()) << VDEBUG_END(log);
    trcallsig.getContacts().clear();
    trcallsig.getContacts().add(toUrl);

    Sptr<BaseUrl> baseUrl = new SipUrl(toUrl);
    Sptr<InviteMsg> invite = new InviteMsg(baseUrl);

    // From: field
    if (getMessage().HasOptionalField(
        H225_Setup_UUIE::e_sourceAddress))
    {
        srcAddressTranslator = getMessage().m_sourceAddress;
    }
    else if (getMessage().HasOptionalField(
        H225_Setup_UUIE::e_sourceCallSignalAddress))
     {
        if(getMessage().m_sourceCallSignalAddress.GetTag() ==
           H225_TransportAddress::e_ipAddress)
        {
            const H225_TransportAddress_ipAddress& ipAddress =
                getMessage().m_sourceCallSignalAddress;
            srcAddressTranslator = ipAddress;
        }
    }

    VDEBUG(log) << fn << ": Setting 'From'" << VDEBUG_END(log);

    if (!srcAddressTranslator.hasIPAddress())
    {
      VDEBUG(log) << fn << ": No source IP address in incoming setup;"
	" using remote address from the socket" << VDEBUG_END(log);
      srcAddressTranslator = trcallsig.getSM()->getH323Call()->
        getCallSigConnection()->getRemoteAddress();
    }

    if((srcAddressTranslator.isEmpty())||(!srcAddressTranslator.hasPhone()))
    {
        PString pString;
		
        if(getQ931().GetCallingPartyNumber(pString))
        {
            std::string callingNum = std::string((const char *)pString);
            srcAddressTranslator.setPhone(callingNum);
        }
    }

    Sptr<BaseUrl> baseFrom = new SipUrl(srcAddressTranslator);
    SipFrom from = SipFrom(baseFrom);

    VDEBUG(log) << fn << ": From***:" << (const char*)(from.encode().logData()) << VDEBUG_END(log);

    if (!(trcallsig.getSM()->getH323Call()->getRouterInfo()->routerMediaControl()))
    {
        VDEBUG(log) << fn << ": Setting 'CanRecvH245Address' token" << VDEBUG_END(log);
        from.setTokenDetails(CanRecvH245Address, CanRecvH245AddressValue);
    }

    string  displayName("Unknown number");    
    if ( getQ931().HasIE(Q931::DisplayIE) )
    {
        displayName = getQ931().GetDisplayName();
    }
    else if ( srcAddressTranslator.hasPhone() )
    {
        displayName = srcAddressTranslator.getPhone();
    }

    from.setDisplayName(Data(displayName.c_str()));
    invite->setFrom(from);

    // Translate callID

    Data cid = convertToHex(getMessage().m_conferenceID, 16);

    if (getMessage().HasOptionalField(H225_Setup_UUIE::e_callIdentifier))
    {
        cid = cid + "." + convertToHex(getMessage().m_callIdentifier.m_guid, 16);
    }

    invite->setCallId(SipCallId(
        cid + "@" + trcallsig.getSM()->getH323Call()->
        getRouterInfo()->callSignalAddress().getIPAsString()));

    trcallsig.invite_ = invite;

    trcallsig.setState(&PatternTrCallSig::inWaitInAdmission, "inWaitInAdmission");

    // Don't send 'invite', send ARQ instead
    Sptr<H323AdmissionEvent> arq =
        new H323AdmissionEvent(
            trcallsig.getSM()->getH323Call()->getFifo()->getFifo());
    arq->srcCallSignalAddress(srcAddressTranslator);
    arq->srcAlias(srcAddressTranslator);
    arq->callIdentifier(trcallsig.getSM()->getH323Call()->
                        getCallSigConnection()->getCallID());
    arq->srcDisplay(displayName);

    trcallsig.getSM()->getRasConnection()->send(arq);
}


void EvtTrSetup::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrSetup::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VWARN(log) << fn << ": Got unexpected Setup message for call "
               << trcallsig.getSM() << VWARN_END(log);
}


router::EvtQ931B* EvtTrSetup::
clone_impl()
{
    return new EvtTrSetup(*this);
}


void EvtTrSetup::
updateRoute()
{
    assert(getConnection());
    assert(getConnection()->getSM());

    TSMMediator& m = dynamic_cast<TSMMediator&>(
        getConnection()->getSM()->getMaster()
    );

    setDirectTarget(&(m.getSM()));
}


#ifndef DISABLE_WRITER

ostream& EvtTrSetup::
writeTo(ostream& out) const
{
    out << "EvtTrSetup";
    mte::EvtBind11<router::EvtSetup, EvtDirectTrCallSig>::writeTo(out);
    return out;
}

#endif
