/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtTrTerminate_cxx_Version = 
    "$Id: EvtTrTerminate.cxx,v 1.15 2001/10/24 21:26:43 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "AckMsg.hxx"
#include "CancelMsg.hxx"
#include "ByeMsg.hxx"
#include "ValAssistant.h"

#include "EvtTrTerminate.hxx"
#include "FeatureTrCallSig.hxx"
#include "TrConnectionSIP.hxx"

using namespace translator;
using Vocal::Logging::VLog;


EvtTrTerminate::
EvtTrTerminate(router::SMCall* sm1, TrSMCall* sm2)
    : mte::EvtBind11<router::EvtTerminate, EvtDirectTrCallSig>(sm1, sm2)
{
}


void EvtTrTerminate::
inIdle(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inIdle");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Just destroy the call
    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inWaitInAdmission(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inIdle");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Just destroy the call
    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inConnectingH2S(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inConnectingH2S");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send CANCEL
    if (!!trcallsig.invite_)
    {
        Sptr<CancelMsg> sipCancel = new CancelMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipCancel);
    }

    // Send BYE
    if (!!trcallsig.invite_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }

    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inWaitOutAdmission(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inConnectingS2H");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send STATUS
    if (!!trcallsig.invite_)
    {
        Sptr<StatusMsg> sipStatus = new StatusMsg(
            *(trcallsig.invite_), msip::respNotFound);
        trcallsig.getSM()->getSIPConnection()->send(sipStatus);
    }

    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inConnectingS2H(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inConnectingS2H");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send STATUS
    if (!!trcallsig.invite_)
    {
        Sptr<StatusMsg> sipStatus = new StatusMsg(
            *(trcallsig.invite_), trcallsig.getReason());
        trcallsig.getSM()->getSIPConnection()->send(sipStatus);
    }

    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inSDPH2S(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inSDPH2S");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    if (!!trcallsig.status_)
    {
        Sptr<AckMsg> sipAck =
            new AckMsg(*(trcallsig.status_));
        trcallsig.getSM()->getSIPConnection()->send(sipAck);
    }

    // Send BYE
    if (!!trcallsig.status_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.status_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }
    else if (!!trcallsig.invite_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }

    trcallsig.getSM()->close();
    VDEBUG(log) << fn << ": exit" << VDEBUG_END(log);
}


void EvtTrTerminate::
inSDPInvite(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inSDPInvite");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send STATUS
    if (!!trcallsig.invite_)
    {
        Sptr<StatusMsg> sipStatus = new StatusMsg(
            *(trcallsig.invite_), trcallsig.getReason());
        trcallsig.getSM()->getSIPConnection()->send(sipStatus);
    }

    // Send BYE
    if (!!trcallsig.invite_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }

    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inWaitAck(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inWaitAck");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send BYE
    if (!!trcallsig.status_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.status_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }
    else if (!!trcallsig.invite_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }

    trcallsig.getSM()->close();
}

        
void EvtTrTerminate::
inWait200(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inWait200");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send CANCEL
    if (!!trcallsig.invite_)
    {
        Sptr<CancelMsg> sipCancel = new CancelMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipCancel);
    }

    // Send BYE
    if (!!trcallsig.invite_)
    {
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.invite_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }

    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inWaitCollision(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate:inWaitCollision:");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    inConnected(trcallsig);
}


void EvtTrTerminate::
inConnected(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inConnected");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Send BYE
    if ( !trcallsig.ack_ &&  !!trcallsig.status_)
    {
        VDEBUG(log) << "Building BYE based on STATUS, we better be the caller." << VDEBUG_END(log);
        
        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.status_));
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }
    else if (!!trcallsig.invite_)
    {
        VDEBUG(log) << "Building BYE based on INVITE, we better be the callee." << VDEBUG_END(log);

        Sptr<ByeMsg> sipBye = new ByeMsg(*(trcallsig.invite_));

        // We need the tag from the ACK's To and we need to add it to the
        // BYE's From.
        //
        if ( !!trcallsig.ack_ )
        {
            Data    toTag = trcallsig.ack_->getTo().getTag();
            SipFrom byeFrom = sipBye->getFrom();
            byeFrom.setTag(toTag);
            
            sipBye->setFrom(byeFrom);
        }
        
        trcallsig.getSM()->getSIPConnection()->send(sipBye);
    }

    trcallsig.getSM()->close();
}


void EvtTrTerminate::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtTrTerminate::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Just destroy the call
    trcallsig.getSM()->close();
}


#ifndef DISABLE_WRITER

ostream& EvtTrTerminate::
writeTo(ostream& out) const
{
    out << "EvtTrTerminate";
    mte::EvtBind11<router::EvtTerminate, EvtDirectTrCallSig>::writeTo(out);
    return out;
}

#endif
