
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FeatureTrCallSig_cxx_Version = 
"$Id: FeatureTrCallSig.cxx,v 1.10 2001/10/19 21:37:37 icahoon Exp $";

#include "ValAssistant.h"
#include "VLog.hxx"

#include "FeatureTrCallSig.hxx"
#include "TrSMCall.hxx"
#include "Q931Defs.hxx"

using namespace translator;
using Vocal::Logging::VLog;


FeatureTrCallSig::
FeatureTrCallSig(TrSMCall* sm)
    : sm_(sm)
    , timer_(new EvtTrCallSigTimer(sm))
{
    reset();
    setEntry(&entry_);
    setExit(&exit_);
}


FeatureTrCallSig::
~FeatureTrCallSig()
{
    const string  fn("FeatureTrCallSig::~FeatureTrCallSig");
    VLog	    	log(fn);

    // Call exit methods for any state
    setState(&PatternTrCallSig::inIdle, "inIdle");

    const string  fn2("FeatureTrCallSig::~FeatureTrCallSig");
    VLog	    	log2(fn2);
}


void FeatureTrCallSig::
reset()
{
    statusCode_ = 0;
    alerted_ = false;
    setStateStealth(&PatternTrCallSig::inIdle, "inIdle");
}


void FeatureTrCallSig::
process_impl(PatternTrCallSig& pattern)
{
    (pattern.*(getState()))(*this);
}


void FeatureTrCallSig::
startTimer(mte::TimeInterval timeout)
{
    assert(this);

    timer_->setTimeout(timeout);

    if (timeout)
    {
        getSM()->startTimer(timer_);
    }
}


void FeatureTrCallSig::
stopTimer()
{
    assert(this);

    getSM()->stopTimer(timer_);
}


int FeatureTrCallSig::
getReason()
{
    if (statusCode_)
    {
	return statusCode_;
    }

    // Map H.323 release reason to SIP status code
    switch (getSM()->getH323Call()->getTerminateReason())
    {
    case H225_ReleaseCompleteReason::e_noBandwidth:
        return msip::respTemporarilyNotAvailable;
        break;

    case H225_ReleaseCompleteReason::e_gatekeeperResources:
        return msip::respTemporarilyNotAvailable;
        break;

    case H225_ReleaseCompleteReason::e_unreachableDestination:
        return msip::respNotFound;
        break;

    case H225_ReleaseCompleteReason::e_destinationRejection:
        return msip::respBusyHere;
        break;

    case H225_ReleaseCompleteReason::e_invalidRevision:
        return msip::respNotImplemented;
        break;

    case H225_ReleaseCompleteReason::e_noPermission:
        return msip::respForbidden;
        break;

    case H225_ReleaseCompleteReason::e_unreachableGatekeeper:
        return msip::respServiceUnavailable;
        break;

    case H225_ReleaseCompleteReason::e_gatewayResources:
        return msip::respServiceUnavailable;
        break;

    case H225_ReleaseCompleteReason::e_badFormatAddress:
        return msip::respBadRequest;
        break;

    case H225_ReleaseCompleteReason::e_adaptiveBusy:
        return msip::respTemporarilyNotAvailable;
        break;

    case H225_ReleaseCompleteReason::e_inConf:
        return msip::respBadRequest;
        break;

    case H225_ReleaseCompleteReason::e_undefinedReason:
        return msip::respInternalServerError;
        break;

    case H225_ReleaseCompleteReason::e_facilityCallDeflection:
        return msip::respBusyHere;
        break;

    case H225_ReleaseCompleteReason::e_securityDenied:
        return msip::respUnauthorized;
        break;

    case H225_ReleaseCompleteReason::e_calledPartyNotRegistered:
        return msip::respNotFound;
        break;

    case H225_ReleaseCompleteReason::e_callerNotRegistered:
        return msip::respInternalServerError;
        break;

    default:
        assert(false);
    }
}


void FeatureTrCallSig::
setReason(int status)
{
    statusCode_ = status;

    // Map SIP status code to H.323 release reason
    switch (status)
    {
    case msip::respNullCode:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respTrying:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respRinging:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respCallIsBeingForwarded:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respQueued:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respOK:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respMultipleChoices:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
    break;

    case msip::respMovedPermanently:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respMovedTemporarily:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respSeeOther:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respUseProxy:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respAlternativeService:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respBadRequest:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respUnauthorized:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_securityDenied);
        break;

    case msip::respPaymentRequired:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respForbidden:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_noPermission);
        break;

    case msip::respNotFound:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respMethodNotAllowed:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respNotAcceptable:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respProxyAuthenticationRequired:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_noPermission);
        break;

    case msip::respRequestTimeout:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_adaptiveBusy);
        break;

    case msip::respConflict:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respGone:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respLengthRequired:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respRequestEntityTooLarge:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_badFormatAddress);
        break;

    case msip::respRequestURITooLarge:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_badFormatAddress);
        break;

    case msip::respUnsupportedMediaType:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respBadExtension:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_badFormatAddress);
        break;

    case msip::respTemporarilyNotAvailable:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_adaptiveBusy);
        break;

    case msip::respCallLegOrTransactionDoesNotExist:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respLoopDetected:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respTooManyHops:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respAddressIncomplete:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_badFormatAddress);
        break;

    case msip::respAmbiguous:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_badFormatAddress);
        break;

    case msip::respBusyHere:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_destinationRejection);
        break;

    case msip::respInternalServerError:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    case msip::respNotImplemented:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_invalidRevision);
        break;

    case msip::respBadGateway:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_gatewayResources);
        break;

    case msip::respServiceUnavailable:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_gatewayResources);
        break;

    case msip::respGatewayTimeout:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_adaptiveBusy);
        break;

    case msip::respSIPVersionNotSupported:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_invalidRevision);
        break;

    case msip::respBusyEverywhere:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_destinationRejection);
        break;

    case msip::respDecline:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_destinationRejection);
        break;

    case msip::respDoesNotExistAnywhere:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_unreachableDestination);
        break;

    case msip::respNotAcceptableGF:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;

    default:
        getSM()->getH323Call()->setTerminateReason(
            H225_ReleaseCompleteReason::e_undefinedReason);
        break;
    }
}


#ifndef DISABLE_WRITER

ostream& FeatureTrCallSig::
writeTo(ostream& out) const
{
    out << "FeatureTrCallSig:"
        << " state=" << getStateName();
    return out;
}


char* FeatureTrCallSig::
getStateName() const
{
    if (false)
        return "";
    else if (getState() == &PatternTrCallSig::inIdle)
        return "Idle";
    else if (getState() == &PatternTrCallSig::inWaitOutAdmission)
        return "WaitOutAdmission";
    else if (getState() == &PatternTrCallSig::inWaitInAdmission)
        return "WaitInAdmission";
    else if (getState() == &PatternTrCallSig::inConnectingS2H)
        return "ConnectingS2H";
    else if (getState() == &PatternTrCallSig::inWaitAck)
        return "WaitAck";
    else if (getState() == &PatternTrCallSig::inSDPInvite)
        return "inSDPInvite";
    else if (getState() == &PatternTrCallSig::inWait200)
        return "inWait200";
    else if (getState() == &PatternTrCallSig::inCollision)
        return "inCollision";
    else if (getState() == &PatternTrCallSig::inConnected)
        return "Connected";
    else if (getState() == &PatternTrCallSig::inConnectingH2S)
        return "ConnectingH2S";
    else if (getState() == &PatternTrCallSig::inSDPH2S)
        return "SDPH2S";
    else if (getState() == &PatternTrCallSig::inSending)
        return "Sending";
    else if (getState() == &PatternTrCallSig::inOther)
        return "Other";
    else
        return "UNKNOWN";
}

#endif
