#ifndef __G722CAPABILITY_H_
#define __G722CAPABILITY_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const G722Capability_hxx_Version = 
"$Id: G722Capability.hxx,v 1.8 2000/12/20 01:47:49 vzubarev Exp $";

#include "AudioCapability.hxx"

namespace Translator
{

#define G722_SMPLS_PER_FRAME 8
#define G722_FRMS_PER_PACKET 200
#define G722_CLCK_RATE 8000

/**
   This class describes the G722 codec capability.
 */
class G722Capability : public AudioCapability
{
  private:
    ///
    static const unsigned g722SubType[3];

  public:
    /// Specific G.722 encoding bit rates
    enum Speed {
      ///
      At64k,
      ///
      At56k,
      ///
      At48k
    };

    /**Create a new G.722 capability.
     */
    G722Capability(
      Speed speed = At64k   /// Encoding bit rate.
    );
    
    ///
    virtual ~G722Capability() {}

    /**Create a copy of the object.
      */
    virtual AudioCapability * clone() const;

    /**Get the sub-type of the capability. This is a code dependent on the
       main type of the capability.

       This returns one of the four possible combinations of mode and speed
       using the enum values of the protocol ASN H245_AudioCapability class.
     */
    virtual unsigned getSubType() const;

    /**
       is called on construction of outgoing TerminalCapabilitySet 
       or OpenLogicalChannel message.
       Fills out message fields with corresponding class members.
     */
    virtual bool getParamSendMsg(
      H245_AudioCapability & pdu,  /// PDU to set information on
      unsigned packetSize          /// Packet size to use in capability
    ) const;

    /**
       is called on receiving of TerminalCapabilitySet
       or OpenLogicalChannel message.
       Sets class members from corresponding message fields.
     */
    virtual bool setParamRecvMsg(
      const H245_AudioCapability & pdu,  /// PDU to get information from
      unsigned & packetSize              /// Packet size to use in capability
    );

  protected:
    ///
    Speed    speed;
    
};

}
#endif
