#if !defined(H323_CALL_TRANSLATOR_DOT_H)
#define H323_CALL_TRANSLATOR_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const H323CallTranslator_hxx_Version = 
"$Id: H323CallTranslator.hxx,v 1.15 2001/10/26 21:47:55 icahoon Exp $";

#include "PollFifoService.hxx"
#include "Sptr.hxx"
#include "Event.hxx"
#include "Q931Protocol.hxx"
#include "VThreadPool.hxx"

namespace translator
{
    class TrRouterSIP;
    class SesSip;
    class SipEventObserver;
    class AsyncEventObserver;
    class TrCallFactoryImp;
    class TrRouterRas;
    class RasEventObserver;
}

namespace router
{
    class TimerEventObserver;
    class RouterCallSig;
    class RouterCallCtl;
    class SesQ931;
    class SesH245;
}

namespace Vocal
{
    namespace SipH323Info {
    	class RouterInfo;
    }
    class Q931ServerProtocol;

    using Vocal::Services::PollFifoService;
    using Vocal::SipH323Info::RouterInfo;
    using Vocal::Services::ControlEvent;
    using Vocal::Services::Event;
    using Vocal::ReturnCode;

    class H323CallTranslator : public PollFifoService
    {
    public:
        H323CallTranslator(RouterInfo &);
        ~H323CallTranslator();

        ReturnCode  onStart();
        ReturnCode  onStop();
        ReturnCode  onShutdown();

        void	    connectToGatekeeper(FifoBase< Sptr<Event> > &);
        void	    connectToSipProxy(FifoBase< Sptr<Event> > & incoming,
	    	    	    	      FifoBase < Sptr<Event> > & outgoing);

    private:
        void	    clear();

        RouterInfo  	    	    	&   routerInfo_;

        FifoBase< Sptr<Event> >     	*   gatekeeperFifo_;
        FifoBase< Sptr<Event> >     	*   sipProxyFifo_;
        FifoBase< Sptr<Event> >     	*   sipProxyOutFifo_;
		
        Q931ServerProtocol  	    	*   q931ServerProtocol_;
		
        translator::TrRouterSIP     	*   trRouterSIP_;
        translator::TrRouterRas     	*   trRouterRas_;
        translator::SesSip	    	    *   sesSip_;
        translator::SipEventObserver    *   sipEventObserver_;
        translator::RasEventObserver    *   rasEventObserver_;
        translator::AsyncEventObserver  *   asyncEventObserver_;
        router::TimerEventObserver      *   timerEventObserver_;

        translator::TrCallFactoryImp    *   callFactory_;
        router::SesQ931	    	        *   sesQ931_;
        router::RouterCallSig           *   routerQ931_;

        router::SesH245                 *   sesH245_;
        router::RouterCallCtl           *   routerH245_;
        VThreadPool                         threadPool_;
    };

} // namespace Vovida

#endif // !defined(H323_CALL_TRANSLATOR_DOT_H)
