/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RasEventObserver_cxx_Version = 
    "$Id: RasEventObserver.cxx,v 1.6 2001/05/29 09:08:01 icahoon Exp $";


#include "VLog.hxx"

#include "RasEventObserver.hxx"
#include "Service.hxx"
#include "TrRouterRas.hxx"
#include "EvtTrACF.hxx"


using namespace translator;
using Vocal::Logging::VLog;

RasEventObserver::
RasEventObserver(
    Service 	    	&   service,
    TrRouterRas	    	&   trRouterRas
    )
    :	EventObserver< Sptr<Event> >(service.getEventSubject()),
    service_(service),
	trRouterRas_(trRouterRas)
{
    service_.subscribe(*this);
}


RasEventObserver::
~RasEventObserver()
{
    service_.unsubscribe(*this);
}


bool RasEventObserver::
interestedIn(const Sptr<Event> & event)
{
    VLog    	    	log;
    VDEBUG(log) << "RasEventObserver::interestedIn " << event->name()
    	    	<< VDEBUG_END(log);

    rasEvent_.dynamicCast(event);
    return ( rasEvent_ != 0 );
}


void RasEventObserver::
onEvent(Sptr<Event>)
{
    const string    	fn("RasEventObserver::onEvent");
    VLog    	    	log(fn);
    VDEBUG(log) << "RasEventObserver::onEvent"
    	    	<< VDEBUG_END(log);

    assert( rasEvent_ != 0 );

#ifndef DISABLE_WRITER
    VDEBUG(log) << fn << ": Received ras event = "
                << *rasEvent_
                << VDEBUG_END(log);
#endif

    // Send RAS event to the appropriate state machine
    EvtTrACF event(&trRouterRas_);

    // Update event's contents & ID
    event.contents_ = rasEvent_.getPtr();
    CallIdentifier& iEvent = event;
    iEvent = rasEvent_->callIdentifier();

    event.start();
    
    rasEvent_ = 0;
}


ostream &           
RasEventObserver::writeTo(ostream & out) const
{
    return ( out << "RasEventObserver" );
}
