#ifndef INCLUDED_TrCallFactoryImp_hxx
#define INCLUDED_TrCallFactoryImp_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TrCallFactoryImp_hxx_Version = 
"$Id: TrCallFactoryImp.hxx,v 1.10 2001/04/03 07:51:16 icahoon Exp $";

#include "PollFifoService.hxx"
#include "RouterInfo.hxx"
#include "VThreadPool.hxx"

#include "TrCallFactory.hxx"
#include "TrCall.hxx"

namespace router
{
    class SesQ931;
    class RouterCallSig;
    class SesH245;
    class RouterCallCtl;
}

namespace translator
{
    using Vocal::Services::PollFifoService;
    using Vocal::SipH323Info::RouterInfo;

    class TrRouterSIP;

    class TrCallFactoryImp: public TrCallFactory
    {
    public:
        TrCallFactoryImp(
            PollFifoService 	    &	service,
            router::SesQ931         &   q931EventSrc,
            router::RouterCallSig   &   callSigRouter,
            router::SesH245         &   h245EventSrc,
            router::RouterCallCtl   &   callCtlRouter,
            TrRouterSIP             &   sipRouter,
            TrRouterRas             &   rasRouter,
            RouterInfo	    	    &	routerInfo,
            FifoBase< Sptr<Event> > *   gatekeeperFifo,
            FifoBase< Sptr<Event> > *   sipProxyFifo,
            VThreadPool             &   threadPool
           );

        /// Create new translator Call object (implementation)
        virtual TrCall* createTrCall();

    private:
    	PollFifoService     	    &	service_;
        router::SesQ931             &   q931EventSrc_;
        router::RouterCallSig       &   callSigRouter_;
        router::SesH245             &   h245EventSrc_;
        router::RouterCallCtl       &   callCtlRouter_;
        TrRouterSIP                 &   sipRouter_;
        TrRouterRas                 &   rasRouter_;
        RouterInfo  	    	    &	routerInfo_;
        FifoBase< Sptr<Event> >     *   gatekeeperFifo_;
        FifoBase< Sptr<Event> >     *   sipProxyFifo_;
        VThreadPool                 &   threadPool_;
    };

}

#endif
