
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const tCapabilityMapTest_cxx_Version = 
"$Id: tCapabilityMapTest.cxx,v 1.9 2001/03/09 06:55:08 icahoon Exp $";
#include "CapabilityMap.hxx"

#include "ptlib.h"
#include "h245.h"
#include "VLog.hxx"


// TEMP CODE: 	This is a temporary work around to avoid calling
//          	gethostbyname in CapabilityMap.
//
string	    SipToH323CallSigGW_HostAddress;
//
// TEMP CODE END



CapabilityMap capabilityMapMaster;

CapabilityMap capabilityMapSlave;

//CapabilityMap capabilityMapSlave1;

using Vocal::Logging::VLog;

using namespace Translator;

VLog log;

void checkTCS()
{

  H245_MultimediaSystemControlMessage msg;
  msg.SetTag(H245_MultimediaSystemControlMessage::e_request);
  
  H245_RequestMessage &msgRq = msg;
  msgRq.SetTag(H245_RequestMessage::e_terminalCapabilitySet);
  
  H245_TerminalCapabilitySet &msgTCS = msgRq;
  
  capabilityMapMaster.makeTCS(msgTCS);

  VDEBUG(log) << "Here is the TCS we made on Master side: "
              << setprecision(2)
	      << msgTCS
	      << VDEBUG_END(log);
  
  capabilityMapSlave.handleTCS(msgTCS);

  VDEBUG(log) << "Here is the TCS we receive on Slave side: "
              << setprecision(2)
              << msgTCS
	      << VDEBUG_END(log);
  
  capabilityMapSlave.makeTCS(msgTCS);
  
  VDEBUG(log) << "Here is the TCS we made on Slave side: "
              << setprecision(2)
              << msgTCS
	      << VDEBUG_END(log);

  capabilityMapMaster.handleTCS(msgTCS);

  VDEBUG(log) << "Here is the TCS we receive on Master side: "
              << setprecision(2)
              << msgTCS
	      << VDEBUG_END(log);

  capabilityMapMaster.makeIntersect();
  capabilityMapSlave.makeIntersect();
  
}

void checkOLC()
{
  bool good;
  
  H245_MultimediaSystemControlMessage msg_Slave;
  msg_Slave.SetTag(H245_MultimediaSystemControlMessage::e_request);
  
  H245_RequestMessage &msgRq_Slave = msg_Slave;
  msgRq_Slave.SetTag(H245_RequestMessage::e_openLogicalChannel);

  H245_OpenLogicalChannel &msgOLC_Slave = msgRq_Slave;

  capabilityMapSlave.makeOLC(msgOLC_Slave);
  
  VDEBUG(log) << "Here is the OLC we made on Slave side: "
              << setprecision(2)
	      << msgOLC_Slave
	      << VDEBUG_END(log);

  // --------------------------------

  H245_MultimediaSystemControlMessage msg_Master;
  msg_Master.SetTag(H245_MultimediaSystemControlMessage::e_request);
  
  H245_RequestMessage &msgRq_Master = msg_Master;
  msgRq_Master.SetTag(H245_RequestMessage::e_openLogicalChannel);

  H245_OpenLogicalChannel &msgOLC_Master = msgRq_Master;

  capabilityMapMaster.makeOLC(msgOLC_Master);
  
  VDEBUG(log) << "Here is the OLC we made on Master side: "
              << setprecision(2)
	      << msgOLC_Master
	      << VDEBUG_END(log);

  // --------------------------------

  H245_MultimediaSystemControlMessage msga_Master;
  msga_Master.SetTag(H245_MultimediaSystemControlMessage::e_response);

  H245_ResponseMessage &msgaRq_Master = msga_Master;
  msgaRq_Master.SetTag(H245_ResponseMessage::e_openLogicalChannelAck);
  
  H245_OpenLogicalChannelAck &msgOLCAck_Master = msgaRq_Master;
  
  VDEBUG(log) << "Here is the OLC we receive on Master side: "
              << setprecision(2)
              << msgOLC_Slave
	      << VDEBUG_END(log);
  
  good = capabilityMapMaster.handleOLC_makeOLCAck(msgOLC_Slave, msgOLCAck_Master);
  
  if(!good)
  {
    std::cout << "There is something wrong with handleOLC_makeOLCAck"
              << std::endl;
    exit(9);	      
  }	      

  VDEBUG(log) << "Here is the OLCAck we made on Master side: "
              << setprecision(2)
              << msgOLCAck_Master
	      << VDEBUG_END(log);

  // --------------------------------

  capabilityMapSlave.handleOLCAck(msgOLCAck_Master);

  VDEBUG(log) << "Here is the OLCAck we receive on Slave side: "
              << setprecision(2)
              << msgOLCAck_Master
	      << VDEBUG_END(log);

  // --------------------------------

  H245_MultimediaSystemControlMessage msga_Slave;
  msga_Slave.SetTag(H245_MultimediaSystemControlMessage::e_response);

  H245_ResponseMessage &msgaRq_Slave = msga_Slave;
  msgaRq_Slave.SetTag(H245_ResponseMessage::e_openLogicalChannelAck);
  
  H245_OpenLogicalChannelAck &msgOLCAck_Slave = msgaRq_Slave;

  VDEBUG(log) << "Here is the OLC we receive on Slave side: "
              << setprecision(2)
              << msgOLC_Master
	      << VDEBUG_END(log);
  
  good = capabilityMapSlave.handleOLC_makeOLCAck(msgOLC_Master, msgOLCAck_Slave);
  
  if(!good)
  {
    std::cout << "There is something wrong with handleOLC_makeOLCAck"
              << std::endl;
    exit(9);	      
  }	      

  VDEBUG(log) << "Here is the OLCAck we made on Slave side: "
              << setprecision(2)
              << msgOLCAck_Slave
	      << VDEBUG_END(log);

  // --------------------------------

  capabilityMapMaster.handleOLCAck(msgOLCAck_Slave);

  VDEBUG(log) << "Here is the OLCAck we receive on Master side: "
              << setprecision(2)
              << msgOLCAck_Slave
	      << VDEBUG_END(log);

  // --------------------------------

}

void checkSDP()
{
  SdpSession msgSDP;
  
  capabilityMapMaster.makeSDP(msgSDP);
  
  VDEBUG(log) << "Here is SDP message we made on Master side: "
              << std::endl
	      << setprecision(3)
	      << msgSDP.encode()
	      << VDEBUG_END(log);
  
  capabilityMapSlave.handleSDP(msgSDP);
  
  VDEBUG(log) << "Here is SDP message we received on Slave1 side: "
              << std::endl
	      << setprecision(3)
	      << msgSDP.encode()
	      << VDEBUG_END(log);
}

main()
{
  log.on(LOG_DEBUG);
  capabilityMapSlave.fillDebug();
  capabilityMapMaster.fillDebug();

  capabilityMapMaster.setMaster(true);
  
  checkTCS();
  checkOLC();
  checkSDP();
}
