/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const tTrCallSig_cxx_Version = 
    "$Id: tTrCallSig.cxx,v 1.8 2000/12/18 23:48:26 bko Exp $";


#include <assert.h>
#include <iostream>

#include "VLog.hxx"

#include "RouterCallSig.hxx"
#include "SesQ931.hxx"

#include "TrCall.hxx"
#include "TrRouterSIP.hxx"
#include "EvtInvite.hxx"
#include "EvtTrConnect.hxx"
#include "EvtMediaReady.hxx"
#include "EvtSipAck.hxx"
#include "EvtTCPConnected.hxx"
#include "SesSip.hxx"

using namespace translator;
using Vocal::Logging::VLog;


// TEMP CODE: 	This is a temporary work around to avoid calling
//          	gethostbyname in CapabilityMap.
//
string	    SipToH323CallSigGW_HostAddress;
//
// TEMP CODE END


int main()
{
    VLog::init(LOG_DEBUG);
    VLog::on(LOG_DEBUG);
    VLog::on(LOG_WARNING);
    
    TrRouterSIP tRouter;
    router::RouterCallSig rRouter;
    TrCall call;
    router::SesQ931 sesQ;
    SesSip sesSip;

    Data url;
    url = "sip:destination@192.168.5.12";
    SipUrl tourl(url);
    int listenPort = 5060;
    int rtpPort = 3456;
    InviteMsg inviteMsg(tourl, listenPort, rtpPort);
    StatusMsg statusMsg(inviteMsg, 200);
    AckMsg ackMsg(statusMsg);
 
    assert(call.getSIPConnection());
    assert(call.getH323Call());
    assert(call.getH323Call()->getCallSigConnection());

    EvtInvite invite(&tRouter);
    sesSip.add(&invite);
    router::EvtTCPConnected tcp(call.getH323Call());
    EvtTrConnect connect(&rRouter);
    router::BH225Connect connectMsg;
    sesQ.add(&connect);
    EvtMediaReady ready(&call);
    EvtSipAck ack(&tRouter);
    sesSip.add(&ack);

    tRouter.add(*(call.getSIPConnection()));

    cerr << "Initial state:" << endl;
    cerr << call;
    
    sesSip.generate(inviteMsg);
    cerr << "State after invite:" << endl;
    cerr << call;

    tcp.start();
    cerr << "State after TCP connected:" << endl;
    cerr << call;

    sesQ.generate(connectMsg, call.getH323Call()->getCallSigConnection());
    cerr << "State after connect:" << endl;
    cerr << call;

    ready.start();
    cerr << "State after ready:" << endl;
    cerr << call;

    sesSip.generate(ackMsg);
    cerr << "State after ack:" << endl;
    cerr << call;

    return 0;
}
