#if !defined(CALL_TRACE_DOT_H)
#define CALL_TRACE_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CallTrace_hxx_Version = 
"$Id: CallTrace.hxx,v 1.6 2001/03/29 23:19:30 icahoon Exp $";


#include "VLog.hxx"
#include "Writer.hxx"
#include "Sptr.hxx"
#include "TransportAddress.hxx"
#include <string>
#include <iomanip>


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 Utilities<br><br><br>
 */
namespace H323Util
{


using Vocal::Transport::TransportAddress;


class calltrace_dir : public Vocal::IO::Writer
{
    public: 
    	
	enum dir
	{
	    IN,
	    OUT
	};

    	#define CT_IN	Vocal::H323Util::calltrace_dir::IN
    	#define CT_OUT	Vocal::H323Util::calltrace_dir::OUT
	
    	calltrace_dir(dir);
	
        ostream & writeTo(ostream &) const;
	
    private:
    
    	dir 	dir_;
};


class calltrace_addr : public Vocal::IO::Writer
{
    public: 
    	
    	calltrace_addr(const TransportAddress &);
    	calltrace_addr(const TransportAddress *);
    	calltrace_addr(Sptr<TransportAddress>);
	
        ostream & writeTo(ostream &) const;
	
    private:
    
    	const TransportAddress *    addr_;
};


class calltrace_prot : public Vocal::IO::Writer
{
    public: 

	enum prot
	{
	    PROT_SIP,
	    PROT_RAS,
	    PROT_Q931,
	    PROT_H245,
            PROT_H245_TUNNEL,
	    PROT_TCP,
	    PROT_RAS_TIMER_EXPIRY,
	    PROT_Q931_TIMER_EXPIRY,
	    PROT_H245_TIMER_EXPIRY
	};
    	
	#define CT_SIP	Vocal::H323Util::calltrace_prot::PROT_SIP
	#define CT_RAS	Vocal::H323Util::calltrace_prot::PROT_RAS
	#define CT_Q931 Vocal::H323Util::calltrace_prot::PROT_Q931
	#define CT_H245 Vocal::H323Util::calltrace_prot::PROT_H245
	#define CT_H245_TUNNEL Vocal::H323Util::calltrace_prot::PROT_H245_TUNNEL
    	#define CT_TCP	Vocal::H323Util::calltrace_prot::PROT_TCP
	#define CT_RAS_TIMER	Vocal::H323Util::calltrace_prot::PROT_RAS_TIMER_EXPIRY
	#define CT_Q931_TIMER	Vocal::H323Util::calltrace_prot::PROT_Q931_TIMER_EXPIRY
	#define CT_H245_TIMER	Vocal::H323Util::calltrace_prot::PROT_H245_TIMER_EXPIRY

    	calltrace_prot(prot);
	
        ostream & writeTo(ostream &) const;
	
    private:
    
    	prot 	prot_;
};


class calltrace_msgtype : public Vocal::IO::Writer
{
    public: 

    	calltrace_msgtype(const string &);
	
        ostream & writeTo(ostream &) const;
	
    private:
    
    	const string &	msgtype_;
};


/** MACRO for logging a call trace message at the LOG_INFO level.<br><br>
 *
 *  @param  log     The VLog to write the message. 
 *  @param  dir     The direction, CT_IN for incoming, CT_OUT for outgoing.
 *  @param  addr    The TransportAddress of the source for incoming messages,
 *  	    	    or the destination for outgoing messages.
 *  @param  prot    The protocol, either PROT_SIP, PROT_RAS, PROT_Q931,
 *  	    	    PROT_H245.
 *  @param  msgType A string with the name of the message.<br><br>
 *  
 *  @see    Vocal::Logging::VLog;
 *  @see    Vocal::Transport::TransportAddress
 *  @see    Vocal::H323Util::CALLTRACE_END
 */
#define     CALLTRACE(log, dir, addr, prot, msgtype)        	    	\
                        VINFO(log)    	    	                        \
		    	<< Vocal::H323Util::calltrace_dir(dir)      	\
			<< Vocal::H323Util::calltrace_addr(addr)    	\
			<< Vocal::H323Util::calltrace_prot(prot)    	\
			<< Vocal::H323Util::calltrace_msgtype(msgtype)	\
			<< setprecision(0) << '\n'


/** MACRO for continuing a call trace message.<br><br>
 *
 *  @see    Vocal::Logging::VLog
 *  @see    Vocal::H323Util::CALLTRACE_END
 */
#define     CALLTRACE_CONT(log)     	    	    	    	    	\
    	    	if ( !((log).logInfo().isOn()) );     	    	    	\
    	    	else (log).logInfo().get(false) 


/** MACRO for terminating a call trace message.<br><br>
 *
 *  @see    Vocal::Logging::VLog
 *  @see    Vocal::H323Util::CALLTRACE_END
 */
#define     CALLTRACE_END(log)   VINFO_END(log)


} // namespace H323Util
} // namespace Vocal


#endif // !defined(CALL_TRACE_DOT_H)
