#ifndef INCLUDED_EvtDirect_hxx
#define INCLUDED_EvtDirect_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtDirect_hxx_Version = 
"$Id: EvtDirect.hxx,v 1.8 2000/12/18 23:48:39 bko Exp $";

namespace mte
{

    /// Event bound directly to destination object (state machine)
    template<class SMEvt, class Src>
    class EvtDirect: public SMEvt
    {
    public:
        typedef typename SMEvt::SMType TargetType;

        /// Construct destination part of event assigning ptr to target object
        EvtDirect(TargetType* pStateMachine = 0);
        
        /// Process event (with given state machine)
        bool process(Src& src);

        /// Assign ptr to target object
        void setDirectTarget(TargetType* pStateMachine);

    private:
        TargetType* pStateMachine_;
    };
    
    //----------------------------
    // Inline and template methods
    //----------------------------

    template<class SMEvt, class Src>
    EvtDirect<SMEvt, Src>::
    EvtDirect(TargetType* pStateMachine)
        : pStateMachine_(pStateMachine)
    {
    }

    template<class SMEvt, class Src>
    inline bool EvtDirect<SMEvt, Src>::
    process(Src& src)
    {
        assert(this);
        assert(pStateMachine_);

        if (pStateMachine_)
        {
            SMEvt::process(*pStateMachine_);
        }
        
        return (pStateMachine_ != 0);
    }

    template<class SMEvt, class Src>
    inline void EvtDirect<SMEvt, Src>::
    setDirectTarget(TargetType* pStateMachine)
    {
        pStateMachine_ = pStateMachine;
    }
}

#endif
