#ifndef INCLUDED_EvtRouter2_hxx
#define INCLUDED_EvtRouter2_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtRouter2_hxx_Version = 
"$Id: EvtRouter2.hxx,v 1.5 2000/12/18 23:48:39 bko Exp $";

#include "IEvtRouter.hxx"

namespace mte
{

    /// Indirect router part of event
    template<class Router1, class Router2, class SM1,
        Router2* (SM1::*GetRt)()>
    class EvtRouter2:
        public IEvtRouter<typename Router1::SMType>,
        public IEvtRouter<typename Router2::SMType>
    {
    public:
        typedef Router1 TargetType;
        typedef typename Router2::SMType SM;
        typedef typename Router2::SrcType Src;

        /// Construct router part of event and assign pointer to event router
        EvtRouter2(Router1* pRouter = 0);
        
        /// Route event to state machine
        bool route(Src& src);

        /// Set pointer to event router
        void setRouter(Router1* pRouter);

        /// Process event with intermediate state machine
        virtual void process(SM1& stateMachine);

    private:
        Router1* pRouter_;
        int path_;
        Src* src_;
    };
    
    //----------------------------
    // Inline and template methods
    //----------------------------

    template<class Router1, class Router2, class SM1,
        Router2* (SM1::*GetRt)()>
    EvtRouter2<Router1, Router2, SM1, GetRt>::
    EvtRouter2(Router1* pRouter)
        : pRouter_(pRouter)
    {
    }

    template<class Router1, class Router2, class SM1,
        Router2* (SM1::*GetRt)()>
    inline void EvtRouter2<Router1, Router2, SM1, GetRt>::
    setRouter(Router1* pRouter)
    {
        assert(this);
        assert(pRouter_);

        pRouter_ = pRouter;
    }

    template<class Router1, class Router2, class SM1,
        Router2* (SM1::*GetRt)()>
    inline bool EvtRouter2<Router1, Router2, SM1, GetRt>::
    route(Src& src)
    {
        assert(this);
        assert(pRouter_);

        path_ = 0;
        src_ = &src;
        (void) pRouter_->route(*this, src);
        return (path_ > 0) ? true : false;
    }

    template<class Router1, class Router2, class SM1,
        Router2* (SM1::*GetRt)()>
    void EvtRouter2<Router1, Router2, SM1, GetRt>::
    process(SM1& stateMachine)
    {
        assert(this);

        Router2* router2 = (stateMachine.*GetRt)();

        if (router2)
        {
            if (router2->route(*this, *src_))
            {
                ++ path_;
            }
        }
    }

}

#endif
