#ifndef HEARTBEAT_HXX
#define HEARTBEAT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Heartbeat_hxx_Version = "$Id: Heartbeat.hxx,v 1.11 2001/02/07 02:33:06 hlo Exp $";


#include "Sptr.hxx"
#include "Data.hxx"


// prototype of startHeartbeating
void startHeartbeating();
void* heartbeat(void*);


class HeartbeatTxThread;
class HeartbeatRxThread;
class HouseKeepingThread;
class ProvisionInterface;


/*
   This class is to provide the heartbeat mechanism.  It is required 
   to have access to Provision Interface to get some parameters.
   It will spawn a Heartbeat transmit thread to send heartbeat to 
   multicast group.
*/

class Heartbeat
{
    public:
        ///
        Heartbeat();
        ///
        virtual ~Heartbeat();
        ///
        void start();
        ///
        void join();
        ///
        void shutdown();

    protected:
        ///
        Sptr < HeartbeatTxThread > itsHeartbeatTxThread;
        ///
        Sptr < HeartbeatRxThread > itsHeartbeatRxThread;
        ///
        Sptr < HouseKeepingThread > itsHouseKeepingThread;
        ///

    private:
        ///
        short unsigned itsSipPort;
        ///
        void copyRhsToThis(const Heartbeat& rhs);
        ///
        Sptr < Data > callAgentId;
        ///
        Heartbeat(const Heartbeat& rhs);
        ///
        Heartbeat& operator=(const Heartbeat& rhs);

};

#endif // HEARTBEAT_HXX
