
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MSServer_cxx_Version = "$Id: MSServer.cxx,v 1.8 2002/02/25 05:57:18 sprajpat Exp $";

#include "callAgent.hxx"
#include "cpLog.h"
#include "MSServer.hxx"
#include "Data.hxx"
#include "Common.h"
#include "CAServerContainer.hxx"


/*
 * Default Constructor
 * @param name the marshal ip address
 */
MSServer::MSServer(const Data& name)
        : Server(SERVER_MS)
{
    msId = name;

}

/*
 * Copy constructor
 * @param rhs the marshal object to copy
 */
MSServer::MSServer(const MSServer& rhs)
        : Server(rhs)
{
    if (this != &rhs)
        copyRhsToThis(rhs);
}

/*
 * Assignment operator
 * @param rhs the marshal object to copy
 * @return MSServer& this object
 */
MSServer&
MSServer::operator=(const MSServer& rhs)
{
    if (this != &rhs)
    {
        Server::operator=(rhs);
        copyRhsToThis(rhs);
    }
    return *this;
}

/*
 * Copy data from an object to self
 * @param rhs the marshal object to copy
 */
void
MSServer::copyRhsToThis(const MSServer& rhs)
{
    msId = rhs.msId;
}

/*
 * Default Destructor
 */
MSServer::~MSServer()
{}

/*
 * Detect the marshal heartbeat up or down
 * @param key the marshal id
 */
void
MSServer::processHeartbeat(const Data& key)
{
    incrHeartbeatCountByOne();

    if (itsStatus == Inactive)
        gotFirstHeartbeat(key);
    else
        if (itsStatus == ActivePending )
            gotHiccup();

}

/*
 * Receive the first heartbeat.  Will change 
 * the status to be active
 * @param key the marshal id
 */
void
MSServer::gotFirstHeartbeat(const Data& key)
{
    setStatus(Active);

    cpLog( LOG_ALERT, "\n");
    cpLog( LOG_ALERT, "** Marshal Up: %s **\n", key.logData() );

}

/*
 * Bookeeping the marshal heartbeat
 * @param agentId the marshal id
 */
void
MSServer::doHouseKeeping(const Data& agentId)
{
    if ( itsStatus != Inactive )
        if ( heartbeatCount == 0 )
        {
            incrNumOfHeartbeatMissed();

            if ( itsStatus == Active)
                setStatus(ActivePending);
            else
            {
                if ( serverIsDead() )
                {
                    setStatus(Inactive);

                    cpLog (LOG_ALERT, "\n");
                    cpLog (LOG_ALERT, "** Marshal Down: %s **\n", agentId.logData() );

                    // clear HB counters
                    clearHeartbeatCount();
                    clearNumOfHeartbeatMissed();

                    // try to switch over to the up Ms
                    //CAServerContainer::instance().switchOverMs(agentId);
                }

            }

        }
        else
        {
            // clear HB counters
            clearHeartbeatCount();
            clearNumOfHeartbeatMissed();
        }

}
