
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const StartupSequence_cxx_Version = "$Id: StartupSequence.cxx,v 1.18 2001/09/20 00:41:50 bko Exp $";

#include "endpoint.hxx"
#include "StartupSequence.hxx"
#include "potStateEngine.hxx"
#include "callProcessing.hxx"
#include "sipProxy.hxx"
#include "CAServerContainer.hxx"
#include "CAServer.hxx"
#include "Sptr.hxx"

void* sendRQNT(void*);

/*
 * Start the send rqnt thread
 * @param isUp a boolean to indicate it is invoked 
 * because of the server up or down.
 */
void startupSequence(bool isUp)
{
    static pthread_t tid;
    int errNum = 0;

    if ((errNum = pthread_create(&tid, NULL, sendRQNT, (void*) isUp)) > 0)
    {
        cpLog (LOG_ERR, "Cannot create startupSequence thread: %d", errNum);
    }

}

/*
 * Send out rqnt if there's no other callAgent
 * It also checks if the other ca is down, then
 * it will send rqnt to all endpoints.
 * @param isUp boolean to indicate it's from up or down callAgent.
 */
void*
sendRQNT(void* isUp)
{
    // if it's up, will detect other CA's heartbeat.
    // else, send RQNT
    if ((bool)isUp)
    {
        // start the delay... and check if there's other CA. If not, then send RQNT
        // todo : get it from provision
        cpLog (LOG_DEBUG, "Wait a little to detect other CA's heartbeat......");
        sleep(ca.startupDelay);

        // loop through the list of CA from CAServerContainer to check the status
        // if no one else heartbeating, then start sending RQNT
        Sptr < vector < Data > > listofActiveServer = new vector < Data > ;
        if ( listofActiveServer.getPtr() == 0 )
        {
            cpLog( LOG_ERR, "Out of memory !!");
            assert( listofActiveServer.getPtr() != 0 );
            return ( NULL );
        }
        CAServerContainer::instance().getListofActive(SERVER_CA, 
						      listofActiveServer);
        if (listofActiveServer->size() == 2)
        {
            cpLog (LOG_DEBUG, "There's another CA - don't send RQNT");
            return NULL;
        }
    }
    else
    {
        cpLog (LOG_DEBUG, "*** The CA is down. Send RQNT now ***");
    }

    // remember to mark it already sent Rqnt
    CAServerContainer::instance().setStartupFlag();

    for (EndpointMap::iterator epMapIter = epMap.begin();
            epMapIter != epMap.end();
            ++epMapIter)
    {
        Endpoint* ep = epMapIter->second;

        cpLog (LOG_DEBUG, "Send RQNT for the endpoint = %s",
               epMapIter->first.c_str());

        if ( ep == 0 )
        {
            cpLog( LOG_DEBUG, "The ep is a NULL pointer !!");
            assert( ep != 0 );
            return ( NULL );
        }
        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);

        addRequestOrSignalEvent (&rqnt, LinePackageRequestOffHookTransition);
        if ( ep->gw == 0 )
        {
            cpLog( LOG_DEBUG, "The ep->gw is a NULL pointer !!");
            assert( ep->gw != 0 );
            return ( NULL );
        }
        if ( ep->gw->asyncStack == 0 )
        {
            cpLog( LOG_DEBUG, "The ep->gw->asyncStack is a NULL pointer !!");
            assert( ep->gw->asyncStack != 0 );
            return ( NULL );
        }
        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
        cpLog (LOG_DEBUG, "RQNT sent !!!!!!");
    }

    return NULL;
}


/*
 * Verify the endpoint is already registered or not
 * @param epId the endpoint id
 * @return bool true if it's already registered, false otherwise
 */
bool
isRegistered(Data epId)
{
    for (int i = 0; i < ENDPOINT_MAX; ++i)
    {
        if ((epId == eps[i].epId) && (eps[i].registered == 1))
            return true;
    }

    return false;
}

/*
 * Mark the endpoint already registered
 * @param epId the endpoint id
 */
void
markRegistered(Data epId)
{
    for (int i = 0; i < ENDPOINT_MAX; ++i)
    {
        if (epId == eps[i].epId)
            eps[i].registered = 1;
    }

}
