#ifndef CALLPROCESSING_HXX
#define CALLPROCESSING_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const callProcessing_h_Version = " $Id: callProcessing.hxx,v 1.1 2001/02/07 00:57:37 medala Exp $";

#include <string>
#include <map>
#include <list>

#include "endpoint.hxx"
#include "Fifo.h"

#define MAX_REDIRECTS 10

/// Maximum number of call processing event data per thread
#define CP_EVENT_DATA_MAX 10

///
typedef struct cpEventData
{
    int featureId;
    char callIdentifier [CALL_ID_LEN_MAX];
    Sptr < MgcCommand > cmd;
    endpoint* ep;
}
CpEventData;

extern CpEventData* newCpEventData (list < CpEventData* > *);
extern void freeCpEventData (list < CpEventData* > *, CpEventData*);

extern list < CpEventData* > eds [];

extern CallData call[];
extern Endpoint eps [];
extern Fifo < CallData* > fifo;
extern map < string, Endpoint* > dnMap;
extern pthread_mutex_t callDataMutex;
extern list < CallData* > callDataList;

extern void giveBusyTone (Endpoint*);
extern void notifyEndpoint (Endpoint*, int);
extern int newRequestIdentifier (void);
extern int newCallIdentifier (void);
extern void cpStateChange (Endpoint*, int);
extern void cpProcessEvent (Endpoint*, int);
extern void cpProcessEvent (CpEventData*, int);
extern void callDataInit (CallData*);
extern CallData* newCallData (Endpoint*);
extern void makeCall (Endpoint*);
extern int startForwardNoAnswerTimer (CallData*);
int tryCallForwardAllCalls (CallData*);
void updateEp (CallData*);
int idleEndpoint (CallData*);
int tryCallWaiting (CallData*);
int tryCallForwardBusy (CallData*);
int getFinalEndpoints (CallData*);

extern void callProcessingStart (void);
extern void sendInternalEvent (Endpoint*,
                                   CallEvent,
                                   char* = 0,
                                   int = -1);
extern int fakeAddressTranslation (const char*);


/* CALLPROCESSING_HXX */
#endif
