
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const potStateEngine_cxx_Version = "$Id: potStateEngine.cxx,v 1.76 2002/02/27 22:19:19 bko Exp $";

#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>

#include "global.h"

#include "cpLog.h"
#include "potStateEngine.hxx"
#include "callState.hxx"
#include "callEvent.hxx"
#include "callProcessing.hxx"
#include "MgcpResponseCode.hxx"
#include "sipProxy.hxx"
#include "Sdp2Session.hxx"

#include "StartupSequence.hxx"


int responseCode;

static void getDialedDigits (CallData*);
//static void initiateTransfer (Endpoint*);
static int completeTransfer (Endpoint*);
static int heldCallOnDisconnect (Endpoint*);
//static void* heldCallActive(void*);
static void* clearXferCall(void*);
static void resumeEndpoint(Endpoint*);

/*
 * Call State-Event Function Matrix for Plain Old Telephone(POT) endpoint type
 */
StateEngine potStateEngine =
{
 { /* CallStateInit */
    initRestart,              /* CallEventRestart */
    initResponse,             /* CallEventResponse */
    callEventError,           /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    callReject,               /* CallEventReceiveCall */
    callEventIgnore,          /* CallEventFarEndAlerting */
    callEventIgnore,          /* CallEventAnswer */
    callEventError,           /* CallEventReleaseCall */
    callEventIgnore,          /* CallEventDeleteCall */
    callEventIgnore,          /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
        },
{ /* CallStateIdle */
    idleRestart,              /* CallEventRestart */
    callEventIgnore,
    newCall,                  /* CallEventNewCall */
    dialingComplete,          /* CallEventDigitsCollected */
    receiveCall,              /* CallEventReceiveCall */
    callEventIgnore,          /* CallEventFarEndAlerting */
    callEventIgnore,          /* CallEventAnswer */
    callEventIgnore,          /* CallEventReleaseCall */
    callEventIgnore,          /* CallEventDeleteCall */
    callEventIgnore,          /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateDialing */
    restartTransient,         /* CallEventRestart */
    callEventIgnore,          /* CallEventResponse */
    callEventError,           /* CallEventNewCall */
    dialingComplete,          /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */
    callEventError,           /* CallEventFarEndAlerting */
    originalCall,             /* CallEventAnswer */
    restartTransient,         /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    featureDialing,           /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateDigitsCollected */
    restartTransient,         /* CallEventRestart */
    connectionReady,          /* CallEventResponse */
    callEventError,           /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */
    callEventError,           /* CallEventFarEndAlerting */
    callEventError,           /* CallEventAnswer */
    restartHalfCall,          /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    callEventIgnore,          /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateCallInitiated */
    restartOutCall,           /* CallEventRestart */
    callEventIgnore,          /* CallEventResponse */
    callEventError,           /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */
    ringBack,                 /* CallEventFarEndAlerting */
    connected,                /* CallEventAnswer */
    disconnect,               /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    featureCallInitiated,     /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateFarEndAlerted */
    restartOutCall,           /* CallEventRestart */
    callEventIgnore,          /* CallEventResponse */
    callEventError,           /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */ /* doCallWaiting */
    ringBack,                 /* CallEventFarEndAlerting */
    connected,                /* CallEventAnswer */
    disconnect,               /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    featureFarEndAlerted,     /* CallEventFeature */
    callEventIgnore,          /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateReceivedCall */
    restartInCall,            /* CallEventRestart */
    ringing,                  /* CallEventResponse */
    glare,                    /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */
    callEventError,           /* CallEventFarEndAlerting */
    callEventError,           /* CallEventAnswer */
    callEventError,           /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    callEventIgnore,          /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateAlerting */
    restartInCall,            /* CallEventRestart */
    connectionReady2,         /* CallEventResponse */
    answer,                   /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */
    callEventError,           /* CallEventFarEndAlerting */
    callEventError,           /* CallEventAnswer */
    callEventError,           /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    callEventIgnore,          /* CallEventFeature */
    modifyConnect,            /* CallEventMDCX early-rtp */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateConnected */
    restartConnected,         /* CallEventRestart */
    callEventIgnore,          /* CallEventResponse */
    callEventIgnore,          /* CallEventNewCall */
    dtmfDigit,                /* CallEventDigitsCollected */ /* DTMF digit */
    glare,                    /* CallEventReceiveCall */ /* doCallWaiting */
    ringBack,                 /* CallEventFarEndAlerting */
    modifyConnect,            /* CallEventAnswer */
    disconnect,               /* CallEventReleaseCall */
    clearCall,                /* CallEventDeleteCall */
    featureConnected,         /* CallEventFeature */
    modifyConnect,            /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
{ /* CallStateDisconnecting */
    restartDisonnect,         /* CallEventRestart */
    disconnectResponse,       /* CallEventResponse */
    callEventIgnore,          /* CallEventNewCall */
    callEventError,           /* CallEventDigitsCollected */
    glare,                    /* CallEventReceiveCall */
    callEventIgnore,          /* CallEventFarEndAlerting */
    callEventIgnore,          /* CallEventAnswer */
    disconnectRelease,        /* CallEventReleaseCall */
    callEventIgnore,          /* CallEventDeleteCall */
    callEventIgnore,          /* CallEventFeature */
    callEventError,           /* CallEventMDCX */
    callEventError            /* CallEventUnknown */
},
};     /* PotStateEngine */



///
void*
initRestart (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "initRestart");

    // This is happening after the endpoint receives a Bye from
    // the far end (causes it to go to disconnecting state)
    // and then it onhook to come in here ( goes to Init state ).
    cout << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;

    cout << "ep is " << ep->epId.logData() << endl;
    cout << "ep->state: " << ep->state << endl;

    if ( ep->call != 0 )
        cout << "ep->call != 0 " << endl;
    else
        cout << "ep->call == 0 " << endl;

    if ( ep->heldCall != 0 )
        cout << "ep->heldCall != 0  " << endl;
    else
        cout << "ep->heldCall == 0 " << endl;

    if ( ep->sipMsg != 0 )
        cout << "ep->sipMsg != 0 " << endl;
    else
        cout << "ep->sipMsg == 0 " << endl;

    if ( ep->sipMsgHeld != 0 )
        cout << "ep->sipMsgHeld != 0 " << endl;
    else
        cout << "ep->sipMsgHeld == 0 " << endl;

    if ( ep->sipMsgXfer != 0 )
        cout << "ep->sipMsgXfer != 0 " << endl;
    else
        cout << "ep->sipMsgXfer == 0 " << endl;

    cout << "sipMsgRouteList is: " << endl;
    printRouteList( ep->sipMsgRouteList );

    cout << "sipMsgHeldRouteList is: " << endl;
    printRouteList( ep->sipMsgHeldRouteList );
    cout << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << "+++++++++++++++++++++++++++++++++++++++++++++" << endl;

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /* Code corresponding to requested events */
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOffHookTransition );

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    return 0;
}    /* initRestart */


///
void*
idleRestart (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "idleRestart");

    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);

    return 0;
}    /* idleRestart */


///
void*
initResponse (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "initResponse");

    if (ep->heldCall != 0)
    {
        cpLog (LOG_ERR, "A held call was found.  Clear it");
        sendInternalEvent (ep,
                           CallEventDeleteCall,
                           ep->heldCall->callIdentifier);

        // clear up this call
        CpEventData* edHeld = newCpEventData (ep->gw->edPool);
        if (edHeld == 0)
        {
            cpLog (LOG_ERR, "initResponse(): edHeld is NULL");
            assert( edHeld != 0 );
            return 0;
        }
        strcpy (edHeld->callIdentifier, ep->heldCall->callIdentifier);
        edHeld->ep = ep;
        clearCall(edHeld);
    }

    // assume there is a call going on (for redundancy callagent)
    if (responseCode == ReasonTransientAlreadyOffHook)
    {
        resumeEndpoint(ep);
    }
    else
    {
        // verify if it is a 200 response, then send REGISTER.
        if (responseCode == ReasonTransactionExecuted)
        {
            if (! isRegistered (ep->epId))
            {
                cpLog (LOG_DEBUG, "In InitResponse: receive 200 response. Send REGISTER now = %s",
                       ep->epId.logData());
                markRegistered(ep->epId);

                // Add a hook here to have the sipProxy send a SIP REGISTER message
                sipEventSend (ep, SipEventSendRegister);
            }
        }

        // before we change state to Idle, we should clean up to get
        // ready for the next call if we ever missed some stuff in previous
        // cleaning effort
        // NOT called for now, because party B's mgcp side and sip side are not in sync after
        // completing a transfer
        cpStateChange (ep, CallStateIdle);
    }

    return 0;
}    /* initResponse */


///
void*
callEventError (void* ptr)
{
    /* Begin Workaround for lost or out of order ack messages */
    // The proper fix should be:
    //   If the RQNT ack is lost and the incoming NTFY matches the
    //   expected transaction id, preform the ack transition and
    //   the process the NTFY
    //   What about the other acks?

    Endpoint* ep = static_cast < Endpoint* > (ptr);

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /* Code corresponding to signals */
    addRequestOrSignalEvent( &rqnt, LinePackageSignalErrorTone );

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    cpLog (LOG_INFO, "callEventError: Endpoint(%s) State(%s) x: %s",
           ep->epId.logData(),
           CallStateString[ep->state],
           ep->requestIdentifier);

    cpStateChange (ep, CallStateInit);
    return ptr;
}    /* callEventError */


///
void*
callReject (void* ptr)
{
    cpLog (LOG_DEBUG, "callReject");
    return 0;
}    /* callReject */


///
void*
callEventIgnore (void* ptr)
{
    cpLog (LOG_DEBUG, "callEventIgnore");
    return 0;
}    /* callEventIgnore */


///
void*
newCall (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "newCall");

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /* Code corresponding to requested events */
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
    addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );

    rqnt.setDigitMap (ca.dialPlan);

    /* Any digit */
#ifndef NCS_PROFILE
    MgcRequestedEvent anyDigit( MgcDTMFPackage::requestWildcardMatch() );
#else
    MgcRequestedEvent anyDigit( MgcNCSLinePackage::requestWildcardMatch() );
#endif
    anyDigit.addAction (MgcpRequestedActionTypeUseDigitMap);
    rqnt.addRequestedEvents (anyDigit);

    /* Code corresponding to signals */
    // if this is a transfer/conference, give special dialtone instead when available
    addRequestOrSignalEvent( &rqnt, LinePackageSignalDialToneOn );

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
    cpStateChange (ep, CallStateDialing);

    return 0;
}    /* newCall */


///
void*
receiveCall (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "receiveCall");

    cpStateChange (ep, CallStateReceivedCall);

    CallData* call = ep->call;
    if (!call)
    {
        cpLog (LOG_ERR, "answer(): call is NULL");
        return ptr;
    }

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /* Code corresponding to requested events */
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOffHookTransition );
    /* Code corresponding to signals */
    addRequestOrSignalEvent( &rqnt, LinePackageSignalRingingOn );

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    return 0;
}    /* receiveCall */


///
void*
restartTransient (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_INFO, "restartTransient at %s (%s)",
           ep->epId.logData(),
           CallStateString [ep->state]);

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /*
      If B keeps A on Hold and onhooks, send rqnt for offhook 
      and signal ringing on to B
    */
    if (ep->call == 0 && ep->heldCall != 0)
    {
        /* Code corresponding to requested events */
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOffHookTransition );
        /* Code corresponding to signals */
        addRequestOrSignalEvent( &rqnt, LinePackageSignalRingingOn );
        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
        cpStateChange (ep, CallStateAlerting);
        return 0;
    }

    if (ep->call != 0)
    {
        if (ep->call->forwardNoAnswerId != 0)
        {
            ep->iq->cancel(call->forwardNoAnswerId);
            ep->call->forwardNoAnswerId = 0;
        }
        if (ep->call->endpoint[0].connectionId[0] != 0 )
        {
            MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
            dlcx.setCallId (ep->call->callIdentifier);
            dlcx.setConnectionId (ep->call->endpoint[0].connectionId);
            ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);
            cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
        }
        callDataInit (ep->call);
        ep->call = 0;
    }
    if (ep->heldCall != 0)
    {
        callDataInit (ep->heldCall);
        ep->heldCall = 0;
    }

    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);
    return 0;
}    /* restartTransient */



/**
when B goes offhook and has a held call
it sends reinvite to A( the held party)
*/
/*
void*
heldCallActive(void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog(LOG_DEBUG, "HeldCallActive");
    if (ep->heldCall != 0)
    {
        ep->call = ep->heldCall;
        ep->heldCall = 0;
        sipEventSend (ep, SipEventSendInvite);
    }
    return 0;
}
*/

///
void*
restartHalfCall (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_INFO, "restartHalfCall %s at %s (%s)",
           ep->call->callIdentifier,
           ep->epId.logData(),
           CallStateString [ep->state]);

    CpEventData* ed = newCpEventData (ep->gw->edPool);
    if (ed == 0)
    {
        cpLog (LOG_ERR, "restartHalfCall(): ed is NULL");
        assert ( ed != 0 );
        return 0;
    }
    strcpy (ed->callIdentifier, ep->call->callIdentifier);
    ed->ep = ep;

    clearCall(ed);

    return 0;
}    /* restartHalfCall */


/****************************************************************************
 Store the dialed digits into the appropriate call block for later processing.
****************************************************************************/
void
getDialedDigits (CallData* call)
{
    cpLog(LOG_DEBUG, "*****************************************");
    cpLog(LOG_DEBUG, "  DIALED DIGITS! DIALED DIGITS! = %s", call->dtmfDigits);
    cpLog(LOG_DEBUG, "*****************************************");

    strcpy (call->dialedDigits, call->dtmfDigits);
    strcpy (call->callDigits, call->dtmfDigits);
    call->dtmfDigits[0] = 0;
}


///
void*
dialingComplete (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    CallData* call;
    cpLog (LOG_DEBUG, "dialingComplete");

    if (ep->call == 0)
    {
        // e.g. for hot line cases where digits were not dialed.
        call = newCallData (ep);
        call->endpoint[0].id = ep->id;
        sprintf (call->callIdentifier, "%x", newCallIdentifier());
    }
    else
    {
        call = ep->call;
    }
    if (!call)
    {
        cpLog (LOG_ERR, "dialingComplete(): call is NULL");
        return ptr;
    }

    // Call Waiting blocked for this call
    if (strcmp(call->dtmfDigits, "*70") == 0)
    {
        call->blockCallWaiting = true;
        // clear out these digits.
        call->dtmfDigits[0] = 0;
        // give dialtone and start the call over again.
        newCall(ep);
        return 0;
    }
    if (strcmp(call->dtmfDigits, "*67") == 0)
    {
        // give dialtone and start the call over again,
        // but don't clear out these digits.
        newCall(ep);
        return 0;
    }
    if (strcmp(call->dtmfDigits, "*82") == 0)
    {
        // give dialtone and start the call over again,
        // but don't clear out these digits.
        newCall(ep);
        return 0;
    }
    // Get and store the originally dialed digits here.
    // Also store the the call digits here too.
    getDialedDigits (call);

    ep->call = call;

    // This will no longer make a call.  It simply creates an RTP
    // connection for the originator.
    makeCall (ep);

    return 0;
}    /* dialingComplete */


/**
 When B gets 200 for Invite in dialing State
 it modifies its connection and changes state
 to Connected
*/
void*
originalCall(void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog(LOG_DEBUG, "originalCall");
    if ( ep->call == 0 )
    {
        cpLog(LOG_INFO, "No call exist for originalCall");
        return 0;
    }
    MgcModifyConnection mdcx (ep->epId.convertString(),
                              ep->call->callIdentifier,
                              ep->call->endpoint[0].connectionId,
                              "sendrecv");
    mdcx.setRemoteConnectionDescriptor (ep->call->endpoint[1].localConnection);

    MgcpConnectionOptions co;

    if (ep->echoCancellation == false)
        co.setEchoCancellation (false);

    cpLog (LOG_DEBUG, "Echo Cancellation %s",
           co.getEchoCancellation() ? "true" : "false" );

    mdcx.setLocalConnectionOptions (co);
    ep->gw->asyncStack->asyncSend(mdcx, ep->gw->handle);
    cpStateChange (ep, CallStateConnected);
    return 0;
}
//originalCall


///
void
callDestination (CallData* call1)
{
    cpLog (LOG_DEBUG, "callDestination");

    // call to the SIP Proxy  to send an INVITE message
    sipEventSend (&eps[call1->endpoint[0].id], SipEventSendInvite);
}    /* callDestination */


///
void*
connectionReady (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "connectionReady");

    cpStateChange (ep, CallStateCallInitiated);

    // In case of a lost INVITE message, allow the MGCP phone to hang up.
    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());
    // Code corresponding to requested events
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
    addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );
    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    callDestination (ep->call);
    return 0;
}    /* connectionReady */


///
void*
restartOutCall (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "restartOutCall");

    CallData* call = ep->call;
    if (!call)
    {
        cpLog (LOG_ERR, "restartOutCall(): call is NULL");
        return ptr;
    }

    if (call->forwardNoAnswerId != 0)
    {
        ep->iq->cancel(call->forwardNoAnswerId);
        cpLog(LOG_DEBUG, "TIMER for forwardNoAnswerId %d was cancelled.",
              call->forwardNoAnswerId);
        call->forwardNoAnswerId = 0;
    }

    callDataInit (ep->call);
    ep->call = 0;

    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);

    return 0;
}    /* restartOutCall */


///
void*
featureFarEndAlerted(void* ptr)
{
    CpEventData* ed = static_cast < CpEventData* > (ptr);

    cpLog(LOG_DEBUG, "featureFarEndAlerted");
    /* If B has a held call and while party c is ringing
       does a flash, it has to send cancel to c and re-invite 
       to A
    */
    if (ed->ep->heldCall != 0)
    {
        sipEventSend(ed->ep, SipEventSendCancel);
        ed->ep->call = ed->ep->heldCall;
        ed->ep->heldCall = 0;
        sipEventSend(ed->ep, SipEventSendInvite);
        cpStateChange (ed->ep, CallStateConnected);
    }
    /*
      If B has no held call, it has send cancel to present call
      and delete its connection
    */
    else
    {
        sipEventSend(ed->ep, SipEventSendCancel);
        if ( ed->ep->call != 0 && ed->ep->call->endpoint[0].connectionId[0] != 0 )
        {
            MgcDeleteConnectionAgent dlcx (ed->ep->epId.convertString());
            dlcx.setCallId (ed->ep->call->callIdentifier);
            dlcx.setConnectionId (ed->ep->call->endpoint[0].connectionId);
            assert( ed->ep->gw != 0 );
            ed->ep->gw->asyncStack->asyncSend (dlcx, ed->ep->gw->handle);
            cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ed->ep->epId.logData());
            if ( ed->ep->call != 0 )
            {
                callDataInit(ed->ep->call);
                ed->ep->call = 0;
            }
        }
        newCall(ed->ep);
    }
    freeCpEventData (ed->ep->gw->edPool, ed);
    return 0;
}

///
void*
ringBack (void* ptr)
{
    // - SIP Proxy calls this function to give ringback tone.

    Endpoint* ep = static_cast < Endpoint* > (ptr);


    if ( ep->call == 0 )
    {
        cpLog(LOG_INFO, "No call exist for ringBack");
        return 0;
    }

    SessionDescriptor sd = ep->call->endpoint[1].localConnection;
    // Call Waiting block this mdcx for 180 messages
    // Let's try blocking it for all 180 messages ...
    if ( sd.isValidDescriptor() )
    {
        if ( !sd.isHold() )
        {
            MgcModifyConnection mdcx (ep->epId.convertString(),
                                      ep->call->callIdentifier,
                                      ep->call->endpoint[0].connectionId,
                                      "sendrecv");
            mdcx.setRemoteConnectionDescriptor (ep->call->endpoint[1].localConnection);
            MgcpConnectionOptions co;

            if (ep->echoCancellation == false)
                co.setEchoCancellation (false);

            cpLog (LOG_DEBUG, "Echo Cancellation %s",
                   co.getEchoCancellation() ? "true" : "false" );

            mdcx.setLocalConnectionOptions (co);

            ep->gw->asyncStack->asyncSend(mdcx, ep->gw->handle);
            cpStateChange (ep, CallStateFarEndAlerted);
            return 0;
        } 
    }//end IF(validSDP)
    cpLog (LOG_DEBUG, "ringBack for ENDPOINT %s\n", ep->epId.logData());
    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /* Code corresponding to requested events */
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
    addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );

    /* Code corresponding to signals */
    addRequestOrSignalEvent( &rqnt, GenericPackageSignalRingBackToneOn );

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    cpStateChange (ep, CallStateFarEndAlerted);
    return 0;
}    /* ringBack */


///
void*
featureCallInitiated(void* ptr)
{
    CpEventData* ed = static_cast < CpEventData* > (ptr);

    cpLog(LOG_DEBUG, "featureCallInitiated");

    /*
        If B has a held call and while party c is ringing
        does a flash, it has to send cancel to c and re-invite 
        to A
     */
    if (ed->ep->heldCall != 0)
    {
        sipEventSend(ed->ep, SipEventSendCancel);
        ed->ep->call = ed->ep->heldCall;
        ed->ep->heldCall = 0;
        sipEventSend(ed->ep, SipEventSendInvite);
    }
    /*
       If B has no held call, it has send cancel to present call
       and delete its connection
    */
    else
    {
        sipEventSend(ed->ep, SipEventSendCancel);
        if ( ed->ep->call != 0 && ed->ep->call->endpoint[0].connectionId[0] != 0 )
        {
            MgcDeleteConnectionAgent dlcx (ed->ep->epId.convertString());
            dlcx.setCallId (ed->ep->call->callIdentifier);
            dlcx.setConnectionId (ed->ep->call->endpoint[0].connectionId);
            assert( ed->ep->gw != 0 );
            ed->ep->gw->asyncStack->asyncSend (dlcx, ed->ep->gw->handle);
            cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ed->ep->epId.logData());
            if ( ed->ep->call != 0 )
            {
                callDataInit(ed->ep->call);
                ed->ep->call = 0;
            }
        }
        newCall(ed->ep);
    }
    freeCpEventData (ed->ep->gw->edPool, ed);
    return 0;
}


/// If there is a held call ...
/// Much of this needs to be changed to messaging instead of
/// directly changing data.
int
heldCallOnDisconnect (Endpoint* ep)
{
    if (ep->heldCall == 0)
    {
        return 0;
    }

    // print this only if there is a held call
    cpLog (LOG_DEBUG, "heldCallOnDisconnect");

    /*
     If B keeps A on hold and while in callInitiated state or dialing state
     onhooks, send cancel if in callintiated state to party C and sent rqnt 
     for offhook and signal for ringing on and change state to alerting
    */
    if ((ep->state == CallStateCallInitiated) || (ep->state == CallStateDialing))
    {
        if (ep->state == CallStateCallInitiated)
        {
            sipEventSend(ep, SipEventSendCancel);
        }
        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
        cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

        /* Code corresponding to requested events */
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOffHookTransition );
        /* Code corresponding to signals */
        addRequestOrSignalEvent( &rqnt, LinePackageSignalRingingOn );
        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
        callDataInit (ep->call);
        ep->call = 0;
        cpStateChange (ep, CallStateAlerting);
        return 1;
    }
    /*
     If party B keeps A on Hold and onhooks, send rqnt for offhook and 
     signal ringing on and change state to Alerting
    */
    if ( ep->call == 0 && ep->heldCall != 0 )
    {
        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
        cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

        addRequestOrSignalEvent( &rqnt, LinePackageRequestOffHookTransition );
        addRequestOrSignalEvent( &rqnt, LinePackageSignalRingingOn );
        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
        cpStateChange (ep, CallStateAlerting);
        return 1;
    }

    // handle the Call Waiting case here
    if (ep->heldCall->callWaited)
    {
        sipEventSend(ep, SipEventSendBye);

        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
        cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

        /* Code corresponding to requested events */
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOffHookTransition );
        /* Code corresponding to signals */
        addRequestOrSignalEvent( &rqnt, LinePackageSignalRingingOn );
        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

        callDataInit (ep->call);
        ep->call = 0;
        cpStateChange (ep, CallStateAlerting);

        return 1;
    }

    // assume for now that this is a transfer attempt

    // If the active call is ringing or connected, assume for now that
    // this a transfer.  In the future, we will need to distinguish whether
    // this held call was held via transfer or call waiting (or some
    // other feature in the future).  If this were call waiting,
    // no matter what the state, the active call is dropped and
    // this set should be rung (similar to the else part below).
    if (completeTransfer (ep))
    {
        return 1;
    }

    // call waiting needs to also be handled here too
    return 1;
}


///
int
completeTransfer(Endpoint* ep)
{
    cpLog (LOG_DEBUG, "completeTransfer");
    int epState = ep->state;

    if (ep->call == 0 || ep->heldCall == 0)
    {
        cpLog( LOG_INFO, "no call or heldCall exist for transfer");
        return 0;
    }
    if ((epState == CallStateConnected) ||
            (epState == CallStateFarEndAlerted))
    {
        // send sip event out for the transfer event
        sipEventSend(ep, SipEventSendXfer);

        // Drop this current call
        CpEventData* edHeld = newCpEventData (ep->gw->edPool);
        if ( edHeld == 0)
        {
            cpLog (LOG_ERR, "completeTransfer(): edHeld is NULL");
            assert( edHeld != 0 );
            return 0;
        }
        // Drop the held call first
        strcpy (edHeld->callIdentifier, ep->heldCall->callIdentifier);
        edHeld->ep = ep;

        if (strcmp(ep->call->endpoint[0].connectionId, ep->heldCall->endpoint[0].connectionId) == 0)
        {
            edHeld->ep->call->endpoint[0].connectionId[0] = 0;
        }

        clearXferCall(edHeld);
        // Change the call state to disconnecting so that clearCall will
        // properly disconnect this call when dropping the active call.
        cpStateChange (ep, CallStateDisconnecting);
        // Now drop the active call
        CpEventData* edActive = newCpEventData (ep->gw->edPool);
        if ( edActive == 0)
        {
            cpLog (LOG_ERR, "completeTransfer(): edActive is NULL");
            assert( edActive != 0 );
            return 0;
        }

        strcpy (edActive->callIdentifier, ep->call->callIdentifier);
        edActive->ep = ep;
        clearXferCall(edActive);

        return 1;
    }
    // - P4: Hook for the unused transient state (Call Initiated)
    // It will be used once remote ringback is supported.
    // Once this state is hit, should we drop this call and ring
    // this endpoint to let it know that there is still a held call?
    // Or should we allow transfer to complete, and assume that this
    // call will complete?
    cpLog (LOG_DEBUG, "Currently cannot handle this scenario");

    return 0;
}  // completeTransfer



///
void* modifyConnect(void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    if ( ep->call == 0 )
    {
        cpLog(LOG_INFO, "No call exist to be modify");
        return 0;
    }
    SessionDescriptor sd = ep->call->endpoint[1].localConnection;
    if ( !sd.isHold() )
    {

        MgcModifyConnection mdcx (ep->epId.convertString(),
                                  ep->call->callIdentifier,
                                  ep->call->endpoint[0].connectionId,
                                  "sendrecv");
        mdcx.setRemoteConnectionDescriptor (ep->call->endpoint[1].localConnection);

        MgcpConnectionOptions co;

        if (ep->echoCancellation == false)
            co.setEchoCancellation (false);

        cpLog (LOG_DEBUG, "Echo Cancellation %s",
               co.getEchoCancellation() ? "true" : "false" );

        mdcx.setLocalConnectionOptions (co);

        ep->gw->asyncStack->asyncSend(mdcx, ep->gw->handle);

    }
    /*
     Modify the Connection to Inactive after getting 200 response
     for Hold Invite
    */
    else
    {
        MgcModifyConnection mdcx (ep->epId.convertString(),
                                  ep->call->callIdentifier,
                                  ep->call->endpoint[0].connectionId,
                                  "inactive");
        ep->gw->asyncStack->asyncSend(mdcx, ep->gw->handle);
    }

    return ( 0 );
}


///
void* featureConnected(void* ptr)
{
    CpEventData* ed = static_cast < CpEventData* > (ptr);

    // Give Call Waiting Tone
    if (ed->featureId == FeatureEventCallWaiting)
    {
        sprintf (ed->ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ed->ep->epId.convertString(), ed->ep->requestIdentifier);
        cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ed->ep->epId.logData());

        /* Code corresponding to requested events */
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
        addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );
        addRequestOrSignalEvent( &rqnt, DTMFPackageRequestWildcardMatch );
        addRequestOrSignalEvent( &rqnt, LinePackageCallWaitingTone);

        ed->ep->gw->asyncStack->asyncSend (rqnt, ed->ep->gw->handle);
        freeCpEventData (ed->ep->gw->edPool, ed);
        return ( 0 );
    }

    // this will send a HOLD (INVITE) message
    if (ed->ep->call == 0 && ed->ep->heldCall != 0)
    {
        ed->ep->call = ed->ep->heldCall;
        ed->ep->heldCall = 0;
        if (ed->ep->call->callWaiting)
        {
            // if there is a ringing "call waiting" call, modify this
            // connection and then send a 200 response in mdcxResponse
            modifyConnect(ed->ep);
        }
        else
        {
            sipEventSend(ed->ep, SipEventSendInvite);
        }
        freeCpEventData (ed->ep->gw->edPool, ed);
        return 0;
    }

    // second flash if there is a held call and start conference
    if (ed->ep->heldCall != 0)
    {
        if (ed->ep->heldCall->callWaited == true)
        {
            // don't swap yet, keep the call & heldCall for Call Waiting.
            sipEventSend(ed->ep, SipEventSendHold);
            freeCpEventData (ed->ep->gw->edPool, ed);
            return ( 0 );
        }
        callDataInit(ed->ep->heldCall);
        ed->ep->heldCall = 0;
        sipEventSend(ed->ep, SipEventSendXfer);
        freeCpEventData (ed->ep->gw->edPool, ed);
        return ( 0 );
    }

    sipEventSend(ed->ep, SipEventSendHold);
    ed->ep->heldCall = ed->ep->call;
    freeCpEventData (ed->ep->gw->edPool, ed);
    return ( 0 );
}


///
void*
disconnect (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "disconnect");

    // Handle the held call during a disconnect
    // e.g. transfer if applicable
    if (heldCallOnDisconnect (ep) == 1)
    {
        return 0;
    }

    cpStateChange (ep, CallStateDisconnecting);

    CallData* call = ep->call;
    if (!call)
    {
        cpLog (LOG_ERR, "disconnect(): call is NULL");
        return ptr;
    }

    if (call->forwardNoAnswerId != 0)
    {
        ep->iq->cancel(call->forwardNoAnswerId);
        cpLog(LOG_DEBUG, "TIMER for forwardNoAnswerId %d was cancelled.",
              call->forwardNoAnswerId);
        call->forwardNoAnswerId = 0;
    }
    MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
    dlcx.setCallId (call->callIdentifier);
    dlcx.setConnectionId (call->endpoint[0].connectionId);


    ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);

    cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());



    // call to the SIP Proxy - to send a BYE message
    sipEventSend (ep, SipEventSendBye);

    return 0;
}    /* disconnect */


///
void*
featureDialing(void* ptr)
{

    CpEventData* ed = static_cast < CpEventData* > (ptr);

    cpLog (LOG_DEBUG, "featureDialing");
    /*
     If B keeps A on hold and in dialing state does a flash
     reconnect to A by sending re-invite to A
    */
    if (ed->ep->heldCall != 0)
    {
        ed->ep->call = ed->ep->heldCall;
        ed->ep->heldCall = 0;
        sipEventSend (ed->ep, SipEventSendInvite);
    }
    /*
     If B has no held call, and does a flash treat it as a new call
    */
    else
    {
        newCall(ed->ep);
    }
    freeCpEventData (ed->ep->gw->edPool, ed);
    return 0;
}

/// far-end disconnected
void*
clearCall (void* ptr)
{
    // - This function is called when the SIP Proxy gets a "BYE" message.
    CpEventData* ed = static_cast < CpEventData* > (ptr);
    Endpoint* ep = ed->ep;
    char callIdentifier[CALL_ID_LEN_MAX];
    strcpy(callIdentifier, ed->callIdentifier);

    cpLog (LOG_DEBUG, "clearCall");

    // first get all data required from ed,
    // then delete it since it is no longer needed.
    freeCpEventData (ep->gw->edPool, ed);

    CallData* call;
    int heldCall = false;

    if ((ep->heldCall != 0) &&
            (strcmp(callIdentifier, ep->heldCall->callIdentifier) == 0))
    {
        cpLog (LOG_DEBUG, "There is a held call");
        if (ep->sipMsgHeld == 0)
        {
            call = ep->heldCall;
            ep->heldCall = 0;
            heldCall = true;
        }
        else if ( ep->sipMsg == 0 && ep->call != 0 )
        {
            call = ep->call;
            ep->call = 0;
        }

        if ( ep->call == 0 && ep->heldCall == 0 &&
                call->endpoint[0].connectionId[0] != 0 )
        {
            MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
            dlcx.setCallId (call->callIdentifier);
            dlcx.setConnectionId (call->endpoint[0].connectionId);
            ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);
            cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
            newCall(ep);
        }

        callDataInit (call);
        return 0;
    }

    else if ((ep->call != 0) &&
             (strcmp(callIdentifier, ep->call->callIdentifier) == 0))
    {
        cpLog (LOG_DEBUG, "CallIdentifier matches active call in clearCall");
        call = ep->call;
        ep->call = 0;
    }
    else
    {
        cpLog (LOG_ERR, "CallIdentifiers do NOT match in clearCall");
        return 0;
    }

    if (call->endpoint[0].connectionId[0] != 0)
    {
        MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
        dlcx.setCallId (call->callIdentifier);
        dlcx.setConnectionId (call->endpoint[0].connectionId);

        ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);

        cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
    }
    callDataInit (call);

    if (heldCall == true)
    {
        // do not change the state of the active call when clearing the
        // held call.
    }

    else
    {
        if ((ep->state == CallStateConnected) ||
                (ep->state == CallStateFarEndAlerted) ||
                (ep->state == CallStateCallInitiated))
        {

            cpStateChange (ep, CallStateDisconnecting);
        }
        else if (ep->state == CallStateAlerting)
            return 0;
        else // epState != CallStateConnected
        {
            restartTransient(ep);
        }
    }

    return 0;
}    /* clearCall */


///
void*
clearXferCall (void* ptr)
{
    // - This function is called when the SIP Proxy gets a "BYE" message.
    CpEventData* ed = static_cast < CpEventData* > (ptr);
    Endpoint* ep = ed->ep;
    char callIdentifier[CALL_ID_LEN_MAX];
    strcpy(callIdentifier, ed->callIdentifier);

    cpLog (LOG_DEBUG, "clearCall");

    // first get all data required from ed,
    // then delete it since it is no longer needed.
    freeCpEventData (ep->gw->edPool, ed);

    CallData* call;
    int heldCall = false;

    if ((ep->heldCall != 0) &&
            (strcmp(callIdentifier, ep->heldCall->callIdentifier) == 0))
    {
        cpLog (LOG_DEBUG, "CallIdentifier matches held call in clearXferCall");
        call = ep->heldCall;
        ep->heldCall = 0;
        heldCall = true;
    }

    else if ((ep->call != 0) &&
             (strcmp(callIdentifier, ep->call->callIdentifier) == 0))
    {
        cpLog (LOG_DEBUG, "CallIdentifier matches active call in clearXferCall");
        call = ep->call;
        ep->call = 0;
    }
    else
    {
        cpLog (LOG_ERR, "CallIdentifiers do NOT match in clearXferCall");
        return 0;
    }

    if (call->endpoint[0].connectionId[0] != 0)
    {
        MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
        dlcx.setCallId (call->callIdentifier);
        dlcx.setConnectionId (call->endpoint[0].connectionId);

        ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);

        cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
    }
    callDataInit (call);

    if (heldCall == true)
    {
        // do not change the state of the active call when clearing the
        // held call.
    }

    else
    {
        if ((ep->state == CallStateConnected) ||
                (ep->state == CallStateFarEndAlerted) ||
                (ep->state == CallStateCallInitiated))
        {

            cpStateChange (ep, CallStateDisconnecting);
        }
        else // epState != CallStateConnected
        {
            restartTransient(ep);
        }
    }

    return 0;
}


///
void*
connected (void* ptr)
{
    // - SIP Proxy calls this function after a 200 response is given
    //   - (e.g. the call is answered)
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "connected");

    CallData* call = ep->call;
    if (!call)
    {
        cpLog (LOG_ERR, "connected(): call is NULL");
        return ptr;
    }

    if (call->forwardNoAnswerId != 0)
    {
        ep->iq->cancel(call->forwardNoAnswerId);
        cpLog(LOG_DEBUG, "TIMER for forwardNoAnswerId %d was cancelled.",
              call->forwardNoAnswerId);
        call->forwardNoAnswerId = 0;
    }

    MgcModifyConnection mdcx (ep->epId.convertString(),
                              call->callIdentifier,
                              call->endpoint[0].connectionId,
                              "sendrecv");

    mdcx.setRemoteConnectionDescriptor (call->endpoint[1].localConnection);

    MgcpConnectionOptions co;
    if (ep->echoCancellation == false)
    {
        co.setEchoCancellation (false);
    }

    cpLog (LOG_DEBUG, "Echo Cancellation %s",
           co.getEchoCancellation() ? "true" : "false");

    mdcx.setLocalConnectionOptions (co);

    //    cpLog(LOG_DEBUG, "MDCX FOR ENDPOINT %d", ep->id);
    //    cpLog(LOG_DEBUG, "Local port = %d", ep->call->endpoint[0].localConnection.getPort());

    ep->gw->asyncStack->asyncSend (mdcx, ep->gw->handle);

    cpLog(LOG_DEBUG, "MDCX FOR ENDPOINT %s\n", ep->epId.logData());

    cpStateChange (ep, CallStateConnected);

    cpLog (LOG_DEBUG, "Connected Call %d of Thread %d at %s",
           ++connectCount[ep->gw->cpThread],
           ep->gw->cpThread,
           ep->epId.logData());
    return 0;
}    /* connected */


///
void*
connectionReady2 (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "connectionReady2");

    CallData* call = ep->call;
    if (!call)
    {
        if (ep->heldCall != 0)
        {
            cpLog (LOG_DEBUG, "connectionReady2(): ignored");
            return ( 0 );
        }
        cpLog (LOG_ERR, "connectionReady2(): call is NULL");
        return ptr;
    }

    // call to the SIP Proxy to send a "200" response message with SDP info.
    sipEventSend (ep, SipEventSendAnswer);

    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    /* Code corresponding to requested events */
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
    addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );
    addRequestOrSignalEvent( &rqnt, DTMFPackageRequestWildcardMatch );

    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    cpStateChange (ep, CallStateConnected);

    return 0;
}    /* connectionReady2 */


///
void*
restartInCall (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "restartInCall");

    if (ep->call)
    {
        callDataInit (ep->call);
        ep->call = 0;
    }
    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);

    return 0;
}    /* restartInCall */


/** when primary call agent out of service and secondary call agent
 * take over control of the endpoints. It send the rqnt to the
 * endpoints, but will receive 401 if the endpoints is in off-hook
 * state. Resend rqnt onhook instead here.
 */
void
resumeEndpoint (Endpoint* ep)
{

    cpLog (LOG_DEBUG, "resumeEndpoint");

    // create the call field for dlcx to work correctly.
    if (ep->call == 0)
    {
        CallData* call = newCallData (ep);
        if (call == 0)
        {
            cpLog (LOG_ERR, "resumeEndpoint(): call is NULL");
            assert( call != 0 );
            return ;
        }
        call->endpoint[0].id = ep->id;
        ep->call = call;
    }

    cpStateChange (ep, CallStateConnected);
    // In case of a lost INVITE message, allow the MGCP phone to hang up.
    sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
    MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());
    // Code corresponding to requested events
    addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
    ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

    return ;
}    /* resumeEndpoint */

///
void*
glare (void* ptr)
{
    //    Endpoint* ep = static_cast<Endpoint*>(ptr);
    cpLog (LOG_DEBUG, "glare");
    return 0;
}    /* glare */


///
void*
ringing (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);

    if ( ep->call == 0 )
    {
        cpLog(LOG_INFO, "ringing(): call is NULL");
        return ptr;
    }

    cpLog (LOG_DEBUG, "ringing");
    cpStateChange (ep, CallStateAlerting);

    MgcCreateConnection crcx (ep->epId.convertString(), ep->call->callIdentifier, "sendrecv");
    crcx.setRemoteConnectionDescriptor (ep->call->endpoint[1].localConnection);

    MgcpConnectionOptions co;
    if (ep->echoCancellation == false)
    {
        co.setEchoCancellation (false);
    }
    cpLog (LOG_DEBUG, "Echo Cancellation %s",
           co.getEchoCancellation() ? "true" : "false");

    crcx.setLocalConnectionOptions (co);
    ep->gw->asyncStack->asyncSend (crcx, ep->gw->handle);

    cpLog(LOG_DEBUG, "CRCX FOR ENDPOINT %s\n", ep->epId.logData());

    return 0;
}    /* ringing */


///
void*
answer (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "answer");

    CallData* call = ep->call;
    if (!call)
    {
        if ( ep->heldCall != 0 )
        {
            ep->call = ep->heldCall;
            ep->heldCall = 0;
            // Call Waiting
            if (ep->call->callWaiting)
            {
                modifyConnect(ep);
            }
            else
            {
                sipEventSend (ep, SipEventSendInvite);
            }
            cpStateChange (ep, CallStateConnected);
            return (0);
        }
        cpLog (LOG_ERR, "answer(): call is NULL");
        return ptr;
    }

    if ( ep->call->endpoint[0].connectionId[0] != 0 )
    {
        cpProcessEvent (ep, CallEventResponse);
        return 0;
    }

    MgcCreateConnection crcx (ep->epId.convertString(), call->callIdentifier, "sendrecv");
    crcx.setRemoteConnectionDescriptor (call->endpoint[1].localConnection);

    MgcpConnectionOptions co;
    if (ep->echoCancellation == false)
    {
        co.setEchoCancellation (false);
    }
    cpLog (LOG_DEBUG, "Echo Cancellation %s",
           co.getEchoCancellation() ? "true" : "false");

    crcx.setLocalConnectionOptions (co);
    ep->gw->asyncStack->asyncSend (crcx, ep->gw->handle);

    cpLog(LOG_DEBUG, "CRCX FOR ENDPOINT %s\n", ep->epId.logData());
    return 0;
}    /* answer */


///
void*
restartConnected (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "restartConnected");

    if (ep->call)
    {
        callDataInit (ep->call);
        ep->call = 0;
    }

    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);

    return 0;
}    /* restartConnected */


///
void*
restartDisonnect (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "restartDisonnect");

    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);

    return 0;
}    /* restartDisonnect */


///
void*
disconnectResponse (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "disconnectResponse");

    if (ep->call != 0)
    {
        // Disconnecting side: therefore, don't wait for l/hu
        callDataInit (ep->call);
        ep->call = 0;
        cpStateChange (ep, CallStateInit);
        cpProcessEvent (ep, CallEventRestart);
    }
    if (ep->state == CallStateAlerting)
    {
        cpStateChange (ep, CallStateInit);
        cpProcessEvent (ep, CallEventRestart);
    }
    return 0;
}    /* disconnectResponse */

///
void*
disconnectRelease (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "disconnectRelease");

    if (ep->call != 0)
    {
        callDataInit (ep->call);
        ep->call = 0;
    }
    cpStateChange (ep, CallStateInit);
    cpProcessEvent (ep, CallEventRestart);
    return 0;
}    /* disconnectRelease */


///
void
addDTMFSignal (MgcNotificationRequest* rqnt, char digit)
{
    switch (digit)
    {
        case '0':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_0 );
        }
        break;
        case '1':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_1 );
        }
        break;
        case '2':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_2 );
        }
        break;
        case '3':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_3 );
        }
        break;
        case '4':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_4 );
        }
        break;
        case '5':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_5 );
        }
        break;
        case '6':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_6 );
        }
        break;
        case '7':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_7 );
        }
        break;
        case '8':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_8 );
        }
        break;
        case '9':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_9 );
        }
        break;
        case '#':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_Hash );
        }
        break;
        case '*':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_Star );
        }
        break;
        case 'A':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_A );
        }
        break;
        case 'B':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_B );
        }
        break;
        case 'C':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_C );
        }
        break;
        case 'D':
        {
            addRequestOrSignalEvent( rqnt, DTMFPackageSignalDTMF_D );
        }
        break;
        default:
        {
            cpLog (LOG_ERR, "addDTMFSignal");
        }
    }    /* switch */
}    /* addDTMFSignal */


///
void*
dtmfDigit (void* ptr)
{
    Endpoint* ep = static_cast < Endpoint* > (ptr);
    cpLog (LOG_DEBUG, "dtmfDigit");

    if ( ep->call == 0 )
    {
        cpLog (LOG_ERR, "dtmfDigit(): call is NULL");
        return ptr;
    }

    CallData* call1 = ep->call;
    if (call1->playDtmfDigit != 0)
    {
        cpLog (LOG_DEBUG, "play digit %c", call1->playDtmfDigit);
        /* Receiving DTMF digit */
        /* Forward DTMF digit to the gateway */
        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
        cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

        addDTMFSignal (&rqnt, call1->playDtmfDigit);

        /* Code corresponding to requested events */
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
        addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );
        addRequestOrSignalEvent( &rqnt, DTMFPackageRequestWildcardMatch );

        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
    }
    if (call1->dtmfDigits[0] != 0)
    {
        cpLog (LOG_DEBUG, "received digit %c from the Gateway", call1->dtmfDigits[0]);

        if (ep->state == CallStateConnected)
        {
            sipEventSend(ep, SipEventSendInfo);
        }

        call1->dtmfDigits[0] = 0;

        /* Sent RQNT again */
        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);
        cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

        /* Code corresponding to requested events */
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
        addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );
        addRequestOrSignalEvent( &rqnt, DTMFPackageRequestWildcardMatch );

        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
    }

    return 0;
}    /* dtmfDigit */


///
bool
saveDigits (Endpoint* ep,
#ifndef NCS_PROFILE
            Sptr < MgcDTMFPackageEvent > event
#else
            Sptr < MgcNCSLinePackageEvent > event
#endif
           )
{
    if (ep->call == 0)
    {
        ep->call = newCallData (ep);

        assert(strlen(ep->call->dtmfDigits) == 0);

        ep->call->endpoint[0].id = ep->id;
        sprintf (ep->call->callIdentifier, "%x", newCallIdentifier());
    }
    if (!call)
    {
        cpLog (LOG_ERR, "saveDigits(): call is NULL");
        return ( false );
    }

    /* support only one digit per event for now */
    switch (event->getType())
    {
#ifndef NCS_PROFILE
        case MgcDTMFPackageDTMF_0:
        {
            strcat (ep->call->dtmfDigits, "0");
            cpLog (LOG_DEBUG, "Got DTMF event 0");
        }
        break;
        case MgcDTMFPackageDTMF_1:
        {
            strcat (ep->call->dtmfDigits, "1");
            cpLog (LOG_DEBUG, "Got DTMF event 1");
        }
        break;
        case MgcDTMFPackageDTMF_2:
        {
            strcat (ep->call->dtmfDigits, "2");
            cpLog (LOG_DEBUG, "Got DTMF event 2");
        }
        break;
        case MgcDTMFPackageDTMF_3:
        {
            strcat (ep->call->dtmfDigits, "3");
            cpLog (LOG_DEBUG, "Got DTMF event 3");
        }
        break;
        case MgcDTMFPackageDTMF_4:
        {
            strcat (ep->call->dtmfDigits, "4");
            cpLog (LOG_DEBUG, "Got DTMF event 4");
        }
        break;
        case MgcDTMFPackageDTMF_5:
        {
            strcat (ep->call->dtmfDigits, "5");
            cpLog (LOG_DEBUG, "Got DTMF event 5");
        }
        break;
        case MgcDTMFPackageDTMF_6:
        {
            strcat (ep->call->dtmfDigits, "6");
            cpLog (LOG_DEBUG, "Got DTMF event 6");
        }
        break;
        case MgcDTMFPackageDTMF_7:
        {
            strcat (ep->call->dtmfDigits, "7");
            cpLog (LOG_DEBUG, "Got DTMF event 7");
        }
        break;
        case MgcDTMFPackageDTMF_8:
        {
            strcat (ep->call->dtmfDigits, "8");
            cpLog (LOG_DEBUG, "Got DTMF event 8");
        }
        break;
        case MgcDTMFPackageDTMF_9:
        {
            strcat (ep->call->dtmfDigits, "9");
            cpLog (LOG_DEBUG, "Got DTMF event 9");
        }
        break;
        case MgcDTMFPackageDTMF_Hash:
        {
            strcat (ep->call->dtmfDigits, "#");
            cpLog (LOG_DEBUG, "Got DTMF event #");
        }
        break;
        case MgcDTMFPackageDTMF_Star:
        {
            strcat (ep->call->dtmfDigits, "*");
            cpLog (LOG_DEBUG, "Got DTMF event *");
        }
        break;
        case MgcDTMFPackageDTMF_A:
        {
            strcat (ep->call->dtmfDigits, "A");
            cpLog (LOG_DEBUG, "Got DTMF event A");
        }
        break;
        case MgcDTMFPackageDTMF_B:
        {
            strcat (ep->call->dtmfDigits, "B");
            cpLog (LOG_DEBUG, "Got DTMF event B");
        }
        break;
        case MgcDTMFPackageDTMF_C:
        {
            strcat (ep->call->dtmfDigits, "C");
            cpLog (LOG_DEBUG, "Got DTMF event C");
        }
        break;
        case MgcDTMFPackageDTMF_D:
        {
            strcat (ep->call->dtmfDigits, "D");
            cpLog (LOG_DEBUG, "Got DTMF event D");
        }
        break;
#else // NCS_PROFILE
        case MgcNCSLinePackageDTMF_0:
        {
            strcat (ep->call->dtmfDigits, "0");
            cpLog (LOG_DEBUG, "Got DTMF event 0");
        }
        break;
        case MgcNCSLinePackageDTMF_1:
        {
            strcat (ep->call->dtmfDigits, "1");
            cpLog (LOG_DEBUG, "Got DTMF event 1");
        }
        break;
        case MgcNCSLinePackageDTMF_2:
        {
            strcat (ep->call->dtmfDigits, "2");
            cpLog (LOG_DEBUG, "Got DTMF event 2");
        }
        break;
        case MgcNCSLinePackageDTMF_3:
        {
            strcat (ep->call->dtmfDigits, "3");
            cpLog (LOG_DEBUG, "Got DTMF event 3");
        }
        break;
        case MgcNCSLinePackageDTMF_4:
        {
            strcat (ep->call->dtmfDigits, "4");
            cpLog (LOG_DEBUG, "Got DTMF event 4");
        }
        break;
        case MgcNCSLinePackageDTMF_5:
        {
            strcat (ep->call->dtmfDigits, "5");
            cpLog (LOG_DEBUG, "Got DTMF event 5");
        }
        break;
        case MgcNCSLinePackageDTMF_6:
        {
            strcat (ep->call->dtmfDigits, "6");
            cpLog (LOG_DEBUG, "Got DTMF event 6");
        }
        break;
        case MgcNCSLinePackageDTMF_7:
        {
            strcat (ep->call->dtmfDigits, "7");
            cpLog (LOG_DEBUG, "Got DTMF event 7");
        }
        break;
        case MgcNCSLinePackageDTMF_8:
        {
            strcat (ep->call->dtmfDigits, "8");
            cpLog (LOG_DEBUG, "Got DTMF event 8");
        }
        break;
        case MgcNCSLinePackageDTMF_9:
        {
            strcat (ep->call->dtmfDigits, "9");
            cpLog (LOG_DEBUG, "Got DTMF event 9");
        }
        break;
        case MgcNCSLinePackageDTMF_Hash:
        {
            strcat (ep->call->dtmfDigits, "#");
            cpLog (LOG_DEBUG, "Got DTMF event #");
        }
        break;
        case MgcNCSLinePackageDTMF_Star:
        {
            strcat (ep->call->dtmfDigits, "*");
            cpLog (LOG_DEBUG, "Got DTMF event *");
        }
        break;
        case MgcNCSLinePackageDTMF_A:
        {
            strcat (ep->call->dtmfDigits, "A");
            cpLog (LOG_DEBUG, "Got DTMF event A");
        }
        break;
        case MgcNCSLinePackageDTMF_B:
        {
            strcat (ep->call->dtmfDigits, "B");
            cpLog (LOG_DEBUG, "Got DTMF event B");
        }
        break;
        case MgcNCSLinePackageDTMF_C:
        {
            strcat (ep->call->dtmfDigits, "C");
            cpLog (LOG_DEBUG, "Got DTMF event C");
        }
        break;
        case MgcNCSLinePackageDTMF_D:
        {
            strcat (ep->call->dtmfDigits, "D");
            cpLog (LOG_DEBUG, "Got DTMF event D");
        }
        break;
#endif
        default:
        {
            cpLog (LOG_ERR, "Unsupported DTMF event %d", event->getType());
            return ( false );
        }
    }    /* switch */

    return ( true );
}    /* saveDigits */


///
int
mapLinePackageEvent (
#ifndef NCS_PROFILE
    Sptr < MgcLinePackageEvent > event,
#else // NCS_PROFILE
    Sptr < MgcNCSLinePackageEvent > event,
#endif
    int* featureEventPtr)
{
    int cpEvent = CallEventUnknown;
    *featureEventPtr = FeatureEventUnknown;

    switch (event->getType())
    {
        /* tbd: a lot */
#ifndef NCS_PROFILE
        case MgcLinePackageOffHookTransition:
        {
            cpLog (LOG_DEBUG, "MgcLinePackageOffHookTransition");
            cpEvent = CallEventNewCall;
        }
        break;
        case MgcLinePackageOnHookTransition:
        {
            cpLog (LOG_DEBUG, "MgcLinePackageOnHookTransition");
            cpEvent = CallEventReleaseCall;
        }
        break;
        case MgcLinePackageFlashHook:
        {
            cpLog (LOG_DEBUG, "MgcLinePackageFlashHook");
            cpEvent = CallEventFeature;
            *featureEventPtr = FeatureEventFlash;
        }
        break;
#else // NCS_PROFILE
        case MgcNCSLinePackageOffHookTransition:
        {
            cpLog (LOG_DEBUG, "MgcNCSLinePackageOffHookTransition");
            cpEvent = CallEventNewCall;
        }
        break;
        case MgcNCSLinePackageOnHookTransition:
        {
            cpLog (LOG_DEBUG, "MgcNCSLinePackageOnHookTransition");
            cpEvent = CallEventReleaseCall;
        }
        break;
        case MgcNCSLinePackageFlashHook:
        {
            cpLog (LOG_DEBUG, "MgcNCSLinePackageFlashHook");
            cpEvent = CallEventFeature;
            *featureEventPtr = FeatureEventFlash;
        }
        break;
#endif
        default:
          cpLog(LOG_ERR, "Unknown Event");
        break;
    }    /* switch */
    return cpEvent;
}    /* mapLinePackageEvent */


///
void
decodeNotify (Endpoint* ep, Sptr < MgcNotify > ntfy)
{
    bool replied = false;
    int cpEvent = CallEventUnknown;
    int featureEvent = FeatureEventUnknown;
    int observedEventCount = ntfy->getNumObservedEvents ();
    cpLog (LOG_DEBUG, "observedEventCount = %d", observedEventCount);

    for (int i = 0; i < observedEventCount; i++)
    {
        Sptr < MgcEvent > observedEvent = ntfy->getItemObservedEvents (i);

        if ( observedEvent == 0 )
        {
            continue;
        }

        switch (observedEvent->getPackageType ())
        {
#ifndef NCS_PROFILE
            case MgcpGenericMediaPackageType:
            {
                cpLog (LOG_DEBUG, "MgcpGenericMediaPackageType");
            }
            break;
            case MgcpDTMFPackageType:
            {
                cpLog (LOG_DEBUG, "MgcpDTMFPackageType");
                Sptr < MgcDTMFPackageEvent > dtmfPtr;
                saveDigits (ep, dtmfPtr.dynamicCast(observedEvent));
                cpEvent = CallEventDigitsCollected;
            }
            break;
            case MgcpLinePackageType:
            {
                cpLog (LOG_DEBUG, "MgcpLinePackageType");
                Sptr < MgcLinePackageEvent > linePtr;
                cpEvent = mapLinePackageEvent(linePtr.dynamicCast(observedEvent), &featureEvent);
            }
            break;
            case MgcpRTPPackageType:
            {
                cpLog (LOG_DEBUG, "MgcpRTPPackageType");
                /* tbd */
            }
            break;
#else // NCS_PROFILE
            case MgcpNCSDefaultPackageType:
            case MgcpNCSLinePackageType:
            case MgcpNCSAllPackagesType:   // when Adsi package is used, this case should also process for Adsi events
            {
                cpLog (LOG_DEBUG, "MgcpNCSLinePackageType");

                Sptr < MgcNCSLinePackageEvent > linePtr;
                cpEvent = mapLinePackageEvent(linePtr.dynamicCast(observedEvent), &featureEvent);

                if ( cpEvent == CallEventUnknown )  // this event is not a call event, so check for digits
                {
                    Sptr < MgcNCSLinePackageEvent > dtmfPtr;
                    bool gotDigits = false;
                    gotDigits = saveDigits (ep, dtmfPtr.dynamicCast(observedEvent));

                    if ( gotDigits )
                        cpEvent = CallEventDigitsCollected;
                }
            }
            break;
            case MgcpNCSAdsiPackageType:
            {
                cpLog (LOG_DEBUG, "MgcpNCSAdsiPackageType");
                /* tbd */
            }
            break;
#endif
            default:
            cpLog (LOG_ERR,
                   "Unsupported Residential Gateway package: %d",
                   observedEvent->getPackageType ());
        }    /* switch */

        if (replied == false)
        {
            /* Assumes the command is always accepted for now */
            ntfy->setResponseCode (TransactionExecuted);
            ntfy->setResponseString (OKResponse);
            ep->gw->asyncStack->asyncReply (ntfy, ep->gw->handle);

            cpLog(LOG_DEBUG, "REPLY FOR NOTIFY ENDPOINT %s\n", ep->epId.logData());

            // ep->gw->reply (ntfy);
            replied = true;
        }
    }    /* for */
 
    // if there are dialed digits from the user, it must have been saved into
    // ep->call->dtmfDigits[], let's search for a possible ending '#'
    // and strip if off if found
    if ( ep != 0 )
    {
        if ( ep->call != 0 )
        {
            int i;
            int temp = strlen(ep->call->dtmfDigits);
            char c = '\0';
            for( i = 0; i < temp; i++ )
            {
                c = ep->call->dtmfDigits[i];
                if( c== '#' ) 
		{
                  ep->call->dtmfDigits[i] = '\0';
                  break;
                }  
            }
        
        }
    }

    // Warning: Process the last observed event only, intermediate
    //          events that are not digits are lost
    switch (cpEvent)
    {
        case (CallEventFeature):
        {
            CpEventData* ed = newCpEventData (ep->gw->edPool);
            if ( ed == 0 )
            {
                cpLog (LOG_ERR, "decodeNotify(): ed is NULL");
                assert( ed != 0 );
                return ;
            }
            ed->featureId = featureEvent;
            ed->ep = ep;
            cpProcessEvent (ed, cpEvent);
        }
        break;
        default:
        cpProcessEvent (ep, cpEvent);
    }    /* switch */

}    /* decodeNotify */


///
void
decodeRestartInProgress (Endpoint* ep, Sptr < MgcRestartInProgress > rsip)
{
    /* tbd: more decoding, send AUEP,... */

    rsip->setResponseCode (TransactionExecuted);
    rsip->setResponseString (OKResponse);

    ep->gw->asyncStack->asyncReply (rsip, ep->gw->handle);

    cpProcessEvent (ep, CallEventRestart);

    // don't send here.  wait till receive 200 rqntResponse
    // Add a hook here to have the sipProxy send a SIP REGISTER message
    //sipEventSend (ep, SipEventSendRegister);

}    /* decodeRestartInProgress */


///
void
decodeDeleteConnectionGw (Endpoint* ep, Sptr < MgcDeleteConnectionGw > dlcx)
{
    /* tbd: implement as CallEventReleaseCall/Restart, but log error and don't send DLCX */

    dlcx->setResponseCode (TransactionExecuted);
    dlcx->setResponseString (OKResponse);

    ep->gw->asyncStack->asyncReply (dlcx, ep->gw->handle);
    // ep->gw->reply (dlcx);

    cpLog(LOG_DEBUG, "REPLY FOR DLCX FOR ENDPOINT %s\n", ep->epId.logData());

    cpProcessEvent (ep, CallEventRestart);
}    /* decodeDeleteConnectionGw */


///
void
crcxResponse (Endpoint* ep, Sptr < MgcCreateConnection > crcx)
{
    cpLog (LOG_DEBUG, "CRCX response FOR ENDPOINT %s\n", ep->epId.logData());
    if (crcx->getResponseCode() != ReasonTransactionExecuted)
    {
        cpLog (LOG_WARNING, "%s crcx %s failed - %d",
               ep->epId.logData(),
               ((ep->call) ? ep->call->callIdentifier : "--"),
               crcx->getResponseCode());
        if (crcx->getResponseCode() == ReasonTransientError)
        {
            if ( ep->call != 0 && ep->call->endpoint[0].connectionId[0] != 0 )
            {
                MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
                dlcx.setCallId (ep->call->callIdentifier);
                dlcx.setConnectionId (ep->call->endpoint[0].connectionId);
                assert( ep->gw != 0 );
                ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);
                cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
                if ( ep->call != 0 )
                {
                    callDataInit(ep->call);
                    ep->call = 0;
                }
                cpStateChange (ep, CallStateInit);
                cpProcessEvent (ep, CallEventRestart);
                return ;
            }
        }
        disconnectResponse(ep);
        return ;
    }
    else
    {
        if (ep->call)
        {
            cpLog (LOG_DEBUG, "connection id = %s",
                   (crcx->getConnectionId ()).c_str());
            strcpy (ep->call->endpoint[0].connectionId,
                    (crcx->getConnectionId ()).c_str());
            ep->call->endpoint[0].localConnection = crcx->getLocalConnectionDescriptor ();
        }
        else
        {
            cpLog(LOG_ERR, "crcxResponse(): call is NULL");
            return ;  //hsinshi temporary keep
        }


        if (ep->call->dialedDigits[0] == 0 )
        {
            sipEventSend (ep, SipEventSendRinging);
            return ;
        }

        cpProcessEvent (ep, CallEventResponse);
    }
}    // crcxResponse


///
void
mdcxResponse (Endpoint* ep, Sptr < MgcModifyConnection > mdcx)
{

    if (mdcx->getResponseCode() != ReasonTransactionExecuted)
    {
        cpLog (LOG_WARNING, "%s mdcx failed for callId %s - response code: %d",
               ep->epId.logData(),
               ((ep->call) ? ep->call->callIdentifier : "--"),
               mdcx->getResponseCode());

        return ;
    }
    if (ep->call == 0)
    {
        cpLog ( LOG_ERR, "mdcxResponse(): call is null" );
        return ;
    }

    if (ep->heldCall == 0)
    {
        if (ep->call->endpoint[1].localConnection.isHold())
        {
            return ;
        }
        sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
        MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);

        /* Code corresponding to requested events */
        if ( ep->state == CallStateAlerting )
        {
            return ;
        }
        addRequestOrSignalEvent( &rqnt, LinePackageRequestOnHookTransition );
        addRequestOrSignalEvent( &rqnt, LinePackageRequestFlashHook );
        addRequestOrSignalEvent( &rqnt, DTMFPackageRequestWildcardMatch );

        ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);

        // If there is a ringing call waiting call, send a 200 response.
        if ( ep->call->callWaiting == true )
        {
            sipEventSend(ep, SipEventSendAnswer);
            ep->call->callWaiting = false;
            return ;
        }
    }
    else
    {
        if (ep->call->endpoint[1].localConnection.isHold())
        {

            if (ep->sipMsgHeld == 0)
            {
                newCall(ep);

                ep->heldCall = ep->call;
                ep->call = 0;
            }
            else
            {
                // Call Waiting, handle the second+ flash cases here
                // ep->call and ep->heldCall are already swapped
                // if ( ep->call->ackReceived == true )
                if ( ep->heldCall->callWaited == true )
                {
                    sipEventSend(ep, SipEventSendAck);
                }
                else
                {
                    CallData* tmp;
                    tmp = ep->heldCall;
                    ep->heldCall = ep->call;
                    ep->call = tmp;
                }
                sipEventSend(ep, SipEventSendInvite);
            }
        }
        else // else for: if (ep->call->endpoint[1].localConnection.isHold())
        {
            sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
            MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);

            /* Code corresponding to requested events */
            addRequestOrSignalEvent(&rqnt, LinePackageRequestOnHookTransition);
            addRequestOrSignalEvent(&rqnt, LinePackageRequestFlashHook);
            addRequestOrSignalEvent(&rqnt, DTMFPackageRequestWildcardMatch);

            ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
            if ( ep->heldCall != 0 )
            {
                // for Call Waiting, handle the first flash case only
                if ( ep->heldCall->callWaiting == true )
                {
                    sipEventSend (ep, SipEventSendAck);
                    // For Call Waiting send a 200 response.
                    // sipMsg and sipMsgHeld (and record route stuff)
                    // will be swapped in sipProxy now.
                    sipEventSend(ep, SipEventSendAnswer);
                    ep->heldCall->callWaiting = false;
                    return ;
                }
            }
        }
    }

    cpLog (LOG_DEBUG, "RQNT FOR ENDPOINT %s\n", ep->epId.logData());

    // - call the SIP Proxy to send an ACK after the connection is complete.
    //
    // note - This one is kinda tricky.  For basic calls, no additional checks
    //        are needed.  However, since this function is called for any
    //        modify connection responses, we should put additional checks,
    //        such as checking the call state, before sending this SIP ACK.
    sipEventSend (ep, SipEventSendAck);
}    // mdcxResponse


///
void
dlcxResponse (Endpoint* ep, Sptr < MgcDeleteConnectionAgent > dlcx)
{
    cpLog (LOG_DEBUG, "DLCX response FOR ENDPOINT %s\n", ep->epId.logData());
    if (dlcx->getResponseCode() != ReasonConnectionDeleted)
    {
        cpLog (LOG_WARNING, "%s dlcx %s failed - %d",
               ep->epId.logData(),
               ((ep->call) ? ep->call->callIdentifier : "--"),
               dlcx->getResponseCode());
    }
    if ( ep->state == CallStateInit )
    {
        initRestart(ep);
        return ;
    }
    if (ep->state == CallStateAlerting)
    {
        cpStateChange (ep, CallStateInit);
        cpProcessEvent (ep, CallEventRestart);
        return ;
    }
    cpProcessEvent (ep, CallEventResponse);
}    // dlcxResponse


///
void
rqntResponse (Endpoint* ep, Sptr < MgcNotificationRequest > rqnt)
{
    cpLog (LOG_DEBUG, "RQNT response FOR ENDPOINT %s\n", ep->epId.logData());
    if (rqnt->getResponseCode() != ReasonTransactionExecuted)
    {
        cpLog (LOG_WARNING, "%s rqnt failed - %d",
               ep->epId.logData(),
               rqnt->getResponseCode());
        if (rqnt->getResponseCode() == ReasonTransientError)
        {
            if ( ep->call != 0 && ep->call->endpoint[0].connectionId[0] != 0 )
            {
                MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
                dlcx.setCallId (ep->call->callIdentifier);
                dlcx.setConnectionId (ep->call->endpoint[0].connectionId);
                assert( ep->gw != 0 );
                ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);
                cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData() );
                if ( ep->call != 0 )
                {
                    callDataInit(ep->call);
                    ep->call = 0;
                }
                cpStateChange (ep, CallStateInit);
                cpProcessEvent (ep, CallEventRestart);
            }
            return ;
        }
        else if (rqnt->getResponseCode() == ReasonTransientAlreadyOffHook)
        {
            if ( ep->state == CallStateInit )
            {
                cpProcessEvent (ep, CallEventResponse);
                return ;
            }
            sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
            MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);

            /* Code corresponding to requested events */
            addRequestOrSignalEvent(&rqnt, LinePackageRequestOnHookTransition);
            addRequestOrSignalEvent(&rqnt, LinePackageRequestFlashHook);
            assert( ep->gw != 0 );
            ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
            return ;
        }
        else if (rqnt->getResponseCode() == ReasonTransientAlreadyOnHook)
        {
            sprintf (ep->requestIdentifier, "%x", newRequestIdentifier());
            MgcNotificationRequest rqnt (ep->epId.convertString(), ep->requestIdentifier);

            /* Code corresponding to requested events */
            addRequestOrSignalEvent(&rqnt, LinePackageRequestOffHookTransition);
            addRequestOrSignalEvent(&rqnt, LinePackageRequestFlashHook);
            assert( ep->gw != 0 );
            ep->gw->asyncStack->asyncSend (rqnt, ep->gw->handle);
            return ;
        }
        else if ( rqnt->getResponseCode() == ReasonUnsupportedSignal )
        {
            if ( ep->call != 0 && ep->call->endpoint[0].connectionId[0] != 0 )
            {
                MgcDeleteConnectionAgent dlcx (ep->epId.convertString());
                dlcx.setCallId (ep->call->callIdentifier);
                dlcx.setConnectionId (ep->call->endpoint[0].connectionId);
                assert( ep->gw != 0 );
                ep->gw->asyncStack->asyncSend (dlcx, ep->gw->handle);
                cpLog(LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
                if ( ep->call != 0 )
                {
                    callDataInit(ep->call);
                    ep->call = 0;
                }
                cpStateChange (ep, CallStateIdle);
                cpProcessEvent (ep, CallEventRestart);
                return ;
            }
        }
    }

    // save the response code for secondary CA to use if primary down
    responseCode = rqnt->getResponseCode();

    cpProcessEvent (ep, CallEventResponse);
}    // rqntResponse


void
auepResponse (Endpoint* ep, Sptr < MgcAuditEndpoint > auep)
{
    cpProcessEvent (ep, CallEventResponse);
}    // auepResponse


// tbd: Move cpStackEvent() to callProcessing.cxx
void
cpStackEvent (Endpoint* ep, Sptr < MgcCommand > cmd)
{
    cpLog (LOG_DEBUG, "%s cpStackEvent", ep->epId.logData());

    switch (cmd->getType())
    {
        case MgcEndpointConfigurationType:
        {
            cpLog (LOG_ERR, "Unsupported EndpointConfiguration Command");
        }
        break;
        case MgcCreateConnectionType:
        {
            Sptr < MgcCreateConnection > crcxPtr;
            crcxResponse (ep, crcxPtr.dynamicCast(cmd));
        }
        break;
        case MgcModifyConnectionType:
        {
            Sptr < MgcModifyConnection > mdcxPtr;
            mdcxResponse (ep, mdcxPtr.dynamicCast(cmd));
        }
        break;
        case MgcDeleteConnectionAgentType:
        {
            Sptr < MgcDeleteConnectionAgent > dlcxPtr;
            dlcxResponse (ep, dlcxPtr.dynamicCast(cmd));
        }
        break;
        case MgcNotificationRequestType:
        {
            Sptr < MgcNotificationRequest > rqntPtr;
            rqntResponse (ep, rqntPtr.dynamicCast(cmd));
        }
        break;
        case MgcNotifyType:
        {
            cpLog (LOG_DEBUG, "NTFY");
            Sptr < MgcNotify > notifyPtr;
            decodeNotify (ep, notifyPtr.dynamicCast(cmd));
        }
        break;
        case MgcAuditEndpointType:
        {
            //            Sptr<MgcAuditEndpoint> auepPtr;
            // 	      auepResponse (ep, auepPtr.dynamicCast(cmd));
        }

        break;
        case MgcAuditConnectionType:
        {
            // tbd:            Sptr<MgcAuditConnection> aucxPtr;
            //                 aucxResponse (ep, aucxPtr.dynamicCast(cmd));
        }

        break;
        case MgcRestartInProgressType:
        {
            cpLog (LOG_DEBUG, "RSIP");
            Sptr < MgcRestartInProgress > rsipPtr;
            decodeRestartInProgress (ep, rsipPtr.dynamicCast(cmd));
        }
        break;
        case MgcDeleteConnectionGwType:
        {
            cpLog (LOG_DEBUG, "DLCX FOR ENDPOINT %s\n", ep->epId.logData());
            Sptr < MgcDeleteConnectionGw > dlcxGwPtr;
            decodeDeleteConnectionGw (ep, dlcxGwPtr.dynamicCast(cmd));
        }
        break;
        default:
        {
            cpLog (LOG_ERR, "Invalid Command %d from gateway", cmd->getType());
        }
    }
}    /* cpStackEvent */


///
void addRequestOrSignalEvent( MgcCommand* cmd, RequestOrSignalEvent e )
{
    // this is an incomplete mapping
    // packages/events can be added and mapped as needed
    switch ( e )
    {
#ifndef NCS_PROFILE
        case LinePackageRequestOnHookTransition:
        cmd->addRequestedEvents( MgcLinePackage::requestOnHookTransition() );
        break;
        case LinePackageRequestOffHookTransition:
        cmd->addRequestedEvents( MgcLinePackage::requestOffHookTransition() );
        break;
        case LinePackageRequestFlashHook:
        cmd->addRequestedEvents( MgcLinePackage::requestFlashHook() );
        break;
        case DTMFPackageRequestWildcardMatch:
        cmd->addRequestedEvents( MgcDTMFPackage::requestWildcardMatch() );
        break;
        case GenericPackageSignalRingBackToneOn:
        cmd->addSignalRequests( MgcGenericPackage::signalRingBackToneOn() );
        break;
        case LinePackageSignalDialToneOn:
        cmd->addSignalRequests( MgcLinePackage::signalDialToneOn() );
        break;
        case LinePackageSignalRingingOn:
        cmd->addSignalRequests( MgcLinePackage::signalRingingOn() );
        break;
        case LinePackageSignalBusyToneOn:
        cmd->addSignalRequests( MgcLinePackage::signalBusyToneOn() );
        break;
        case LinePackageCallWaitingTone:
        cmd->addSignalRequests( MgcLinePackage::signalCallWaitingTone() );
        // cmd->addSignalRequests( MgcLinePackage::signalCallWaitingTone1() );
        break;
        case LinePackageSignalErrorTone:
        cmd->addSignalRequests( MgcLinePackage::signalErrorTone() );
        break;
        case DTMFPackageSignalDTMF_0:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_0() );
        break;
        case DTMFPackageSignalDTMF_1:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_1() );
        break;
        case DTMFPackageSignalDTMF_2:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_2() );
        break;
        case DTMFPackageSignalDTMF_3:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_3() );
        break;
        case DTMFPackageSignalDTMF_4:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_4() );
        break;
        case DTMFPackageSignalDTMF_5:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_5() );
        break;
        case DTMFPackageSignalDTMF_6:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_6() );
        break;
        case DTMFPackageSignalDTMF_7:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_7() );
        break;
        case DTMFPackageSignalDTMF_8:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_8() );
        break;
        case DTMFPackageSignalDTMF_9:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_9() );
        break;
        case DTMFPackageSignalDTMF_Hash:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_Hash() );
        break;
        case DTMFPackageSignalDTMF_Star:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_Star() );
        break;
        case DTMFPackageSignalDTMF_A:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_A() );
        break;
        case DTMFPackageSignalDTMF_B:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_B() );
        break;
        case DTMFPackageSignalDTMF_C:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_C() );
        break;
        case DTMFPackageSignalDTMF_D:
        cmd->addSignalRequests( MgcDTMFPackage::signalDTMF_D() );
        break;
#else  // NCS_PROFILE
        case LinePackageRequestOnHookTransition:
        cmd->addRequestedEvents( MgcNCSLinePackage::requestOnHookTransition() );
        break;
        case LinePackageRequestOffHookTransition:
        cmd->addRequestedEvents( MgcNCSLinePackage::requestOffHookTransition() );
        break;
        case LinePackageRequestFlashHook:
        cmd->addRequestedEvents( MgcNCSLinePackage::requestFlashHook() );
        break;
        case DTMFPackageRequestWildcardMatch:
        cmd->addRequestedEvents( MgcNCSLinePackage::requestWildcardMatch() );
        break;
        case GenericPackageSignalRingBackToneOn:
        cmd->addSignalRequests( MgcNCSLinePackage::signalRingBackTone() );
        break;
        case LinePackageSignalDialToneOn:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDialTone() );
        break;
        case LinePackageSignalRingingOn:
        cmd->addSignalRequests( MgcNCSLinePackage::signalRinging() );
        break;
        case LinePackageSignalBusyToneOn:
        cmd->addSignalRequests( MgcNCSLinePackage::signalBusyTone() );
        break;
        case LinePackageCallWaitingTone:
        cmd->addSignalRequests( MgcNCSLinePackage::signalCallWaitingTone1() );
        break;
        case LinePackageSignalErrorTone:
        cmd->addSignalRequests( MgcNCSLinePackage::signalReorderTone() );
        break;
        case DTMFPackageSignalDTMF_0:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_0() );
        break;
        case DTMFPackageSignalDTMF_1:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_1() );
        break;
        case DTMFPackageSignalDTMF_2:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_2() );
        break;
        case DTMFPackageSignalDTMF_3:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_3() );
        break;
        case DTMFPackageSignalDTMF_4:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_4() );
        break;
        case DTMFPackageSignalDTMF_5:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_5() );
        break;
        case DTMFPackageSignalDTMF_6:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_6() );
        break;
        case DTMFPackageSignalDTMF_7:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_7() );
        break;
        case DTMFPackageSignalDTMF_8:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_8() );
        break;
        case DTMFPackageSignalDTMF_9:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_9() );
        break;
        case DTMFPackageSignalDTMF_Hash:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_Hash() );
        break;
        case DTMFPackageSignalDTMF_Star:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_Star() );
        break;
        case DTMFPackageSignalDTMF_A:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_A() );
        break;
        case DTMFPackageSignalDTMF_B:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_B() );
        break;
        case DTMFPackageSignalDTMF_C:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_C() );
        break;
        case DTMFPackageSignalDTMF_D:
        cmd->addSignalRequests( MgcNCSLinePackage::signalDTMF_D() );
        break;
#endif
        default:
        break;
    }

    return ;
}
