/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const sipProxy_cxx_Version 
    = "$Id: sipProxy.cxx,v 1.66 2002/03/20 04:32:47 bko Exp $";


#include "sipProxy.hxx"
#include "global.h"

#include <stdio.h>
#include "Sdp2Session.hxx"

#include "cpLog.h"
#include "potStateEngine.hxx"
#include "callProcessing.hxx"
#include "SipMsg.hxx"
#include "SipVia.hxx"
#include "InviteMsg.hxx"
#include "StatusMsg.hxx"
#include "AckMsg.hxx"
#include "ByeMsg.hxx"
#include "CancelMsg.hxx"
#include "OptionsMsg.hxx"
#include "InfoMsg.hxx"
#include "SipUrl.hxx"
#include "Sdp2Connection.hxx"
#include "Sdp2Media.hxx"
#include "Sdp2Session.hxx"
#include "SipContentData.hxx"
#include "SipSdp.hxx"
#include "SipContact.hxx"
#include "RegisterMsg.hxx"
#include "TransferMsg.hxx"
#include "SipCSeq.hxx"
#include "SipRoute.hxx"
#include "SipTransferTo.hxx"

bool processSipEventSendInvite(Endpoint*);
bool processSipEventSendTrying(Endpoint*);
bool processSipEventSendRinging(Endpoint*);
bool processSipEventSendAnswer(Endpoint*);
bool processSipEventSendAck(Endpoint*);
bool processSipEventSendBusy(Endpoint*);
bool processSipEventSendByeOrCancel(Endpoint*);
bool processSipEventSendByeResponse(Endpoint*);
bool processSipEventSendCancelResponse(Endpoint*);
bool processSipEventSendInfo(Endpoint*);
bool processSipEventSendInfoResponse(Endpoint*);
bool processSipEventSendRegister(Endpoint*);
bool processSipEventSendHold(Endpoint*);
bool processSipEventSendXfer(Endpoint*);
bool processSipEventSendConf(Endpoint*);

bool processSipEventReceiveInvite(Endpoint*);
bool processSipEventReceiveTrying(Endpoint*);
bool processSipEventReceiveRinging(Endpoint*);
bool processSipEventReceiveAnswer(Endpoint*);
bool processSipEventReceiveTransfer(Endpoint*);
bool processSipEventReceiveTransferResponse(Endpoint*);
bool processSipEventReceiveRedirect(Endpoint*);
bool processSipEventReceiveBusy(Endpoint*);
bool processSipEventReceiveAck(Endpoint*);
bool processSipEventReceiveBye(Endpoint*);
bool processSipEventReceiveByeOrCancelResponse(Endpoint*);
bool processSipEventReceiveCancel(Endpoint*);
bool processSipEventReceiveInfo(Endpoint*);
bool processSipEventReceiveInfoResponse(Endpoint*);

void processInviteMsg(Endpoint*, SipMsg*, InviteMsg*);
void processXferMsg(Endpoint*, SipMsg*, TransferMsg*);
void processAckMsg(Endpoint*, AckMsg*);
void processByeMsg(Endpoint*, SipMsg*, ByeMsg*);
void processCancelMsg(Endpoint*, SipMsg*, CancelMsg*);
void processOptionsMsg(Endpoint*, OptionsMsg*);
void processInfoMsg(Endpoint*, SipMsg*, InfoMsg*);
void processStatusMsg(Endpoint*, SipMsg*, StatusMsg*);

Vocal::SDP::SdpSession
convert (SdpSession x)
{
    Vocal::SDP::SdpSession y;

    y.decode(x.encode());

    return y;
}

SdpSession 
convert(Vocal::SDP::SdpSession x)
{
    LocalScopeAllocator l;
    SdpSession y;
    y.decode(x.encode().getData(l));
    return y;
}

/// get the size of route list
int sizeRouteList( vector < SipRoute* > * routeList )
{
    int i = 0;

    if ( routeList != 0 )
    {
        vector < SipRoute* > ::iterator iter;
        iter = routeList->begin();
        while (iter != routeList->end())
        {
            if (*iter)
            {
//                cout << ((*iter)->encode()).logData() << endl;
                cpLog ( LOG_DEBUG, ((*iter)->encode()).logData() );
                i++;
            }
            ++iter;
        }
    }

    return ( i );
}

/// print out route list
void printRouteList( vector < SipRoute* > * routeList )
{
    if ( routeList != 0 )
    {
        vector < SipRoute* > ::iterator iter;
        iter = routeList->begin();
        while (iter != routeList->end())
        {
            if (*iter)
            {
//                cout << ((*iter)->encode()).logData() << endl;
                cpLog ( LOG_DEBUG, ((*iter)->encode()).logData() );
            }
            ++iter;
        }
    }
}

/// copy over, reverting the order if not sameOrder, the route list from recordrouteList into routeListSave
void saveRouteList( vector <SipRoute*>* routeListSave, 
		    SipRecordRouteList recordrouteList, 
		    bool sameOrder )
{
    if ( routeListSave != 0 )
    {
        SipRecordRouteList::iterator iter = recordrouteList.begin();
        while ( iter != recordrouteList.end() )
        {
            SipRoute* route = new SipRoute;
            assert( route != 0 );
            route->setUrl( (*iter)->getUrl() );
            if ( !sameOrder )
            {
                routeListSave->insert( routeListSave->begin(), route );
            }
            else
            {
                routeListSave->push_back( route );
            }
            iter++;
        }
    }
}

// delete all SipRoute* nodes in routeList and free memory associated with them
void clearRouteList( vector < SipRoute* > * routeList )
{
    if ( routeList != 0 )
    {
        vector < SipRoute* > ::iterator iter;
        iter = routeList->begin();
        while (iter != routeList->end())
        {
            if (*iter)
            {
                delete (*iter);
            }
            ++iter;
        }
        routeList->clear();
    }
}


// Change MGCP address format to SIP address format
string translate2sip (string user, string host)
{
    string address = "sip:" + user + "@" + host;

    return address;
}

// Change SIP address format to MGCP address format
string translate2mgcp (string before)
{
    string after = before;

    if (after.find("sip:") != string::npos)
    {

        // Get rid of "sip:" if found
        // after.replace(after.find("sip:"), 4, "");
        after.erase(0, 4);
    }

    // Get rid of ":5060" if found
    unsigned int colon = after.find(":");
    if (colon != string::npos)
    {
        // after.replace(after.find(":5060"), 5, "");
        after.erase(colon, after.length() - colon);
    }

    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "before translating, epId = " << before << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "after translating, epId = " << after << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    return after;
}


/*****************************************************************
  This is where all outgoing sip messages are sent out.
  Most originated from the MGCP Call Agent, but some are
  immediate responses to incoming SIP messages.
*****************************************************************/

void
sipEventSend (Endpoint* ep, SipEventSend event)
{
    assert( ep != 0 );
    cpLog (LOG_DEBUG, "sipEventSend: %s process event %s in state %s",
           ep->epId.logData(),
           SipEventSendString[event],
           CallStateString[ep->state]);

    switch (event)
    {
        case SipEventSendInvite:
        {
            if (!processSipEventSendInvite(ep))
               return;
        }
        break;
        case SipEventSendTrying:
        {
            if (!processSipEventSendTrying(ep))
               return;
        }
        break;
        case SipEventSendRinging:
        {
            if (!processSipEventSendRinging(ep))
               return;
        }
        break;
        case SipEventSendAnswer:
        {
            if (!processSipEventSendAnswer(ep))
               return;
        }
        break;
        case SipEventSendAck:
        {
            if (!processSipEventSendAck(ep))
               return;
        }
        break;
        case SipEventSendBusy:
        {
            if (!processSipEventSendBusy(ep))
               return;
        }
        break;
        case SipEventSendBye:
        case SipEventSendCancel:
        {
            if (!processSipEventSendByeOrCancel(ep))
               return;
        }
        break;
        case SipEventSendByeResponse:
        {
            if (!processSipEventSendByeResponse(ep))
               return;
        }
        break;
        case SipEventSendCancelResponse:
        {
            if (!processSipEventSendCancelResponse(ep))
               return;
        }
        break;
        case SipEventSendInfo:
        {
            if (!processSipEventSendInfo(ep))
               return;
        }
        break;
        case SipEventSendInfoResponse:
        {
            if (!processSipEventSendInfoResponse(ep))
               return;
        }
        break;
        case SipEventSendRegister:
        {
            if (!processSipEventSendRegister(ep))
               return;
        }
        break;
        case SipEventSendHold:
        {
            if (!processSipEventSendHold(ep))
               return;
        }
        break;
        case SipEventSendXfer:
        {
            if (!processSipEventSendXfer(ep))
               return;
        }
        break;
        case SipEventSendConf:
        {
            if (!processSipEventSendConf(ep))
               return;
        }

        break;
        default:
        {
            cpLog (LOG_ERR, "Unknown Sip event %d", event);
        }
    } // end of switch (event)
}    // sipEventSend


// Note that this switch statement could probably be organized to better
// fit the SIP message heirarchy (sp?).
// Perhaps each case can be called directly after finding out what type of
// SIP message was received?

/*****************************************************************
   This is where all incoming sip messages are passed along
   to the MGCP Call Agent.
*****************************************************************/
void
sipEventReceive (Endpoint* ep, SipEventReceive event)
{
    assert( ep != 0 );
    cpLog (LOG_DEBUG, "sipEventReceive: %s process event %s in state %s",
           ep->epId.logData(),
           SipEventReceiveString[event],
           CallStateString[ep->state]);
    switch (event)
    {
        case SipEventReceiveInvite:
        {
            if (!processSipEventReceiveInvite(ep))
               return;
        }

        break;
        case SipEventReceiveTrying:
        {
            if (!processSipEventReceiveTrying(ep))
               return;
        }

        break;
        case SipEventReceiveRinging:
        {
            if (!processSipEventReceiveRinging(ep))
               return;
        }

        break;
        case SipEventReceiveAnswer:
        {
            if (!processSipEventReceiveAnswer(ep))
               return;
        }
        break;
        case SipEventReceiveTransfer:
        {
            if (!processSipEventReceiveTransfer(ep))
               return;
        }
        break;
        case SipEventReceiveTransferResponse:
        {
            if (!processSipEventReceiveTransferResponse(ep))
               return;
        }
        break;
        case SipEventReceiveRedirect:
        {
            if (!processSipEventReceiveRedirect(ep))
               return;
        }

        break;
        case SipEventReceiveBusy:
        {
            if (!processSipEventReceiveBusy(ep))
               return;
        }

        break;
        case SipEventReceiveAck:
        {
            if (!processSipEventReceiveAck(ep))
               return;
        }
        break;
        case SipEventReceiveBye:
        {
            if (!processSipEventReceiveBye(ep))
               return;
        }
        break;
        case SipEventReceiveByeResponse:
        case SipEventReceiveCancelResponse:
        {
            if (!processSipEventReceiveByeOrCancelResponse(ep))
               return;
        }

        break;
        case SipEventReceiveCancel:
        {
            if (!processSipEventReceiveCancel(ep))
               return;
        }
        break;
        case SipEventReceiveInfo:
        {
            if (!processSipEventReceiveInfo(ep))
               return;
        }
        break;
        case SipEventReceiveInfoResponse:
        {
            if (!processSipEventReceiveInfoResponse(ep))
               return;
        }
        break;
        case SipEventReceiveRegisterResponse:
        {
            /* PROCESSING MOVED TO RECEIVING POINT
                     cout << "*****************" << endl;
                     cout << "RECEIVED A RESPONSE TO THE REGISTER MESSAGE" << endl;
                     cout << "*****************" << endl;
                     assert(ep);
                     if (ep->state == CallStateInit)
                     {
                        break;
                     }
                     StatusMsg* statusMsg= dynamic_cast<StatusMsg*>(ep->sipMsg);
                     if (!statusMsg )
                     {
                         break;
                     }
                     // Data expires
                     int expires = statusMsg->getExpires().getDelta();
                     cout << "expires = " << expires << endl;
                     cout << "ca.registerRetrans = " << ca.registerRetrans << endl;
                     if (expires > ca.registerRetrans)
                     {
                         expires = ca.registerRetrans;
                     }
                     QueueEventData* ed = newQueueEventData ();
                     ed->type = TimeoutEvent;
                     ed->ep = ep;
                     ed->featureId = FeatureEventSipRegister;
                     // Should the next REGISTER message to be sent a little sooner
                     // to eliminate even a small window of being un-registered?
                     ep->iq->addDelayMs(ed, expires * 1000);
            */
        }
        break;
        default:
        {
            cpLog (LOG_ERR, "Unknown Sip event %d", event);
        }
    }  // end of switch (event)
}  // sipEventReceive


Endpoint* getEpId (SipUrl url)
{
    string epId = url.getNameAddr().convertString();

    epId = translate2mgcp (epId);

    EndpointMap::iterator p = epMap.find (epId);

    if (p == epMap.end())
    {
        for (int i = 0; i < 10; i++)
        {
            string user = url.getUserValue().convertString();

            /*
                        epId = eps[i].epId;
                        if (epId.find(user) != string::npos)
                        {
                            cpLog (LOG_DEBUG, "USING ONLY USER NAME ... ignoring host/IP");
                            return &eps[i];
                        }
            */
            map < string, Endpoint* > ::iterator p = dnMap.find(user);
            if (p != dnMap.end())
            {
                return p->second;
            }

        }

        // tbd: check for wild cards
        cpLog (LOG_WARNING, "Unknown endpoint id %s", epId.c_str());
        return 0;
    }

    cpLog (LOG_DEBUG, "%s SIP stack event", epId.c_str());

    return p->second;
}


Endpoint* getToEpId(SipMsg* msg)
{
    // This will work for SIP to SIP calls (no redirection)
    SipCommand* x = dynamic_cast < SipCommand * > (msg);
    if ( !x )
    {
        return ( NULL );
    }

    SipRequestLine requestLine = x->getRequestLine();
    Endpoint* ep = 0;

    Sptr <BaseUrl> baseUrl = requestLine.getUrl(); 
    if (baseUrl.getPtr() != 0) 
    { 
	if (baseUrl->getType() == SIP_URL) 
	{ 
	    Sptr<SipUrl> sipUrl; 
	    sipUrl.dynamicCast(baseUrl); 
	    ep = getEpId( *sipUrl ); 
	} 
    } 

    //  Endpoint* ep = getEpId(requestLine.getUrl());
    return ep;
}


Endpoint* getFromEpId(SipMsg* msg)
{
    if ( !msg )
    {
        return ( NULL );
    }

    SipFrom from = msg->getFrom();
    Endpoint* ep = 0;
    
    Sptr <BaseUrl> baseUrl = from.getUrl(); 
    if (baseUrl.getPtr() != 0) 
    { 
	if (baseUrl->getType() == SIP_URL) 
	{ 
	    Sptr<SipUrl> sipUrl; 
	    sipUrl.dynamicCast(baseUrl); 
            ep = getEpId( *sipUrl );
	} 
    } 

    // Endpoint* ep = getEpId(from.getUrl());
    return ep;
}


// Check if Call Waiting is allowed.
// If so, return true.
// Otherwise return false.
bool DoCallWaiting(Endpoint* ep, SipMsg* msg)
{
    assert( ep != 0 );
    if ( ep->state != CallStateConnected )
    {
        return false;
    }
    if ( !(ep->call) )
    {
        return false;
    }
    if (ep->call->blockCallWaiting == true )
    {
        return false;
    }
    if (ep->sipMsgHeld != NULL)
    {
        return false;
    }


    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (msg);
    if ( inviteMsg )
    {
        cout << "****** 5 ********" << endl;  // *5*
        cout << "SEND A SIP 180 INVITE RESPONSE FOR CALL WAITING" << endl;
        cout << "****** 5 ********" << endl;  // *5*
        StatusMsg status( *inviteMsg, 180 /*ringing*/ );
        assert( sipProxyStack );
        sipProxyStack->sendReply(status);
    }

    ep->heldCall = newCallData(ep);
    if ( ep->heldCall == 0 )
    {
        cpLog( LOG_ERR, "Call Data could not be allocated");
        return ( false );
    }

    if ( ep->sipMsgHeld != 0 )
    {
        delete( ep->sipMsgHeld );
        ep->sipMsgHeld = 0;
    }
    ep->sipMsgHeld = msg;

    if ( !msg )
    {
        return ( false );
    }

    saveRouteList( ep->sipMsgHeldRouteList, (msg->getrecordrouteList()), 1 );
    int numContact = msg->getNumContact();
    if ( numContact )
    {
        SipContact contact = msg->getContact( numContact - 1 );
        SipRoute* route = new SipRoute;
        assert( route != 0 );
        route->setUrl( contact.getUrl() );
        assert( ep->sipMsgHeldRouteList != 0 );
        ep->sipMsgHeldRouteList->push_back( route );
    }
    printRouteList( ep->sipMsgHeldRouteList );

    // moved up a few line in order to return false for not being
    // able to allocate call data for the held call so that the calling
    // function free the msg upon return false

    //    ep->heldCall = newCallData(ep);
    //    if (ep->heldCall)
    //    {
    ep->heldCall->callWaiting = true;
    // ep->heldCall->callWaiting is also true only for the very first time.
    ep->heldCall->callWaited = true;
    ep->heldCall->endpoint[0].id = ep->id;

    // For Call Waiting, if multiple RTP paths are allowed, do this.
    strcpy( ep->heldCall->callIdentifier, ep->call->callIdentifier );
    ep->heldCall->callIdentifier[CALL_ID_LEN_MAX - 1] = 0;
    strcpy( ep->heldCall->endpoint[0].connectionId, ep->call->endpoint[0].connectionId );
    ep->heldCall->endpoint[0].connectionId[CONNECTION_ID_LEN_MAX - 1] = 0;
    ep->heldCall->endpoint[0].localConnection = ep->call->endpoint[0].localConnection;
    // set the Sdp info for the far end
    Sptr<SipContentData> sipContent = msg->getContentData(0);
    Sptr<SipSdp> sipsdp;
    sipsdp.dynamicCast(sipContent);
    if ( sipsdp == 0 )
    {
        cpLog( LOG_DEBUG, "DoCallWaiting(): No Sdp info in the sip message.");
        return ( false );
    }

    SdpSession sdp = convert(sipsdp->getSdpDescriptor());
    ep->heldCall->endpoint[1].localConnection = sdp;

    sendInternalEvent( ep, CallEventFeature, NULL, FeatureEventCallWaiting);
    return true;
    //    }
    //    cpLog( LOG_ERR, "Call Data could not be allocated");

    //    return false;
}


/*****************************************************************
   This is where all incoming sip messages are identified and sent
   to the appropriate CA procedure for further processing.
*****************************************************************/
void sipStackEvent (SipMsg* msg)
{
    Endpoint* ep = 0;

    if ( !msg )
    {
        cpLog( LOG_DEBUG, "sipStackEvent():  msg == 0 ");
        return ;
    }

    cout << "GOT A SIP MESSAGE!!!" << endl;
    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (msg);
    if ( inviteMsg )
    {
        cout << "GOT A INVITE SIP MESSAGE!!!" << endl;
        ep = getToEpId(inviteMsg);
        processInviteMsg(ep, msg, inviteMsg);
        return ;
    } // inviteMsg


    TransferMsg* xferMsg = dynamic_cast < TransferMsg* > (msg);
    if ( xferMsg )
    {
        cout << "GOT AN TRANSFER SIP MESSAGE!!!" << endl;
        ep = getToEpId(xferMsg);
        processXferMsg(ep, msg, xferMsg);
        return ;
    } // xferMsg


    AckMsg* ackMsg = dynamic_cast < AckMsg* > (msg);
    if ( ackMsg )
    {
        cout << "GOT AN ACK SIP MESSAGE!!!" << endl;
        ep = getToEpId(ackMsg);
        processAckMsg(ep, ackMsg);
        return ;
    } // ackMsg

    ByeMsg* byeMsg = dynamic_cast < ByeMsg* > (msg);
    if ( byeMsg )
    {
        cout << "GOT A BYE SIP MESSAGE!!!" << endl;
        ep = getToEpId(byeMsg);
        processByeMsg(ep ,msg, byeMsg);

        return ;
    } // byeMsg

    CancelMsg* cancelMsg = dynamic_cast < CancelMsg* > (msg);
    if ( cancelMsg )
    {
        cout << "GOT A CANCEL SIP MESSAGE!!!" << endl;
        ep = getToEpId(cancelMsg);
        processCancelMsg(ep, msg, cancelMsg);

        return ;
    } // cancelMsg

    OptionsMsg* optionsMsg = dynamic_cast < OptionsMsg* > (msg);
    if ( optionsMsg )
    {
        cout << "GOT AN OPTIONS SIP MESSAGE!!!" << endl;
        ep = getToEpId(optionsMsg);
        processOptionsMsg(ep, optionsMsg);

        return ;
    } // optionsMsg

    InfoMsg* infoMsg = dynamic_cast < InfoMsg* > (msg);
    if ( infoMsg )
    {
        cout << "GOT AN INFO SIP MESSAGE!!!" << endl;
        ep = getToEpId(infoMsg);
        processInfoMsg(ep, msg, infoMsg);

        return ;
    } // infoMsg

    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (msg);
    if ( statusMsg )
    {
        cout << "GOT A STATUS SIP MESSAGE!!!" << endl;
        ep = getFromEpId(msg);
        processStatusMsg(ep, msg, statusMsg);
        return ;
    } // statusMsg
    cpLog (LOG_WARNING, "Unknown SIP message");

    // free the unknown sip message if not returned yet
    delete( msg );
    return ;
}


// procedures for SipEventSend
bool processSipEventSendInvite(Endpoint* ep)
{
     cout << "****** 1 ********" << endl;  // *1*
     cout << "SEND A SIP INVITE MESSAGE" << endl;
     cout << "****** 1 ********" << endl;  // *1*

     Sptr<SipUrl> toUrl = new SipUrl;
     if ( !ep->call )
     {
         cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
         return false;
     }

     //    toUrl.setPhoneNumber( ep->call->callDigits );
     toUrl->setUserValue( ep->call->callDigits, "phone" );
     toUrl->setHost( ca.sipProxyHost );
     toUrl->setPort( ca.sipProxyPort );
     // toUrl.setUserParm ("phone");


     InviteMsg* inviteMessage = 0;


     if ( (ep->heldCall != NULL) && (ep->sipMsgHeld == 0) )
     {
         // If there is a held call, save the previous sipMsg
         // in sipMsgHeld, before overwriting the field with new active Invite
         // this new invite going out should have the same call id as the held call
         // because it is doing transfering


         if ( !ep->sipMsg )
         {
             cpLog( LOG_DEBUG, "The endpoint ep->sipMsg == 0 ");
             return false;
         }
	 //  inviteMessage = new InviteMsg(toUrl);
	 inviteMessage = new InviteMsg(toUrl, ca.sipMyPort);
         assert( inviteMessage != 0 );

         inviteMessage->setCallId( ep->sipMsg->getCallId() );

         SipTo to (toUrl);
         inviteMessage->setTo( to );

         //SipUrl fromUrl;
	 Sptr<SipUrl> fromUrl = new SipUrl;
         //fromUrl.setPhoneNumber( ep->dn );
	 fromUrl->setUserValue( ep->dn, "phone" );
         fromUrl->setHost( ca.sipMyHost );
         fromUrl->setPort( ca.sipMyPort );

         SipFrom from (fromUrl);
         inviteMessage->setFrom( from );


         if ( ep->sipMsgHeld != 0 )
         {
             delete( ep->sipMsgHeld );
             ep->sipMsgHeld = 0;
         }
         ep->sipMsgHeld = ep->sipMsg;
         ep->sipMsg = 0;

         // swap the sipmsg route list as well
         vector < SipRoute* > * temp = ep->sipMsgHeldRouteList;
         ep->sipMsgHeldRouteList = ep->sipMsgRouteList;
         clearRouteList( temp );
         ep->sipMsgRouteList = temp;

     }
     else if (( ep->sipMsg != 0 ) || ( (ep->call != 0) && (ep->sipMsg == 0) && (ep->sipMsgHeld != 0) && ( ep->heldCall == 0 ) ))
     {
         // this can happen when transferring party B flashes the second time

         if ( ep->sipMsgHeld != 0 )
         {
             if ( ep->heldCall == 0 )
             {
                 // No ep->heldCall object, and there is still a ep->sipMsgHeld
                 // swap the held pointer to the active one
                 // also swap the held route list to the active one
                 if ( ep->sipMsg != 0 )
                 {
                     delete( ep->sipMsg );
                     ep->sipMsg = 0;
                 }

                 ep->sipMsg = ep->sipMsgHeld;
                 ep->sipMsgHeld = 0;
                 clearRouteList( ep->sipMsgRouteList );

                 vector < SipRoute* > * temp = ep->sipMsgRouteList;
                 ep->sipMsgRouteList = ep->sipMsgHeldRouteList;
                 ep->sipMsgHeldRouteList = temp;
             }
             else if ( ep->heldCall != 0 )
             {
                 // there is both a heldCall object and a sipMsgHeld pointer
                 // only swap them with active call, also swap route list
                 SipMsg* tempSipMsg = ep->sipMsg;
                 vector < SipRoute* > * tempRouteList = ep->sipMsgRouteList;

                 ep->sipMsg = ep->sipMsgHeld;
                 ep->sipMsgRouteList = ep->sipMsgHeldRouteList;
                 ep->sipMsgHeld = tempSipMsg;
                 ep->sipMsgHeldRouteList = tempRouteList;
             }
         }

         // this occurs when transferring party (B) pressed flash button
         // the second time, after sending a hold Invite to the being-transferred
         // party (A) and got a 200 back from A for the hold Invite
         // this invite should be based on the 200 status message
         // for the previous hold Invite to A
         // and be sent to A to put B back talking with A
         // also need to set the route list from stored information

         StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
         int statusCode = 0;

         if ( statusMsg != 0 )
         {
             statusCode = statusMsg->getStatusLine().getStatusCode();
         }
         else
         {
             cpLog( LOG_ERR, "The previous status message was NOT found !!");
             return true;
//                    break;
         }

         if ( statusCode == 200 )
         {
             inviteMessage = new InviteMsg( *statusMsg );
             assert( inviteMessage != 0 );

             inviteMessage->setCallId( statusMsg->getCallId() );

             SipCSeq cseq = statusMsg->getCSeq();
             int num = cseq.getCSeqData().convertInt();
             num++;
             cseq.setCSeq( num );
             inviteMessage->setCSeq( cseq );

             // set the route list info from the 200 for the very first valid invite (sdp!=0)
             int i = sizeRouteList( ep->sipMsgRouteList );
             if (i > 0)
             {
                 inviteMessage->setRouteList( *(ep->sipMsgRouteList) );

                 SipRoute siproute = inviteMessage->getRoute(0);
                 inviteMessage->removeRoute(0);

                 SipRequestLine reqLine = inviteMessage->getRequestLine();
                 reqLine.setUrl( siproute.getUrl() );
                 inviteMessage->setRequestLine( reqLine );
             }

             //SipUrl reInviteToUrl = (inviteMessage->getTo()).getUrl();
	     Sptr<BaseUrl> baseUrl = (inviteMessage->getTo()).getUrl();
	     Sptr<SipUrl> reInviteToUrl;
	     reInviteToUrl.dynamicCast( baseUrl );
	     assert( reInviteToUrl != 0 );
		     
             // update the To Url's user field for A calls B
	     Sptr<BaseUrl> statusBaseToUrl = (statusMsg->getTo()).getUrl();  
	     Sptr<SipUrl> sipUrl; 
	     sipUrl.dynamicCast(statusBaseToUrl); 
	     assert( sipUrl != 0 );

	     //   if ((statusMsg->getTo()).getUrl().getUserParm() == "phone")
	     if ( sipUrl->getUserParam() == "phone" )
             {
	         //reInviteToUrl.setPhoneNumber(( statusMsg->getTo()).getUrl().getUser() );
	         reInviteToUrl->setUserValue( sipUrl->getUserValue(), "phone" );
                 SipTo reInviteTo( reInviteToUrl );
                 inviteMessage->setTo( reInviteTo );
             }
             else
             {
                 reInviteToUrl->setUser((( statusMsg->getTo()).getUrl())->getUser() );
                 SipTo reInviteTo( reInviteToUrl );
                 inviteMessage->setTo( reInviteTo );
             }
         }
         else
         {
             cpLog( LOG_ERR, "The status message found was NOT 200 !!");
             return true;
//                    break;
         }
     }
     else // a normal first invite going out
     {
         inviteMessage = new InviteMsg( toUrl );
         assert( inviteMessage != 0 );

         SipTo to (toUrl);
         inviteMessage->setTo( to );

         Sptr<SipUrl> fromUrl = new SipUrl;
         //fromUrl.setPhoneNumber( ep->dn );
	 fromUrl->setUserValue( ep->dn, "phone" );
         fromUrl->setHost( ca.sipMyHost );
         fromUrl->setPort( ca.sipMyPort );

         SipFrom from (fromUrl);
         inviteMessage->setFrom( from );
     }

     // via is default to 5060.  Change it here for now???
     SipVia sipVia = inviteMessage->getVia ();
     sipVia.setprotoVersion( "2.0" );
     sipVia.setHost( Data( ca.sipMyHost ) );
     sipVia.setPort (ca.sipMyPort);
     inviteMessage->flushViaList();
     inviteMessage->setVia (sipVia, 0);

     cout << "####################" << endl;
     cout << "InviteMsg = " << inviteMessage->encode().logData() << endl;
     cout << "####################" << endl;

     if ( ep->sipMsg != 0 )
     {
         delete( ep->sipMsg );
         ep->sipMsg = 0;
     }
     ep->sipMsg = dynamic_cast < SipMsg* > (inviteMessage);

     if ( !ep->call )
     {
         cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
         return false;
     }

     SdpSession sdp = ep->call->endpoint[0].localConnection;
     Sptr<SipSdp> sipSdp;
     sipSdp.dynamicCast (inviteMessage->getContentData(0));
     if (!sipSdp)
     {
         /*
                cout << "ERROR: INVALID SDP was found ... ABORT CALL" << endl;
                giveBusyTone (ep);
                break;
         */

         SipSdp sipSdp;
         sipSdp.setSdpDescriptor( convert(sdp) );
         inviteMessage->setContentData( &sipSdp );
     }
     else
     {
         sipSdp->setSdpDescriptor(convert(sdp));
     }

     // Set Contact: header
     //myUrl.setUser( );
     // SipUrl myUrl(fromEpId);
     Sptr<SipUrl> myUrl = new SipUrl;
     //myUrl.setPhoneNumber( ep->dn );
     myUrl->setUserValue( ep->dn, "phone" );
     myUrl->setHost(ca.sipMyHost);
     myUrl->setPort(ca.sipMyPort);
     SipContact me;
     me.setUrl( myUrl );
     inviteMessage->setNumContact( 0 );    // Clear
     inviteMessage->setContact( me );

     sipProxyStack->sendAsync(*inviteMessage);

     if (ep->call)
     {
         ep->call->ackReceived = false;
     }
    return true;
}

bool processSipEventSendTrying(Endpoint* ep)
{
    cout << "****** 3 ********" << endl;  // *3*
    cout << "SEND A SIP 100 INVITE RESPONSE" << endl;
    cout << "****** 3 ********" << endl;  // *3*

    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
    if (!inviteMsg)
    {
       cpLog (LOG_ERR, "No INVITE message could be found");
       return true;
//       break;
    }
    StatusMsg status( *inviteMsg, 100 /*trying*/ );
    assert( sipProxyStack );
    sipProxyStack->sendReply(status);
    return true;
};

bool processSipEventSendRinging(Endpoint* ep)
{
    cout << "****** 5 ********" << endl;  // *5*
    cout << "SEND A SIP 180 or 183 INVITE RESPONSE" << endl;
    cout << "****** 5 ********" << endl;  // *5*
    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
    if (!inviteMsg)
    {
        cpLog (LOG_ERR, "No INVITE message could be found");
        return true;
//        break;
    }

    if ( strcmp(ca.earlyRtp, "off") )
    {
        // if later on we decide to use 183 with Sdp info for early RTP
        // replace with this line of code to send 183 instead
        //        StatusMsg status( *inviteMsg, 183 /*ringing*/ );
        StatusMsg status( *inviteMsg, 180 /*ringing*/ );

        if ( !ep->call )
        {
            cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
            return false;
        }

        SdpSession sdp = ep->call->endpoint[0].localConnection;
        Sptr<SipSdp> sipSdp;
	sipSdp.dynamicCast (status.getContentData(0));

        // check to see if the previous invite was a hold invite
        // (i.e. if contains sdp=0)
        // if so, send a Hold sdp as well in the 183
        Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
        Sptr<SipSdp> inviteSipSdp;
	inviteSipSdp.dynamicCast (sipContent);
        if (inviteSipSdp != 0)
        {
            SdpSession inviteSdp = convert(inviteSipSdp->getSdpDescriptor());
            if ( inviteSdp.isHold() )
            {
                // set Hold sdp in the 183
                sdp.setHold();
            }
        }

        if (!sipSdp)
        {
            SipSdp sipSdp2;
            sipSdp2.setSdpDescriptor( convert(sdp) );
            status.setContentData( &sipSdp2 );
        }
        else
        {
            sipSdp->setSdpDescriptor( convert(sdp) );
        }

        assert( sipProxyStack );
        sipProxyStack->sendReply(status);
    }
    else
    {
        StatusMsg status( *inviteMsg, 180 /*ringing*/ );
        assert( sipProxyStack );
        sipProxyStack->sendReply(status);
    }
    return true;
}

bool processSipEventSendAnswer(Endpoint* ep)
{
    cout << "****** 7 ********" << endl;  // *7*
    cout << "SEND A SIP 200 INVITE RESPONSE WITH SDP INFO" << endl;
    cout << "****** 7 ********" << endl;  // *7*

    // Call Waiting response to the ringing held call when answered
    if (ep->heldCall)
    {
        if (ep->heldCall->callWaited == true)
        {
            // if (ep->call->ackReceived == false)
            {
                SipMsg* tempSipMsg = ep->sipMsg;
                vector < SipRoute* > * tempRouteList = ep->sipMsgRouteList;

                ep->sipMsg = ep->sipMsgHeld;
                ep->sipMsgRouteList = ep->sipMsgHeldRouteList;
                ep->sipMsgHeld = tempSipMsg;
                ep->sipMsgHeldRouteList = tempRouteList;
            }
        }
    }
    else if (ep->call)
    {
        if (ep->call->callWaiting == true)
        {
            // if (ep->call->ackReceived == false)
            {
                SipMsg* tempSipMsg = ep->sipMsg;
                vector < SipRoute* > * tempRouteList = ep->sipMsgRouteList;

                ep->sipMsg = ep->sipMsgHeld;
                ep->sipMsgRouteList = ep->sipMsgHeldRouteList;
                ep->sipMsgHeld = tempSipMsg;
                ep->sipMsgHeldRouteList = tempRouteList;
            }
        }
    }

    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
    if (!inviteMsg)
    {
        cpLog (LOG_ERR, "No INVITE message could be found either");
        giveBusyTone (ep);
        return true;
  //      break;
    }

    StatusMsg status( *inviteMsg, 200 /*connected*/ );

    if ( !ep->call )
    {
        cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
        return false;
    }

    SdpSession sdp = ep->call->endpoint[0].localConnection;

    // check to see if the invite is a hold invite
    // (i.e. if contains sdp=0)
    // if so, send a Hold sdp as well in the 200
    Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
    Sptr<SipSdp> inviteSipSdp;
    inviteSipSdp.dynamicCast (sipContent);
    if (inviteSipSdp != 0)
    {
        SdpSession inviteSdp = convert(inviteSipSdp->getSdpDescriptor());
        if ( inviteSdp.isHold() )
        {
            // set Hold sdp in the 200
            sdp.setHold();
        }
    }

    
    Sptr<SipSdp> sipSdpPtr;
    sipSdpPtr.dynamicCast( status.getContentData(0) );
    if ( sipSdpPtr == 0 )
    {
        SipSdp sipSdp;
        sipSdp.setSdpDescriptor( convert(sdp) );
        status.setContentData( &sipSdp );
    }
    else 
    {
        sipSdpPtr->setSdpDescriptor(convert(sdp));
    }


    // for the internal trial
    // string contactEpId = translate2sip (ep->dn, ca.sipMyHost);
    // SipUrl contactUrl ( contactEpId );
    Sptr<SipUrl> contactUrl = new SipUrl;
    //contactUrl.setPhoneNumber( ep->dn );
    contactUrl->setUserValue( ep->dn, "phone" );
    contactUrl->setHost( ca.sipMyHost);
    contactUrl->setPort( ca.sipMyPort );
    // contactUrl.setUserParm( "phone" );
    SipContact contact;
    contact.setUrl (contactUrl);
    status.setContact (contact);

    assert( sipProxyStack );
    sipProxyStack->sendReply(status);
    return true;
}

bool processSipEventSendAck(Endpoint* ep)
{
    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
    if (!statusMsg)
    {
        cpLog (LOG_INFO, "No STATUS message could be found");
        return true;
 //       break;
    }
    if ( statusMsg->getStatusLine().getStatusCode() < 200 )
    {
        cpLog (LOG_ERR, "STATUS message was < 200");
        return true;
//        break;
    }

    cout << "****** 9 ********" << endl;  // *9*
    cout << "SEND A SIP ACK MESSAGE" << endl;
    cout << "****** 9 ********" << endl;  // *9*

    AckMsg ack( *statusMsg );

    /* Sending or receiving Bye has already cleaned up the ep->sipMsgRouteList
              and we don't have route list info here now !!!! */
    // only when we get 200 for an invite, we can get a route list
    if ( statusMsg->getStatusLine().getStatusCode() == 200 )
    {
        int i = 0;
        i = sizeRouteList( ep->sipMsgRouteList );
        if ( i > 0 )
        {
            assert( ep->sipMsgRouteList != 0 );
            ack.setRouteList( *(ep->sipMsgRouteList) );

            SipRoute siproute = ack.getRoute(0);
            ack.removeRoute(0);

            SipRequestLine reqLine = ack.getRequestLine();
            reqLine.setUrl( siproute.getUrl() );
            ack.setRequestLine( reqLine );
        }
    }
    // ACKs for status >= 300 is "hop-by-hop".  Not "end-to-end".
    else
    {
        SipRequestLine reqLine = ack.getRequestLine();	
        //SipUrl reqUrl = reqLine.getUrl();
	Sptr<BaseUrl> baseUrl = reqLine.getUrl();
	Sptr<SipUrl> reqUrl;
	reqUrl.dynamicCast( baseUrl );
	if ( reqUrl != 0 )
	{
            reqUrl->setHost( ca.sipProxyHost );
	    reqUrl->setPort( ca.sipProxyPort );
	    reqLine.setUrl( reqUrl );
	    ack.setRequestLine( reqLine );
	}
    }

    assert( sipProxyStack );
    sipProxyStack->sendAsync(ack);
    return true;
}

bool processSipEventSendBusy(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "SEND A SIP 486 INVITE RESPONSE" << endl;
    cout << "*****************" << endl;

    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
    if (!inviteMsg)
    {
        cpLog (LOG_ERR, "No INVITE message could be found");
        return true;
   //     break;
    }
    StatusMsg status( *inviteMsg, 486/*busy*/ );

    // for the internal trial
    // string tempEpId = translate2sip (ep->dn, ca.sipMyHost);
    // SipUrl url ( tempEpId );
    Sptr<SipUrl> url = new SipUrl;
    //url.setPhoneNumber( ep->dn );
    url->setUserValue( ep->dn, "phone");
    url->setHost( ca.sipMyHost );
    url->setPort( ca.sipMyPort );
    // url.setUserParm( "phone" );
    SipContact contact;
    contact.setUrl (url);
    status.setContact (contact);

    assert( sipProxyStack );
    sipProxyStack->sendReply(status);
    return true;
}

bool processSipEventSendByeOrCancel(Endpoint* ep)
{
    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
    if (inviteMsg)
    {
        if ( !ep->call )
        {
            cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
            return false;
        }
        // tempoarily check for BYE/CANCEL here
        if (ep->call->ackReceived == true)
        {
            cpLog (LOG_DEBUG, "INVITE message found");
            cout << "*****************" << endl;
            cout << "SEND A SIP BYE MESSAGE" << endl;
            cout << "*****************" << endl;
            ByeMsg bye( *inviteMsg );

            // for the internal trial
            bye.flushViaList();
            SipVia sipVia;
            sipVia.setHost (ca.sipMyHost);
            sipVia.setPort (ca.sipMyPort);
            bye.setVia (sipVia, 0);

            // set the route list
            int i = 0;
            i = sizeRouteList( ep->sipMsgRouteList );
            if ( i > 0 )
            {
                assert( ep->sipMsgRouteList != 0 );
                bye.setRouteList( *(ep->sipMsgRouteList) );

                SipRoute siproute = bye.getRoute(0);
                bye.removeRoute(0);

                SipRequestLine reqLine = bye.getRequestLine();
                reqLine.setUrl( siproute.getUrl() );
                bye.setRequestLine( reqLine );
            }

            assert( sipProxyStack );
            sipProxyStack->sendAsync(bye);

            // clean up the sipMsg pointer here because this call is byeing
            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }

            clearRouteList( ep->sipMsgRouteList );
            return true;
  //          break;
        }
        else
        {
            cpLog (LOG_DEBUG, "INVITE message found");
            cout << "*****************" << endl;
            cout << "SEND A SIP CANCEL MESSAGE" << endl;
            cout << "*****************" << endl;
            CancelMsg cancel( *inviteMsg );

            // set the route list
            int i = 0;
            i = sizeRouteList( ep->sipMsgRouteList );
            if ( i > 0 )
            {
                assert( ep->sipMsgRouteList != 0 );
                cancel.setRouteList( *(ep->sipMsgRouteList) );

                SipRoute siproute = cancel.getRoute(0);
                cancel.removeRoute(0);

                SipRequestLine reqLine = cancel.getRequestLine();
                reqLine.setUrl( siproute.getUrl() );
                cancel.setRequestLine( reqLine );
            }

            assert( sipProxyStack );
            sipProxyStack->sendAsync(cancel);

            // clean up the sipMsg pointer here because this call is canceling
            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }

            clearRouteList( ep->sipMsgRouteList );

            // check to see if there is a held call,
            // if so, make the held call the active one
            if ( ep->sipMsgHeld != 0 )
            {
                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                }
                ep->sipMsg = ep->sipMsgHeld;
                ep->sipMsgHeld = 0;

                // also swap the route list
                vector < SipRoute* > * temp = ep->sipMsgRouteList;
                ep->sipMsgRouteList = ep->sipMsgHeldRouteList;
                ep->sipMsgHeldRouteList = temp;
            }
            return true;
 //           break;
        }
    }

    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
    if (statusMsg)
    {
        cpLog (LOG_DEBUG, "STATUS message found");
        int status = statusMsg->getStatusLine().getStatusCode();
        if (status < 200)
        {
            cpLog (LOG_ERR, "The STATUS message was < 200");
            cout << "****** 11 ********" << endl;  // *11*
            // cout << "SEND A SIP CANCEL MESSAGE" << endl;
            cout << "****** 11 ********" << endl;  // *11*

            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;

                // also clean up the route list
                clearRouteList( ep->sipMsgRouteList );
            }
            return true;
//            break;
        }

        if (status != 200)
        {
            cout << "****** 11 ********" << endl;  // *11*
            cout << "BYE MESSAGE NOT SENT SINCE A STATUS " << status
            << " MSG WAS FOUND" << endl;
            cout << "****** 11 ********" << endl;  // *11*

            // If there is still a held call, wait until the held call
            // become active to do clean up after swapping (in SipEventSendInvite ).
            if ( !ep->heldCall )
            {
                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;

                    // also clean up route list
                    clearRouteList( ep->sipMsgRouteList );
                }
            }
            return true;
            //break;
        }
        else
        {
            cout << "****** 11 ********" << endl;  // *11*
            cout << "SEND A SIP BYE MESSAGE BASED ON A STATUS MSG" << endl;
            cout << "****** 11 ********" << endl;  // *11*
            ByeMsg bye( *statusMsg );

            bye.flushViaList();
            SipVia sipVia;
            sipVia.setHost (ca.sipMyHost);
            sipVia.setPort (ca.sipMyPort);
            bye.setVia (sipVia, 0);

            // set the route list
            int i = 0;
            i = sizeRouteList( ep->sipMsgRouteList );
            if ( i > 0 )
            {
                assert( ep->sipMsgRouteList != 0 );
                bye.setRouteList( *(ep->sipMsgRouteList) );

                SipRoute siproute = bye.getRoute(0);
                bye.removeRoute(0);

                SipRequestLine reqLine = bye.getRequestLine();
                reqLine.setUrl( siproute.getUrl() );
                bye.setRequestLine( reqLine );
            }

            assert( sipProxyStack );
            sipProxyStack->sendAsync(bye);
        }
        // clean up the sipMsg pointer here because this call is byeing
        if ( ep->sipMsg != 0 )
        {
            delete( ep->sipMsg );
            ep->sipMsg = 0;
        }

        clearRouteList( ep->sipMsgRouteList );
        return true;
//        break;
    }
    cpLog (LOG_ERR, "No INVITE, ACK nor STATUS message could be found");
    cout << "****** 11 ********" << endl;  // *11*
    cout << "****** 11 ********" << endl;  // *11*
    return true;
}

bool processSipEventSendByeResponse(Endpoint* ep)
{
    cout << "****** 13 ********" << endl;  // *13*
    cout << "SEND A SIP BYE 200 RESPONSE" << endl;
    cout << "****** 13 ********" << endl;  // *13*

    ByeMsg* byeMsg = dynamic_cast < ByeMsg* > (ep->sipMsg);
    if (!byeMsg)
    {
        cpLog (LOG_ERR, "No BYE message could be found");
        return true;
//        break;
    }
    StatusMsg status( *byeMsg, 200 /* OK */);
    assert( sipProxyStack );
    sipProxyStack->sendReply(status);
    return true;
}
bool processSipEventSendCancelResponse(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "SEND A SIP CANCEL 200 RESPONSE" << endl;
    cout << "*****************" << endl;

    CancelMsg* cancelMsg = dynamic_cast < CancelMsg* > (ep->sipMsg);
    if (!cancelMsg)
    {
        cpLog (LOG_ERR, "No CANCEL message could be found");
        return true;
//        break;
    }
    StatusMsg status( *cancelMsg, 200 /* OK */);
    assert( sipProxyStack );
    sipProxyStack->sendReply(status);
    return true;
}

bool processSipEventSendInfo(Endpoint* ep)
{
/* InfoMsg no longer carries dtmf digits, remove its current code for now
    cout << "*****************" << endl;
    cout << "SEND A SIP INFO MESSAGE" << endl;
    cout << "*****************" << endl;

    if (!(ep->call))
    {
        cpLog (LOG_ERR, "No call at %s", ep->epId.logData());
        return true;
    }

    if (ep->state != CallStateConnected)
    {
        cpLog (LOG_ERR, "Call state for %s is not connected", ep->epId.logData());
        return true;
    }

    char digit = ep->call->dtmfDigits[0];
    sipmgcp::MgNotificationEvent dtmfEvent;

    switch ( digit )
    {
        case '*':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_Star();
        }
        break;
        case '#':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_Hash();
        }
        break;
        case '0':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_0();
        }
        break;
        case '1':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_1();
        }
        break;
        case '2':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_2();
        }
        break;
        case '3':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_3();
        }
        break;
        case '4':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_4();
        }
        break;
        case '5':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_5();
        }
        break;
        case '6':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_6();
        }
        break;
        case '7':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_7();
        }
        break;
        case '8':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_8();
        }
        break;
        case '9':
        {
            dtmfEvent = sipmgcp::MgDTMFPackage::notifyDTMF_9();
        }
        break;
        default:
        {
            return false;
        }
    }

    SipUrl toUrl;
    SipUrl fromUrl;

    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
    if ( statusMsg )
    {
        toUrl = statusMsg->getTo().getUrl();
        fromUrl = statusMsg->getFrom().getUrl();
    }
    else
    {
        InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
        if ( inviteMsg )
        {
            toUrl = inviteMsg->getFrom().getUrl();
            fromUrl = inviteMsg->getTo().getUrl();
        }
        else
        {
            cpLog (LOG_ERR, "when sending a DTMF digit, neither INVITE NOR STATUS 200 was found");
            return false;
        }
    }

    // Encode in MGCP DTMF event
    //            string myAddr = from.getUser() + "@" +  from.getHost();
    string myAddr = translate2sip(ep->dn, ca.sipMyHost);
    sipmgcp::MgNotify dtmfNotify( myAddr, "", dtmfEvent );
    sipmgcp::MgcpSession dtmfSession;
    dtmfSession.setMgCmd( &dtmfNotify );
    SipMgcp dtmfContentData( dtmfSession );

    // Construct an INFO message
    SipUrl url;
    if (toUrl.getUserParm() == "phone")
    {
        url.setPhoneNumber (toUrl.getUser() );
    }
    else
    {
        url.setUser( toUrl.getUser() );
    }
    //            url.setHost( UaConfiguration::instance()->getProxyServer() );
    url.setHost( ca.sipProxyHost );

    InfoMsg info( url, ca.sipProxyPort );

    // Set the Record Route
    int i = sizeRouteList( ep->sipMsgRouteList );
    if ( i > 0 )
    {
        assert( ep->sipMsgRouteList != 0 );
        info.setRouteList( *(ep->sipMsgRouteList) );
        SipRoute siproute = info.getRoute(0);
        info.removeRoute(0);

        SipRequestLine reqLine = info.getRequestLine();
        reqLine.setUrl( siproute.getUrl() );
        info.setRequestLine( reqLine );
    }

    // Set the via
    SipVia sipVia;
    sipVia.setprotoVersion( "2.0" );
    sipVia.setHost( Data( ca.sipMyHost ) );
    sipVia.setPort( ca.sipMyPort );
    info.flushViaList();
    info.setVia( sipVia, 0 );

    SipFrom from ( fromUrl );
    info.setFrom( from );

    SipCallId callId = ep->sipMsg->getCallId();
    info.setCallId( callId );

    SipCSeq sipCSeq = info.getCSeq();
    // sipCSeq.setCSeq( cseq );
    sipCSeq.setCSeq( ++(ep->call->lastInfoCseq) );
    info.setCSeq( sipCSeq );
    info.setContentData( &dtmfContentData );

    sipProxyStack->sendAsync( info );
// InfoMsg is no longer used to carry dtmf digits, remove current code for now
*/
    return true;
}

bool processSipEventSendInfoResponse(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "SEND A SIP INFO RESPONSE MESSAGE" << endl;
    cout << "*****************" << endl;

    InfoMsg* infoMsg = dynamic_cast < InfoMsg* > (ep->sipMsg);
    if (!infoMsg)
    {
        cpLog (LOG_ERR, "No INFO message could be found");
        return true;
//        break;
    }
    StatusMsg status( *infoMsg, 200 /* ok */ );
    assert( sipProxyStack );
    sipProxyStack->sendReply(status);
    return true;
}

bool processSipEventSendRegister(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "SEND A REGISTER MESSAGE" << endl;
    cout << "*****************" << endl;
    // Just for now just send this out.
    // Eventually, save this message to handle re-registration, etc.
    RegisterMsg registerMsg;

    //change the register request url
    SipRequestLine reqLine = registerMsg.getRequestLine();
    //SipUrl reqUrl = reqLine.getUrl();
    Sptr<BaseUrl> baseUrl = reqLine.getUrl();
    Sptr<SipUrl> reqUrl;
    reqUrl.dynamicCast( baseUrl );
    if ( reqUrl != 0 )
    {
        // reqUrl.setUser ( ep->dn );
        // reqUrl->setPhoneNumber( ep->dn );
        reqUrl->setUserValue( ep->dn, "phone" );
	reqUrl->setHost( ca.sipProxyHost );
	reqUrl->setPort( ca.sipProxyPort );
        // reqUrl.setUserParm( "phone" );
	reqLine.setUrl(reqUrl);
	registerMsg.setRequestLine(reqLine);
    }

    // string epIdString = translate2sip (ep->dn, ca.sipMyHost);
    // SipUrl sipUrl ( epIdString );
    Sptr<SipUrl> sipUrl = new SipUrl;
    //sipUrl.setPhoneNumber( ep->dn );
    sipUrl->setUserValue( ep->dn, "phone");
    sipUrl->setHost( ca.sipMyHost );
    sipUrl->setPort( ca.sipMyPort );
    // sipUrl.setUserParm( "phone" );

    //correct the from field
    SipFrom sipfrom ( sipUrl );
    registerMsg.setFrom( sipfrom );

    //correct the to field
    SipTo sipto (sipUrl);
    registerMsg.setTo(sipto);

    //correct the via field
    SipVia sipvia = registerMsg.getVia();
    sipvia.setPort( ca.sipMyPort );
    registerMsg.removeVia();
    registerMsg.setVia(sipvia);


    //add myself to the contact list
    SipContact myContact;
    myContact.setUrl(sipUrl);
    myContact.setAction( "proxy" );
    registerMsg.setNumContact(0);
    registerMsg.setContact(myContact);

    //add the default expire
    SipExpires sipexpires;
    sipexpires.setDelta(ca.registerRetrans);
    registerMsg.setExpires(sipexpires);

    sipProxyStack->sendAsync(registerMsg);
    return true;
}

bool processSipEventSendHold(Endpoint* ep)
{
    // the endpoint is in Connected state and has only
    // one active call, and wants to send an Invite with
    // Sdp=0 to put the far end on hold
    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
    if ( statusMsg )
    {
        if (statusMsg->getStatusLine().getStatusCode() == 200)
        {
            // this endpoint invited the far end into the connection previously,
            // because its ep->sipMsg points to a 200 message

            // send a re-Invite with Sdp=0 and increment CSeq
            cout << "SEND A SIP re-INVITE MESSAGE with Sdp = 0" << endl;
            InviteMsg* inviteMessage = 0;

            inviteMessage = new InviteMsg( *statusMsg );

            SipVia sipVia;
            sipVia.setprotoVersion( "2.0" );
            sipVia.setHost( Data( ca.sipMyHost ) );
            sipVia.setPort( ca.sipMyPort );
            inviteMessage->flushViaList();
            inviteMessage->setVia( sipVia, 0 );

            SipCSeq cseq = statusMsg->getCSeq();
            int num = cseq.getCSeqData().convertInt();
            num++;
            cseq.setCSeq( num );
            inviteMessage->setCSeq( cseq );


            inviteMessage->flushContentData();
            //	SipContentType contentType = statusMsg->getContentType();
            //	inviteMessage->setContentType((SipContentType) contentType);


            Sptr<SipSdp> sipSdp;
	    sipSdp.dynamicCast ( inviteMessage->getContentData( 0 ) );
            if ( sipSdp == 0 )
            {
                SdpSession sdpDesc = ep->call->endpoint[0].localConnection;
                sdpDesc.setHold();
                SipSdp sipSdp;
                sipSdp.setSdpDescriptor( convert(sdpDesc) );
                inviteMessage->setContentData( &sipSdp );
            }
            else
            {
                SdpSession sdpDesc = convert(sipSdp->getSdpDescriptor());
                sdpDesc.setHold();
                sipSdp->setSdpDescriptor( convert(sdpDesc) );
            }

            // set the route list
            assert( ep->sipMsgRouteList != 0 );
            inviteMessage->setRouteList( *(ep->sipMsgRouteList) );

            SipRoute siproute = inviteMessage->getRoute(0);
            inviteMessage->removeRoute(0);

            SipRequestLine reqLine = inviteMessage->getRequestLine();
            reqLine.setUrl( siproute.getUrl() );
            inviteMessage->setRequestLine( reqLine );


            sipProxyStack->sendAsync(*inviteMessage);

            if (ep->call)
            {
               ep->call->ackReceived = false;
            }

            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }
            ep->sipMsg = dynamic_cast < SipMsg* > (inviteMessage);
        }
    }
    else
    {
        InviteMsg* oldInviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
        if ( oldInviteMsg )
        {
            // this endpoint was invited into the connection previously,
            // because its ep->sipMsg points to an Invite message

            // send an Invite to the far end with Sdp=0 to put it on hold
            cout << "SEND A SIP INVITE MESSAGE with Sdp = 0" << endl;

            InviteMsg* inviteMessage = 0;
            inviteMessage = new InviteMsg((oldInviteMsg->getFrom()).getUrl(),
                                          ca.sipMyPort );

            SipFrom from( (oldInviteMsg->getTo()).getUrl() );
            inviteMessage->setFrom( from );

            inviteMessage->setCallId( oldInviteMsg->getCallId() );


            int i = sizeRouteList( ep->sipMsgRouteList );
            if ( i > 0 )
            {
                assert( ep->sipMsgRouteList != 0 );
                inviteMessage->setRouteList( *(ep->sipMsgRouteList) );
                SipRoute siproute = inviteMessage->getRoute(0);
                inviteMessage->removeRoute(0);

                SipRequestLine reqLine = inviteMessage->getRequestLine();
                reqLine.setUrl( siproute.getUrl() );
                inviteMessage->setRequestLine( reqLine );
            }

            SipVia sipVia;
            sipVia.setprotoVersion( "2.0" );
            sipVia.setHost( Data( ca.sipMyHost ) );
            sipVia.setPort( ca.sipMyPort );
            inviteMessage->flushViaList();
            inviteMessage->setVia( sipVia, 0 );

            SipCSeq cseq = oldInviteMsg->getCSeq();
            int num = cseq.getCSeqData().convertInt();
            num++;
            cseq.setCSeq( num );
            inviteMessage->setCSeq( cseq );

            Sptr<SipSdp> sipSdp;
	    sipSdp.dynamicCast ( inviteMessage->getContentData( 0 ) );
            if ( sipSdp != 0 )
            {
                SdpSession sdpDesc = convert(sipSdp->getSdpDescriptor());
                sdpDesc.setHold();
                sipSdp->setSdpDescriptor( convert(sdpDesc) );
            }

            sipProxyStack->sendAsync(*inviteMessage);

            if (ep->call)
            {
                ep->call->ackReceived = false;
            }

            // have sipMsgBuf pointer points to the previous message
            // in case this hold invite would fail ( with 486 response) for party C flash
            if ( ep->sipMsgBuf != 0 )
            {
                delete( ep->sipMsgBuf );
                ep->sipMsgBuf = 0;
            }

            ep->sipMsgBuf = ep->sipMsg;
            ep->sipMsg = dynamic_cast < SipMsg* > (inviteMessage);
        }
        else
        {
            cpLog( LOG_DEBUG, "Can't find either status or invite message to send hold for ");
        }

    }
    return true;
}

bool processSipEventSendXfer(Endpoint* ep)
{
    // We know that the endpoint wants to do a transfer
    // so save that information here
    if ( ep->state == CallStateFarEndAlerted )
    {
        ep->callType = BlindXfer;
    }
    else if ( ep->state == CallStateConnected )
    {
        if (ep->heldCall == 0)
        {
            ep->callType = Conference;
        }
        else
        {
            ep->callType = ConsultationXfer;
        }
    }
    else
    {
        cpLog( LOG_DEBUG, "In a wrong call state to send Xfer");
        return true;
//        break;
    }


    // find the previous Invite if blind transfer
    InviteMsg* prevInviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);

    InviteMsg* inviteMessage = 0;
    SipCSeq cseq;

    if ( !prevInviteMsg )
    {
        // find the 200 for the previous Invite if consultation transfer
        StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);

        if ( !statusMsg )
        {
            cout << "No previous Invite or 200 for an Invite was found, can NOT complete the transfer !!" << endl;
            return true;
//            break;
        }
        else
        {
            // create the re-invite based on the status message
            inviteMessage = new InviteMsg( *statusMsg );
            assert( inviteMessage != 0 );
            cseq = statusMsg->getCSeq();
        }
    }
    else
    {
        // create the re-invite based on the previous invite message
        inviteMessage = new InviteMsg( *prevInviteMsg );
        assert( inviteMessage != 0 );
        cseq = prevInviteMsg->getCSeq();
        inviteMessage->setCallId( prevInviteMsg->getCallId() );
    }

    // send a re-Invite with sdp=0 to put the far end on hold while completing the transfer
    cout << "SEND A SIP re-INVITE MESSAGE with Sdp = 0" << endl;

        SipVia sipVia;
        sipVia.setprotoVersion( "2.0" );
        sipVia.setHost( Data( ca.sipMyHost ) );
        sipVia.setPort( ca.sipMyPort );
        inviteMessage->flushViaList();
        inviteMessage->setVia( sipVia, 0 );

        int num = cseq.getCSeqData().convertInt();
        num++;
        cseq.setCSeq( num );
        inviteMessage->setCSeq( cseq );

        Sptr<SipSdp> sipSdp;
	sipSdp.dynamicCast ( inviteMessage->getContentData( 0 ) );
        if ( sipSdp != 0 )
        {
            Vocal::SDP::SdpSession sdpDesc = sipSdp->getSdpDescriptor();
            sdpDesc.setHold();
            sipSdp->setSdpDescriptor( sdpDesc );
        }
        else
        {
            SdpSession sdpDesc 
		= ep->call->endpoint[0].localConnection;
            sdpDesc.setHold();
            SipSdp sipSdp;
            sipSdp.setSdpDescriptor( convert(sdpDesc) );
            inviteMessage->setContentData( &sipSdp );
        }

        int i = sizeRouteList( ep->sipMsgRouteList );
        if ( i > 0 )
        {
            assert( ep->sipMsgRouteList != 0 );
            inviteMessage->setRouteList( *(ep->sipMsgRouteList) );
            SipRoute siproute = inviteMessage->getRoute(0);
            inviteMessage->removeRoute(0);

            SipRequestLine reqLine = inviteMessage->getRequestLine();
            reqLine.setUrl( siproute.getUrl() );
            inviteMessage->setRequestLine( reqLine );
        }

        sipProxyStack->sendAsync(*inviteMessage);

        if (ep->call)
        {
            ep->call->ackReceived = false;
        }

        if ( ep->sipMsg != 0 )
        {
            delete( ep->sipMsg );
            ep->sipMsg = 0;
        }
        ep->sipMsg = dynamic_cast < SipMsg* > (inviteMessage);
    return true;
}

bool processSipEventSendConf(Endpoint* ep)
{
    cpLog( LOG_ERR, "No support for conference yet !!");

    // We know that the endpoint wants to do a conference.
    // so save that information here
    ep->callType = Conference;

    // more to come
    return true;
}

// procedures for SipEventReceive
bool processSipEventReceiveInvite(Endpoint* ep)
{
    cout << "****** 2 ********" << endl;  // *2*
    cout << "RECEIVED A SIP INVITE MESSAGE" << endl;
    cout << "****** 2 ********" << endl;  // *2*

    bool containSdpHold = false;

    if ( !ep->sipMsg )
    {
        cpLog( LOG_DEBUG, "The endpoint ep->sipMsg == 0 ");
        return false;
    }

    Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
    Sptr<SipSdp> sipsdp;
    sipsdp.dynamicCast (sipContent);
    if (sipsdp != 0)
    {
        Vocal::SDP::SdpSession sdp = sipsdp->getSdpDescriptor();
        if ( !ep->call )
        {
            cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
            return false;
        }
        ep->call->endpoint[1].localConnection = convert(sdp);

        containSdpHold = sdp.isHold();
    }
    sipEventSend (ep, SipEventSendTrying);

    if ( ep->state == CallStateIdle )
    {
        // a new call
        sendInternalEvent (ep, CallEventReceiveCall);
    }
    else if ( containSdpHold           // this invite is a Hold Invite (i.e. contains sdp=0)
              || (ep->sipMsgHeld != 0) // or this invite comes from a different source than the previous caller(held party)
            )
    {
        sendInternalEvent (ep, CallEventMDCX);
        if ( ep->state == CallStateConnected )
        {
            sipEventSend (ep, SipEventSendAnswer);
        }
        else if ( ep->state == CallStateAlerting )
        {
            sipEventSend (ep, SipEventSendRinging );
        }
        else
        {
            cout << "Wrong state to receive an Invite in transfer" << endl;
        }
    }
    else
    {
        cout << "Confusing Invite message, no processing for it" << endl;
    }
    // sendInternalEvent (ep, CallEventReceiveCall);
    return true;
}

bool processSipEventReceiveTrying(Endpoint* ep)
{
    cout << "****** 4 ********" << endl;  // *4*
    cout << "RECEIVED A SIP 100 INVITE RESPONSE ... DO NOTHING" << endl;
    cout << "****** 4 ********" << endl;  // *4*
    return true;
}

bool processSipEventReceiveRinging(Endpoint* ep)
{
    cout << "****** 6 ********" << endl;  // *6*
    cout << "RECEIVED A SIP 180 INVITE RESPONSE" << endl;
    cout << "****** 6 ********" << endl;  // *6*

    // check to see if this is a 180 for a Invite with sdp=0 to put far end on hold
    // in blind transfer
    bool containSdpHold = false;
    InviteMsg* prevMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
    if ( prevMsg )
    {
        Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
        Sptr<SipSdp> sipsdp;
	sipsdp.dynamicCast (sipContent);

        if (sipsdp != 0)
        {
            containSdpHold = (sipsdp->getSdpDescriptor()).isHold();
        }
    }

    if ( containSdpHold && (ep->callType == BlindXfer) )  // a 180 indicating a blind transfer
    {
        // send a transfer to the held party based on a 200 status message received from it previously
        StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsgHeld);
        if ( statusMsg )
        {
            if ( (statusMsg->getStatusLine().getStatusCode()) == 200 )
            {
                // a saved 200 for a previous Invite with sdp=0 to put the far end on hold
                Sptr<BaseUrl> toUrl;
                toUrl = (prevMsg->getTo()).getUrl();

                TransferMsg* xferMsg = new TransferMsg( *statusMsg, toUrl );
                assert( xferMsg != 0 );

                cout << "****** Transfer ********" << endl;
                cout << "SENDING A SIP TRANSFER MESSAGE" << endl;
                cout << "******* Transfer *******" << endl;

                int i = 0;
                i = sizeRouteList( ep->sipMsgHeldRouteList );
                if ( i > 0 )
                {
                    assert( ep->sipMsgHeldRouteList != 0 );
                    xferMsg->setRouteList( *(ep->sipMsgHeldRouteList) );
                    SipRoute siproute = xferMsg->getRoute(0);
                    xferMsg->removeRoute(0);

                    SipRequestLine reqLine = xferMsg->getRequestLine();
                    reqLine.setUrl( siproute.getUrl() );
                    xferMsg->setRequestLine( reqLine );
                }

                // re-set Via as UA
                SipVia sipVia;
                sipVia.setprotoVersion( "2.0" );
                sipVia.setHost( Data( ca.sipMyHost ) );
                sipVia.setPort( ca.sipMyPort );
                xferMsg->flushViaList(); ;
                xferMsg->setVia( sipVia, 0 );

                sipProxyStack->sendAsync(*xferMsg);

                if ( ep->sipMsgXfer != 0 )
                {
                    delete( ep->sipMsgXfer );
                    ep->sipMsgXfer = 0;
                }
                ep->sipMsgXfer = dynamic_cast < SipMsg* > (xferMsg);
            }
            else
            {
                cpLog( LOG_ERR, "No 200 status message to create the transfer from");
            }
        }
        else
        {
            cpLog( LOG_ERR, "No status message to create the transfer from");
        }
    }
    else // a normal 180 or 183 for a basic call, OR a 180 or 183 for an Invite triggered by a received transfer message
    {
        sendInternalEvent (ep, CallEventFarEndAlerting);

        if ( ep->sipMsgXfer != 0 )
        {
            // There is an outstanding received transfer message, this 180 must be for a blind transfer
            // so send a 200 for the transfer

            // make sure that the transfer's TransferTo field is the same as the last Invite's To field
            // SipTransferTo to = ep->sipMsgXfer->getTransferTo();
            // SipUrl xferToUrl = to.getUrl();

            // InviteMsg* prevMsg= dynamic_cast<InviteMsg*>(ep->sipMsg);
            //  if ( prevMsg )
            //  {
            //  SipUrl inviteToUrl = (ep->sipMsg->getTo()).getUrl();
            //	 if ( inviteToUrl == xferToUrl )
            //	 {
            // we need to send a 200 for the received transfer
            TransferMsg* xferMsg = dynamic_cast < TransferMsg* > (ep->sipMsgXfer);
            if ( xferMsg == 0 )
            {
                cpLog( LOG_DEBUG, "The endpoint ep->sipMsgXfer does not contain a TransferMsg. ");
                return false;
            }

            StatusMsg status( *xferMsg, 200 /* a 200 for transfer*/ );
            assert( sipProxyStack );
            sipProxyStack->sendReply(status);
            // clean up the xfer pointer here since transfer is completed
            if ( ep->sipMsgXfer != 0 )
            {
                delete( ep->sipMsgXfer );
                ep->sipMsgXfer = 0;
            }

            //	 }
            //    }
        }

    }
    //sendInternalEvent (ep, CallEventFarEndAlerting);
    return true;
}

bool processSipEventReceiveAnswer(Endpoint* ep)
{
    cout << "****** 8 ********" << endl;  // *8*
    cout << "RECEIVED A SIP 200 INVITE RESPONSE" << endl;
    cout << "****** 8 ********" << endl;  // *8*
    if (!ep->call)
    {
        // for transfering party to complete the transfer
        // there could not be any call left once he onhook
        bool containSdpHold = false;
        if ( !ep->sipMsg )
        {
            cpLog( LOG_DEBUG, "The endpoint ep->sipMsg == 0 ");
            return false;
        }

        Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
        Sptr<SipSdp> sipsdp;
	sipsdp.dynamicCast (sipContent);

        if (sipsdp != 0)
        {
            containSdpHold = (sipsdp->getSdpDescriptor()).isHold();
        }

        if ( !(( containSdpHold ) && ( ep->callType == ConsultationXfer )) )
            // the previous Invite was NOT a hold Invite for completing a consultation transfer
        {

            cout << "ERROR: a 200 response to an invite message" << endl;
            cout << "was received, but there was no active call." << endl;
            return true;
//            break;
        }
    }

    /*if (ep->state != CallStateFarEndAlerted)
    {
        cout << "ERROR: 200 responses received, but call state" << endl;
        cout << "was not CallStateFarEndAlerted, nor a 200 for Invite(hold)." << endl;
    break;
}
    */
    // If this 200 is for an Invite (Hold) in a consultation transfer or conference,
    // an ACK must have been sent out.  Now send a transfer message to the held party.
    if (( ep->callType == ConsultationXfer ) || (ep->callType == Conference))
    {
        // send a transfer to the held party based on a 200 status message received from it previously
        StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsgHeld);
        if ( statusMsg )
        {
            if ( statusMsg->getStatusLine().getStatusCode() == 200 )
            {
                // SipUrl toUrl;
                string conferenceServer = ca.conferenceServer;
                Sptr<SipUrl> toUrl = new SipUrl( conferenceServer );
                // a saved 200 for a previous Invite with sdp=0 to put the held party on hold
                if ( ep->callType == ConsultationXfer )
                {
                    // toUrl = (statusMsg->getTo()).getUrl();
                    if ( !ep->sipMsg )
                    {
                        cpLog( LOG_DEBUG, "The endpoint ep->sipMsg == 0 ");
                        return false;
                    }
		    
                    //toUrl = (ep->sipMsg->getTo()).getUrl();
		    Sptr<BaseUrl> baseUrl = (ep->sipMsg->getTo()).getUrl();
		    toUrl.dynamicCast( baseUrl );

                }

		assert( toUrl != 0 );
                TransferMsg* xferMsg = new TransferMsg( *statusMsg, toUrl );
                assert( xferMsg != 0 );


                SipVia sipVia;
                sipVia.setprotoVersion( "2.0" );
                sipVia.setHost( Data( ca.sipMyHost ) );
                sipVia.setPort( ca.sipMyPort );

                xferMsg->flushViaList();
                xferMsg->setVia( sipVia, 0 );


                int i = 0;
                i = sizeRouteList( ep->sipMsgHeldRouteList );
                if ( i > 0 )
                {
                    assert( ep->sipMsgHeldRouteList != 0 );
                    xferMsg->setRouteList( *(ep->sipMsgHeldRouteList) );
                    SipRoute siproute = xferMsg->getRoute(0);
                    xferMsg->removeRoute(0);

                    SipRequestLine reqLine = xferMsg->getRequestLine();
                    reqLine.setUrl( siproute.getUrl() );
                    xferMsg->setRequestLine( reqLine );
                }


                cout << "****** Transfer ********" << endl;
                cout << "SENDING A SIP TRANSFER MESSAGE" << endl;
                cout << "******* Transfer *******" << endl;

                sipProxyStack->sendAsync(*xferMsg);

                if ( ep->sipMsgXfer != 0 )
                {
                    delete( ep->sipMsgXfer );
                    ep->sipMsgXfer = 0;
                }
                ep->sipMsgXfer = dynamic_cast < SipMsg* > (xferMsg);

                return true;
//                break;
            }
        }
    }
    else
    {
        if ( !ep->call )
        {
            cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
            return false;
        }

        if ( ((ep->call->endpoint[1]).localConnection).isHold() )
        {
            if ( ep->heldCall )
            {
                if ( ep->heldCall->callWaited == true )
                {
                    // for Call Waiting, swap the active and held call
                    // so that the MDCX will activate correctly
                    CallData* tmp;
                    tmp = ep->heldCall;
                    ep->heldCall = ep->call;
                    ep->call = tmp;
                    ep->heldCall->callWaiting = ep->call->callWaiting;
                    ep->heldCall->callWaited = ep->call->callWaited;
                    ep->call->callWaiting = false;
                    ep->call->callWaited = false;
                }
            }
            // this must be a 200 for an Invite(Hold) to put far end on hold
            // so don't overwrite the sdp with stuff in the 200 which would
            // contain valid sdp of the far end. This isHold value
            // will be checked by mgcp side to trigger a MDCX(inactive)
            sendInternalEvent (ep, CallEventAnswer);
        }
        else // a normal 200 (answer), update the sdp info
        {
            // Done in case 200 for Invite

            //SipContentData* sipContent = ep->sipMsg->getContentData(0);
            //SipSdp* sipsdp = dynamic_cast<SipSdp*>(sipContent);

            // if (sipsdp)
            //{
            // SdpSession sdp = sipsdp->getSdpDescriptor();
            // ep->call->endpoint[1].localConnection = sdp;
            //}

            //string statusMsgCallId = ep->sipMsg->getCallId().logData();
            sendInternalEvent (ep, CallEventAnswer);

            // for now, if there is a outstanding Transfer(i.e. in consultation transfer),
            // send a 200 for it
            // in the future with RSVP, we may need to send the 200 for transfer
            // at a later time, i.e. after getting a successful 200 for MDCX from mgcp
            if ( ep->sipMsgXfer )
            {
                TransferMsg* xferMsg = dynamic_cast < TransferMsg* > (ep->sipMsgXfer);
                if ( xferMsg == 0 )
                {
                    cpLog( LOG_DEBUG, "The endpoint ep->sipMsgXfer does not contain a TransferMsg.");
                    return false;
                }

                StatusMsg status( *xferMsg, 200/*200 for transfer*/ );
                assert( sipProxyStack );
                sipProxyStack->sendReply(status);


                // clean up the Xfer pointer here because the transfer is completed
                if ( ep->sipMsgXfer != 0 )
                {
                    delete( ep->sipMsgXfer );
                    ep->sipMsgXfer = 0;
                }

            }
        }
    }
    return true;
}

bool processSipEventReceiveTransfer(Endpoint* ep)
{
    // send a 100 trying message for the transfer
    TransferMsg* xferMsg = dynamic_cast < TransferMsg* > (ep->sipMsgXfer);
    if ( xferMsg )
    {
        cpLog( LOG_INFO, "Sending 100 trying for a transfer");
        StatusMsg status( *xferMsg, 100 /* transfer trying */);
        assert( sipProxyStack );
        sipProxyStack->sendReply( status );
    }
    else
    {
        cpLog( LOG_ERR, "No transfer message to send 100 for.");
        return true;
//        break;
    }

    SipTransferTo to = ep->sipMsgXfer->getTransferTo();
    //SipUrl toUrl = to.getUrl();
    Sptr<BaseUrl> baseUrl = to.getUrl();
    Sptr<SipUrl> toUrl;
    toUrl.dynamicCast( baseUrl );
    if ( toUrl != 0 )
    {
        // Invite the TransferTo party via the local marshal
      toUrl->setHost( ca.sipProxyHost );
      toUrl->setPort( ca.sipProxyPort );
      to.setUrl ( toUrl );
    }

    // send an Invite to the toUrl in the transfer message
    {
        if ( !ep->call )
        {
            cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
            return false;
        }

        InviteMsg* inviteMessage = new InviteMsg(toUrl, ca.sipMyPort);

        SdpSession sdp = ep->call->endpoint[0].localConnection;
        Sptr<SipSdp> sipSdp;
	sipSdp.dynamicCast (inviteMessage->getContentData(0));
        if (!sipSdp)
        {
            cout << "ERROR: INVALID SDP was found ... ABORT CALL" << endl;
            giveBusyTone (ep);
            delete( inviteMessage );
            return true;
//            break;
        }

        //SipSdp sipSdp;
        //sipSdp.setSdpDescriptor( sdp );
        sipSdp->setSdpDescriptor(convert(sdp));
        //inviteMessage->setContentData( sipSdp );


        SipTo to (toUrl);
        inviteMessage->setTo( to );

        // string sipString = translate2sip (ep->dn, ca.sipMyHost);
        // SipUrl fromUrl( sipString );
        Sptr<SipUrl> fromUrl = new SipUrl;
        //fromUrl.setPhoneNumber ( ep->dn );
	fromUrl->setUserValue( ep->dn, "phone");
        fromUrl->setHost ( ca.sipMyHost );
        fromUrl->setPort ( ca.sipMyPort );
        SipFrom from (fromUrl);
        inviteMessage->setFrom( from );


        // this invite should have the same call id as the transfer that triggered it
        inviteMessage->setCallId( ep->sipMsgXfer->getCallId() );

        Sptr<SipUrl> myUrl = new SipUrl;
        //myUrl.setPhoneNumber( ep->dn );
	myUrl->setUserValue( ep->dn, "phone");
        myUrl->setHost( ca.sipMyHost );
        myUrl->setPort( ca.sipMyPort );
        SipContact me;
        me.setUrl( myUrl );
        inviteMessage->setNumContact( 0 );    // Clear
        inviteMessage->setContact( me );

        sipProxyStack->sendAsync(*inviteMessage);

        if (ep->call)
        {
            ep->call->ackReceived = false;
        }

        if ( ep->sipMsg != 0 )
        {
            delete( ep->sipMsg );
            ep->sipMsg = 0;
        }
        ep->sipMsg = dynamic_cast < SipMsg* > (inviteMessage);
    }
    return true;
}

bool processSipEventReceiveTransferResponse(Endpoint* ep)
{
    printRouteList( ep->sipMsgRouteList );

    // Transfer is completed
    // send a Bye message to the held call party

    // send a Bye to the held party because they were connected before
    //sipEventSend (ep, SipEventSendBye);

    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsgHeld);
    if ( statusMsg )
    {
        ByeMsg bye( *statusMsg );

        // for the internal trial
        bye.flushViaList();
        SipVia sipVia;
        sipVia.setHost (ca.sipMyHost);
        sipVia.setPort (ca.sipMyPort);
        bye.setVia (sipVia, 0);

        // transfer completed, Bye to the held party (A)
        // set the route list
        int i = 0;
        i = sizeRouteList( ep->sipMsgHeldRouteList);
        if ( i > 0 )
        {
            assert( ep->sipMsgHeldRouteList != 0 );
            bye.setRouteList( *(ep->sipMsgHeldRouteList) );

            SipRoute siproute = bye.getRoute(0);
            bye.removeRoute(0);

            SipRequestLine reqLine = bye.getRequestLine();
            reqLine.setUrl( siproute.getUrl() );
            bye.setRequestLine( reqLine );
        }

        assert( sipProxyStack );
        sipProxyStack->sendAsync(bye);

        if ( ep->sipMsgHeld != 0 )
        {
            delete( ep->sipMsgHeld );
            ep->sipMsgHeld = 0;
        }

        // clean up the route list for the held party as well
        clearRouteList( ep->sipMsgHeldRouteList );

        if (ep->callType == Conference)
        {
            if ( ep->sipMsg )
            {
                // For conference, if there is another call,
                // send another transfer message for it.
                // Since transfer is sent to the held party,
                // first swap the active and held data

                SipMsg* active = ep->sipMsg;
                ep->sipMsg = ep->sipMsgHeld;
                ep->sipMsgHeld = active;

                vector < SipRoute* > * temp = ep->sipMsgHeldRouteList;
                ep->sipMsgHeldRouteList = ep->sipMsgRouteList;
                ep->sipMsgRouteList = temp;

                // Clean up the first transfer message
                if ( ep->sipMsgXfer != 0 )
                {
                    delete( ep->sipMsgXfer );
                    ep->sipMsgXfer = 0;
                }

                sipEventReceive (ep, SipEventReceiveAnswer);
                return true;
//                break;
            }
            else
            {
                // Two TRANSFER and two BYE messages have been sent.
                // Since conference is basically completed, reset callType.
                ep->callType = BasicCall;

                // Now send a new INVITE for itself to the MCU
                string conferenceServer = ca.conferenceServer;
                Sptr<SipUrl> toUrl = new SipUrl(conferenceServer);

                // send an Invite to the toUrl in the transfer message
                InviteMsg* inviteMessage = new InviteMsg(toUrl, ca.sipMyPort);
                if ( inviteMessage )
                {
                    if ( !ep->call )
                    {
                        cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
                        delete( inviteMessage );
                        return false;
                    }

                    SdpSession sdp = ep->call->endpoint[0].localConnection;

                    Sptr<SipSdp> sipSdp;
		    sipSdp.dynamicCast (inviteMessage->getContentData(0));
                    if (!sipSdp)
                    {
                        cout << "ERROR: INVALID SDP was found ... ABORT CALL" << endl;
                        giveBusyTone (ep);

                        // Clean up the transfer messages
                        if ( ep->sipMsgXfer != 0 )
                        {
                            delete( ep->sipMsgXfer );
                            ep->sipMsgXfer = 0;
                        }

                        delete( inviteMessage );
                        return true;
//                        break;
                    }

                    //SipSdp sipSdp;
                    //sipSdp.setSdpDescriptor( sdp );
                    sipSdp->setSdpDescriptor(convert(sdp));
                    //inviteMessage->setContentData( sipSdp );

                    SipTo to (toUrl);
                    inviteMessage->setTo( to );

                    string sipString = translate2sip (ep->dn, ca.sipMyHost);
                    Sptr<SipUrl> fromUrl = new SipUrl( sipString );
                    SipFrom from (fromUrl);
                    inviteMessage->setFrom( from );


                    // this invite should have the same call id as the transfer that triggered it
                    if ( !ep->sipMsgXfer )
                    {
                        cpLog( LOG_DEBUG, "The endpoint ep->sipMsgXfer == 0 ");
                        delete( inviteMessage );
                        return false;
                    }

                    inviteMessage->setCallId( ep->sipMsgXfer->getCallId() );



                    Sptr<SipUrl> myUrl = new SipUrl( sipString );
                    //myUrl.setUser( );
                    myUrl->setHost(ca.sipMyHost);
                    myUrl->setPort(ca.sipMyPort);
                    SipContact me;
                    me.setUrl( myUrl );
                    inviteMessage->setNumContact( 0 );    // Clear
                    inviteMessage->setContact( me );


                    sipProxyStack->sendAsync(*inviteMessage);

                    ep->sipMsg = dynamic_cast < SipMsg* > (inviteMessage);
                }
                else
                {
                    cpLog( LOG_ERR, "Out of memory !!!");
                    assert( 0 );
                    return true;
//                    break;
                }
            }

            // Clean up the transfer messages
            if ( ep->sipMsgXfer != 0 )
            {
                delete( ep->sipMsgXfer );
                ep->sipMsgXfer = 0;
            }

            return true;
//            break;
        }

    }
    else
    {
        cpLog( LOG_DEBUG, "No status message to create a Bye from.");
    }


    // for a blind transfer, send a cancel to the active call party
    if ( ep->callType == BlindXfer )
    {
        //sipEventSend (ep, SipEventSendCancel);

        InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
        if (inviteMsg)
        {
            printRouteList( ep->sipMsgRouteList );

            CancelMsg cancel( *inviteMsg );

            assert( sipProxyStack );
            sipProxyStack->sendAsync(cancel);

            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }


            // clean up the route list for the active party as well
            clearRouteList( ep->sipMsgRouteList );


        }
        else
        {
            cpLog( LOG_DEBUG, "No Invite message to create a Cancel from.");
        }

    }
    else if ( ep->callType == ConsultationXfer )
    {
        // for a consultation transfer, send a Bye to the active party
        //sipEventSend (ep, SipEventSendBye);

        StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
        if ( statusMsg )
        {
            ByeMsg bye( *statusMsg );

            // for the internal trial
            bye.flushViaList();
            SipVia sipVia;
            sipVia.setHost (ca.sipMyHost);
            sipVia.setPort (ca.sipMyPort);
            bye.setVia (sipVia, 0);

            // transfer completed, Bye to the active party (C)
            // set the route list

            int i = 0;
            i = sizeRouteList(ep->sipMsgRouteList);
            if ( i > 0 )
            {
                assert( ep->sipMsgRouteList != 0 );
                bye.setRouteList( *(ep->sipMsgRouteList) );

                SipRoute siproute = bye.getRoute(0);
                bye.removeRoute(0);

                SipRequestLine reqLine = bye.getRequestLine();
                reqLine.setUrl( siproute.getUrl() );
                bye.setRequestLine( reqLine );
            }

            assert( sipProxyStack );
            sipProxyStack->sendAsync(bye);

            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }


            // clean up the route list for the active party as well
            clearRouteList( ep->sipMsgRouteList );


        }
        else
        {
            cpLog( LOG_DEBUG, "No status message to create a Bye from.");
        }

    }
    else
    {
        cpLog( LOG_DEBUG, "Wrong call type to receive a transfer response.");
    }

    // clean up callType and calls  in endpoint because transfer is completed
    // Bye and/or Cancel have been sent out
    ep->callType = BasicCall;

    if ( ep->sipMsgXfer != 0 )
    {
        delete( ep->sipMsgXfer );
        ep->sipMsgXfer = 0;
    }
    return true;
}

bool processSipEventReceiveRedirect(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "RECEIVED A SIP 3xx INVITE RESPONSE" << endl;
    cout << "*****************" << endl;
    // kudo - if available
    // ep->call->endpoint[1].localConnection =

    // Send another INVITE message
    // Same call-id, but increased sequence # ???
    // Based on the received 3xx status message ???
    return true;
}

bool processSipEventReceiveBusy(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "RECEIVED A SIP 4xx MESSAGE" << endl;
    cout << "*****************" << endl;
    giveBusyTone (ep);
    // kudo - do we need to send something back e.g. cancel/bye
    // sipEventSend (ep, SipEventSendBye);
    // This will always delete the active call.
    // If held calls need to be supported, this needs to be changed.
    // sendInternalEvent (ep, CallEventDeleteCall, ep->call->callIdentifier);
    // Should call giveBusyTone from MGCP CA side
    return true;
}

bool processSipEventReceiveAck(Endpoint* ep)
{
    cout << "****** 10 ********" << endl;  // *10*
    cout << "RECEIVED A SIP ACK MESSAGE ... DO NOTHING" << endl;
    cout << "****** 10 ********" << endl;  // *10*
    // kudo - if available
    // ep->call->endpoint[1].localConnection =
    // sendInternalEvent (ep, ???);

    // temperarily set this flag to distinguish CANCEL vs BYE
    if (ep->call)
    {
        ep->call->ackReceived = true;
    }
    return true;
}

bool processSipEventReceiveBye(Endpoint* ep)
{
    cout << "****** 12 ********" << endl;  // *12*
    cout << "RECEIVED A SIP BYE MESSAGE" << endl;
    cout << "****** 12 ********" << endl;  // *12*
    sipEventSend (ep, SipEventSendByeResponse);


    // If there is a transfer message pointer,
    // or there is a held call, and this Bye comes from
    // the same source as one of those, clean up and don't do anything
    // SipUrl byeFromUrl = (ep->sipMsg->getFrom()).getUrl();


    // clean up the active call sipMsg pointer here because the call is byed from the far end

    if ( ep->sipMsg != 0 )
    {
        delete( ep->sipMsg );
        ep->sipMsg = 0;
    }

    // also clean up the route list
    clearRouteList( ep->sipMsgRouteList );

    // This will always delete the active call.
    // If held calls need to be supported, this needs to be changed.
    if (ep->call)
    {
        sendInternalEvent (ep,
                           CallEventDeleteCall,
                           ep->call->callIdentifier);

    }
    else
    {
        cout << "BUT THERE WAS NO CALL TO CANCEL" << endl;
    }
    return true;
}


bool processSipEventReceiveByeOrCancelResponse(Endpoint* ep)
{
    cout << "****** 14 ********" << endl;  // *14*
    cout << "RECEIVED A SIP BYE OR CANCEL 200 RESPONSE ... DO NOTHING" << endl;
    cout << "****** 14 ********" << endl;  // *14*
    return true;
}

bool processSipEventReceiveCancel(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "RECEIVED A SIP CANCEL MESSAGE" << endl;
    cout << "*****************" << endl;
    sipEventSend (ep, SipEventSendCancelResponse);

    // This will always delete the active call.
    // If held calls need to be supported, this needs to be changed.
    if (ep->call)
    {
        sendInternalEvent (ep, CallEventDeleteCall, ep->call->callIdentifier);
    }
    else
    {
        cout << "BUT THERE WAS NO CALL TO CANCEL" << endl;
    }


    // clean up the sipMsg pointer because the far end is canceling
    if ( ep->sipMsg )
    {
        delete( ep->sipMsg );
        ep->sipMsg = 0;
    }

    // also clean up the route list info
    clearRouteList( ep->sipMsgRouteList );

    return true;
}

bool processSipEventReceiveInfo(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "RECEIVED A SIP DIGIT MESSAGE" << endl;
    cout << "*****************" << endl;
/* InfoMsg is no longer used to carry dtmf digits, removed current code for now
    if (!(ep->call))
    {
        cpLog (LOG_ERR, "No call at %s", ep->epId.logData());
        return true;
//        break;
    }
    if (ep->state != CallStateConnected)
    {
        cpLog (LOG_ERR, "Call state for %s is not connected", ep->epId.logData());
        return true;
//        break;
    }
    // get digit from sip message here
    int i = 0;
    if ( !ep->sipMsg )
    {
        cpLog( LOG_DEBUG, "The endpoint ep->sipMsg == 0");
        return false;
    }

    vector < SipContentData* > dataList = ep->sipMsg->getContentDataList();
    vector < SipContentData* > ::iterator iter = dataList.begin();
    while ( iter != dataList.end() )
    {
        Data mediaType = ((*iter)->getCtype()).getMediatype();
        if ( mediaType == APPLICATION_MGCP )
        {
            SipContentData* content = (*iter);
            SipMgcp* sipMgcpContent = dynamic_cast < SipMgcp* > ( content );
            if ( sipMgcpContent != 0 )
            {
                // Get MGCP Notify
                sipmgcp::MgcpSession mgcpSession = sipMgcpContent->getMgcpDescriptor();
                Sptr < sipmgcp::MgCommand > mgCmd = mgcpSession.getMgCmd();
                Sptr < sipmgcp::MgNotify > mgNotify;
                mgNotify.dynamicCast( mgCmd );
                if ( mgNotify != 0 )
                {
                    vector < Sptr < sipmgcp::MgNotificationEvent > > dtmfEvents = mgNotify->getObservedEventsList();
                    vector < Sptr < sipmgcp::MgNotificationEvent > > ::iterator dtmfIter = dtmfEvents.begin();
                    while ( dtmfIter != dtmfEvents.end() )
                    {
                        // DeviceSignalType dtmfDigit;
                        if ( sipmgcp::MgcpDTMFPackageType != (*dtmfIter)->getEvent().getPackageType() )
                        {
                            dtmfIter++;
                            continue;
                        }

                        sipmgcp::MgcpDTMFPackageItemType dtmfType = (*dtmfIter)->getEvent().getDTMFPackageType();
                        char digit = ' ';
                        switch ( dtmfType )
                        {
                            case sipmgcp::MgcDTMFPackageDTMF_Star:
                            // dtmfDigit = DeviceSignalDigitStar;
                            digit = '*';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_Hash:
                            // dtmfDigit = DeviceSignalDigitHash;
                            digit = '#';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_0:
                            // dtmfDigit = DeviceSignalDigit0;
                            digit = '0';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_1:
                            // dtmfDigit = DeviceSignalDigit1;
                            digit = '1';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_2:
                            // dtmfDigit = DeviceSignalDigit2;
                            digit = '2';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_3:
                            // dtmfDigit = DeviceSignalDigit3;
                            digit = '3';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_4:
                            // dtmfDigit = DeviceSignalDigit4;
                            digit = '4';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_5:
                            // dtmfDigit = DeviceSignalDigit5;
                            digit = '5';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_6:
                            // dtmfDigit = DeviceSignalDigit6;
                            digit = '6';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_7:
                            // dtmfDigit = DeviceSignalDigit7;
                            digit = '7';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_8:
                            // dtmfDigit = DeviceSignalDigit8;
                            digit = '8';
                            break;
                            case sipmgcp::MgcDTMFPackageDTMF_9:
                            // dtmfDigit = DeviceSignalDigit9;
                            digit = '9';
                            break;
                            default:
                            dtmfIter++;
                            continue;
                            break;
                        }
                        cpLog(LOG_DEBUG, "DTMF digit %c has been received", digit);
                        ep->call->playDtmfDigit = digit;
                        sendInternalEvent (ep, CallEventDigitsCollected);
                    }
                }
            }
            else
            {
                cpLog( LOG_ERR, "Content data %d is not MGCP", i );
            }
        }
        i++;
        iter++;
    }
    sipEventSend (ep, SipEventSendInfoResponse);
// InfoMsg is no longer used to carry dtmf digits, remove current code for now
*/
    return true;
}

bool processSipEventReceiveInfoResponse(Endpoint* ep)
{
    cout << "*****************" << endl;
    cout << "RECEIVED A SIP DIGIT RESPONSE MESSAGE ... DO NOTHING" << endl;
    cout << "*****************" << endl;
    return true;
}

// procedures for sipStackEvent
void processInviteMsg(Endpoint* ep, SipMsg* msg, InviteMsg* inviteMsg)
{
    if (ep)
    {

        bool containSdpHold = false;
        Sptr<SipContentData> sipContent = msg->getContentData(0);
        Sptr<SipSdp> sipsdp;
	sipsdp.dynamicCast (sipContent);

        if (sipsdp != 0)
        {
            containSdpHold = (sipsdp->getSdpDescriptor()).isHold();
        }

        if ( (ep->state == CallStateAlerting) || (ep->state == CallStateConnected) )
        {

            // this Invite has Sdp=0, meaning it's not a new Invite
            // but rather a Hold Invite to put this endpoint on hold
            // Later we want to check to see if it is from the same source that wants to put us on hold
            // because we don't want to be put on hold from different source than who we were ringing for
            // or talking to.  TBD
            if ( containSdpHold )
            {
                // we need to make sure that this hold invite is NOT from the party C (one that receives transfer) doing
                // a flash

                // if we have a held call and we are talking with the active call (party C), and if this hold
                // invite is from the active call, it means that party C did a flash trying to put us on hold.
                // if so, we should reject this invite by sending back a 100 and then a 486, and not updating our
                // sipMsg pointer so that we don't lose the previous 200 with the call, and can send a Bye later
                // if our side hangs up so that things can get cleaned up properly
                if ( ep->sipMsgHeld != 0 )
                {
                    StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
                    if ( statusMsg != 0 )
                    {
                        int statusCode = 0;
                        statusCode = statusMsg->getStatusLine().getStatusCode();
                        if ( statusCode == 200 )
                        {
                            Sptr<BaseUrl> baseStatusToUrl = (statusMsg->getTo()).getUrl();
                            Sptr<BaseUrl> baseInviteFromUrl = (inviteMsg->getFrom()).getUrl();

			    Sptr<SipUrl> statusToUrl;
			    Sptr<SipUrl> inviteFromUrl;
			    
			    statusToUrl.dynamicCast( baseStatusToUrl );
			    inviteFromUrl.dynamicCast( baseInviteFromUrl );

			    assert( statusToUrl != 0 );
			    assert( inviteFromUrl != 0 );

                            if ( *statusToUrl == *inviteFromUrl )
                            {
                                // reject this hold invite by sending back a 100 and 486,
                                // but not updating sipMsg pointer
                                SipMsg * temp = ep->sipMsg;
                                ep->sipMsg = msg;
                                sipEventSend (ep, SipEventSendTrying);
                                sipEventSend (ep, SipEventSendBusy);

                                // free the memory for msg before restoring ep->sipMsg
                                if ( ep->sipMsg != 0 )
                                {
                                    delete( ep->sipMsg );
                                    ep->sipMsg = 0;
                                }
                                ep->sipMsg = temp;

                                return ;
                            }
                        }
                    }

                }

                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                }
                ep->sipMsg = msg;
                sipEventReceive (ep, SipEventReceiveInvite);
            }
            else
            {
                InviteMsg* previnviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
                if ( previnviteMsg )
                {
                    //  we received two Invites, if they are from different sources, we
                    // are getting (receiving) a transfer or a call waiting call.
                    // make sure the From fields of these two Invites are different
                    Sptr<BaseUrl> basefrom1 = (previnviteMsg->getFrom()).getUrl();
                    Sptr<BaseUrl> basefrom2 = (msg->getFrom()).getUrl();

		    Sptr<SipUrl> from1;
		    Sptr<SipUrl> from2;

		    from1.dynamicCast( basefrom1);
		    from2.dynamicCast( basefrom2);

		    assert( from1 != 0 );
		    assert( from2 != 0 );

                    SipCallId callId1 = previnviteMsg->getCallId();
                    SipCallId callId2 = msg->getCallId();

                    if ( !( *from1 == *from2 ) )
                    {

                        if ( callId1 == callId2 )
                        {

                            // they are from different sources, so make the previous sipMsg to be the held
                            // to make room for the new incoming Invite
                            if ( ep->sipMsgHeld != 0 )
                            {
                                delete( ep->sipMsgHeld );
                                ep->sipMsgHeld = 0;
                            }
                            ep->sipMsgHeld = ep->sipMsg;
                            ep->sipMsg = msg;


                            // swap the route list and save the new Invite's route list
                            vector < SipRoute* > * temp = ep->sipMsgHeldRouteList;
                            ep->sipMsgHeldRouteList = ep->sipMsgRouteList;

                            clearRouteList( temp );
                            ep->sipMsgRouteList = temp;

                            saveRouteList( ep->sipMsgRouteList, (msg->getrecordrouteList()), 1 );
                            int numContact = msg->getNumContact();
                            if ( numContact )
                            {
                                SipContact contact = msg->getContact( numContact - 1 );
                                SipRoute* route = new SipRoute;
                                assert( route != 0 );
                                route->setUrl( contact.getUrl() );
                                assert( ep->sipMsgRouteList != 0 );
                                ep->sipMsgRouteList->push_back( route );
                            }


                            sipEventReceive (ep, SipEventReceiveInvite);

                        }
                        else
                        {
                            // Check Call Waiting
                            if (DoCallWaiting(ep, msg) == true )
                            {
                                return ;
                            }

                            cpLog( LOG_ERR, "Abnormal: two invites are from different sources, but callids are different");
                            cpLog( LOG_ERR, "The endpoint is NOT in a state to receive such an Invite !! Sending back a 486 for this Invite !!");

                            StatusMsg status( *inviteMsg, 486/*busy*/ );
                            assert( sipProxyStack );
                            sipProxyStack->sendReply(status);

                            // free the invite message
                            delete( inviteMsg );
                        }
                    }
                    else
                    {

                        if ( callId1 == callId2 )
                        {
                            //  we received two invites from the same source and they have the same call id
                            //  check to see if the first invite is a hold Invite (sdp==0)
                            //  and the second invite contains a valid sdp, if so, this means the transferring
                            //  party (B) is doing a second flash, trying to get back to A.
                            //  upon receiving such a second invite, A should modify its connection to talk to B
                            bool prevContainSdpHold = false;
                            if ( !ep->sipMsg )
                            {
                                cpLog( LOG_DEBUG, "The endpoint ep->sipMsg == 0 ");
                                return ;
                            }

                            Sptr<SipContentData> prevSipContent = ep->sipMsg->getContentData(0);
                            Sptr<SipSdp> prevSipSdp;
			    prevSipSdp.dynamicCast (prevSipContent);

                            if (prevSipSdp != 0)
                            {
                                prevContainSdpHold = (prevSipSdp->getSdpDescriptor()).isHold();
                            }

                            if ( prevContainSdpHold && !( containSdpHold ) )
                            {
                                // A should modify its connection from inactive to sendrecv to talk to B
                                if ( ep->sipMsg != 0 )
                                {
                                    delete( ep->sipMsg );
                                    ep->sipMsg = 0;
                                }
                                ep->sipMsg = msg;
                                sipEventSend (ep, SipEventSendTrying);

                                // set the Sdp info for the far end
                                if ( !sipsdp )
                                {
                                    cpLog( LOG_DEBUG, "No Sdp info in the message");
                                    return ;
                                }

                                SdpSession sdp = convert(sipsdp->getSdpDescriptor());
                                if ( !ep->call )
                                {
                                    cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
                                    return ;
                                }

                                ep->call->endpoint[1].localConnection = sdp;

                                sendInternalEvent( ep, CallEventMDCX );

                                // send 200 answer back
                                sipEventSend(ep, SipEventSendAnswer );
                            }
                            else
                            {
                                cpLog( LOG_ERR, "Two Invites from the same source with the same call id, but the previous invite is NOT a hold invite and the second invite does NOT contain a valid Sdp !!");
                                // free the invite message
                                delete( inviteMsg );
                            }
                        }
                        else
                        {    // from the same source with valid sdp,
                            // but with different call ids, reject the invite by sending a 486
                            // because we are either in alerting or connected state


                            cpLog( LOG_ERR, "Abnormal: two invites are from the same source, but with different call ids !! Sending back a 486 !!");
                            StatusMsg status( *inviteMsg, 486/*busy*/ );
                            assert( sipProxyStack );
                            sipProxyStack->sendReply(status);

                            // free the invite message
                            delete( inviteMsg );
                        }
                    }

                }
                else // Is this an error condition ??
                {


                    // Check Call Waiting
                    if ( DoCallWaiting(ep, msg) == true )
                    {
                        return ;
                    }

                    cpLog ( LOG_ERR, "No previous invite message was found !! Sending back a 486 !!");

                    StatusMsg status( *inviteMsg, 486/*busy*/ );
                    assert( sipProxyStack );
                    sipProxyStack->sendReply(status);

                    // free the invite message
                    delete( inviteMsg );

                    //                      sipEventSend (ep, SipEventSendBusy);
                    //    StatusMsg status( *inviteMsg, 486/*busy*/ );

                    // for the internal trial
                    /*
                     string tempEpId = translate2sip (ep->dn, ca.sipMyHost);
                     SipUrl url ( tempEpId );
                     url.setHost( ca.sipMyHost);
                     url.setPort( ca.sipMyPort );
                     url.setUserParm( "phone" );
                     SipContact contact;
                     contact.setUrl (url);
                     status.setContact (contact);

                     assert( sipProxyStack );
                    sipProxyStack->sendReply(status);*/



                }
            }

        }
        else if (ep->state == CallStateIdle ) // call state is Idle, this is a new call
        {
            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }
            ep->sipMsg = msg;

            ep->call = newCallData(ep);
            assert( ep->call != 0 );
            ep->call->endpoint[0].id = ep->id;
            sprintf (ep->call->callIdentifier, "%x", newCallIdentifier());


            // got a first Invite in Idle state, save the Record Route info
            saveRouteList( ep->sipMsgRouteList, (msg->getrecordrouteList()), 1 );
            int numContact = msg->getNumContact();
            if ( numContact )
            {
                SipContact contact = msg->getContact( numContact - 1 );
                SipRoute* route = new SipRoute;
                assert( route != 0 );
                route->setUrl( contact.getUrl() );
                assert( ep->sipMsgRouteList != 0 );
                ep->sipMsgRouteList->push_back( route );
            }


            sipEventReceive (ep, SipEventReceiveInvite);
        }
        else
        {
            // The endpoint is NOT in a state to receive Invite
            cpLog( LOG_DEBUG, "The endpoint is NOT in a state to receive invite !!");
            StatusMsg status( *inviteMsg, 486/*busy*/ );
            assert( sipProxyStack );
            sipProxyStack->sendReply(status);

            // free the invite message
            delete( inviteMsg );
        }
    }
    else
    {
        // VON invalid DN reponse ... just pretend it is busy for now?
        // sipEventSend (ep, SipEventSendBusy);
        StatusMsg status( *inviteMsg, 486/*busy*/ );
        assert( sipProxyStack );
        sipProxyStack->sendReply(status);

        // free the invite message
        delete( inviteMsg );
    }
    return ;
} // inviteMsg

void processXferMsg(Endpoint* ep, SipMsg* msg, TransferMsg* xferMsg)
{
    if (ep)
    {
        // save the transfer message to wait for a 200 for it
        if ( ep->sipMsgXfer != 0 )
        {
            delete( ep->sipMsgXfer );
            ep->sipMsgXfer = 0;
        }
        ep->sipMsgXfer = msg;
        // now swap the active call with the held call sip messages
        // before we send Invite to the far end

        if ( ep->sipMsgHeld != 0 );
        {
            delete( ep->sipMsgHeld );
            ep->sipMsgHeld = 0;
        }

        ep->sipMsgHeld = ep->sipMsg;
        ep->sipMsg = 0;


        // swap the route list as well
        vector < SipRoute* > * temp = ep->sipMsgHeldRouteList;
        ep->sipMsgHeldRouteList = ep->sipMsgRouteList;
        clearRouteList( temp );
        ep->sipMsgRouteList = temp;


        sipEventReceive (ep, SipEventReceiveTransfer);
    }
    else
    {
        cpLog( LOG_DEBUG, "An unknown endpoint in Transfer message");
        // free the transfer message
        delete( xferMsg );
    }
    return ;
} // xferMsg

void processAckMsg(Endpoint* ep, AckMsg* ackMsg)
{
    if (ep)
    {
        // Tempoarily don't save the ACK message.  Keep the original INVITE
        // ep->sipMsg = msg;
        sipEventReceive (ep, SipEventReceiveAck);
    }
    else
    {
        cpLog( LOG_DEBUG, "An unknown endpoint in Ack message");
    }

    // free the Ack message
    delete( ackMsg );
    return ;
} // ackMsg

void processByeMsg(Endpoint* ep, SipMsg* msg, ByeMsg* byeMsg)
{
    if (ep)
    {

        Sptr<BaseUrl> baseByeFromUrl = (byeMsg->getFrom()).getUrl();
	Sptr<SipUrl> byeFromUrl;
	byeFromUrl.dynamicCast( baseByeFromUrl );
	assert( byeFromUrl != 0 );

        if ( ep->sipMsgHeld )
        {
            InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsgHeld);
            if ( inviteMsg )
            {
                Sptr<BaseUrl> baseInviteFromUrl = (inviteMsg->getFrom()).getUrl();
		Sptr<SipUrl> inviteFromUrl;
		inviteFromUrl.dynamicCast( baseInviteFromUrl );
		assert( inviteFromUrl != 0 );

                if ( *inviteFromUrl == *byeFromUrl )
                {
                    // bye is from the same source as the held Invite
                    // meaning the far end of held party is byeing, so clean up
                    // Note that sending a bye or receiving a bye is when we need to clean
                    // up the sipMsg or sipMsgHeld pointer, depending who we are sending to or receiving from
                    if ( ep->sipMsgHeld != 0 )
                    {
                        delete( ep->sipMsgHeld );
                        ep->sipMsgHeld = 0;
                    }


                    // clean up the route list for the held party
                    clearRouteList( ep->sipMsgHeldRouteList );


                    // send a 200 back for the Bye
                    // keep the sipMsg pointer and take advantage of
                    // SipEventSendByeResponse procedure
                    SipMsg * temp = ep->sipMsg;
                    ep->sipMsg = msg;
                    sipEventSend (ep, SipEventSendByeResponse);

                    // free the memory for msg before restoring ep->sipMsg
                    if ( ep->sipMsg != 0 )
                    {
                        delete( ep->sipMsg );
                        ep->sipMsg = 0;
                    }
                    ep->sipMsg = temp;
                    return ;
                }
            }

            StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsgHeld);
            if ( statusMsg )
            {
	        //SipUrl statusToUrl = (statusMsg->getTo()).getUrl();
	        Sptr<BaseUrl> baseStatusToUrl = (statusMsg->getTo()).getUrl();
		Sptr<SipUrl> statusToUrl;
		statusToUrl.dynamicCast( baseStatusToUrl );
		assert( statusToUrl != 0 );
                
                // check to see if the Bye is from the held party that was connected before
                if ( *statusToUrl == *byeFromUrl )
                {
                    if ( ep->sipMsgHeld != 0 )
                    {
                        delete( ep->sipMsgHeld );
                        ep->sipMsgHeld = 0;
                    }


                    // clean up the route list for the held party
                    clearRouteList( ep->sipMsgHeldRouteList );


                    // send a 200 back for the Bye
                    // keep the sipMsg pointer and take advantage of
                    // SipEventSendByeResponse procedure
                    SipMsg * temp = ep->sipMsg;
                    ep->sipMsg = msg;
                    sipEventSend (ep, SipEventSendByeResponse);

                    // free the memory for msg before restoring ep->sipMsg
                    if ( ep->sipMsg != 0 )
                    {
                        delete( ep->sipMsg );
                        ep->sipMsg = 0;
                    }
                    ep->sipMsg = temp;

                    // This should delete the held call on the mgcp side
                    if (ep->heldCall)
                    {
                        sendInternalEvent (ep,
                                           CallEventDeleteCall,
                                           ep->heldCall->callIdentifier);

                    }
                    else
                    {
                        cout << "BUT THERE WAS NO CALL TO CANCEL" << endl;
                    }

                    return ;
                }
            }
        }
        else  // no sipMsgHeld, check for the active sipMsg
        {
            StatusMsg* statusMsg = dynamic_cast < StatusMsg* > (ep->sipMsg);
            if ( statusMsg )
            {
	        //SipUrl statusToUrl = (statusMsg->getTo()).getUrl();
	        Sptr<BaseUrl> baseStatusToUrl = (statusMsg->getTo()).getUrl();
		Sptr<SipUrl> statusToUrl;
		statusToUrl.dynamicCast( baseStatusToUrl );
		assert( statusToUrl != 0 );

                // check to see if the Bye is from the held party that was connected before
                if ( *statusToUrl == *byeFromUrl )
                {
                    // send a 200 back for the Bye
                    if ( ep->sipMsg != 0 )
                    {
                        delete( ep->sipMsg );
                        ep->sipMsg = 0;
                    }
                    ep->sipMsg = msg;
                    sipEventSend (ep, SipEventSendByeResponse);


                    if ( ep->sipMsg != 0 )
                    {
                        delete( ep->sipMsg );
                        ep->sipMsg = 0;
                    }


                    // clean up the route list for the held party
                    clearRouteList( ep->sipMsgRouteList );


                    // This should delete the held call on the mgcp side
                    if (ep->heldCall)
                    {
                        sendInternalEvent (ep,
                                           CallEventDeleteCall,
                                           ep->heldCall->callIdentifier);

                    }
                    else
                    {
                        cout << "BUT THERE WAS NO CALL TO CANCEL" << endl;
                    }

                    return ;
                }

            }
        }

        if ( ep->sipMsg != 0 )
        {
            delete( ep->sipMsg );
            ep->sipMsg = 0;
        }
        ep->sipMsg = msg;
        sipEventReceive (ep, SipEventReceiveBye);
    }
    else
    {
        cpLog( LOG_DEBUG, "An unknown endpoint in Bye message");

        // free the Bye message
        delete( byeMsg );
    }

    return ;
} // byeMsg

void processCancelMsg(Endpoint* ep, SipMsg* msg, CancelMsg* cancelMsg)
{
    if (ep)
    {

        // If there is a held call, check to see if this Bye is
        // from the held party, if so, clean up and don't do anything
        //SipUrl cancelFromUrl = ( cancelMsg->getFrom()).getUrl();
        Sptr<BaseUrl> baseCancelFromUrl = ( cancelMsg->getFrom()).getUrl();
	Sptr<SipUrl> cancelFromUrl;
	cancelFromUrl.dynamicCast( baseCancelFromUrl );
	assert( cancelFromUrl != 0 );

        if ( ep->sipMsgHeld )
        {
            InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsgHeld);
            if ( inviteMsg )
            {
	        //SipUrl inviteFromUrl = ( inviteMsg->getFrom()).getUrl();
	        Sptr<BaseUrl> baseInviteFromUrl = ( inviteMsg->getFrom()).getUrl();
		Sptr<SipUrl> inviteFromUrl;
		inviteFromUrl.dynamicCast( baseInviteFromUrl );
		assert( inviteFromUrl != 0 );

                if ( *inviteFromUrl == *cancelFromUrl )
                {
                    // cancel is from the same source as the held Invite
                    // meaning the far end of held party is byeing, so clean up
                    if ( ep->sipMsgHeld != 0 )
                    {
                        delete( ep->sipMsgHeld );
                        ep->sipMsgHeld = 0;
                    }


                    // also clean up the route list of the held party
                    clearRouteList( ep->sipMsgHeldRouteList );


                    // send a 200 back for the Cancel
                    // keep the sipMsg pointer and take advantage of
                    // SipEventSendByeResponse procedure
                    SipMsg * temp = ep->sipMsg;
                    ep->sipMsg = msg;
                    sipEventSend (ep, SipEventSendCancelResponse);

                    // free the memory before restoring ep->sipMsg
                    if ( ep->sipMsg != 0 )
                    {
                        delete( ep->sipMsg );
                        ep->sipMsg = 0;
                    }
                    ep->sipMsg = temp;

                    // This should delete the held call on the mgcp side
                    if (ep->heldCall)
                    {
                        sendInternalEvent (ep,
                                           CallEventDeleteCall,
                                           ep->heldCall->callIdentifier);

                    }
                    else
                    {
                        cout << "BUT THERE WAS NO CALL TO CANCEL" << endl;
                    }
                    return ;
                }
            }
        }


        if ( ep->sipMsg != 0 )
        {
            delete( ep->sipMsg );
            ep->sipMsg = 0;
        }
        ep->sipMsg = msg;
        sipEventReceive (ep, SipEventReceiveCancel);
    }
    else
    {
        cpLog( LOG_DEBUG, "An unknown endpoint in cancel message");

        // free the cancel message
        delete( cancelMsg );
    }

    return ;
} // cancelMsg

void processOptionsMsg(Endpoint* ep, OptionsMsg* optionsMsg)
{
    if (ep)
    {
        // ep->sipMsg = msg;
        // sipEventReceive (ep, SipEventReceiveOptions);
        cpLog (LOG_ERR, "GOT AN OPTIONS MESSAGE ... IGNORED FOR NOW");

        // free the options message
        delete( optionsMsg );
    }
    else
    {
        cpLog( LOG_DEBUG, "An unknown endpoint in options message");

        // free the options message
        delete( optionsMsg );
    }

    return ;
} // optionsMsg

void processInfoMsg(Endpoint* ep, SipMsg* msg, InfoMsg* infoMsg)
{
    if (ep)
    {
        // ep->sipMsg = msg;
        SipMsg * temp = ep->sipMsg;
        ep->sipMsg = msg;
        sipEventReceive (ep, SipEventReceiveInfo);

        if ( ep->sipMsg != 0 )
        {
            delete( ep->sipMsg );
            ep->sipMsg = 0;
        }
        ep->sipMsg = temp;
    }
    else
    {
        cpLog( LOG_DEBUG, "An unknown endpoint in info message");

        // free the info message
        delete( infoMsg );
    }

    return ;
} // infoMsg

void processStatusMsg(Endpoint* ep, SipMsg* msg, StatusMsg* statusMsg)
{
    if (ep)
    {
        Data method = statusMsg->getCSeq().getMethod();
        int status = statusMsg->getStatusLine().getStatusCode();
        if (method == REGISTER_METHOD)
        {
            //SipMsg* currentSipMsg = ep->sipMsg;
            //ep->sipMsg = msg;
            if (status == 200)
            {
                //sipEventReceive(ep, SipEventReceiveRegisterResponse);

                cout << "*****************" << endl;
                cout << "RECEIVED A RESPONSE TO THE REGISTER MESSAGE" << endl;
                cout << "*****************" << endl;

                if (ep->state == CallStateInit)
                {
                    // set the register flag to false and skip inserting
                    // expire timer into timeout queue for register message
                    // instead, rely on RQNT 200 response to send register
                    // message again

                    ep->registered = false;

                    delete( msg );
                    msg = 0;
                    return ;
                }
                // Data expires
                int expires = statusMsg->getExpires().getDelta().convertInt();
                cout << "expires = " << expires << endl;
                cout << "ca.registerRetrans = " << ca.registerRetrans << endl;
                if (expires > ca.registerRetrans)
                {
                    expires = ca.registerRetrans;
                }
                QueueEventData* ed = newQueueEventData ();
                assert( ed != 0 );
                ed->type = TimeoutEvent;
                ed->ep = ep;
                ed->featureId = FeatureEventSipRegister;
                // Should the next REGISTER message to be sent a little sooner
                // to eliminate even a small window of being un-registered?
                assert( ep->iq != 0 );
                ep->iq->addDelayMs(ed, expires * 1000);
            }
            else
            {
                cpLog (LOG_ERR, "STATUS != 200 received for REGISTER");
            }

            delete( msg );
            msg = 0;
            //ep->sipMsg = currentSipMsg;
            return ;
        }
        cout << "THE STATUS MESSAGE WAS A " << status << endl;

        // process any status code above 400 here
        if ( ( status >= 400 ) && ( status <= 699 ) )
        {
            // check to see if this is a 486 fail response for a hold invite we sent out,
            // if so, it means our hold invite was rejected.
            // In this case, we do NOT give busy tone, and pretend nothing happened
            // by reverting back to the previous message before the hold invite we sent
            // to maintain speech path until the other side send us Bye to clean
            // up things
            bool holdInviteRejected = false;
            bool containSdpHold = false;
            InviteMsg* prevInviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
            if ( prevInviteMsg )
            {
                Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
                Sptr<SipSdp> sipsdp;
		sipsdp.dynamicCast (sipContent);
                if (sipsdp != 0)
                {
                    SdpSession sdp = convert(sipsdp->getSdpDescriptor());
                    containSdpHold = sdp.isHold();
                }

                if ( containSdpHold && ( status == 486 ) )
                {
                    holdInviteRejected = true;
                }
            }

            if ( ep->sipMsg != 0 )
            {
                delete( ep->sipMsg );
                ep->sipMsg = 0;
            }
            ep->sipMsg = msg;

            // if the previous message was NOT a hold invite, save route list
            // into a temporay place, and swap back after sending ack
            sipEventSend (ep, SipEventSendAck);

            if ( holdInviteRejected == false )
            {
                sipEventReceive (ep, SipEventReceiveBusy);
            }
            else if ( ep->sipMsgBuf != 0 )
            {
                // previous hold Invite was rejected by a 486, revert back
                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                }

                ep->sipMsg = ep->sipMsgBuf;
                ep->sipMsgBuf = 0;

                // if the ep has a heldCall which points to the same
                // call as the active call because of the first flash
                // clean up the heldCall pointer for the mgcp side because
                // it doesn't know that the flash (hold invite) failed.
                if ( ep->heldCall == ep->call )
                {
                    ep->heldCall = 0;
                }
            }
            return ;
        }

        switch ( status )
        {
            case 100:
            {
                sipEventReceive (ep, SipEventReceiveTrying);

                // free the 100 status message
                delete( statusMsg );
            }
            break;
            case 180:
            case 182:
            /* activate this block if later on we decide to use 180 with no Sdp for late RTP
                 and use 183 with Sdp for early RTP
                 for now, we assume both 180 and 183 carry Sdp info and it is up to the mgcp side
                 to check for sdp in localConnection to decide whether to give local ringback (no Sdp)
                 or to create rtp path to listen to remote ringback
                          {
                              sipEventReceive (ep, SipEventReceiveRinging);
                          }
                          break;
            */
            case 183:
            {
                // the previous message has to be an invite sent out
                InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
                if ( inviteMsg )
                {
                    bool containSdpHold = false;
                    Sptr<SipContentData> sipContent = inviteMsg->getContentData(0);
                    Sptr<SipSdp> sipsdp;
		    sipsdp.dynamicCast (sipContent);

                    if (sipsdp != 0)
                    {
                        // check to see if the previous invite was a hold invite
                        containSdpHold = (sipsdp->getSdpDescriptor()).isHold();
                    }
                    else
                    {
                        cpLog( LOG_ERR, "The previous invite did not contain Sdp info !!");
                    }

                    if ( containSdpHold )  // the previous Invite was a hold Invite
                    {
                        if ( ep->callType != BlindXfer )
                        {
                            // set the far end sdp to 0, not done for the blind transfer case
                            // since the call may be cleaned up already
                            if ( !ep->call )
                            {
                                cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
                                return ;
                            }

                            ((ep->call->endpoint[1]).localConnection).setHold();
                        }
                    }
                    else  // a 183 for a normal Invite
                    {
                        Sptr<SipContentData> sipContent = msg->getContentData(0);
                        Sptr<SipSdp> sipsdp;
			sipsdp.dynamicCast (sipContent);

                        if (sipsdp != 0)
                        {
                            SdpSession sdp = convert(sipsdp->getSdpDescriptor());
                            if ( !ep->call )
                            {
                                cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
                                delete( msg );
                                return ;
                            }

                            ep->call->endpoint[1].localConnection = sdp;
                        }
                        else
                        {
                            // cpLog( LOG_ERR, "This 183 does not contain Sdp info !! ");
                            cpLog( LOG_INFO, "This 180 or 183 does not contain Sdp info !! ");
                        }
                    }

                    sipEventReceive (ep, SipEventReceiveRinging);
                }
                else
                {
                    cpLog( LOG_ERR, "The previous message was NOT an invite !!");
                }

                // free the 180, 182 or 183 status message
                delete( statusMsg );
            }
            break;
            case 200:
            {

                Data method = statusMsg->getCSeq().getMethod();

                if ( method == TRANSFER_METHOD )    // a 200 for a Transfer
                {
                    TransferMsg* xferMsg = dynamic_cast < TransferMsg* > (ep->sipMsgXfer);
                    if ( xferMsg )
                    {
                        if ( ep->sipMsgXfer != 0 )
                        {
                            delete( ep->sipMsgXfer );
                            ep->sipMsgXfer = 0;
                        }
                        ep->sipMsgXfer = msg;
                        sipEventReceive (ep, SipEventReceiveTransferResponse);

                    }
                    else
                    {
                        cpLog( LOG_DEBUG, "got a 200 for a transfer message, but no transfer message was found !!");

                        // free the 200 for transfer status message
                        delete( statusMsg );
                    }
                    break;

                }

                if ( method == INVITE_METHOD )    // a 200 for an Invite
                {
                    InviteMsg* inviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
                    if ( inviteMsg )
                    {
                        bool containSdpHold = false;
                        Sptr<SipContentData> sipContent = inviteMsg->getContentData(0);
                        Sptr<SipSdp> sipsdp;
			sipsdp.dynamicCast (sipContent);

                        if (sipsdp != 0)
                        {
                            containSdpHold = (sipsdp->getSdpDescriptor()).isHold();
                        }

                        // if the previous message was a hold Invite, this 200 means the hold
                        // invite is successful, we can delete sipMsgBuf pointed message
                        if ( containSdpHold )
                        {
                            if ( ep->sipMsgBuf != 0 )
                            {
                                delete( ep->sipMsgBuf );
                                ep->sipMsgBuf = 0;
                            }
                        }

                        if ( ep->sipMsg != 0 )
                        {
                            delete( ep->sipMsg );
                            ep->sipMsg = 0;
                        }
                        ep->sipMsg = msg;


                        if ( containSdpHold )  // the previous Invite was a hold Invite
                        {
                            if (( ep->callType == ConsultationXfer ) || (ep->callType == Conference))
                            {
                                // for consultation transfer, send an Ack here
                                sipEventSend (ep, SipEventSendAck);
                            }
                            else
                            {
                                // set the far end sdp to 0, not done for the consultation case
                                // since the call may be cleaned up already
                                if ( !ep->call )
                                {
                                    cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
                                    return ;
                                }

                                ((ep->call->endpoint[1]).localConnection).setHold();
                            }
                        }
                        else  // a 200 for a normal Invite
                        {
                            Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
                            Sptr<SipSdp> sipsdp;
			    sipsdp.dynamicCast (sipContent);

                            if (sipsdp != 0)
                            {
                                SdpSession sdp = convert(sipsdp->getSdpDescriptor());
                                if ( !ep->call )
                                {
                                    cpLog( LOG_DEBUG, "The endpoint does not have an active call.");
                                    return ;
                                }

                                ep->call->endpoint[1].localConnection = sdp;
                            }

                            if (!((ep->state == CallStateDialing) ||
                                    ((ep->state == CallStateConnected) && (ep->sipMsgHeld == 0))))
                            {
                                // if the ep state is in Dialing, we don't want to update route list because
                                // it is abnormal case to reinvite the held party into connection
                                // only if the state is either in FarEndAlerted or Connected, we do update route list


                                // a 200 back for a first invite going out, save the record route list
                                // first clean up before save new route list, in case transferring party
                                // B flashes second time to re-invite held party A and get a 200 back for that
                                clearRouteList( ep->sipMsgRouteList );

                                saveRouteList( ep->sipMsgRouteList, (msg->getrecordrouteList()), 0 );
                                int numContact = msg->getNumContact();
                                if ( numContact )
                                {
                                    SipContact contact = msg->getContact( numContact - 1 );
                                    SipRoute* route = new SipRoute;
                                    assert( route != 0 );
                                    route->setUrl( contact.getUrl() );
                                    assert( ep->sipMsgRouteList != 0 );
                                    ep->sipMsgRouteList->push_back( route );
                                }

                                printRouteList( ep->sipMsgRouteList );

                            }
                        }

                        sipEventReceive (ep, SipEventReceiveAnswer);

                    }
                    break;

                }

                if ( method == BYE_METHOD )   // a 200 for a Bye
                {
                    // check to see which Bye (active or possibly held) this is for
		    //SipUrl respTo = (msg->getTo()).getUrl();
		    Sptr<BaseUrl> baseRespTo = (msg->getTo()).getUrl();
		    Sptr<SipUrl> respTo;
		    respTo.dynamicCast( baseRespTo );
		    assert( respTo != 0 );

                    ByeMsg* byeMsg = dynamic_cast < ByeMsg* > (ep->sipMsg);
                    ByeMsg* byeMsgHeld = dynamic_cast < ByeMsg* > (ep->sipMsgHeld);

                    if ( byeMsg )
                    {
		        //SipUrl byeMsgTo = (ep->sipMsg->getTo()).getUrl();
		        Sptr<BaseUrl> baseByeMsgTo = (ep->sipMsg->getTo()).getUrl();
			Sptr<SipUrl> byeMsgTo;
			byeMsgTo.dynamicCast( baseByeMsgTo );
			assert( byeMsgTo != 0 );

                        //	SipUrl byeMsgHeldTo = (ep->sipMsgHeld->getTo()).getUrl();
                        if ( *byeMsgTo == *respTo )
                        {
                            // this must be a 200 for a Bye message
                            // we sent out previouly on the active call
                            if ( ep->sipMsg != 0 )
                            {
                                delete( ep->sipMsg );
                                ep->sipMsg = 0;
                            }
                            ep->sipMsg = msg;
                            sipEventReceive (ep, SipEventReceiveByeResponse);

                            // this active call is done, clean up
                            if ( ep->sipMsg != 0 )
                            {
                                delete( ep->sipMsg );
                                ep->sipMsg = 0;
                            }

                            break;
                        }

                    }

                    if ( byeMsgHeld )
                    {
		        //SipUrl byeMsgHeldTo = (ep->sipMsgHeld->getTo()).getUrl();
		        Sptr<BaseUrl> baseByeMsgHeldTo = (ep->sipMsgHeld->getTo()).getUrl();
			Sptr<SipUrl> byeMsgHeldTo;
			byeMsgHeldTo.dynamicCast( baseByeMsgHeldTo );
			assert( byeMsgHeldTo != 0 );

                        //	SipUrl byeMsgHeldTo = (ep->sipMsgHeld->getTo()).getUrl();
                        if ( *byeMsgHeldTo == *respTo )
                        {
                            // this must be a 200 for a Bye message
                            // we sent out previouly on the held call
                            if ( ep->sipMsgHeld != 0 )
                            {
                                delete( ep->sipMsgHeld );
                                ep->sipMsgHeld = 0;
                            }
                            ep->sipMsgHeld = msg;
                            sipEventReceive (ep, SipEventReceiveByeResponse);

                            // the held call is done, clean up
                            if ( ep->sipMsgHeld != 0 )
                            {
                                delete( ep->sipMsgHeld );
                                ep->sipMsgHeld = 0;
                            }

                            break;
                        }

                    }

                    // free the 200 for Bye message if not done so above
                    delete( statusMsg );
                    break;
                }

                if ( method == CANCEL_METHOD )    // a 200 for a Cancel
                {
                    CancelMsg* cancelMsg = dynamic_cast < CancelMsg* > (ep->sipMsg);
                    if ( cancelMsg )
                    {
                        // this must be a 200 for a cancel we sent out previously
                        if ( ep->sipMsg != 0 )
                        {
                            delete( ep->sipMsg );
                            ep->sipMsg = 0;
                        }
                        ep->sipMsg = msg;
                        sipEventReceive (ep, SipEventReceiveCancelResponse);

                        // this active call is done, clean up
                        if ( ep->sipMsg != 0 )
                        {
                            delete( ep->sipMsg );
                            ep->sipMsg = 0;
                        }

                        break;
                    }

                    // free the 200 for cancel message if not done so above
                    delete( statusMsg );
                    break;

                }

                if ( method == INFO_METHOD )    // a 200 for a Info message
                {
                    sipEventReceive(ep, SipEventReceiveInfoResponse);

                    // free the 200 for info message
                    delete( statusMsg );
                    break;
                }

                // if still not break out, free the un-processed status message
                delete( statusMsg );
            }
            break;
            case 300:
            case 302:
            {
                cout << "*****************" << endl;
                cout << "SEND A NEW INVITE DUE TO  A 3XX MESSAGE" << endl;
                cout << "*****************" << endl;
                SipContact sipContact = msg->getContact(0);
                //SipUrl contactUrl = sipContact.getUrl();
		Sptr<BaseUrl> baseContactUrl = sipContact.getUrl();
		Sptr<SipUrl> contactUrl;
		contactUrl.dynamicCast( baseContactUrl );
		assert( contactUrl != 0 );
		
                // Invite the new party via the local marshal
                contactUrl->setHost( ca.sipProxyHost );
                contactUrl->setPort( ca.sipProxyPort );

                if (ep->sipMsg == 0)
                {
                    // we no longer have the previous invite to redirect
                    // just send ack for the 302 and return
                    // free the memory for the 302 after sending the ack
                    ep->sipMsg = msg;
                    sipEventSend( ep, SipEventSendAck );
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                    cpLog (LOG_DEBUG, "the sipMsg is NULL when received 302");
                    break;
                }

                InviteMsg* invite = dynamic_cast<InviteMsg*>(ep->sipMsg);
                if (invite == 0)
                {
                    // it's not the invite message, return;
                    delete (ep->sipMsg);
                    cpLog (LOG_DEBUG, "Failed casting to InviteMsg when receive 302.");
                    ep->sipMsg = msg;
                    sipEventSend( ep, SipEventSendAck );
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                    break;
		    }

                InviteMsg* inviteMessage = new InviteMsg(
						   *invite,
                                               contactUrl);
                assert( inviteMessage != 0 );

                // via is default to 5060.  Change it here for now???
                SipVia sipVia = inviteMessage->getVia ();
                sipVia.setPort (ca.sipMyPort);
                inviteMessage->flushViaList();
                inviteMessage->setVia (sipVia, 0);

                bool containSdpHold = false;
                InviteMsg* prevInviteMsg = dynamic_cast < InviteMsg* > (ep->sipMsg);
                if ( prevInviteMsg )
                {
                    Sptr<SipContentData> sipContent = ep->sipMsg->getContentData(0);
                    Sptr<SipSdp> sipsdp;
		    sipsdp.dynamicCast (sipContent);
                    if (sipsdp != 0)
                    {
                        SdpSession sdp = convert(sipsdp->getSdpDescriptor());
                        containSdpHold = sdp.isHold();
                    }
                }

                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                }
                ep->sipMsg = msg;

                // save route list from the 302 from a previous invite ( NOT hold invite !)
                // swap the sipRouteList into a temporary pointer
                // just for sending the Ack back using saved route list
                sipEventSend (ep, SipEventSendAck);

                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                }
                ep->sipMsg = inviteMessage;
                sipProxyStack->sendAsync(*inviteMessage);

                if (ep->call)
                {
                    ep->call->ackReceived = false;
                }

            }
            break;
            default:
            {
                cpLog (LOG_WARNING, "Unknown response code %d", status);
                if ( ep->sipMsg != 0 )
                {
                    delete( ep->sipMsg );
                    ep->sipMsg = 0;
                }
                ep->sipMsg = msg;
                sipEventReceive (ep, SipEventReceiveBusy);   // overflow!
            } // default
        }  // switch

        return ;
    } // (ep)
    cpLog (LOG_WARNING, "Unknown endpoint in the status message.");

    // free the msg for the unknown endpoint
    delete( msg );
    return ;
} // statusMsg
